@@ -39,9 +39,9 @@ class BRUSH_EXPORT KisAutoBrushFactory : public KisBrushFactory
 public:
 
     KisAutoBrushFactory() {}
-    virtual ~KisAutoBrushFactory() {}
+    ~KisAutoBrushFactory() override {}
 
-    virtual QString id() const {
+    QString id() const override {
         return "auto_brush";
     }
 
@@ -50,7 +50,7 @@ public:
      * object. If this call leads to the creation of a resource, it should be
      * added to the resource provider, too.
      */
-    KisBrushSP getOrCreateBrush(const QDomElement& brushDefinition, bool forceCopy);
+    KisBrushSP createBrush(const QDomElement& brushDefinition) override;
 
 };
 
