@@ -3,7 +3,7 @@
    Copyright (C) 2006-2010 Thomas Zander <zander@kde.org>
    Copyright (C) 2006-2010 Thorsten Zachmann <zachmann@kde.org>
    Copyright (C) 2007-2009,2011 Jan Hambrecht <jaham@gmx.net>
-   CopyRight (C) 2010 Boudewijn Rempt <boud@kogmbh.com>
+   CopyRight (C) 2010 Boudewijn Rempt <boud@valdyas.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -70,8 +70,11 @@
 #include <QByteArray>
 #include <FlakeDebug.h>
 
+#include "kis_assert.h"
+
 #include <limits>
 #include "KoOdfGradientBackground.h"
+#include <KisHandlePainterHelper.h>
 
 // KoShapePrivate
 
@@ -79,12 +82,8 @@ KoShapePrivate::KoShapePrivate(KoShape *shape)
     : q_ptr(shape),
       size(50, 50),
       parent(0),
-      userData(0),
-      appData(0),
-      stroke(0),
       shadow(0),
       border(0),
-      clipPath(0),
       filterEffectStack(0),
       transparency(0.0),
       zIndex(0),
@@ -102,9 +101,7 @@ KoShapePrivate::KoShapePrivate(KoShape *shape)
       textRunAroundDistanceRight(0.0),
       textRunAroundDistanceBottom(0.0),
       textRunAroundThreshold(0.0),
-      textRunAroundContour(KoShape::ContourFull),
-      anchor(0),
-      minimumHeight(0.0)
+      textRunAroundContour(KoShape::ContourFull)
 {
     connectors[KoConnectionPoint::TopConnectionPoint] = KoConnectionPoint::defaultConnectionPoint(KoConnectionPoint::TopConnectionPoint);
     connectors[KoConnectionPoint::RightConnectionPoint] = KoConnectionPoint::defaultConnectionPoint(KoConnectionPoint::RightConnectionPoint);
@@ -113,24 +110,68 @@ KoShapePrivate::KoShapePrivate(KoShape *shape)
     connectors[KoConnectionPoint::FirstCustomConnectionPoint] = KoConnectionPoint(QPointF(0.5, 0.5), KoConnectionPoint::AllDirections, KoConnectionPoint::AlignCenter);
 }
 
+KoShapePrivate::KoShapePrivate(const KoShapePrivate &rhs, KoShape *q)
+    : q_ptr(q),
+      size(rhs.size),
+      shapeId(rhs.shapeId),
+      name(rhs.name),
+      localMatrix(rhs.localMatrix),
+      connectors(rhs.connectors),
+      parent(0), // to be initialized later
+      shapeManagers(), // to be initialized later
+      toolDelegates(), // FIXME: how to initialize them?
+      userData(rhs.userData ? rhs.userData->clone() : 0),
+      stroke(rhs.stroke),
+      fill(rhs.fill),
+      inheritBackground(rhs.inheritBackground),
+      inheritStroke(rhs.inheritStroke),
+      dependees(), // FIXME: how to initialize them?
+      shadow(0), // WARNING: not implemented in Krita
+      border(0), // WARNING: not implemented in Krita
+      clipPath(rhs.clipPath ? rhs.clipPath->clone() : 0),
+      clipMask(rhs.clipMask ? rhs.clipMask->clone() : 0),
+      additionalAttributes(rhs.additionalAttributes),
+      additionalStyleAttributes(rhs.additionalStyleAttributes),
+      filterEffectStack(0), // WARNING: not implemented in Krita
+      transparency(rhs.transparency),
+      hyperLink(rhs.hyperLink),
+
+      zIndex(rhs.zIndex),
+      runThrough(rhs.runThrough),
+      visible(rhs.visible),
+      printable(rhs.visible),
+      geometryProtected(rhs.geometryProtected),
+      keepAspect(rhs.keepAspect),
+      selectable(rhs.selectable),
+      detectCollision(rhs.detectCollision),
+      protectContent(rhs.protectContent),
+
+      textRunAroundSide(rhs.textRunAroundSide),
+      textRunAroundDistanceLeft(rhs.textRunAroundDistanceLeft),
+      textRunAroundDistanceTop(rhs.textRunAroundDistanceTop),
+      textRunAroundDistanceRight(rhs.textRunAroundDistanceRight),
+      textRunAroundDistanceBottom(rhs.textRunAroundDistanceBottom),
+      textRunAroundThreshold(rhs.textRunAroundThreshold),
+      textRunAroundContour(rhs.textRunAroundContour)
+{
+}
+
 KoShapePrivate::~KoShapePrivate()
 {
     Q_Q(KoShape);
-    if (parent)
+    if (parent) {
         parent->removeShape(q);
-    Q_FOREACH (KoShapeManager *manager, shapeManagers) {
-        manager->remove(q);
-        manager->removeAdditional(q);
     }
-    delete userData;
-    delete appData;
-    if (stroke && !stroke->deref())
-        delete stroke;
+
+    Q_FOREACH (KoShapeManager *manager, shapeManagers) {
+        manager->shapeInterface()->notifyShapeDestructed(q);
+    }
+    shapeManagers.clear();
+
     if (shadow && !shadow->deref())
         delete shadow;
     if (filterEffectStack && !filterEffectStack->deref())
         delete filterEffectStack;
-    delete clipPath;
 }
 
 void KoShapePrivate::shapeChanged(KoShape::ChangeType type)
@@ -138,31 +179,16 @@ void KoShapePrivate::shapeChanged(KoShape::ChangeType type)
     Q_Q(KoShape);
     if (parent)
         parent->model()->childChanged(q, type);
-    q->shapeChanged(type);
-    Q_FOREACH (KoShape * shape, dependees)
-        shape->shapeChanged(type, q);
-}
 
-void KoShapePrivate::updateStroke()
-{
-    Q_Q(KoShape);
-    if (stroke == 0)
-        return;
-    KoInsets insets;
-    stroke->strokeInsets(q, insets);
-    QSizeF inner = q->size();
-    // update left
-    q->update(QRectF(-insets.left, -insets.top, insets.left,
-                     inner.height() + insets.top + insets.bottom));
-    // update top
-    q->update(QRectF(-insets.left, -insets.top,
-                     inner.width() + insets.left + insets.right, insets.top));
-    // update right
-    q->update(QRectF(inner.width(), -insets.top, insets.right,
-                     inner.height() + insets.top + insets.bottom));
-    // update bottom
-    q->update(QRectF(-insets.left, inner.height(),
-                     inner.width() + insets.left + insets.right, insets.bottom));
+    q->shapeChanged(type);
+
+    Q_FOREACH (KoShape * shape, dependees) {
+        shape->shapeChanged(type, q);
+    }
+
+    Q_FOREACH (KoShape::ShapeChangeListener *listener, listeners) {
+       listener->notifyShapeChangedImpl(type, q);
+    }
 }
 
 void KoShapePrivate::addShapeManager(KoShapeManager *manager)
@@ -271,8 +297,8 @@ KoShape::KoShape()
     notifyChanged();
 }
 
-KoShape::KoShape(KoShapePrivate &dd)
-    : d_ptr(&dd)
+KoShape::KoShape(KoShapePrivate *dd)
+    : d_ptr(dd)
 {
 }
 
@@ -280,9 +306,26 @@ KoShape::~KoShape()
 {
     Q_D(KoShape);
     d->shapeChanged(Deleted);
+    d->listeners.clear();
     delete d_ptr;
 }
 
+KoShape *KoShape::cloneShape() const
+{
+    KIS_SAFE_ASSERT_RECOVER_NOOP(0 && "not implemented!");
+    qWarning() << shapeId() << "cannot be cloned";
+    return 0;
+}
+
+void KoShape::paintStroke(QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &paintcontext)
+{
+    Q_UNUSED(paintcontext);
+
+    if (stroke()) {
+        stroke()->paint(this, painter, converter);
+    }
+}
+
 void KoShape::scale(qreal sx, qreal sy)
 {
     Q_D(KoShape);
@@ -406,6 +449,29 @@ QRectF KoShape::boundingRect() const
     return bb;
 }
 
+QRectF KoShape::boundingRect(const QList<KoShape *> &shapes)
+{
+    QRectF boundingRect;
+    Q_FOREACH (KoShape *shape, shapes) {
+        boundingRect |= shape->boundingRect();
+    }
+    return boundingRect;
+}
+
+QRectF KoShape::absoluteOutlineRect(KoViewConverter *converter) const
+{
+    return absoluteTransformation(converter).map(outline()).boundingRect();
+}
+
+QRectF KoShape::absoluteOutlineRect(const QList<KoShape *> &shapes, KoViewConverter *converter)
+{
+    QRectF absoluteOutlineRect;
+    Q_FOREACH (KoShape *shape, shapes) {
+        absoluteOutlineRect |= shape->absoluteOutlineRect(converter);
+    }
+    return absoluteOutlineRect;
+}
+
 QTransform KoShape::absoluteTransformation(const KoViewConverter *converter) const
 {
     Q_D(const KoShape);
@@ -474,6 +540,19 @@ KoShape::ChildZOrderPolicy KoShape::childZOrderPolicy()
 
 bool KoShape::compareShapeZIndex(KoShape *s1, KoShape *s2)
 {
+    /**
+     * WARNING: Our definition of zIndex is not yet compatible with SVG2's
+     *          definition. In SVG stacking context of groups with the same
+     *          zIndex are **merged**, while in Krita the contents of groups
+     *          is never merged. One group will always below than the other.
+     *          Therefore, when zIndex of two groups inside the same parent
+     *          coinside, the resulting painting order in Krita is
+     *          **UNDEFINED**.
+     *
+     *          To avoid this trouble we use  KoShapeReorderCommand::mergeInShape()
+     *          inside KoShapeCreateCommand.
+     */
+
     // First sort according to runThrough which is sort of a master level
     KoShape *parentShapeS1 = s1->parent();
     KoShape *parentShapeS2 = s2->parent();
@@ -555,20 +634,51 @@ bool KoShape::compareShapeZIndex(KoShape *s1, KoShape *s2)
 void KoShape::setParent(KoShapeContainer *parent)
 {
     Q_D(KoShape);
-    if (d->parent == parent)
+
+    if (d->parent == parent) {
         return;
+    }
+
     KoShapeContainer *oldParent = d->parent;
     d->parent = 0; // avoids recursive removing
-    if (oldParent)
-        oldParent->removeShape(this);
+
+    if (oldParent) {
+        oldParent->shapeInterface()->removeShape(this);
+    }
+
+    KIS_SAFE_ASSERT_RECOVER_NOOP(parent != this);
+
     if (parent && parent != this) {
         d->parent = parent;
-        parent->addShape(this);
+        parent->shapeInterface()->addShape(this);
     }
+
     notifyChanged();
     d->shapeChanged(ParentChanged);
 }
 
+bool KoShape::inheritsTransformFromAny(const QList<KoShape *> ancestorsInQuestion) const
+{
+    bool result = false;
+
+    KoShape *shape = const_cast<KoShape*>(this);
+    while (shape) {
+        KoShapeContainer *parent = shape->parent();
+        if (parent && !parent->inheritsTransform(shape)) {
+            break;
+        }
+
+        if (ancestorsInQuestion.contains(shape)) {
+            result = true;
+            break;
+        }
+
+        shape = parent;
+    }
+
+    return result;
+}
+
 int KoShape::zIndex() const
 {
     Q_D(const KoShape);
@@ -587,7 +697,7 @@ void KoShape::update() const
     }
 }
 
-void KoShape::update(const QRectF &rect) const
+void KoShape::updateAbsolute(const QRectF &rect) const
 {
 
     if (rect.isEmpty() && !rect.isNull()) {
@@ -597,9 +707,8 @@ void KoShape::update(const QRectF &rect) const
     Q_D(const KoShape);
 
     if (!d->shapeManagers.empty() && isVisible()) {
-        QRectF rc(absoluteTransformation(0).mapRect(rect));
         Q_FOREACH (KoShapeManager * manager, d->shapeManagers) {
-            manager->update(rc);
+            manager->update(rect);
         }
     }
 }
@@ -621,27 +730,29 @@ QRectF KoShape::outlineRect() const
 QPainterPath KoShape::shadowOutline() const
 {
     Q_D(const KoShape);
-    if (d->fill) {
+    if (background()) {
         return outline();
     }
 
     return QPainterPath();
 }
 
-QPointF KoShape::absolutePosition(KoFlake::Position anchor) const
+QPointF KoShape::absolutePosition(KoFlake::AnchorPosition anchor) const
 {
-    QPointF point;
-    switch (anchor) {
-    case KoFlake::TopLeftCorner: break;
-    case KoFlake::TopRightCorner: point = QPointF(size().width(), 0.0); break;
-    case KoFlake::BottomLeftCorner: point = QPointF(0.0, size().height()); break;
-    case KoFlake::BottomRightCorner: point = QPointF(size().width(), size().height()); break;
-    case KoFlake::CenteredPosition: point = QPointF(size().width() / 2.0, size().height() / 2.0); break;
+    const QRectF rc = outlineRect();
+
+    QPointF point = rc.topLeft();
+
+    bool valid = false;
+    QPointF anchoredPoint = KoFlake::anchorToPoint(anchor, rc, &valid);
+    if (valid) {
+        point = anchoredPoint;
     }
+
     return absoluteTransformation(0).map(point);
 }
 
-void KoShape::setAbsolutePosition(const QPointF &newPosition, KoFlake::Position anchor)
+void KoShape::setAbsolutePosition(const QPointF &newPosition, KoFlake::AnchorPosition anchor)
 {
     Q_D(KoShape);
     QPointF currentAbsPosition = absolutePosition(anchor);
@@ -687,42 +798,30 @@ void KoShape::notifyChanged()
 void KoShape::setUserData(KoShapeUserData *userData)
 {
     Q_D(KoShape);
-    delete d->userData;
-    d->userData = userData;
+    d->userData.reset(userData);
 }
 
 KoShapeUserData *KoShape::userData() const
 {
     Q_D(const KoShape);
-    return d->userData;
-}
-
-void KoShape::setApplicationData(KoShapeApplicationData *appData)
-{
-    Q_D(KoShape);
-    // appdata is deleted by the application.
-    d->appData = appData;
-}
-
-KoShapeApplicationData *KoShape::applicationData() const
-{
-    Q_D(const KoShape);
-    return d->appData;
+    return d->userData.data();
 }
 
 bool KoShape::hasTransparency() const
 {
     Q_D(const KoShape);
-    if (! d->fill)
-        return true;
-    else
-        return d->fill->hasTransparency() || d->transparency > 0.0;
+    QSharedPointer<KoShapeBackground> bg = background();
+
+    return !bg || bg->hasTransparency() || d->transparency > 0.0;
 }
 
 void KoShape::setTransparency(qreal transparency)
 {
     Q_D(KoShape);
     d->transparency = qBound<qreal>(0.0, transparency, 1.0);
+
+    d->shapeChanged(TransparencyChanged);
+    notifyChanged();
 }
 
 qreal KoShape::transparency(bool recursive) const
@@ -776,7 +875,7 @@ QSizeF KoShape::size() const
 QPointF KoShape::position() const
 {
     Q_D(const KoShape);
-    QPointF center(0.5*size().width(), 0.5*size().height());
+    QPointF center = outlineRect().center();
     return d->localMatrix.map(center) - center;
 }
 
@@ -973,34 +1072,10 @@ void KoShape::setTextRunAroundContour(KoShape::TextRunAroundContour contour)
     d->textRunAroundContour = contour;
 }
 
-void KoShape::setAnchor(KoShapeAnchor *anchor)
-{
-    Q_D(KoShape);
-    d->anchor = anchor;
-}
-
-KoShapeAnchor *KoShape::anchor() const
-{
-    Q_D(const KoShape);
-    return d->anchor;
-}
-
-void KoShape::setMinimumHeight(qreal height)
-{
-    Q_D(KoShape);
-    d->minimumHeight = height;
-}
-
-qreal KoShape::minimumHeight() const
-{
-    Q_D(const KoShape);
-    return d->minimumHeight;
-}
-
-
 void KoShape::setBackground(QSharedPointer<KoShapeBackground> fill)
 {
     Q_D(KoShape);
+    d->inheritBackground = false;
     d->fill = fill;
     d->shapeChanged(BackgroundChanged);
     notifyChanged();
@@ -1009,7 +1084,32 @@ void KoShape::setBackground(QSharedPointer<KoShapeBackground> fill)
 QSharedPointer<KoShapeBackground> KoShape::background() const
 {
     Q_D(const KoShape);
-    return d->fill;
+
+    QSharedPointer<KoShapeBackground> bg;
+
+    if (!d->inheritBackground) {
+        bg = d->fill;
+    } else if (parent()) {
+        bg = parent()->background();
+    }
+
+    return bg;
+}
+
+void KoShape::setInheritBackground(bool value)
+{
+    Q_D(KoShape);
+
+    d->inheritBackground = value;
+    if (d->inheritBackground) {
+        d->fill.clear();
+    }
+}
+
+bool KoShape::inheritBackground() const
+{
+    Q_D(const KoShape);
+    return d->inheritBackground;
 }
 
 void KoShape::setZIndex(int zIndex)
@@ -1119,6 +1219,9 @@ void KoShape::setKeepAspectRatio(bool keepAspect)
 {
     Q_D(KoShape);
     d->keepAspect = keepAspect;
+
+    d->shapeChanged(KeepAspectRatioChange);
+    notifyChanged();
 }
 
 bool KoShape::keepAspectRatio() const
@@ -1151,26 +1254,46 @@ bool KoShape::collisionDetection()
     return d->detectCollision;
 }
 
-KoShapeStrokeModel *KoShape::stroke() const
+KoShapeStrokeModelSP KoShape::stroke() const
 {
     Q_D(const KoShape);
-    return d->stroke;
+
+    KoShapeStrokeModelSP stroke;
+
+    if (!d->inheritStroke) {
+        stroke = d->stroke;
+    } else if (parent()) {
+        stroke = parent()->stroke();
+    }
+
+    return stroke;
 }
 
-void KoShape::setStroke(KoShapeStrokeModel *stroke)
+void KoShape::setStroke(KoShapeStrokeModelSP stroke)
 {
     Q_D(KoShape);
-    if (stroke)
-        stroke->ref();
-    d->updateStroke();
-    if (d->stroke)
-        d->stroke->deref();
+
+    d->inheritStroke = false;
     d->stroke = stroke;
-    d->updateStroke();
     d->shapeChanged(StrokeChanged);
     notifyChanged();
 }
 
+void KoShape::setInheritStroke(bool value)
+{
+    Q_D(KoShape);
+    d->inheritStroke = value;
+    if (d->inheritStroke) {
+        d->stroke.clear();
+    }
+}
+
+bool KoShape::inheritStroke() const
+{
+    Q_D(const KoShape);
+    return d->inheritStroke;
+}
+
 void KoShape::setShadow(KoShapeShadow *shadow)
 {
     Q_D(KoShape);
@@ -1212,7 +1335,7 @@ KoBorder *KoShape::border() const
 void KoShape::setClipPath(KoClipPath *clipPath)
 {
     Q_D(KoShape);
-    d->clipPath = clipPath;
+    d->clipPath.reset(clipPath);
     d->shapeChanged(ClipPathChanged);
     notifyChanged();
 }
@@ -1220,7 +1343,19 @@ void KoShape::setClipPath(KoClipPath *clipPath)
 KoClipPath * KoShape::clipPath() const
 {
     Q_D(const KoShape);
-    return d->clipPath;
+    return d->clipPath.data();
+}
+
+void KoShape::setClipMask(KoClipMask *clipMask)
+{
+    Q_D(KoShape);
+    d->clipMask.reset(clipMask);
+}
+
+KoClipMask* KoShape::clipMask() const
+{
+    Q_D(const KoShape);
+    return d->clipMask.data();
 }
 
 QTransform KoShape::transform() const
@@ -1279,7 +1414,7 @@ QString KoShape::saveStyle(KoGenStyle &style, KoShapeSavingContext &context) con
 {
     Q_D(const KoShape);
     // and fill the style
-    KoShapeStrokeModel *sm = stroke();
+    KoShapeStrokeModelSP sm = stroke();
     if (sm) {
         sm->fillStyle(style, context);
     }
@@ -1403,10 +1538,8 @@ void KoShape::loadStyle(const KoXmlElement &element, KoShapeLoadingContext &cont
     styleStack.setTypeProperties("graphic");
 
     d->fill.clear();
-    if (d->stroke && !d->stroke->deref()) {
-        delete d->stroke;
-        d->stroke = 0;
-    }
+    d->stroke.clear();
+
     if (d->shadow && !d->shadow->deref()) {
         delete d->shadow;
         d->shadow = 0;
@@ -1623,7 +1756,7 @@ QSharedPointer<KoShapeBackground> KoShape::loadOdfFill(KoShapeLoadingContext &co
     return bg;
 }
 
-KoShapeStrokeModel *KoShape::loadOdfStroke(const KoXmlElement &element, KoShapeLoadingContext &context) const
+KoShapeStrokeModelSP KoShape::loadOdfStroke(const KoXmlElement &element, KoShapeLoadingContext &context) const
 {
     KoStyleStack &styleStack = context.odfLoadingContext().styleStack();
     KoOdfStylesReader &stylesReader = context.odfLoadingContext().stylesReader();
@@ -1632,7 +1765,7 @@ KoShapeStrokeModel *KoShape::loadOdfStroke(const KoXmlElement &element, KoShapeL
     if (stroke == "solid" || stroke == "dash") {
         QPen pen = KoOdfGraphicStyles::loadOdfStrokeStyle(styleStack, stroke, stylesReader);
 
-        KoShapeStroke *stroke = new KoShapeStroke();
+        QSharedPointer<KoShapeStroke> stroke(new KoShapeStroke());
 
         if (styleStack.hasProperty(KoXmlNS::calligra, "stroke-gradient")) {
             QString gradientName = styleStack.property(KoXmlNS::calligra, "stroke-gradient");
@@ -1655,7 +1788,7 @@ KoShapeStrokeModel *KoShape::loadOdfStroke(const KoXmlElement &element, KoShapeL
     } else if (stroke.isEmpty()) {
         QPen pen = KoOdfGraphicStyles::loadOdfStrokeStyle(styleStack, "solid", stylesReader);
         if (KoOdfWorkaround::fixMissingStroke(pen, element, context, this)) {
-            KoShapeStroke *stroke = new KoShapeStroke();
+            QSharedPointer<KoShapeStroke> stroke(new KoShapeStroke());
 
 #ifndef NWORKAROUND_ODF_BUGS
             KoOdfWorkaround::fixPenWidth(pen, context);
@@ -1671,7 +1804,7 @@ KoShapeStrokeModel *KoShape::loadOdfStroke(const KoXmlElement &element, KoShapeL
 #endif
     }
 
-    return 0;
+    return KoShapeStrokeModelSP();
 }
 
 KoShapeShadow *KoShapePrivate::loadOdfShadow(KoShapeLoadingContext &context) const
@@ -1834,9 +1967,8 @@ void KoShape::loadOdfClipContour(const KoXmlElement &element, KoShapeLoadingCont
         ps->loadContourOdf(child, context, scaleFactor);
         ps->setTransformation(transformation());
 
-        KoClipData *cd = new KoClipData(ps);
-        KoClipPath *clipPath = new KoClipPath(this, cd);
-        d->clipPath = clipPath;
+        KoClipPath *clipPath = new KoClipPath({ps}, KoFlake::UserSpaceOnUse);
+        d->clipPath.reset(clipPath);
     }
 }
 
@@ -1893,14 +2025,14 @@ QTransform KoShape::parseOdfTransform(const QString &transform)
                 scaleMatrix.scale(params[0].toDouble(), params[0].toDouble());
             matrix = matrix * scaleMatrix;
         } else if (cmd == "skewx") {
-            QPointF p = absolutePosition(KoFlake::TopLeftCorner);
+            QPointF p = absolutePosition(KoFlake::TopLeft);
             QTransform shearMatrix;
             shearMatrix.translate(p.x(), p.y());
             shearMatrix.shear(tan(-params[0].toDouble()), 0.0F);
             shearMatrix.translate(-p.x(), -p.y());
             matrix = matrix * shearMatrix;
         } else if (cmd == "skewy") {
-            QPointF p = absolutePosition(KoFlake::TopLeftCorner);
+            QPointF p = absolutePosition(KoFlake::TopLeft);
             QTransform shearMatrix;
             shearMatrix.translate(p.x(), p.y());
             shearMatrix.shear(0.0F, tan(-params[0].toDouble()));
@@ -2119,7 +2251,7 @@ void KoShape::saveOdfClipContour(KoShapeSavingContext &context, const QSizeF &or
 
     debugFlake << "shape saves contour-polygon";
     if (d->clipPath && !d->clipPath->clipPathShapes().isEmpty()) {
-        // This will loose data as odf can only save one set of contour wheras
+        // This will loose data as odf can only save one set of contour whereas
         // svg loading and at least karbon editing can produce more than one
         // TODO, FIXME see if we can save more than one clipshape to odf
         d->clipPath->clipPathShapes().first()->saveContourOdf(context, originalSize);
@@ -2136,6 +2268,17 @@ void KoShape::applyConversion(QPainter &painter, const KoViewConverter &converte
     painter.scale(zoomX, zoomY);
 }
 
+KisHandlePainterHelper KoShape::createHandlePainterHelper(QPainter *painter, KoShape *shape, const KoViewConverter &converter, qreal handleRadius)
+{
+    const QTransform originalPainterTransform = painter->transform();
+
+    painter->setTransform(shape->absoluteTransformation(&converter) * painter->transform());
+    KoShape::applyConversion(*painter, converter);
+
+    // move c-tor
+    return KisHandlePainterHelper(painter, originalPainterTransform, handleRadius);
+}
+
 QPointF KoShape::shapeToDocument(const QPointF &point) const
 {
     return absoluteTransformation(0).map(point);
@@ -2286,3 +2429,67 @@ KoShapePrivate *KoShape::priv()
     Q_D(KoShape);
     return d;
 }
+
+KoShape::ShapeChangeListener::~ShapeChangeListener()
+{
+    Q_FOREACH(KoShape *shape, m_registeredShapes) {
+        shape->removeShapeChangeListener(this);
+    }
+}
+
+void KoShape::ShapeChangeListener::registerShape(KoShape *shape)
+{
+    KIS_SAFE_ASSERT_RECOVER_RETURN(!m_registeredShapes.contains(shape));
+    m_registeredShapes.append(shape);
+}
+
+void KoShape::ShapeChangeListener::unregisterShape(KoShape *shape)
+{
+    KIS_SAFE_ASSERT_RECOVER_RETURN(m_registeredShapes.contains(shape));
+    m_registeredShapes.removeAll(shape);
+}
+
+void KoShape::ShapeChangeListener::notifyShapeChangedImpl(KoShape::ChangeType type, KoShape *shape)
+{
+    KIS_SAFE_ASSERT_RECOVER_RETURN(m_registeredShapes.contains(shape));
+
+    notifyShapeChanged(type, shape);
+
+    if (type == KoShape::Deleted) {
+        unregisterShape(shape);
+    }
+}
+
+void KoShape::addShapeChangeListener(KoShape::ShapeChangeListener *listener)
+{
+    Q_D(KoShape);
+
+    KIS_SAFE_ASSERT_RECOVER_RETURN(!d->listeners.contains(listener));
+    listener->registerShape(this);
+    d->listeners.append(listener);
+}
+
+void KoShape::removeShapeChangeListener(KoShape::ShapeChangeListener *listener)
+{
+    Q_D(KoShape);
+
+    KIS_SAFE_ASSERT_RECOVER_RETURN(d->listeners.contains(listener));
+    d->listeners.removeAll(listener);
+    listener->unregisterShape(this);
+}
+
+QList<KoShape *> KoShape::linearizeSubtree(const QList<KoShape *> &shapes)
+{
+    QList<KoShape *> result;
+
+    Q_FOREACH (KoShape *shape, shapes) {
+        result << shape;
+
+        KoShapeContainer *container = dynamic_cast<KoShapeContainer*>(shape);
+        if (container) {
+            result << linearizeSubtree(container->shapes());
+        }
+    }
+
+    return result;
+}
