@@ -24,6 +24,7 @@
 #define KOSHAPE_H
 
 #include "KoFlake.h"
+#include "KoFlakeTypes.h"
 #include "KoConnectionPoint.h"
 
 #include <QSharedPointer>
@@ -52,11 +53,13 @@ class KoShapePrivate;
 class KoFilterEffectStack;
 class KoSnapData;
 class KoClipPath;
+class KoClipMask;
 class KoShapePaintingContext;
 class KoShapeAnchor;
 class KoBorder;
 struct KoInsets;
 class KoShapeBackground;
+class KisHandlePainterHelper;
 
 
 /**
@@ -113,6 +116,7 @@ public:
         ShearChanged,   ///< used after a shear()
         SizeChanged,    ///< used after a setSize()
         GenericMatrixChange,    ///< used after the matrix was changed without knowing which property explicitly changed
+        KeepAspectRatioChange, ///< used after setKeepAspectRatio()
         ParentChanged,   ///< used after a setParent()
         CollisionDetected, ///< used when another shape moved in our boundingrect
         Deleted, ///< the shape was deleted
@@ -125,7 +129,8 @@ public:
         TextRunAroundChanged, ///< used after a setTextRunAroundSide()
         ChildChanged, ///< a child of a container was changed/removed. This is propagated to all parents
         ConnectionPointChanged, ///< a connection point has changed
-        ClipPathChanged ///< the shapes clip path has changed
+        ClipPathChanged, ///< the shapes clip path has changed
+        TransparencyChanged ///< the shapetransparency value has changed
     };
 
     /// The behavior text should do when intersecting this shape.
@@ -165,7 +170,13 @@ public:
     virtual ~KoShape();
 
     /**
-     * @brief Paint the shape
+     * @brief creates a deep copy of the shape or shape's subtree 
+     * @return a cloned shape
+     */
+    virtual KoShape* cloneShape() const;
+
+    /**
+     * @brief Paint the shape fill
      * The class extending this one is responsible for painting itself.  Since we do not
      * assume the shape is square the paint must also clear its background if it will draw
      * something transparent on top.
@@ -182,6 +193,15 @@ public:
      */
     virtual void paint(QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &paintcontext) = 0;
 
+    /**
+     * @brief paintStroke paints the shape's stroked outline
+     * @param painter used for painting the shape
+     * @param converter to convert between internal and view coordinates.
+     * @see applyConversion()
+     * @param paintcontext the painting context.
+     */
+    virtual void paintStroke(QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &paintcontext);
+
     /**
      * @brief Paint the shape's border
      * This is a helper function that could be called from the paint() method of all shapes. 
@@ -320,6 +340,26 @@ public:
      */
     virtual QRectF boundingRect() const;
 
+    /**
+     * Get the united bounding box of a group of shapes. This is a utility
+     * function used in many places in Krita.
+     */
+    static QRectF boundingRect(const QList<KoShape*> &shapes);
+
+    /**
+     * @return the bounding rect of the outline of the shape measured
+     *         in absolute coordinate system. Please note that in contrast to
+     *         boundingRect() this rect doesn't include the stroke and other
+     *         insets.
+     */
+    QRectF absoluteOutlineRect(KoViewConverter *converter = 0) const;
+
+    /**
+     * Same as a member function, but applies to a list of shapes and returns a
+     * united rect.
+     */
+    static QRectF absoluteOutlineRect(const QList<KoShape*> &shapes, KoViewConverter *converter = 0);
+
     /**
      * @brief Add a connector point to the shape
      *
@@ -484,6 +524,10 @@ public:
      * or have a complex fill.
      * Setting such a background will allow the shape to be filled and will be able to tell
      * if it is transparent or not.
+     *
+     * If the shape inherited the background from its parent, its stops inheriting it, that
+     * is inheritBackground property resets to false.
+     *
      * @param background the new shape background.
      */
     void setBackground(QSharedPointer<KoShapeBackground> background);
@@ -497,6 +541,19 @@ public:
      */
     QSharedPointer<KoShapeBackground> background() const;
 
+    /**
+     * @brief setInheritBackground marks a shape as inhiriting the background
+     * from the parent shape. NOTE: The currently selected background is destroyed.
+     * @param value true if the shape should inherit the filling background
+     */
+    void setInheritBackground(bool value);
+
+    /**
+     * @brief inheritBackground shows if the shape inherits background from its parent
+     * @return true if the shape inherits the fill
+     */
+    bool inheritBackground() const;
+
     /**
      * Returns true if there is some transparency, false if the shape is fully opaque.
      * The default implementation will just return if the background has some transparency,
@@ -651,6 +708,15 @@ public:
      */
     void setParent(KoShapeContainer *parent);
 
+
+    /**
+     * @brief inheritsTransformFromAny checks if the shape inherits transformation from
+     *        any of the shapes listed in \p ancestorsInQuestion. The inheritance is checked
+     *        in recursive way.
+     * @return true if there is a (transitive) transformation-wise parent found in \p ancestorsInQuestion
+     */
+    bool inheritsTransformFromAny(const QList<KoShape*> ancestorsInQuestion) const;
+
     /**
      * Request a repaint to be queued.
      * The repaint will be of the entire Shape, including its selection handles should this
@@ -663,13 +729,13 @@ public:
     /**
      * Request a repaint to be queued.
      * The repaint will be restricted to the parameters rectangle, which is expected to be
-     * in points (the internal coordinates system of KoShape) and it is expected to be
+     * in absolute coordinates of the canvas and it is expected to be
      * normalized.
      * <p>This method will return immediately and only request a repaint. Successive calls
      * will be merged into an appropriate repaint action.
      * @param rect the rectangle (in pt) to queue for repaint.
      */
-    virtual void update(const QRectF &rect) const;
+    virtual void updateAbsolute(const QRectF &rect) const;
 
     /// Used by compareShapeZIndex() to order shapes
     enum ChildZOrderPolicy {
@@ -731,13 +797,26 @@ public:
      * Returns the currently set stroke, or 0 if there is no stroke.
      * @return the currently set stroke, or 0 if there is no stroke.
      */
-    KoShapeStrokeModel *stroke() const;
+    KoShapeStrokeModelSP stroke() const;
 
     /**
-     * Set a new stroke, removing the old one.
+     * Set a new stroke, removing the old one. The stroke inheritance becomes disabled.
      * @param stroke the new stroke, or 0 if there should be no stroke.
      */
-    void setStroke(KoShapeStrokeModel *stroke);
+    void setStroke(KoShapeStrokeModelSP stroke);
+
+    /**
+     * @brief setInheritStroke marks a shape as inhiriting the stroke
+     * from the parent shape. NOTE: The currently selected stroke is destroyed.
+     * @param value true if the shape should inherit the stroke style
+     */
+    void setInheritStroke(bool value);
+
+    /**
+     * @brief inheritStroke shows if the shape inherits the stroke from its parent
+     * @return true if the shape inherits the stroke style
+     */
+    bool inheritStroke() const;
 
     /**
      * Return the insets of the stroke.
@@ -763,9 +842,15 @@ public:
     /// Returns the currently set clip path or 0 if there is no clip path set
     KoClipPath * clipPath() const;
 
+    /// Sets a new clip mask, removing the old one. The mask is owned by the shape.
+    void setClipMask(KoClipMask *clipMask);
+
+    /// Returns the currently set clip mask or 0 if there is no clip mask set
+    KoClipMask* clipMask() const;
+
     /**
      * Setting the shape to keep its aspect-ratio has the effect that user-scaling will
-     * keep the width/hight ratio intact so as not to distort shapes that rely on that
+     * keep the width/height ratio intact so as not to distort shapes that rely on that
      * ratio.
      * @param keepAspect the new value
      */
@@ -773,7 +858,7 @@ public:
 
     /**
      * Setting the shape to keep its aspect-ratio has the effect that user-scaling will
-     * keep the width/hight ratio intact so as not to distort shapes that rely on that
+     * keep the width/height ratio intact so as not to distort shapes that rely on that
      * ratio.
      * @return whether to keep aspect ratio of this shape
      */
@@ -785,7 +870,7 @@ public:
      * @param anchor The place on the (unaltered) shape that you want the position of.
      * @return the point that is the absolute, centered position of this shape.
      */
-    QPointF absolutePosition(KoFlake::Position anchor = KoFlake::CenteredPosition) const;
+    QPointF absolutePosition(KoFlake::AnchorPosition anchor = KoFlake::Center) const;
 
     /**
      * Move this shape to an absolute position where the end location will be the same
@@ -801,7 +886,7 @@ public:
      * @param newPosition the new absolute center of the shape.
      * @param anchor The place on the (unaltered) shape that you set the position of.
      */
-    void setAbsolutePosition(const QPointF &newPosition, KoFlake::Position anchor = KoFlake::CenteredPosition);
+    void setAbsolutePosition(const QPointF &newPosition, KoFlake::AnchorPosition anchor = KoFlake::Center);
 
     /**
      * Set a data object on the shape to be used by an application.
@@ -815,19 +900,6 @@ public:
      */
     KoShapeUserData *userData() const;
 
-    /**
-     * Set a data object on the shape to be used by an application.
-     * This is specifically useful when an application wants to have data that is per shape
-     * and should be deleted when the shape is destructed.
-     * @param applicationData the new application data, or 0 to delete the current one.
-     */
-    void setApplicationData(KoShapeApplicationData *applicationData);
-
-    /**
-     * Return the current applicationData.
-     */
-    KoShapeApplicationData *applicationData() const;
-
     /**
      * Return the Id of this shape, identifying the type of shape by the id of the factory.
      * @see KoShapeFactoryBase::shapeId()
@@ -900,6 +972,13 @@ public:
      */
     static void applyConversion(QPainter &painter, const KoViewConverter &converter);
 
+    /**
+     * A convenience method that creates a handles helper with applying transformations at
+     * the same time. Please note that you shouldn't save/restore additionally. All the work
+     * on restoring original painter's transformations is done by the helper.
+     */
+    static KisHandlePainterHelper createHandlePainterHelper(QPainter *painter, KoShape *shape, const KoViewConverter &converter, qreal handleRadius = 0.0);
+
     /**
      * @brief Transforms point from shape coordinates to document coordinates
      * @param point in shape coordinates
@@ -1090,9 +1169,31 @@ public:
      */
     KoShapePrivate *priv();
 
+public:
+
+    struct KRITAFLAKE_EXPORT ShapeChangeListener {
+        virtual ~ShapeChangeListener();
+        virtual void notifyShapeChanged(ChangeType type, KoShape *shape) = 0;
+
+    private:
+        friend class KoShape;
+        friend class KoShapePrivate;
+        void registerShape(KoShape *shape);
+        void unregisterShape(KoShape *shape);
+        void notifyShapeChangedImpl(ChangeType type, KoShape *shape);
+
+        QList<KoShape*> m_registeredShapes;
+    };
+
+    void addShapeChangeListener(ShapeChangeListener *listener);
+    void removeShapeChangeListener(ShapeChangeListener *listener);
+
+public:
+    static QList<KoShape*> linearizeSubtree(const QList<KoShape*> &shapes);
+
 protected:
     /// constructor
-    KoShape(KoShapePrivate &);
+    KoShape(KoShapePrivate *);
 
     /* ** loading saving helper methods */
     /// attributes from ODF 1.1 chapter 9.2.15 Common Drawing Shape Attributes
@@ -1155,7 +1256,7 @@ protected:
     virtual void loadStyle(const KoXmlElement &element, KoShapeLoadingContext &context);
 
     /// Loads the stroke style
-    KoShapeStrokeModel *loadOdfStroke(const KoXmlElement &element, KoShapeLoadingContext &context) const;
+    KoShapeStrokeModelSP loadOdfStroke(const KoXmlElement &element, KoShapeLoadingContext &context) const;
 
     /// Loads the fill style
     QSharedPointer<KoShapeBackground> loadOdfFill(KoShapeLoadingContext &context) const;
