@@ -105,7 +105,7 @@ void KoShapeManager::Private::paintGroup(KoShapeGroup *group, QPainter &painter,
     std::sort(shapes.begin(), shapes.end(), KoShape::compareShapeZIndex);
     Q_FOREACH (KoShape *child, shapes) {
         // we paint recursively here, so we do not have to check recursively for visibility
-        if (!child->isVisible(false))
+        if (!child->isVisible())
             continue;
         KoShapeGroup *childGroup = dynamic_cast<KoShapeGroup*>(child);
         if (childGroup) {
@@ -133,23 +133,14 @@ KoShapeManager::KoShapeManager(KoCanvasBase *canvas)
     connect(d->selection, SIGNAL(selectionChanged()), this, SIGNAL(selectionChanged()));
 }
 
-void KoShapeManager::Private::unlinkFromShapesRecursively(const QList<KoShape*> &shapes)
-{
-    Q_FOREACH (KoShape *shape, shapes) {
-        shape->priv()->removeShapeManager(q);
-
-        KoShapeContainer *container = dynamic_cast<KoShapeContainer*>(shape);
-        if (container) {
-            unlinkFromShapesRecursively(container->shapes());
-        }
-    }
-}
-
 KoShapeManager::~KoShapeManager()
 {
-    d->unlinkFromShapesRecursively(d->shapes);
-    d->shapes.clear();
-
+    Q_FOREACH (KoShape *shape, d->shapes) {
+        shape->priv()->removeShapeManager(this);
+    }
+    Q_FOREACH (KoShape *shape, d->additionalShapes) {
+        shape->priv()->removeShapeManager(this);
+    }
     delete d;
 }
 
@@ -157,7 +148,9 @@ void KoShapeManager::setShapes(const QList<KoShape *> &shapes, Repaint repaint)
 {
     //clear selection
     d->selection->deselectAll();
-    d->unlinkFromShapesRecursively(d->shapes);
+    Q_FOREACH (KoShape *shape, d->shapes) {
+        shape->priv()->removeShapeManager(this);
+    }
     d->aggregate4update.clear();
     d->tree.clear();
     d->shapes.clear();
@@ -267,7 +260,7 @@ void KoShapeManager::paint(QPainter &painter, const KoViewConverter &converter,
     // also filter shapes with a parent which has filter effects applied
     QList<KoShape*> sortedShapes;
     foreach (KoShape *shape, unsortedShapes) {
-        if (!shape->isVisible())
+        if (!shape->isVisible(true))
             continue;
         bool addShapeToList = true;
         // check if one of the shapes ancestors have filter effects
@@ -488,7 +481,7 @@ KoShape *KoShapeManager::shapeAt(const QPointF &position, KoFlake::ShapeSelectio
     KoShape *firstUnselectedShape = 0;
     for (int count = sortedShapes.count() - 1; count >= 0; count--) {
         KoShape *shape = sortedShapes.at(count);
-        if (omitHiddenShapes && ! shape->isVisible())
+        if (omitHiddenShapes && ! shape->isVisible(true))
             continue;
         if (! shape->hitTest(position))
             continue;
@@ -538,7 +531,7 @@ QList<KoShape *> KoShapeManager::shapesAt(const QRectF &rect, bool omitHiddenSha
      
         KoShape *shape = shapes.at(count);
        
-        if (omitHiddenShapes && !shape->isVisible()) {
+        if (omitHiddenShapes && !shape->isVisible(true)) {
             shapes.removeAt(count);
         } else {
             const QPainterPath outline = shape->absoluteTransformation(0).map(shape->outline());
@@ -573,7 +566,7 @@ void KoShapeManager::update(const QRectF &rect, const KoShape *shape, bool selec
 void KoShapeManager::notifyShapeChanged(KoShape *shape)
 {
     Q_ASSERT(shape);
-    if (d->aggregate4update.contains(shape)) {
+    if (d->aggregate4update.contains(shape) || d->additionalShapes.contains(shape)) {
         return;
     }
     const bool wasEmpty = d->aggregate4update.isEmpty();
@@ -585,10 +578,9 @@ void KoShapeManager::notifyShapeChanged(KoShape *shape)
         Q_FOREACH (KoShape *child, container->shapes())
             notifyShapeChanged(child);
     }
-
-    if (wasEmpty && !d->aggregate4update.isEmpty()) {
-        d->updateTreeCompressor.start();
-    }
+    if (wasEmpty && !d->aggregate4update.isEmpty())
+        QTimer::singleShot(100, this, SLOT(updateTree()));
+    emit shapeChanged(shape);
 }
 
 QList<KoShape*> KoShapeManager::shapes() const
@@ -601,7 +593,7 @@ QList<KoShape*> KoShapeManager::topLevelShapes() const
     QList<KoShape*> shapes;
     // get all toplevel shapes
     Q_FOREACH (KoShape *shape, d->shapes) {
-        if (!shape->parent() || dynamic_cast<KoShapeLayer*>(shape->parent())) {
+        if (shape->parent() == 0) {
             shapes.append(shape);
         }
     }
