@@ -44,30 +44,31 @@
 #include <KoEmbeddedDocumentSaver.h>
 #include "KoShapeLoadingContext.h"
 #include "KoShapeSavingContext.h"
-#include "KoShapeContainerDefaultModel.h"
+#include "SimpleShapeContainerModel.h"
 #include "KoShapeBackground.h"
+#include "KisQPainterStateSaver.h"
 
 #include <FlakeDebug.h>
 
 
 // The XML of a frame looks something like this:
-// 
+//
 // 1. <draw:frame ...attributes...>
 // 2.   <draw:object xlink:href="./Object1" ...more attributes>
 // 3.   <draw:image xlink:href="./ObjectReplacements/Object1" ...more attributes>
 // 4. </draw:frame>
 //
 // or
-// 
+//
 // 1. <draw:frame ...attributes...>
-// 2.   <math:math>...inline xml here...</math:math>    
+// 2.   <math:math>...inline xml here...</math:math>
 // 3.   <draw:image xlink:href="./ObjectReplacements/Object1" ...more attributes>
 // 4. </draw:frame>
 //
 // We define each Xml statement on lines 2 and 3 above as an "object".
 // (Strictly only the first child element is an object in the ODF sense,
 // but we have to have some terminology here.)
-// 
+//
 // In an ODF frame, only the first line, i.e. the first object
 // contains the real contents.  All the rest of the objects are used /
 // shown if we cannot handle the first one.  The most common cases are
@@ -76,7 +77,7 @@
 //
 // Sometimes, e.g. in the case of an embedded document, the reference
 // points not to a file but to a directory structure inside the ODF
-// store. 
+// store.
 //
 // When we load and save in the UnavailShape, we have to be general
 // enough to cover all possible cases of references and inline XML,
@@ -171,7 +172,7 @@ KoUnavailShape::Private::~Private()
 
 KoUnavailShape::KoUnavailShape()
 : KoFrameShape( "", "" )
-, KoShapeContainer(new KoShapeContainerDefaultModel())
+, KoShapeContainer(new SimpleShapeContainerModel())
 , d(new Private(this))
 {
     setShapeId(KoUnavailShape_SHAPEID);
@@ -185,9 +186,9 @@ KoUnavailShape::~KoUnavailShape()
     delete d;
 }
 
-
 void KoUnavailShape::paint(QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &paintContext)
 {
+    KisQPainterStateSaver saver(&painter);
     applyConversion(painter, converter);
 
     // If the frame is empty, just draw a background.
@@ -554,34 +555,6 @@ void KoUnavailShape::Private::storeXmlRecursive(const KoXmlElement &el, KoXmlWri
     const QByteArray name(el.nodeName().toLatin1());
     writer.startElement(name.constData());
 
-    // Copy all the attributes, including namespaces.
-    QList< QPair<QString, QString> >  attributeNames = el.attributeFullNames();
-    for (int i = 0; i < attributeNames.size(); ++i) {
-        QPair<QString, QString> attrPair(attributeNames.value(i));
-        if (attrPair.first.isEmpty()) {
-            writer.addAttribute(attrPair.second.toLatin1(), el.attribute(attrPair.second));
-        }
-        else {
-            // This somewhat convoluted code is because we need the
-            // namespace, not the namespace URI.
-            QString nsShort = KoXmlNS::nsURI2NS(attrPair.first.toLatin1());
-            // in case we don't find the namespace in our list create a own one and use that
-            // so the document created on saving is valid.
-            if (nsShort.isEmpty()) {
-                nsShort = unknownNamespaces.value(attrPair.first);
-                if (nsShort.isEmpty()) {
-                    nsShort = QString("ns%1").arg(unknownNamespaces.size() + 1);
-                    unknownNamespaces.insert(attrPair.first, nsShort);
-                }
-                QString name = QString("xmlns:") + nsShort;
-                writer.addAttribute(name.toLatin1(), attrPair.first.toLatin1());
-            }
-            QString attr(nsShort + ':' + attrPair.second);
-            writer.addAttribute(attr.toLatin1(), el.attributeNS(attrPair.first,
-                                                               attrPair.second));
-        }
-    }
-
     // Child elements
     // Loop through all the child elements of the draw:frame.
     KoXmlNode n = el.firstChild();
