@@ -31,8 +31,6 @@
 #include <QTextLayout>
 #include <QTextLine>
 
-#include <QStack>
-
 #include <KoSvgTextShape.h>
 #include <KoXmlWriter.h>
 #include <KoXmlReader.h>
@@ -40,16 +38,12 @@
 
 #include <SvgParser.h>
 #include <SvgWriter.h>
-#include <SvgUtil.h>
 #include <SvgSavingContext.h>
 #include <SvgGraphicContext.h>
 
 #include <html/HtmlSavingContext.h>
 #include <html/HtmlWriter.h>
 
-#include "kis_dom_utils.h"
-#include <boost/optional.hpp>
-
 
 struct KoSvgTextShapeMarkupConverter::Private {
     Private(KoSvgTextShape *_shape) : shape(_shape) {}
@@ -371,105 +365,6 @@ bool KoSvgTextShapeMarkupConverter::convertFromHtml(const QString &htmlText, QSt
     return true;
 }
 
-void postCorrectBlockHeight(QTextDocument *doc,
-                            qreal currLineAscent,
-                            qreal prevLineAscent,
-                            qreal prevLineDescent,
-                            int prevBlockCursorPosition,
-                            qreal currentBlockAbsoluteLineOffset)
-{
-    KIS_SAFE_ASSERT_RECOVER_RETURN(prevBlockCursorPosition >= 0);
-
-    QTextCursor postCorrectionCursor(doc);
-    postCorrectionCursor.setPosition(prevBlockCursorPosition);
-    if (!postCorrectionCursor.isNull()) {
-        const qreal relativeLineHeight =
-                ((currentBlockAbsoluteLineOffset - currLineAscent + prevLineAscent) /
-                 (prevLineAscent + prevLineDescent)) * 100.0;
-
-        QTextBlockFormat format = postCorrectionCursor.blockFormat();
-        format.setLineHeight(relativeLineHeight, QTextBlockFormat::ProportionalHeight);
-        postCorrectionCursor.setBlockFormat(format);
-        postCorrectionCursor = QTextCursor();
-    }
-}
-
-QTextFormat findMostCommonFormat(const QList<QTextFormat> &allFormats)
-{
-    QTextCharFormat mostCommonFormat;
-
-    QSet<int> propertyIds;
-
-    /**
-     * Get all existing property ids
-     */
-    Q_FOREACH (const QTextFormat &format, allFormats) {
-        const QMap<int, QVariant> formatProperties = format.properties();
-        Q_FOREACH (int id, formatProperties.keys()) {
-            propertyIds.insert(id);
-        }
-    }
-
-    /**
-     * Filter out properties that do not exist in some formats. Otherwise, the
-     * global format may override the default value used in these formats
-     * (and yes, we do not have access to the default values to use them
-     * in difference calculation algorithm
-     */
-    Q_FOREACH (const QTextFormat &format, allFormats) {
-        for (auto it = propertyIds.begin(); it != propertyIds.end();) {
-            if (!format.hasProperty(*it)) {
-                it = propertyIds.erase(it);
-            } else {
-                ++it;
-            }
-        }
-        if (propertyIds.isEmpty()) break;
-    }
-
-    if (!propertyIds.isEmpty()) {
-        QMap<int, QMap<QVariant, int>> propertyFrequency;
-
-        /**
-         * Calculate the frequency of values used in *all* the formats
-         */
-        Q_FOREACH (const QTextFormat &format, allFormats) {
-            const QMap<int, QVariant> formatProperties = format.properties();
-
-            Q_FOREACH (int id, propertyIds) {
-                KIS_SAFE_ASSERT_RECOVER_BREAK(formatProperties.contains(id));
-                propertyFrequency[id][formatProperties.value(id)]++;
-            }
-        }
-
-        /**
-         * Add the most popular property value to the set of most common properties
-         */
-        for (auto it = propertyFrequency.constBegin(); it != propertyFrequency.constEnd(); ++it) {
-            const int id = it.key();
-            const QMap<QVariant, int> allValues = it.value();
-
-            int maxCount = 0;
-            QVariant maxValue;
-
-            for (auto valIt = allValues.constBegin(); valIt != allValues.constEnd(); ++valIt) {
-                if (valIt.value() > maxCount) {
-                    maxCount = valIt.value();
-                    maxValue = valIt.key();
-                }
-            }
-
-            KIS_SAFE_ASSERT_RECOVER_BREAK(maxCount > 0);
-            mostCommonFormat.setProperty(id, maxValue);
-        }
-
-    }
-
-    return mostCommonFormat;
-}
-
-
-
 bool KoSvgTextShapeMarkupConverter::convertDocumentToSvg(const QTextDocument *doc, QString *svgText)
 {
     QBuffer svgBuffer;
@@ -479,98 +374,77 @@ bool KoSvgTextShapeMarkupConverter::convertDocumentToSvg(const QTextDocument *do
 
     svgWriter.setAutoFormatting(true);
 
+    QTextBlock block = doc->begin();
 
+    /**
+     * Find the most commonly used format.
+     * This is what we put into the text-style.
+     * We cannot just check the first textCharFormat we find
+     * because users might be using things like drop-caps.
+     */
+    QList<int> formatFrequency;
+    QList<QTextCharFormat> allFormats;
+    allFormats.append(QTextCharFormat());
+    formatFrequency.append(1);
+    while (block.isValid()) {
+        //copied from QTextDocument.cpp
+        for (int f=0; f<block.textFormats().size(); f++) {
+            for(int i=0; i<allFormats.size(); i++) {
+                //props should proly compare itself to the main text format...
+                QTextCharFormat diff = formatDifference(block.textFormats().at(f).format, allFormats.at(i)).toCharFormat();
 
-    QTextCharFormat mostCommonCharFormat;
-    QTextBlockFormat mostCommonBlockFormat;
-
-    {
-        QTextBlock block = doc->begin();
-
-        QList<QTextFormat> allCharFormats;
-        QList<QTextFormat> allBlockFormats;
-
-        while (block.isValid()) {
-            allBlockFormats.append(block.blockFormat());
-            Q_FOREACH (const QTextLayout::FormatRange &range, block.textFormats()) {
-                QTextFormat format =  range.format;
-                allCharFormats.append(format);
+                if (diff.properties().isEmpty()) {
+                    formatFrequency[i] = formatFrequency.at(i) + block.textFormats().at(f).length;
+                } else {
+                    allFormats.append(block.textFormats().at(f).format);
+                    formatFrequency.append(block.textFormats().at(f).length);
+                }
             }
-            block = block.next();
         }
-
-        mostCommonCharFormat = findMostCommonFormat(allCharFormats).toCharFormat();
-        mostCommonBlockFormat = findMostCommonFormat(allBlockFormats).toBlockFormat();
+        block = block.next();
+    }
+    QTextCharFormat mostCommonCharFormat = QTextCharFormat();
+    int top = 0;
+    for(int i=0; i<formatFrequency.size(); i++) {
+        if (formatFrequency.at(i)>top) {
+            top = formatFrequency.at(i);
+            mostCommonCharFormat = allFormats.at(i);
+        }
     }
 
     //Okay, now the actual writing.
 
-    QTextBlock block = doc->begin();
-
+    block = doc->begin();
     svgWriter.writeStartElement("text");
-
-    {
-        const QString commonTextStyle = style(mostCommonCharFormat, mostCommonBlockFormat);
-        if (!commonTextStyle.isEmpty()) {
-            svgWriter.writeAttribute("style", commonTextStyle);
-        }
-    }
-
-    int prevBlockRelativeLineSpacing = mostCommonBlockFormat.lineHeight();
-    int prevBlockLineType = mostCommonBlockFormat.lineHeightType();
-    qreal prevBlockAscent = 0.0;
-    qreal prevBlockDescent= 0.0;
-
+    svgWriter.writeAttribute("style", style(mostCommonCharFormat, block.blockFormat()));
+    //insert the style of the first block and the first block text format into the text style.
     while (block.isValid()) {
-        const QTextBlockFormat blockFormatDiff = formatDifference(block.blockFormat(), mostCommonBlockFormat).toBlockFormat();
-        QTextCharFormat blockCharFormatDiff = QTextCharFormat();
-        const QVector<QTextLayout::FormatRange> formats = block.textFormats();
-        if (formats.size()==1) {
-            blockCharFormatDiff = formatDifference(formats.at(0).format, mostCommonCharFormat).toCharFormat();
-        }
+        QTextLayout *layout = block.layout();
+        QVector<QTextLayout::FormatRange> formats = block.textFormats();
+        QTextBlockFormat blockFormatDiff = formatDifference(block.blockFormat(), doc->firstBlock().blockFormat()).toBlockFormat();
 
-        const QTextLayout *layout = block.layout();
-        const QTextLine line = layout->lineAt(0);
+        int lineHeight = doc->firstBlock().blockFormat().lineHeight();
+        int lineHeightType = doc->firstBlock().blockFormat().lineHeightType();
+        if (blockFormatDiff.hasProperty(QTextFormat::LineHeight)) {
+            lineHeight = blockFormatDiff.lineHeight();
+            lineHeightType = blockFormatDiff.lineHeightType();
+        }
+        if (lineHeightType == QTextBlockFormat::SingleHeight) {
+            //single line will set lineHeight to 0...
+            lineHeight = 100;
+        }
+        QTextLine line = layout->lineAt(0);
+
+        double lineHeightPt = line.height()*double(lineHeight)/100.0;
 
         svgWriter.writeStartElement("tspan");
-
-        {
-            const QString blockStyleString = style(blockCharFormatDiff, blockFormatDiff);
-            if (!blockStyleString.isEmpty()) {
-                svgWriter.writeAttribute("style", blockStyleString);
-            }
-        }
-
-        if (block.blockNumber() > 0) {
-            const qreal lineHeightPt =
-                line.ascent() - prevBlockAscent +
-                (prevBlockAscent + prevBlockDescent) * qreal(prevBlockRelativeLineSpacing) / 100.0;
-
+        if (block.blockNumber()>0) {
             svgWriter.writeAttribute("x", "0");
-            svgWriter.writeAttribute("dy", KisDomUtils::toString(lineHeightPt) + "pt");
+            svgWriter.writeAttribute("dy", QString::number(lineHeightPt)+"pt");
         }
 
-        prevBlockRelativeLineSpacing =
-            blockFormatDiff.hasProperty(QTextFormat::LineHeight) ?
-                blockFormatDiff.lineHeight() :
-                mostCommonBlockFormat.lineHeight();
-
-        prevBlockLineType =
-            blockFormatDiff.hasProperty(QTextFormat::LineHeightType) ?
-                blockFormatDiff.lineHeightType() :
-                mostCommonBlockFormat.lineHeightType();
-
-        if (prevBlockLineType == QTextBlockFormat::SingleHeight) {
-            //single line will set lineHeight to 100%
-            prevBlockRelativeLineSpacing = 100;
-        }
-
-        prevBlockAscent = line.ascent();
-        prevBlockDescent = line.descent();
-
         QString text = block.text();
 
-
         if (formats.size()>1) {
             QStringList texts;
             QVector<QTextCharFormat> charFormats;
@@ -585,9 +459,10 @@ bool KoSvgTextShapeMarkupConverter::convertDocumentToSvg(const QTextDocument *do
 
             for (int c = 0; c<texts.size(); c++) {
                 QTextCharFormat diff = formatDifference(charFormats.at(c), mostCommonCharFormat).toCharFormat();
+
                 if (!diff.properties().isEmpty()) {
                     svgWriter.writeStartElement("tspan");
-                    svgWriter.writeAttribute("style", style(diff, QTextBlockFormat(), mostCommonCharFormat));
+                    svgWriter.writeAttribute("style", style(diff, blockFormatDiff, mostCommonCharFormat));
                     svgWriter.writeCharacters(texts.at(c));
                     svgWriter.writeEndElement();
                 } else {
@@ -613,161 +488,103 @@ bool KoSvgTextShapeMarkupConverter::convertDocumentToSvg(const QTextDocument *do
     return true;
 }
 
-void parseTextAttributes(const QXmlStreamAttributes &elementAttributes,
-                         QTextCharFormat &charFormat,
-                         QTextBlockFormat &blockFormat)
-{
-    QString styleString;
-
-    // we convert all the presentation attributes into styles
-    QString presentationAttributes;
-    for (int a = 0; a < elementAttributes.size(); a++) {
-        if (elementAttributes.at(a).name() != "style") {
-            presentationAttributes
-                    .append(elementAttributes.at(a).name().toString())
-                    .append(":")
-                    .append(elementAttributes.at(a).value().toString())
-                    .append(";");
-        }
-    }
-
-    if (presentationAttributes.endsWith(";")) {
-        presentationAttributes.chop(1);
-    }
-
-    if (elementAttributes.hasAttribute("style")) {
-        styleString = elementAttributes.value("style").toString();
-        if (styleString.endsWith(";")) {
-            styleString.chop(1);
-        }
-    }
-
-    if (!styleString.isEmpty() || !presentationAttributes.isEmpty()) {
-        //add attributes to parse them as part of the style.
-        styleString.append(";")
-                .append(presentationAttributes);
-        QStringList styles = styleString.split(";");
-        QVector<QTextFormat> formats = KoSvgTextShapeMarkupConverter::stylesFromString(styles, charFormat, blockFormat);
-
-        charFormat = formats.at(0).toCharFormat();
-        blockFormat = formats.at(1).toBlockFormat();
-    }
-}
-
 bool KoSvgTextShapeMarkupConverter::convertSvgToDocument(const QString &svgText, QTextDocument *doc)
 {
     QXmlStreamReader svgReader(svgText.trimmed());
     doc->clear();
     QTextCursor cursor(doc);
-
-    struct BlockFormatRecord {
-        BlockFormatRecord() {}
-        BlockFormatRecord(QTextBlockFormat _blockFormat,
-                          QTextCharFormat _charFormat)
-            : blockFormat(_blockFormat),
-              charFormat(_charFormat)
-        {}
-
-        QTextBlockFormat blockFormat;
-        QTextCharFormat charFormat;
-    };
-
-    QStack<BlockFormatRecord> formatStack;
-    formatStack.push(BlockFormatRecord(cursor.blockFormat(), cursor.charFormat()));
-
-    qreal currBlockAbsoluteLineOffset = 0.0;
-    int prevBlockCursorPosition = -1;
-    qreal prevLineDescent = 0.0;
-    qreal prevLineAscent = 0.0;
+    bool newBlock = false;
+    QString dyString;
+    QTextBlockFormat mainBlockFormat = cursor.blockFormat();
+    QTextCharFormat mainCharFormat = cursor.charFormat();
+    QTextCharFormat oldFormat = cursor.charFormat();
 
     while (!svgReader.atEnd()) {
         QXmlStreamReader::TokenType token = svgReader.readNext();
         switch (token) {
         case QXmlStreamReader::StartElement:
         {
-            bool newBlock = false;
-            QTextBlockFormat newBlockFormat;
-            QTextCharFormat newCharFormat;
-            qreal absoluteLineOffset = 1.0;
+            newBlock = false;
+            if (svgReader.name() == "tspan") {
+                if (svgReader.attributes().hasAttribute("dy")) {
+                    newBlock = true;
+                    oldFormat = mainCharFormat;
+                } else {
+                    oldFormat = cursor.charFormat();
+                }
 
-            // fetch format of the parent block and make it default
-            if (formatStack.size() >= 2) {
-                newBlockFormat = formatStack[formatStack.size() - 2].blockFormat;
-                newCharFormat = formatStack[formatStack.size() - 2].charFormat;
             }
 
-            {
-                const QXmlStreamAttributes elementAttributes = svgReader.attributes();
-                parseTextAttributes(elementAttributes, newCharFormat, newBlockFormat);
-
-                // mnemonic for a newline is (dy != 0 && x == 0)
-
-                if (svgReader.name() != "text" &&
-                    elementAttributes.hasAttribute("dy") &&
-                    elementAttributes.hasAttribute("x")) {
-
-                    QString xString = elementAttributes.value("x").toString();
-                    if (xString.contains("pt")) {
-                        xString = xString.remove("pt").trimmed();
-                    }
-
-                    if (KisDomUtils::toDouble(xString) == 0.0) {
-
-                        QString dyString = elementAttributes.value("dy").toString();
-                        if (dyString.contains("pt")) {
-                            dyString = dyString.remove("pt").trimmed();
-                        }
-
-                        KIS_SAFE_ASSERT_RECOVER_NOOP(formatStack.isEmpty() == (svgReader.name() == "text"));
-
-                        absoluteLineOffset = KisDomUtils::toDouble(dyString);
-                        newBlock = absoluteLineOffset > 0;
-                    }
+            QString styleString;
+            QString attributes;
+            for (int a = 0; a<svgReader.attributes().size(); a++) {
+                if (svgReader.attributes().at(a).name() != "style")
+                    attributes.append(svgReader.attributes().at(a).name().toString())
+                            .append(":")
+                            .append(svgReader.attributes().at(a).value().toString())
+                            .append(";");
+            }
+            if (attributes.endsWith(";")) {
+                attributes.chop(1);
+            }
+            if (svgReader.attributes().hasAttribute("style")) {
+                styleString = svgReader.attributes().value("style").toString();
+                if (styleString.endsWith(";")) {
+                    styleString.chop(1);
                 }
             }
+            if (styleString.size()>0 || attributes.size()>0) {
+                //add attributes to parse them as part of the style.
+                styleString.append(";")
+                        .append(attributes);
+                QStringList styles = styleString.split(";");
+                QVector<QTextFormat> formats = stylesFromString(styles, cursor.charFormat(), cursor.blockFormat());
+                cursor.mergeCharFormat(formats.at(0).toCharFormat());
+                cursor.mergeBlockFormat(formats.at(1).toBlockFormat());
+
+                if (svgReader.name()=="text") {
+                    mainBlockFormat = cursor.blockFormat();
+                    mainCharFormat = cursor.charFormat();
+                    oldFormat = mainCharFormat;
+                }
+            }
+            if (svgReader.attributes().hasAttribute("dy")) {
+                dyString = svgReader.attributes().value("dy").toString();
+            } else {
+                dyString.clear();
+            }
 
             //hack
             doc->setTextWidth(100);
             doc->setTextWidth(-1);
-
-            if (newBlock && absoluteLineOffset > 0) {
-                KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(!formatStack.isEmpty(), false);
-                KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(cursor.block().layout()->lineCount() == 1, false);
-
-                QTextLine line = cursor.block().layout()->lineAt(0);
-
-                if (prevBlockCursorPosition >= 0) {
-                    postCorrectBlockHeight(doc, line.ascent(), prevLineAscent, prevLineDescent,
-                                           prevBlockCursorPosition, currBlockAbsoluteLineOffset);
+            QTextLine line = cursor.block().layout()->lineForTextPosition(cursor.positionInBlock());
+            //Can't do this until there's a first QLine.
+            int lineHeight = 100;
+            if (!dyString.isEmpty()) {
+                QTextBlockFormat format = cursor.blockFormat();
+                if (cursor.block().layout()->lineCount()>0) {
+                    //Always interpret no unit as points..
+                    if (dyString.contains("pt")) {
+                        dyString = dyString.remove("pt").trimmed();
+                    }
+                    qreal dy = dyString.toDouble();
+                    lineHeight = (dy/line.height())*100;
                 }
-
-                prevBlockCursorPosition = cursor.position();
-                prevLineAscent  = line.ascent();
-                prevLineDescent = line.descent();
-                currBlockAbsoluteLineOffset = absoluteLineOffset;
-
-                cursor.insertBlock();
-                cursor.setCharFormat(formatStack.top().charFormat);
-                cursor.setBlockFormat(formatStack.top().blockFormat);
+                format.setLineHeight(lineHeight, QTextBlockFormat::ProportionalHeight);
+                cursor.setBlockFormat(format);
             }
 
-
-
-            cursor.mergeCharFormat(newCharFormat);
-            cursor.mergeBlockFormat(newBlockFormat);
-
-            formatStack.push(BlockFormatRecord(cursor.blockFormat(), cursor.charFormat()));
-
+            if (newBlock) {
+                cursor.setBlockCharFormat(cursor.charFormat());
+                cursor.insertBlock();
+            }
             break;
+
         }
         case QXmlStreamReader::EndElement:
         {
-            if (svgReader.name() != "text") {
-                formatStack.pop();
-                KIS_SAFE_ASSERT_RECOVER(!formatStack.isEmpty()) { break; }
-
-                cursor.setCharFormat(formatStack.top().charFormat);
-                cursor.setBlockFormat(formatStack.top().blockFormat);
+            if (!newBlock && svgReader.name()!="text" ) {
+                cursor.setCharFormat(oldFormat);
             }
             break;
         }
@@ -780,16 +597,9 @@ bool KoSvgTextShapeMarkupConverter::convertSvgToDocument(const QString &svgText,
             break;
         }
         default:
-            break;
+            ;
         }
     }
-
-    if (prevBlockCursorPosition >= 0) {
-        QTextLine line = cursor.block().layout()->lineAt(0);
-        postCorrectBlockHeight(doc, line.ascent(), prevLineAscent, prevLineDescent,
-                               prevBlockCursorPosition, currBlockAbsoluteLineOffset);
-    }
-
     if (svgReader.hasError()) {
         d->errors << svgReader.errorString();
         return false;
@@ -822,11 +632,6 @@ QString KoSvgTextShapeMarkupConverter::style(QTextCharFormat format, QTextBlockF
         if (propertyId == QTextCharFormat::FontPointSize) {
             c.append("font-size").append(":")
                     .append(format.properties()[propertyId].toString()+"pt");
-            style.append(c);
-            c.clear();
-            QFontMetricsF metrics(format.fontFamily());
-            qreal xRatio = metrics.xHeight()/metrics.height();
-            c.append("font-size-adjust").append(":").append(QString::number(xRatio));
         }
         if (propertyId == QTextCharFormat::FontPixelSize) {
             c.append("font-size").append(":")
@@ -870,7 +675,7 @@ QString KoSvgTextShapeMarkupConverter::style(QTextCharFormat format, QTextBlockF
         if (propertyId == QTextCharFormat::FontKerning) {
             QString val = "normal";
             if(!format.fontKerning()) {
-                val = "0";
+                val = "none";
             }
             c.append("kerning").append(":")
                     .append(val);
@@ -927,32 +732,32 @@ QString KoSvgTextShapeMarkupConverter::style(QTextCharFormat format, QTextBlockF
         }
     }
 
+    if (format.hasProperty(QTextCharFormat::FontUnderline)
+            || format.hasProperty(QTextCharFormat::FontOverline)
+            || format.hasProperty(QTextCharFormat::FontStrikeOut)) {
+        QString c;
         if (format.underlineStyle()!=QTextCharFormat::NoUnderline ||
                 format.underlineStyle() != QTextCharFormat::SpellCheckUnderline) {
             QStringList values;
-            QString c;
 
-            if (format.fontUnderline()) {
+            if (format.hasProperty(QTextCharFormat::FontUnderline)) {
+
                 values.append("underline");
-            }
-            if(format.fontOverline()) {
+
+            } else if(format.hasProperty(QTextCharFormat::FontOverline)) {
                 values.append("overline");
-            }
-            if(format.fontStrikeOut()) {
-                values.append("line-through");
+            } else {
+                values.append("strike-through");
             }
             c.append("text-decoration").append(":")
                     .append(values.join(" "));
-
-            if (!values.isEmpty()) {
-                style.append(c);
-            }
         }
+        if (!c.isEmpty()) {
+            style.append(c);
+        }
+    }
 
     if (blockFormat.hasProperty(QTextBlockFormat::BlockAlignment)) {
-        // TODO: Alignment works incorrectly! The offsets should be calculated
-        //       according to the shape width/height!
-
         QString c;
         QString val;
         if (blockFormat.alignment()==Qt::AlignRight) {
@@ -974,13 +779,10 @@ QString KoSvgTextShapeMarkupConverter::style(QTextCharFormat format, QTextBlockF
 
 QVector<QTextFormat> KoSvgTextShapeMarkupConverter::stylesFromString(QStringList styles, QTextCharFormat currentCharFormat, QTextBlockFormat currentBlockFormat)
 {
-    Q_UNUSED(currentBlockFormat);
-
     QVector<QTextFormat> formats;
     QTextCharFormat charFormat;
     charFormat.setTextOutline(currentCharFormat.textOutline());
     QTextBlockFormat blockFormat;
-    SvgGraphicsContext *context = new SvgGraphicsContext();
 
     for (int i=0; i<styles.size(); i++) {
         if (!styles.at(i).isEmpty()){
@@ -993,8 +795,29 @@ QVector<QTextFormat> KoSvgTextShapeMarkupConverter::stylesFromString(QStringList
             }
 
             if (property == "font-size") {
-                qreal val = SvgUtil::parseUnitX(context, value);
-                charFormat.setFontPointSize(val);
+                if (value.contains("%")) {
+                    value = value.remove("%");
+                    int fontSize = currentCharFormat.fontPointSize();
+                    if (fontSize<0) {
+                        fontSize = 10;
+                    }
+                    charFormat.setFontPointSize(qreal(value.toInt()/100*fontSize));
+                } else if (value.contains("em")) {
+                    value = value.remove("em");
+                    int fontSize = currentCharFormat.fontPointSize();
+                    if (fontSize<0) {
+                        fontSize = 10;
+                    }
+                    charFormat.setFontPointSize(qreal(value.toInt()*fontSize));
+                } else if (value.contains("px")) {
+                    value = value.remove("px");
+                    QFont font = charFormat.font();
+                    font.setPixelSize(value.toInt());
+                    charFormat.setFont(font);
+                } else {
+                    value = value.remove("pt");
+                    charFormat.setFontPointSize(value.toDouble());
+                }
             }
 
             if (property == "font-variant") {
@@ -1025,14 +848,11 @@ QVector<QTextFormat> KoSvgTextShapeMarkupConverter::stylesFromString(QStringList
                 charFormat.setFontUnderline(false);
                 charFormat.setFontOverline(false);
                 charFormat.setFontStrikeOut(false);
-                QStringList values = value.split(" ");
-                if (values.contains("line-through")) {
+                if (value == "strike-through") {
                     charFormat.setFontStrikeOut(true);
-                }
-                if (values.contains("overline")) {
+                } else if (value == "overline") {
                     charFormat.setFontOverline(true);
-                }
-                if(values.contains("underline")){
+                } else {
                     charFormat.setFontUnderline(true);
                 }
             }
@@ -1050,24 +870,18 @@ QVector<QTextFormat> KoSvgTextShapeMarkupConverter::stylesFromString(QStringList
             }
 
             if (property == "letter-spacing") {
-                qreal val = SvgUtil::parseUnitX(context, value);
-                charFormat.setFontLetterSpacingType(QFont::AbsoluteSpacing);
-                charFormat.setFontLetterSpacing(val);
+                charFormat.setFontLetterSpacing(value.toDouble());
             }
 
             if (property == "word-spacing") {
-                qreal val = SvgUtil::parseUnitX(context, value);
-                charFormat.setFontWordSpacing(val);
+                charFormat.setFontWordSpacing(value.toDouble());
             }
 
             if (property == "kerning") {
                 if (value=="normal") {
                     charFormat.setFontKerning(true);
                 } else {
-                    qreal val = SvgUtil::parseUnitX(context, value);
                     charFormat.setFontKerning(false);
-                    charFormat.setFontLetterSpacingType(QFont::AbsoluteSpacing);
-                    charFormat.setFontLetterSpacing(charFormat.fontLetterSpacing() + val);
                 }
             }
 
@@ -1094,6 +908,7 @@ QVector<QTextFormat> KoSvgTextShapeMarkupConverter::stylesFromString(QStringList
             }
 
             if (property == "text-anchor") {
+                qDebug()<<"setting alignment"<< value;
                 if (value == "end") {
                     blockFormat.setAlignment(Qt::AlignRight);
                 } else if (value == "middle") {
