@@ -363,13 +363,11 @@ void KoPathTool::convertToPath()
     QList<KoParameterShape*> shapesToConvert;
     Q_FOREACH (KoShape *shape, m_pointSelection.selectedShapes()) {
         KoParameterShape * parameterShape = dynamic_cast<KoParameterShape*>(shape);
-        if (parameterShape && parameterShape->isParametricShape()) {
+        if (parameterShape && parameterShape->isParametricShape())
             shapesToConvert.append(parameterShape);
-        }
     }
-    if (!shapesToConvert.isEmpty()) {
+    if (shapesToConvert.count())
         d->canvas->addCommand(new KoParameterToPathCommand(shapesToConvert));
-    }
     updateOptionsWidget();
 }
 
@@ -831,16 +829,14 @@ void KoPathTool::mouseDoubleClickEvent(KoPointerEvent *event)
     // check if we are doing something else at the moment
     if (m_currentStrategy) return;
 
-    if (!m_activeHandle && m_activeSegment && m_activeSegment->isValid()) {
-        QList<KoPathPointData> segments;
-        segments.append(
-            KoPathPointData(m_activeSegment->path,
-                            m_activeSegment->path->pathPointIndex(m_activeSegment->segmentStart)));
+    QScopedPointer<PathSegment> s(segmentAtPoint(event->point));
 
-        KoPathPointInsertCommand *cmd = new KoPathPointInsertCommand(segments, m_activeSegment->positionOnSegment);
+    if (s && s->isValid()) {
+        QList<KoPathPointData> segments;
+        segments.append(KoPathPointData(s->path, s->path->pathPointIndex(s->segmentStart)));
+        KoPathPointInsertCommand *cmd = new KoPathPointInsertCommand(segments, s->positionOnSegment);
         d->canvas->addCommand(cmd);
 
-        m_pointSelection.clear();
         foreach (KoPathPoint * p, cmd->insertedPoints()) {
             m_pointSelection.add(p, false);
         }
@@ -848,23 +844,18 @@ void KoPathTool::mouseDoubleClickEvent(KoPointerEvent *event)
         event->accept();
     } else if (!m_activeHandle && !m_activeSegment && m_activatedTemporarily) {
         emit done();
-        event->accept();
-    } else if (!m_activeHandle && !m_activeSegment) {
-        KoShapeManager *shapeManager = canvas()->shapeManager();
-        KoSelection *selection = shapeManager->selection();
-
-        selection->deselectAll();
-        event->accept();
     }
 }
 
 KoPathTool::PathSegment* KoPathTool::segmentAtPoint(const QPointF &point)
 {
     Q_D(KoToolBase);
+    const int clickProximity = 5;
 
+    // convert click proximity to point using the current zoom level
+    QPointF clickOffset = d->canvas->viewConverter()->viewToDocument(QPointF(clickProximity, clickProximity));
     // the max allowed distance from a segment
-    const QRectF grabRoi = handleGrabRect(point);
-    const qreal distanceThreshold = 0.5 * KisAlgebra2D::maxDimension(grabRoi);
+    const qreal maxSquaredDistance = clickOffset.x()*clickOffset.x();
 
     QScopedPointer<PathSegment> segment(new PathSegment);
 
@@ -874,25 +865,23 @@ KoPathTool::PathSegment* KoPathTool::segmentAtPoint(const QPointF &point)
             continue;
 
         // convert document point to shape coordinates
-        const QPointF p = shape->documentToShape(point);
+        QPointF p = shape->documentToShape(point);
         // our region of interest, i.e. a region around our mouse position
-        const QRectF roi = shape->documentToShape(grabRoi);
-
-        qreal minDistance = std::numeric_limits<qreal>::max();
+        QRectF roi(p - clickOffset, p + clickOffset);
 
+        qreal minSqaredDistance = HUGE_VAL;
         // check all segments of this shape which intersect the region of interest
-        const QList<KoPathSegment> segments = shape->segmentsAt(roi);
-
+        QList<KoPathSegment> segments = shape->segmentsAt(roi);
         foreach (const KoPathSegment &s, segments) {
-            const qreal nearestPointParam = s.nearestPoint(p);
-            const QPointF nearestPoint = s.pointAt(nearestPointParam);
-            const qreal distance = kisDistance(p, nearestPoint);
-
+            qreal nearestPointParam = s.nearestPoint(p);
+            QPointF nearestPoint = s.pointAt(nearestPointParam);
+            QPointF diff = p - nearestPoint;
+            qreal squaredDistance = diff.x()*diff.x() + diff.y()*diff.y();
             // are we within the allowed distance ?
-            if (distance > distanceThreshold)
+            if (squaredDistance > maxSquaredDistance)
                 continue;
             // are we closer to the last closest point ?
-            if (distance < minDistance) {
+            if (squaredDistance < minSqaredDistance) {
                 segment->path = shape;
                 segment->segmentStart = s.first();
                 segment->positionOnSegment = nearestPointParam;
@@ -922,7 +911,6 @@ void KoPathTool::activate(ToolActivation activation, const QSet<KoShape*> &shape
     useCursor(m_selectCursor);
     m_canvasConnections.addConnection(d->canvas->selectedShapesProxy(), SIGNAL(selectionChanged()), this, SLOT(slotSelectionChanged()));
     m_canvasConnections.addConnection(d->canvas->selectedShapesProxy(), SIGNAL(selectionContentChanged()), this, SLOT(updateActions()));
-    m_shapeFillResourceConnector.connectToCanvas(d->canvas);
 
     initializeWithShapes(shapes.toList());
 }
@@ -946,22 +934,17 @@ void KoPathTool::clearActivePointSelectionReferences()
     m_pointSelection.clear();
 }
 
-#include "kis_pointer_utils.h"
-
 void KoPathTool::initializeWithShapes(const QList<KoShape*> shapes)
 {
     QList<KoPathShape*> selectedShapes;
     Q_FOREACH (KoShape *shape, shapes) {
         KoPathShape *pathShape = dynamic_cast<KoPathShape*>(shape);
 
-        if (pathShape && pathShape->isShapeEditable()) {
+        if (pathShape && pathShape->isEditable()) {
             selectedShapes.append(pathShape);
         }
     }
 
-    const QRectF oldBoundingRect =
-        KoShape::boundingRect(implicitCastList<KoShape*>(m_pointSelection.selectedShapes()));
-
     if (selectedShapes != m_pointSelection.selectedShapes()) {
         clearActivePointSelectionReferences();
         m_pointSelection.setSelectedShapes(selectedShapes);
@@ -974,7 +957,6 @@ void KoPathTool::initializeWithShapes(const QList<KoShape*> shapes)
         // current canvas
         repaint(shape->boundingRect());
     }
-    repaint(oldBoundingRect);
 
     updateOptionsWidget();
     updateActions();
@@ -1088,7 +1070,6 @@ void KoPathTool::deactivate()
 {
     Q_D(KoToolBase);
 
-    m_shapeFillResourceConnector.disconnect();
     m_canvasConnections.clear();
     m_pointSelection.clear();
     m_pointSelection.setSelectedShapes(QList<KoPathShape*>());
