@@ -39,12 +39,16 @@ class KisLockedPropertiesProxy: public KisPropertiesConfiguration
 {
 public:
     KisLockedPropertiesProxy(KisPropertiesConfiguration *, KisLockedPropertiesSP);
-    ~KisLockedPropertiesProxy();
+    ~KisLockedPropertiesProxy() override;
 
     using KisPropertiesConfiguration::getProperty;
-    QVariant getProperty(const QString &name) const;
+    QVariant getProperty(const QString &name) const override;
     using KisPropertiesConfiguration::setProperty;
-    void setProperty(const QString & name, const QVariant & value);
+    void setProperty(const QString & name, const QVariant & value) override;
+
+    bool hasProperty(const QString& name) const override;
+
+    QList<QString> getPropertiesKeys() const;
 
 private:
     Q_DISABLE_COPY(KisLockedPropertiesProxy)
