@@ -33,17 +33,20 @@
 #include "kis_painter.h"
 #include "kis_paint_device.h"
 #include "kis_paintop_registry.h"
+#include "kis_timing_information.h"
 #include <brushengine/kis_paint_information.h>
 #include "kis_paintop_config_widget.h"
 #include <brushengine/kis_paintop_preset.h>
 #include "kis_paintop_settings_update_proxy.h"
 #include <time.h>
-#include<kis_types.h>
+#include <kis_types.h>
 #include <kis_signals_blocker.h>
 
 #include <brushengine/kis_locked_properties_server.h>
 #include <brushengine/kis_locked_properties_proxy.h>
 
+#include "KisPaintopSettingsIds.h"
+
 
 struct Q_DECL_HIDDEN KisPaintOpSettings::Private {
     Private()
@@ -77,11 +80,13 @@ struct Q_DECL_HIDDEN KisPaintOpSettings::Private {
     };
 
     KisPaintopSettingsUpdateProxy* updateProxyNoCreate() const {
-        return preset ? preset->updateProxyNoCreate() : 0;
+        auto presetSP = preset.toStrongRef();
+        return presetSP ? presetSP->updateProxyNoCreate() : 0;
     }
 
     KisPaintopSettingsUpdateProxy* updateProxyCreate() const {
-        return preset ? preset->updateProxy() : 0;
+        auto presetSP = preset.toStrongRef();
+        return presetSP ? presetSP->updateProxy() : 0;
     }
 };
 
@@ -142,6 +147,33 @@ void KisPaintOpSettings::setRandomOffset(const KisPaintInformation &paintInforma
 
 }
 
+bool KisPaintOpSettings::hasMaskingSettings() const
+{
+    return getBool(KisPaintOpUtils::MaskingBrushEnabledTag, false);
+}
+
+KisPaintOpSettingsSP KisPaintOpSettings::createMaskingSettings() const
+{
+    if (!hasMaskingSettings()) return KisPaintOpSettingsSP();
+
+    const KoID pixelBrushId(KisPaintOpUtils::MaskingBrushPaintOpId, QString());
+
+    KisPaintOpSettingsSP maskingSettings = KisPaintOpRegistry::instance()->settings(pixelBrushId);
+    this->getPrefixedProperties(KisPaintOpUtils::MaskingBrushPresetPrefix, maskingSettings);
+
+    const bool useMasterSize = this->getBool(KisPaintOpUtils::MaskingBrushUseMasterSizeTag, true);
+    if (useMasterSize) {
+        const qreal masterSizeCoeff = getDouble(KisPaintOpUtils::MaskingBrushMasterSizeCoeffTag, 1.0);
+        maskingSettings->setPaintOpSize(masterSizeCoeff * paintOpSize());
+    }
+
+    return maskingSettings;
+}
+
+QString KisPaintOpSettings::maskingBrushCompositeOp() const
+{
+    return getString(KisPaintOpUtils::MaskingBrushCompositeOpTag, COMPOSITE_MULT);
+}
 
 KisPaintOpSettingsSP KisPaintOpSettings::clone() const
 {
@@ -308,6 +340,32 @@ QString KisPaintOpSettings::indirectPaintingCompositeOp() const
     return COMPOSITE_ALPHA_DARKEN;
 }
 
+bool KisPaintOpSettings::isAirbrushing() const
+{
+    return getBool(AIRBRUSH_ENABLED, false);
+}
+
+qreal KisPaintOpSettings::airbrushInterval() const
+{
+    qreal rate = getDouble(AIRBRUSH_RATE, 1.0);
+    if (rate == 0.0) {
+        return LONG_TIME;
+    }
+    else {
+        return 1000.0 / rate;
+    }
+}
+
+bool KisPaintOpSettings::useSpacingUpdates() const
+{
+    return getBool(SPACING_USE_UPDATES, false);
+}
+
+bool KisPaintOpSettings::needsAsynchronousUpdates() const
+{
+    return false;
+}
+
 QPainterPath KisPaintOpSettings::brushOutline(const KisPaintInformation &info, OutlineMode mode)
 {
     QPainterPath path;
@@ -379,9 +437,11 @@ void KisPaintOpSettings::setCanvasMirroring(bool xAxisMirrored, bool yAxisMirror
 void KisPaintOpSettings::setProperty(const QString & name, const QVariant & value)
 {
     if (value != KisPropertiesConfiguration::getProperty(name) &&
-            !d->disableDirtyNotifications && this->preset()) {
-
-        this->preset()->setPresetDirty(true);
+            !d->disableDirtyNotifications) {
+        KisPaintOpPresetSP presetSP = preset().toStrongRef();
+        if (presetSP) {
+            presetSP->setPresetDirty(true);
+        }
     }
 
     KisPropertiesConfiguration::setProperty(name, value);
@@ -408,6 +468,21 @@ void KisPaintOpSettings::setLodUserAllowed(KisPropertiesConfigurationSP config,
     config->setProperty("lodUserAllowed", value);
 }
 
+bool KisPaintOpSettings::lodSizeThresholdSupported() const
+{
+    return true;
+}
+
+qreal KisPaintOpSettings::lodSizeThreshold() const
+{
+    return getDouble("lodSizeThreshold", 100.0);
+}
+
+void KisPaintOpSettings::setLodSizeThreshold(qreal value)
+{
+    setProperty("lodSizeThreshold", value);
+}
+
 #include "kis_standard_uniform_properties_factory.h"
 
 QList<KisUniformPaintOpPropertySP> KisPaintOpSettings::uniformProperties(KisPaintOpSettingsSP settings)
