@@ -34,6 +34,28 @@
 class KisPaintOpConfigWidget;
 class KisPaintopSettingsUpdateProxy;
 
+/**
+ * Configuration property used to control whether airbrushing is enabled.
+ */
+const QString AIRBRUSH_ENABLED = "PaintOpSettings/isAirbrushing";
+
+/**
+ * Configuration property used to control airbrushing rate. The value should be in dabs per second.
+ */
+const QString AIRBRUSH_RATE = "PaintOpSettings/rate";
+
+/**
+ * Configuration property used to control whether airbrushing is configured to ignore distance-based
+ * spacing.
+ */
+const QString AIRBRUSH_IGNORE_SPACING = "PaintOpSettings/ignoreSpacing";
+
+/**
+ * Configuration property used to control whether the spacing settings can be updated between
+ * painted dabs.
+ */
+const QString SPACING_USE_UPDATES = "PaintOpSettings/updateSpacingBetweenDabs";
+
 /**
  * This class is used to cache the settings for a paintop
  * between two creations. There is one KisPaintOpSettings per input device (mouse, tablet,
@@ -59,7 +81,7 @@ public:
     /**
      *
      */
-    virtual void setOptionsWidget(KisPaintOpConfigWidget* widget);
+    void setOptionsWidget(KisPaintOpConfigWidget* widget);
 
     /**
      * This function is called by a tool when the mouse is pressed. It's useful if
@@ -104,19 +126,36 @@ public:
     virtual QString indirectPaintingCompositeOp() const;
 
     /**
-     * Whether this paintop wants to deposit paint even when not moving, i.e. the
-     * tool needs to activate its timer.
+     * Whether this paintop wants to deposit paint even when not moving, i.e. the tool needs to
+     * activate its timer. If this is true, painting updates need to be generated at regular
+     * intervals even in the absence of input device events, e.g. when the cursor is not moving.
+     *
+     * The default implementation checks the property AIRBRUSH_ENABLED, defaulting to false if the
+     * property is not found. This should be suitable for most paintops.
      */
-    virtual bool isAirbrushing() const {
-        return false;
-    }
+    virtual bool isAirbrushing() const;
 
     /**
-    * If this paintop deposit the paint even when not moving, the tool needs to know the rate of it in miliseconds
-    */
-    virtual int rate() const {
-        return 100;
-    }
+     * Indicates the minimum time interval that might be needed between airbrush dabs, in
+     * milliseconds. A lower value means painting updates need to happen more frequently. This value
+     * should be ignored if isAirbrushing() is false.
+     *
+     * The default implementation uses the property AIRBRUSH_RATE, defaulting to an interval of
+     * one second if the property is not found. This should be suitable for most paintops.
+     */
+    virtual qreal airbrushInterval() const;
+
+    /**
+     * Indicates whether this configuration allows spacing information to be updated between painted
+     * dabs during a stroke.
+     */
+    virtual bool useSpacingUpdates() const;
+
+    /**
+     * Indicates if the tool should call paintOp->doAsynchronousUpdate() inbetween
+     * paintAt() calls to do the asynchronous rendering
+     */
+    virtual bool needsAsynchronousUpdates() const;
 
     /**
      * This enum defines the current mode for painting an outline.
@@ -251,6 +290,11 @@ public:
     static bool isLodUserAllowed(const KisPropertiesConfigurationSP config);
     static void setLodUserAllowed(KisPropertiesConfigurationSP config, bool value);
 
+    virtual bool lodSizeThresholdSupported() const;
+
+    qreal lodSizeThreshold() const;
+    void setLodSizeThreshold(qreal value);
+
     /**
     * @return the option widget of the paintop (can be 0 is no option widgets is set)
     */
@@ -264,6 +308,26 @@ public:
      */
     virtual void setRandomOffset(const KisPaintInformation &paintInformation);
 
+    /**
+     * @return true if this preset demands a secondary masked brush running
+     *         alongside it
+     */
+    bool hasMaskingSettings() const;
+
+    /**
+     * @return a newly created settings object representing a preset of the masking
+     *         brush that should be run alongside the current brush
+     */
+    KisPaintOpSettingsSP createMaskingSettings() const;
+
+    /**
+     * @return a composite op id of the masked brush rendering algorithm.
+     *
+     * Please take into account that the brush itself always paints in alpha-
+     * darken mode, but the final result is combined with this composite op.
+     */
+    QString maskingBrushCompositeOp() const;
+
 protected:
 
     /**
