@@ -74,7 +74,7 @@ KisFilterConfigurationSP KisFilterRegistry::cloneConfiguration(const KisFilterCo
 {
     Q_ASSERT(kfc);
     KisFilterSP filter = value(kfc->name());
-    KisFilterConfigurationSP newkfc = filter->defaultConfiguration(0);
+    KisFilterConfigurationSP newkfc(filter->defaultConfiguration());
     newkfc->fromXML(kfc->toXML());
     return newkfc;
 }
