@@ -368,7 +368,7 @@ public:
      * @param visible the new visibility state
      * @param isLoading if true, the property is set during loading.
      */
-    virtual void setVisible(bool visible, bool loading = false);
+    virtual void setVisible(bool visibile, bool loading = false);
 
     /**
      * Return the locked status of this node. Locked nodes cannot be
@@ -380,7 +380,7 @@ public:
      * Set the locked status of this node. Locked nodes cannot be
      * edited.
      */
-    virtual void setUserLocked(bool l);
+    void setUserLocked(bool l);
 
     /**
      * @return true if the node can be edited:
