@@ -55,10 +55,10 @@ KisColorSpaceConvertVisitor::~KisColorSpaceConvertVisitor()
 bool KisColorSpaceConvertVisitor::visit(KisGroupLayer * layer)
 {
     convertPaintDevice(layer);
-    KisLayerSP child = dynamic_cast<KisLayer*>(layer->firstChild().data());
+    KisLayerSP child = qobject_cast<KisLayer*>(layer->firstChild().data());
     while (child) {
         child->accept(*this);
-        child = dynamic_cast<KisLayer*>(child->nextSibling().data());
+        child = qobject_cast<KisLayer*>(child->nextSibling().data());
     }
 
     layer->resetCache();
@@ -109,24 +109,29 @@ bool KisColorSpaceConvertVisitor::convertPaintDevice(KisLayer* layer)
         }
     }
 
+    KisImageSP image = m_image.toStrongRef();
+    if (!image) {
+        return false;
+    }
+
     if (layer->original()) {
         KUndo2Command* cmd = layer->original()->convertTo(m_dstColorSpace, m_renderingIntent, m_conversionFlags);
         if (cmd) {
-            m_image->undoAdapter()->addCommand(cmd);
+            image->undoAdapter()->addCommand(cmd);
         }
     }
 
     if (layer->paintDevice()) {
         KUndo2Command* cmd = layer->paintDevice()->convertTo(m_dstColorSpace, m_renderingIntent, m_conversionFlags);
         if (cmd) {
-            m_image->undoAdapter()->addCommand(cmd);
+            image->undoAdapter()->addCommand(cmd);
         }
     }
 
     if (layer->projection()) {
         KUndo2Command* cmd = layer->projection()->convertTo(m_dstColorSpace, m_renderingIntent, m_conversionFlags);
         if (cmd) {
-            m_image->undoAdapter()->addCommand(cmd);
+            image->undoAdapter()->addCommand(cmd);
         }
     }
 
@@ -143,9 +148,13 @@ bool KisColorSpaceConvertVisitor::convertPaintDevice(KisLayer* layer)
 
 bool KisColorSpaceConvertVisitor::visit(KisColorizeMask *mask)
 {
+    KisImageSP image = m_image.toStrongRef();
+    if (!image) {
+        return false;
+    }
     KUndo2Command* cmd = mask->setColorSpace(m_dstColorSpace, m_renderingIntent, m_conversionFlags);
     if (cmd) {
-        m_image->undoAdapter()->addCommand(cmd);
+        image->undoAdapter()->addCommand(cmd);
     }
     return true;
 }
