@@ -127,56 +127,67 @@ class KisImage::KisImagePrivate
 public:
     KisImagePrivate(KisImage *_q, qint32 w, qint32 h,
                     const KoColorSpace *c,
-                    KisUndoStore *u,
+                    KisUndoStore *undo,
                     KisImageAnimationInterface *_animationInterface)
         : q(_q)
         , lockedForReadOnly(false)
         , width(w)
         , height(h)
-        , colorSpace(c)
+        , colorSpace(c ? c : KoColorSpaceRegistry::instance()->rgb8())
         , nserver(1)
-        , undoStore(u)
-        , legacyUndoAdapter(u, _q)
-        , postExecutionUndoAdapter(u, _q)
+        , undoStore(undo ? undo : new KisDumbUndoStore())
+        , legacyUndoAdapter(undoStore.data(), _q)
+        , postExecutionUndoAdapter(undoStore.data(), _q)
         , recorder(_q)
         , signalRouter(_q)
         , animationInterface(_animationInterface)
-        , scheduler(_q)
+        , scheduler(_q, _q)
         , axesCenter(QPointF(0.5, 0.5))
+    {
         {
-            {
-                KisImageConfig cfg;
-                if (cfg.enableProgressReporting()) {
-                    scheduler.setProgressProxy(&compositeProgressProxy);
-                }
-
-                // Each of these lambdas defines a new factory function.
-                scheduler.setLod0ToNStrokeStrategyFactory(
-                    [=](bool forgettable) {
-                        return KisLodSyncPair(
-                            new KisSyncLodCacheStrokeStrategy(KisImageWSP(q), forgettable),
-                            KisSyncLodCacheStrokeStrategy::createJobsData(KisImageWSP(q)));
-                    });
-
-                scheduler.setSuspendUpdatesStrokeStrategyFactory(
-                    [=]() {
-                        return KisSuspendResumePair(
-                            new KisSuspendProjectionUpdatesStrokeStrategy(KisImageWSP(q), true),
-                            KisSuspendProjectionUpdatesStrokeStrategy::createSuspendJobsData(KisImageWSP(q)));
-                    });
-
-                scheduler.setResumeUpdatesStrokeStrategyFactory(
-                    [=]() {
-                        return KisSuspendResumePair(
-                            new KisSuspendProjectionUpdatesStrokeStrategy(KisImageWSP(q), false),
-                            KisSuspendProjectionUpdatesStrokeStrategy::createResumeJobsData(KisImageWSP(q)));
-                    });
+            KisImageConfig cfg;
+            if (cfg.enableProgressReporting()) {
+                scheduler.setProgressProxy(&compositeProgressProxy);
             }
 
-            connect(q, SIGNAL(sigImageModified()), KisMemoryStatisticsServer::instance(), SLOT(notifyImageChanged()));
+            // Each of these lambdas defines a new factory function.
+            scheduler.setLod0ToNStrokeStrategyFactory(
+                [=](bool forgettable) {
+                    return KisLodSyncPair(
+                        new KisSyncLodCacheStrokeStrategy(KisImageWSP(q), forgettable),
+                        KisSyncLodCacheStrokeStrategy::createJobsData(KisImageWSP(q)));
+                });
+
+            scheduler.setSuspendUpdatesStrokeStrategyFactory(
+                [=]() {
+                    return KisSuspendResumePair(
+                        new KisSuspendProjectionUpdatesStrokeStrategy(KisImageWSP(q), true),
+                        KisSuspendProjectionUpdatesStrokeStrategy::createSuspendJobsData(KisImageWSP(q)));
+                });
+
+            scheduler.setResumeUpdatesStrokeStrategyFactory(
+                [=]() {
+                    return KisSuspendResumePair(
+                        new KisSuspendProjectionUpdatesStrokeStrategy(KisImageWSP(q), false),
+                        KisSuspendProjectionUpdatesStrokeStrategy::createResumeJobsData(KisImageWSP(q)));
+                });
         }
 
+        connect(q, SIGNAL(sigImageModified()), KisMemoryStatisticsServer::instance(), SLOT(notifyImageChanged()));
+    }
 
+    ~KisImagePrivate() {
+        /**
+         * Stop animation interface. It may use the rootLayer.
+         */
+        delete animationInterface;
+
+        /**
+         * First delete the nodes, while strokes
+         * and undo are still alive
+         */
+        rootLayer.clear();
+    }
 
     KisImage *q;
 
@@ -200,7 +211,7 @@ public:
 
     KisNameServer nserver;
 
-    KisUndoStore *undoStore;
+    QScopedPointer<KisUndoStore> undoStore;
     KisLegacyUndoAdapter legacyUndoAdapter;
     KisPostExecutionUndoAdapter postExecutionUndoAdapter;
 
@@ -230,23 +241,14 @@ public:
 KisImage::KisImage(KisUndoStore *undoStore, qint32 width, qint32 height, const KoColorSpace * colorSpace, const QString& name)
         : QObject(0)
         , KisShared()
+        , m_d(new KisImagePrivate(this, width, height,
+                                  colorSpace, undoStore,
+                                  new KisImageAnimationInterface(this)))
 {
+    // make sure KisImage belongs to the GUI thread
+    moveToThread(qApp->thread());
+
     setObjectName(name);
-    // Handle undoStore == 0 and colorSpace == 0 cases
-    if (!undoStore) {
-        undoStore = new KisDumbUndoStore();
-    }
-
-    const KoColorSpace *c;
-    if (colorSpace != 0) {
-        c = colorSpace;
-    } else {
-        c = KoColorSpaceRegistry::instance()->rgb8();
-    }
-    m_d = new KisImagePrivate(this, width, height,
-                              c, undoStore,
-                              new KisImageAnimationInterface(this));
-
     setRootLayer(new KisGroupLayer(this, "root", OPACITY_OPAQUE_U8));
 }
 
@@ -259,18 +261,6 @@ KisImage::~KisImage()
      */
     waitForDone();
 
-    /**
-     * Stop animation interface. It may use the rootLayer.
-     */
-    delete m_d->animationInterface;
-
-    /**
-     * First delete the nodes, while strokes
-     * and undo are still alive
-     */
-    m_d->rootLayer = 0;
-
-    delete m_d->undoStore;
     delete m_d;
     disconnect(); // in case Qt gets confused
 }
@@ -290,6 +280,9 @@ KisImage::KisImage(const KisImage& rhs, KisUndoStore *undoStore, bool exactCopy)
                               undoStore ? undoStore : new KisDumbUndoStore(),
                               new KisImageAnimationInterface(*rhs.animationInterface(), this)))
 {
+    // make sure KisImage belongs to the GUI thread
+    moveToThread(qApp->thread());
+
     setObjectName(rhs.objectName());
 
     m_d->xres = rhs.m_d->xres;
@@ -330,6 +323,11 @@ KisImage::KisImage(const KisImage& rhs, KisUndoStore *undoStore, bool exactCopy)
         newAnnotations << annotation->clone();
     }
     m_d->annotations = newAnnotations;
+
+    KIS_ASSERT_RECOVER_NOOP(!rhs.m_d->projectionUpdatesFilter);
+    KIS_ASSERT_RECOVER_NOOP(!rhs.m_d->disableUIUpdateSignals);
+    KIS_ASSERT_RECOVER_NOOP(!rhs.m_d->disableDirtyRequests);
+
     m_d->blockLevelOfDetail = rhs.m_d->blockLevelOfDetail;
 }
 
@@ -587,7 +585,7 @@ void KisImage::cropImage(const QRect& newRect)
 
 void KisImage::cropNode(KisNodeSP node, const QRect& newRect)
 {
-    bool isLayer = dynamic_cast<KisLayer*>(node.data());
+    bool isLayer = qobject_cast<KisLayer*>(node.data());
     KUndo2MagicString actionName = isLayer ?
         kundo2_i18n("Crop Layer") :
         kundo2_i18n("Crop Mask");
@@ -763,7 +761,11 @@ void KisImage::rotateImage(double radians)
 
 void KisImage::rotateNode(KisNodeSP node, double radians)
 {
-    rotateImpl(kundo2_i18n("Rotate Layer"), node, false, radians);
+    if (node->inherits("KisMask")) {
+        rotateImpl(kundo2_i18n("Rotate Mask"), node, false, radians);
+    } else {
+        rotateImpl(kundo2_i18n("Rotate Layer"), node, false, radians);
+    }
 }
 
 void KisImage::shearImpl(const KUndo2MagicString &actionName,
@@ -830,8 +832,13 @@ void KisImage::shearNode(KisNodeSP node, double angleX, double angleY)
 {
     QPointF shearOrigin = QRectF(bounds()).center();
 
-    shearImpl(kundo2_i18n("Shear layer"), node, false,
-              angleX, angleY, shearOrigin);
+    if (node->inherits("KisMask")) {
+        shearImpl(kundo2_i18n("Shear Mask"), node, false,
+                  angleX, angleY, shearOrigin);
+    } else {
+        shearImpl(kundo2_i18n("Shear Layer"), node, false,
+                  angleX, angleY, shearOrigin);
+    }
 }
 
 void KisImage::shear(double angleX, double angleY)
@@ -873,7 +880,9 @@ bool KisImage::assignImageProfile(const KoColorProfile *profile)
     m_d->colorSpace = dstCs;
 
     KisChangeProfileVisitor visitor(srcCs, dstCs);
-    return m_d->rootLayer->accept(visitor);
+    bool retval = m_d->rootLayer->accept(visitor);
+    m_d->signalRouter.emitNotification(ProfileChangedSignal);
+    return retval;
 
 }
 
@@ -929,10 +938,10 @@ QPointF KisImage::documentToPixel(const QPointF &documentCoord) const
     return QPointF(documentCoord.x() * xRes(), documentCoord.y() * yRes());
 }
 
-QPoint KisImage::documentToIntPixel(const QPointF &documentCoord) const
+QPoint KisImage::documentToImagePixelFloored(const QPointF &documentCoord) const
 {
     QPointF pixelCoord = documentToPixel(documentCoord);
-    return QPoint((int)pixelCoord.x(), (int)pixelCoord.y());
+    return QPoint(qFloor(pixelCoord.x()), qFloor(pixelCoord.y()));
 }
 
 QRectF KisImage::documentToPixel(const QRectF &documentRect) const
@@ -940,11 +949,6 @@ QRectF KisImage::documentToPixel(const QRectF &documentRect) const
     return QRectF(documentToPixel(documentRect.topLeft()), documentToPixel(documentRect.bottomRight()));
 }
 
-QRect KisImage::documentToIntPixel(const QRectF &documentRect) const
-{
-    return documentToPixel(documentRect).toAlignedRect();
-}
-
 QPointF KisImage::pixelToDocument(const QPointF &pixelCoord) const
 {
     return QPointF(pixelCoord.x() / xRes(), pixelCoord.y() / yRes());
@@ -1125,13 +1129,12 @@ void KisImage::setUndoStore(KisUndoStore *undoStore)
 
     m_d->legacyUndoAdapter.setUndoStore(undoStore);
     m_d->postExecutionUndoAdapter.setUndoStore(undoStore);
-    delete m_d->undoStore;
-    m_d->undoStore = undoStore;
+    m_d->undoStore.reset(undoStore);
 }
 
 KisUndoStore* KisImage::undoStore()
 {
-    return m_d->undoStore;
+    return m_d->undoStore.data();
 }
 
 KisUndoAdapter* KisImage::undoAdapter() const
@@ -1439,7 +1442,7 @@ void KisImage::addSpontaneousJob(KisSpontaneousJob *spontaneousJob)
 
 void KisImage::setProjectionUpdatesFilter(KisProjectionUpdatesFilterSP filter)
 {
-    // udpate filters are *not* recursive!
+    // update filters are *not* recursive!
     KIS_ASSERT_RECOVER_NOOP(!filter || !m_d->projectionUpdatesFilter);
 
     m_d->projectionUpdatesFilter = filter;
@@ -1484,6 +1487,16 @@ void KisImage::notifyProjectionUpdated(const QRect &rc)
     }
 }
 
+void KisImage::setWorkingThreadsLimit(int value)
+{
+    m_d->scheduler.setThreadsLimit(value);
+}
+
+int KisImage::workingThreadsLimit() const
+{
+    return m_d->scheduler.threadsLimit();
+}
+
 void KisImage::notifySelectionChanged()
 {
     /**
@@ -1506,24 +1519,25 @@ void KisImage::notifySelectionChanged()
 }
 
 void KisImage::requestProjectionUpdateImpl(KisNode *node,
-                                           const QRect &rect,
+                                           const QVector<QRect> &rects,
                                            const QRect &cropRect)
 {
-    if (rect.isEmpty()) return;
+    if (rects.isEmpty()) return;
 
-    KisNodeGraphListener::requestProjectionUpdate(node, rect);
-    m_d->scheduler.updateProjection(node, rect, cropRect);
+    m_d->scheduler.updateProjection(node, rects, cropRect);
 }
 
-void KisImage::requestProjectionUpdate(KisNode *node, const QRect& rect)
+void KisImage::requestProjectionUpdate(KisNode *node, const QVector<QRect> &rects, bool resetAnimationCache)
 {
     if (m_d->projectionUpdatesFilter
-        && m_d->projectionUpdatesFilter->filter(this, node, rect)) {
+        && m_d->projectionUpdatesFilter->filter(this, node, rects, resetAnimationCache)) {
 
         return;
     }
 
-    m_d->animationInterface->notifyNodeChanged(node, rect, false);
+    if (resetAnimationCache) {
+        m_d->animationInterface->notifyNodeChanged(node, rects, false);
+    }
 
     /**
      * Here we use 'permitted' instead of 'active' intentively,
@@ -1532,29 +1546,31 @@ void KisImage::requestProjectionUpdate(KisNode *node, const QRect& rect)
      * supporting the wrap-around mode will not make much harm.
      */
     if (m_d->wrapAroundModePermitted) {
-        const QRect boundRect = effectiveLodBounds();
-        KisWrappedRect splitRect(rect, boundRect);
+        QVector<QRect> allSplitRects;
 
-        Q_FOREACH (const QRect &rc, splitRect) {
-            requestProjectionUpdateImpl(node, rc, boundRect);
+        const QRect boundRect = effectiveLodBounds();
+        Q_FOREACH (const QRect &rc, rects) {
+            KisWrappedRect splitRect(rc, boundRect);
+            allSplitRects.append(splitRect);
         }
+
+        requestProjectionUpdateImpl(node, allSplitRects, boundRect);
+
     } else {
-        requestProjectionUpdateImpl(node, rect, bounds());
+        requestProjectionUpdateImpl(node, rects, bounds());
     }
+
+    KisNodeGraphListener::requestProjectionUpdate(node, rects, resetAnimationCache);
 }
 
 void KisImage::invalidateFrames(const KisTimeRange &range, const QRect &rect)
 {
-    if (m_d->animationInterface) {
-        m_d->animationInterface->invalidateFrames(range, rect);
-    }
+    m_d->animationInterface->invalidateFrames(range, rect);
 }
 
 void KisImage::requestTimeSwitch(int time)
 {
-    if (m_d->animationInterface) {
-        m_d->animationInterface->requestTimeSwitchNonGUI(time);
-    }
+    m_d->animationInterface->requestTimeSwitchNonGUI(time);
 }
 
 QList<KisLayerCompositionSP> KisImage::compositions()
@@ -1597,6 +1613,10 @@ bool checkMasksNeedConversion(KisNodeSP root, const QRect &bounds)
 
 void KisImage::setWrapAroundModePermitted(bool value)
 {
+    if (m_d->wrapAroundModePermitted != value) {
+        requestStrokeEnd();
+    }
+
     m_d->wrapAroundModePermitted = value;
 
     if (m_d->wrapAroundModePermitted &&
@@ -1687,11 +1707,10 @@ void KisImage::setProofingConfiguration(KisProofingConfigurationSP proofingConfi
 
 KisProofingConfigurationSP KisImage::proofingConfiguration() const
 {
-    if (!m_d->proofingConfig) {
-        KisImageConfig cfg;
-        m_d->proofingConfig = cfg.defaultProofingconfiguration();
+    if (m_d->proofingConfig) {
+        return m_d->proofingConfig;
     }
-    return m_d->proofingConfig;
+    return KisProofingConfigurationSP();
 }
 
 QPointF KisImage::mirrorAxesCenter() const
