@@ -81,7 +81,7 @@ class KRITAIMAGE_EXPORT KisImage : public QObject,
 public:
 
     /// @param colorSpace can be null. in that case it will be initialised to a default color space.
-    KisImage(KisUndoStore *undoStore, qint32 width, qint32 height, const KoColorSpace * colorSpace, const QString& name);
+    KisImage(KisUndoStore *undoStore, qint32 width, qint32 height, const KoColorSpace *colorSpace, const QString& name);
     ~KisImage() override;
 
 public: // KisNodeGraphListener implementation
@@ -92,7 +92,7 @@ public: // KisNodeGraphListener implementation
     void nodeChanged(KisNode * node) override;
     void invalidateAllFrames() override;
     void notifySelectionChanged() override;
-    void requestProjectionUpdate(KisNode *node, const QRect& rect) override;
+    void requestProjectionUpdate(KisNode *node, const QVector<QRect> &rects, bool resetAnimationCache) override;
     void invalidateFrames(const KisTimeRange &range, const QRect &rect) override;
     void requestTimeSwitch(int time) override;
 
@@ -101,6 +101,16 @@ public: // KisProjectionUpdateListener implementation
 
 public:
 
+    /**
+     * Set the number of threads used by the image's working threads
+     */
+    void setWorkingThreadsLimit(int value);
+
+    /**
+     * Return the number of threads available to the image's working threads
+     */
+    int workingThreadsLimit() const;
+
     /**
      * Makes a copy of the image with all the layers. If possible, shallow
      * copies of the layers are made.
@@ -111,7 +121,7 @@ public:
      * expected to use the copied image anywhere except for saving. Don't use
      * this option if you plan to work with the copied image later.
      */
-    KisImage* clone(bool exactCopy = false);
+    KisImage *clone(bool exactCopy = false);
 
     /**
      * Render the projection onto a QImage.
@@ -259,7 +269,7 @@ public:
      * undo stack using the adapter. This adapter is used for a backward
      * compatibility for old commands created before strokes. It blocks
      * all the porcessing at the scheduler, waits until it's finished
-     * adn executes commands exclusively.
+     * and executes commands exclusively.
      */
     KisUndoAdapter* undoAdapter() const;
 
@@ -324,11 +334,11 @@ public:
     QPointF documentToPixel(const QPointF &documentCoord) const;
 
     /**
-     * Convert a document coordinate to an integer pixel coordinate.
+     * Convert a document coordinate to an integer pixel coordinate rounded down.
      *
      * @param documentCoord PostScript Pt coordinate to convert.
      */
-    QPoint documentToIntPixel(const QPointF &documentCoord) const;
+    QPoint documentToImagePixelFloored(const QPointF &documentCoord) const;
 
     /**
      * Convert a document rectangle to a pixel rectangle.
@@ -337,13 +347,6 @@ public:
      */
     QRectF documentToPixel(const QRectF &documentRect) const;
 
-    /**
-     * Convert a document rectangle to an integer pixel rectangle.
-     *
-     * @param documentRect PostScript Pt rectangle to convert.
-     */
-    QRect documentToIntPixel(const QRectF &documentRect) const;
-
     /**
      * Convert a pixel coordinate to a document coordinate.
      *
@@ -842,14 +845,14 @@ public Q_SLOTS:
      */
     KisProjectionUpdatesFilterSP projectionUpdatesFilter() const override;
 
-    void refreshGraphAsync(KisNodeSP root = 0) override;
+    void refreshGraphAsync(KisNodeSP root = KisNodeSP()) override;
     void refreshGraphAsync(KisNodeSP root, const QRect &rc) override;
     void refreshGraphAsync(KisNodeSP root, const QRect &rc, const QRect &cropRect) override;
 
     /**
      * Triggers synchronous recomposition of the projection
      */
-    void refreshGraph(KisNodeSP root = 0);
+    void refreshGraph(KisNodeSP root = KisNodeSP());
     void refreshGraph(KisNodeSP root, const QRect& rc, const QRect &cropRect);
     void initialRefreshGraph();
 
@@ -941,7 +944,7 @@ private:
     void refreshHiddenArea(KisNodeSP rootNode, const QRect &preparedArea);
 
     void requestProjectionUpdateImpl(KisNode *node,
-                                     const QRect& rect,
+                                     const QVector<QRect> &rects,
                                      const QRect &cropRect);
 
     friend class KisImageResizeCommand;
@@ -955,6 +958,7 @@ private:
     friend class KisReselectGlobalSelectionCommand;
     friend class KisSetGlobalSelectionCommand;
     friend class KisImageTest;
+    friend class Document; // For libkis
 
     /**
      * Replaces the current global selection with globalSelection. If
