@@ -18,6 +18,8 @@
 
 #include "kis_image_animation_interface.h"
 
+#include <QFileInfo>
+
 #include "kis_global.h"
 #include "kis_image.h"
 #include "kis_regenerate_frame_stroke_strategy.h"
@@ -37,6 +39,8 @@ struct KisImageAnimationInterface::Private
           externalFrameActive(false),
           frameInvalidationBlocked(false),
           cachedLastFrameValue(-1),
+          audioChannelMuted(false),
+          audioChannelVolume(0.5),
           m_currentTime(0),
           m_currentUITime(0)
     {
@@ -50,6 +54,9 @@ struct KisImageAnimationInterface::Private
           playbackRange(rhs.playbackRange),
           framerate(rhs.framerate),
           cachedLastFrameValue(-1),
+          audioChannelFileName(rhs.audioChannelFileName),
+          audioChannelMuted(rhs.audioChannelMuted),
+          audioChannelVolume(rhs.audioChannelVolume),
           m_currentTime(rhs.m_currentTime),
           m_currentUITime(rhs.m_currentUITime)
     {
@@ -63,6 +70,9 @@ struct KisImageAnimationInterface::Private
     KisTimeRange playbackRange;
     int framerate;
     int cachedLastFrameValue;
+    QString audioChannelFileName;
+    bool audioChannelMuted;
+    qreal audioChannelVolume;
 
     KisSwitchTimeStrokeStrategy::SharedTokenWSP switchToken;
 
@@ -87,7 +97,8 @@ private:
 
 
 KisImageAnimationInterface::KisImageAnimationInterface(KisImage *image)
-    : m_d(new Private)
+    : QObject(image),
+      m_d(new Private)
 {
     m_d->image = image;
 
@@ -156,6 +167,43 @@ int KisImageAnimationInterface::framerate() const
     return m_d->framerate;
 }
 
+QString KisImageAnimationInterface::audioChannelFileName() const
+{
+    return m_d->audioChannelFileName;
+}
+
+void KisImageAnimationInterface::setAudioChannelFileName(const QString &fileName)
+{
+    QFileInfo info(fileName);
+
+    KIS_SAFE_ASSERT_RECOVER_NOOP(fileName.isEmpty() || info.isAbsolute());
+    m_d->audioChannelFileName = fileName.isEmpty() ? fileName : info.absoluteFilePath();
+
+    emit sigAudioChannelChanged();
+}
+
+bool KisImageAnimationInterface::isAudioMuted() const
+{
+    return m_d->audioChannelMuted;
+}
+
+void KisImageAnimationInterface::setAudioMuted(bool value)
+{
+    m_d->audioChannelMuted = value;
+    emit sigAudioChannelChanged();
+}
+
+qreal KisImageAnimationInterface::audioVolume() const
+{
+    return m_d->audioChannelVolume;
+}
+
+void KisImageAnimationInterface::setAudioVolume(qreal value)
+{
+    m_d->audioChannelVolume = value;
+    emit sigAudioVolumeChanged();
+}
+
 void KisImageAnimationInterface::setFramerate(int fps)
 {
     m_d->framerate = fps;
@@ -286,6 +334,13 @@ KisUpdatesFacade* KisImageAnimationInterface::updatesFacade() const
 void KisImageAnimationInterface::notifyNodeChanged(const KisNode *node,
                                                    const QRect &rect,
                                                    bool recursive)
+{
+    notifyNodeChanged(node, QVector<QRect>({rect}), recursive);
+}
+
+void KisImageAnimationInterface::notifyNodeChanged(const KisNode *node,
+                                                   const QVector<QRect> &rects,
+                                                   bool recursive)
 {
     if (externalFrameActive() || m_d->frameInvalidationBlocked) return;
     if (node->inherits("KisSelectionMask")) return;
@@ -293,18 +348,25 @@ void KisImageAnimationInterface::notifyNodeChanged(const KisNode *node,
     KisKeyframeChannel *channel =
         node->getKeyframeChannel(KisKeyframeChannel::Content.id());
 
-    if (recursive) {
-        KisTimeRange affectedRange;
-        KisTimeRange::calculateTimeRangeRecursive(node, currentTime(), affectedRange, false);
+    KisTimeRange invalidateRange;
 
-        invalidateFrames(affectedRange, rect);
+    if (recursive) {
+        KisTimeRange::calculateTimeRangeRecursive(node, currentTime(), invalidateRange, false);
     } else if (channel) {
         const int currentTime = m_d->currentTime();
-
-        invalidateFrames(channel->affectedFrames(currentTime), rect);
+        invalidateRange = channel->affectedFrames(currentTime);
     } else {
-        invalidateFrames(KisTimeRange::infinite(0), rect);
+        invalidateRange = KisTimeRange::infinite(0);
     }
+
+
+    // we compress the updated rect (atm, no one uses it anyway)
+    QRect unitedRect;
+    Q_FOREACH (const QRect &rc, rects) {
+        unitedRect |= rc;
+    }
+
+    invalidateFrames(invalidateRange, unitedRect);
 }
 
 void KisImageAnimationInterface::invalidateFrames(const KisTimeRange &range, const QRect &rect)
