@@ -35,7 +35,7 @@
 #include "kis_global.h"
 #include <cmath>
 
-#ifdef Q_OS_MAC
+#ifdef Q_OS_OSX
 #include <errno.h>
 #endif
 
@@ -236,7 +236,7 @@ QString KisImageConfig::swapDir(bool requestDefault)
     // tell us otherwise.
 
     // the other option here would be to use a "garbled name" temp file (i.e. no name
-    // KRITA_SWAP_FILE_XXXXXX) in an obsure /var/folders place, which is not
+    // KRITA_SWAP_FILE_XXXXXX) in an obscure /var/folders place, which is not
     // nice to the user. having a clearly named swap file in the home folder is
     // much nicer to Krita's users.
 
@@ -247,8 +247,12 @@ QString KisImageConfig::swapDir(bool requestDefault)
 #else
     QString swap = QDir::tempPath();
 #endif
-    return !requestDefault ?
+    QString configuredSwap = !requestDefault ?
             m_config.readEntry("swaplocation", swap) : swap;
+    if (configuredSwap.isEmpty()) {
+        configuredSwap = swap;
+    }
+    return swap;
 }
 
 void KisImageConfig::setSwapDir(const QString &swapDir)
@@ -343,7 +347,7 @@ void KisImageConfig::setLazyFrameCreationEnabled(bool value)
 #include <sys/sysctl.h>
 #elif defined Q_OS_WIN
 #include <windows.h>
-#elif defined Q_OS_MAC
+#elif defined Q_OS_OSX
 #include <sys/types.h>
 #include <sys/sysctl.h>
 #endif
@@ -389,7 +393,7 @@ int KisImageConfig::totalRAM()
 #   if defined ENV32BIT
     totalMemory = qMin(totalMemory, 2000);
 #   endif
-#elif defined Q_OS_MAC
+#elif defined Q_OS_OSX
     int mib[2] = { CTL_HW, HW_MEMSIZE };
     u_int namelen = sizeof(mib) / sizeof(mib[0]);
     uint64_t size;
@@ -443,7 +447,7 @@ KisProofingConfigurationSP KisImageConfig::defaultProofingconfiguration()
     if (m_config.readEntry("defaultProofingBlackpointCompensation", true)) {
         proofingConfig->conversionFlags  |= KoColorConversionTransformation::ConversionFlag::BlackpointCompensation;
     } else {
-                proofingConfig->conversionFlags  = proofingConfig->conversionFlags & ~KoColorConversionTransformation::ConversionFlag::BlackpointCompensation;
+        proofingConfig->conversionFlags  = proofingConfig->conversionFlags & ~KoColorConversionTransformation::ConversionFlag::BlackpointCompensation;
     }
     QColor def(Qt::green);
     m_config.readEntry("defaultProofingGamutwarning", def);
@@ -478,3 +482,38 @@ void KisImageConfig::setUseLodForColorizeMask(bool value)
 {
     m_config.writeEntry("useLodForColorizeMask", value);
 }
+
+int KisImageConfig::maxNumberOfThreads(bool defaultValue) const
+{
+    return (defaultValue ? QThread::idealThreadCount() : m_config.readEntry("maxNumberOfThreads", QThread::idealThreadCount()));
+}
+
+void KisImageConfig::setMaxNumberOfThreads(int value)
+{
+    if (value == QThread::idealThreadCount()) {
+        m_config.deleteEntry("maxNumberOfThreads");
+    } else {
+        m_config.writeEntry("maxNumberOfThreads", value);
+    }
+}
+
+int KisImageConfig::frameRenderingClones(bool defaultValue) const
+{
+    const int defaultClonesCount = qMax(1, maxNumberOfThreads(defaultValue) / 2);
+    return defaultValue ? defaultClonesCount : m_config.readEntry("frameRenderingClones", defaultClonesCount);
+}
+
+void KisImageConfig::setFrameRenderingClones(int value)
+{
+    m_config.writeEntry("frameRenderingClones", value);
+}
+
+int KisImageConfig::fpsLimit(bool defaultValue) const
+{
+    return defaultValue ? 100 : m_config.readEntry("fpsLimit", 100);
+}
+
+void KisImageConfig::setFpsLimit(int value)
+{
+    m_config.writeEntry("fpsLimit", value);
+}
