@@ -283,7 +283,14 @@ namespace KisLayerUtils {
 
         void redo() override {
             foreach (KisNodeSP node, m_info->allSrcNodes()) {
-                forceAllDelayedNodesUpdate(node);
+                recursiveApplyNodes(node,
+                    [] (KisNodeSP node) {
+                        KisDelayedUpdateNodeInterface *delayedUpdate =
+                            dynamic_cast<KisDelayedUpdateNodeInterface*>(node.data());
+                        if (delayedUpdate) {
+                            delayedUpdate->forceUpdateTimedNode();
+                        }
+                    });
             }
         }
 
@@ -839,11 +846,6 @@ namespace KisLayerUtils {
             applicator.applyCommand(new FillSelectionMasks(info));
             applicator.applyCommand(new CreateMergedLayer(info), KisStrokeJobData::BARRIER);
 
-            // NOTE: shape layer may have emitted spontaneous jobs during layer creation,
-            //       wait for them to complete!
-            applicator.applyCommand(new RefreshDelayedUpdateLayers(info), KisStrokeJobData::BARRIER);
-            applicator.applyCommand(new KUndo2Command(), KisStrokeJobData::BARRIER);
-
             // in two-layer mode we disable pass trhough only when the destination layer
             // is not a group layer
             applicator.applyCommand(new DisablePassThroughForHeadsOnly(info, true));
@@ -1355,19 +1357,7 @@ namespace KisLayerUtils {
         return recursiveFindNode(root,
             [uuid] (KisNodeSP node) {
                 return node->uuid() == uuid;
-        });
-    }
-
-    void forceAllDelayedNodesUpdate(KisNodeSP root)
-    {
-        KisLayerUtils::recursiveApplyNodes(root,
-        [] (KisNodeSP node) {
-            KisDelayedUpdateNodeInterface *delayedUpdate =
-                    dynamic_cast<KisDelayedUpdateNodeInterface*>(node.data());
-            if (delayedUpdate) {
-                delayedUpdate->forceUpdateTimedNode();
-            }
-        });
+            });
     }
 
 }
