@@ -249,7 +249,7 @@ QRect KisNode::accessRect(const QRect &rect, PositionToFilthy pos) const
     return rect;
 }
 
-void KisNode::childNodeChanged(KisNodeSP /*changedChildNode*/)
+void KisNode::childNodeChanged(KisNodeSP changedChildNode)
 {
 }
 
@@ -604,12 +604,12 @@ void KisNode::setDirty(const QRect & rect)
 
 void KisNode::setDirtyDontResetAnimationCache()
 {
-    setDirtyDontResetAnimationCache(QVector<QRect>({extent()}));
+    setDirtyDontResetAnimationCache({extent()});
 }
 
 void KisNode::setDirtyDontResetAnimationCache(const QRect &rect)
 {
-    setDirtyDontResetAnimationCache(QVector<QRect>({rect}));
+    setDirtyDontResetAnimationCache({rect});
 }
 
 void KisNode::setDirtyDontResetAnimationCache(const QVector<QRect> &rects)
