@@ -425,6 +425,10 @@ public:
 
         if (m_data) {
             imageData += estimateDataSize(m_data.data());
+
+            if (!m_frames.isEmpty()) {
+                qWarning() << "WARNING: Extra data object:" << estimateDataSize(m_data.data());
+            }
         }
 
         if (m_lodData) {
@@ -1439,7 +1443,7 @@ void KisPaintDevice::clear(const QRect & rc)
 
 void KisPaintDevice::fill(const QRect & rc, const KoColor &color)
 {
-    KIS_ASSERT_RECOVER_RETURN(*color.colorSpace() == *colorSpace());
+    Q_ASSERT(*color.colorSpace() == *colorSpace());
     m_d->currentStrategy()->fill(rc, color.data());
 }
 
