@@ -42,10 +42,6 @@
 #include <kis_debug.h>
 #include <klocalizedstring.h>
 
-#include <KoColorSpace.h>
-#include <KoColor.h>
-#include <KoCompositeOpRegistry.h>
-
 #include "kis_image.h"
 #include "filter/kis_filter.h"
 #include "kis_layer.h"
@@ -55,9 +51,7 @@
 #include "kis_vec.h"
 #include "kis_iterator_ng.h"
 #include "kis_random_accessor_ng.h"
-#include "kis_paintop.h"
-#include "kis_selection.h"
-#include "kis_fill_painter.h"
+
 #include "filter/kis_filter_configuration.h"
 #include "kis_pixel_selection.h"
 #include <brushengine/kis_paint_information.h>
@@ -67,72 +61,15 @@
 #include <kis_distance_information.h>
 #include <KoColorSpaceMaths.h>
 #include "kis_lod_transform.h"
-
+#include "kis_algebra_2d.h"
+#include "krita_utils.h"
 
 
 // Maximum distance from a Bezier control point to the line through the start
 // and end points for the curve to be considered flat.
 #define BEZIER_FLATNESS_THRESHOLD 0.5
-#define trunc(x) ((int)(x))
-#ifndef Q_OS_WIN
 
-#endif
-
-struct Q_DECL_HIDDEN KisPainter::Private {
-    Private(KisPainter *_q) : q(_q) {}
-    Private(KisPainter *_q, const KoColorSpace *cs)
-        : q(_q), paintColor(cs), backgroundColor(cs) {}
-
-    KisPainter *q;
-
-    KisPaintDeviceSP            device;
-    KisSelectionSP              selection;
-    KisTransaction*             transaction;
-    KoUpdater*                  progressUpdater;
-
-    QVector<QRect>              dirtyRects;
-    KisPaintOp*                 paintOp;
-    KoColor                     paintColor;
-    KoColor                     backgroundColor;
-    KoColor                     customColor;
-    KisFilterConfigurationSP    generator;
-    KisPaintLayer*              sourceLayer;
-    FillStyle                   fillStyle;
-    StrokeStyle                 strokeStyle;
-    bool                        antiAliasPolygonFill;
-    const KoPattern*           pattern;
-    QPointF                     duplicateOffset;
-    quint32                     pixelSize;
-    const KoColorSpace*         colorSpace;
-    KoColorProfile*             profile;
-    const KoCompositeOp*        compositeOp;
-    const KoAbstractGradient*   gradient;
-    KisPaintOpPresetSP          paintOpPreset;
-    QImage                      polygonMaskImage;
-    QPainter*                   maskPainter;
-    KisFillPainter*             fillPainter;
-    KisPaintDeviceSP            polygon;
-    qint32                      maskImageWidth;
-    qint32                      maskImageHeight;
-    QPointF                     axesCenter;
-    bool                        mirrorHorizontally;
-    bool                        mirrorVertically;
-    bool                        isOpacityUnit; // TODO: move into ParameterInfo
-    KoCompositeOp::ParameterInfo paramInfo;
-    KoColorConversionTransformation::Intent renderingIntent;
-    KoColorConversionTransformation::ConversionFlags conversionFlags;
-
-    bool tryReduceSourceRect(const KisPaintDevice *srcDev,
-                             QRect *srcRect,
-                             qint32 *srcX,
-                             qint32 *srcY,
-                             qint32 *srcWidth,
-                             qint32 *srcHeight,
-                             qint32 *dstX,
-                             qint32 *dstY);
-
-    void fillPainterPathImpl(const QPainterPath& path, const QRect &requestedRect);
-};
+#include "kis_painter_p.h"
 
 KisPainter::KisPainter()
     : d(new Private(this))
@@ -273,12 +210,14 @@ KisPaintDeviceSP KisPainter::convertToAlphaAsAlpha(KisPaintDeviceSP src)
 {
     const KoColorSpace *srcCS = src->colorSpace();
     const QRect processRect = src->extent();
-    KisPaintDeviceSP dst = new KisPaintDevice(KoColorSpaceRegistry::instance()->alpha8());
+    KisPaintDeviceSP dst(new KisPaintDevice(KoColorSpaceRegistry::instance()->alpha8()));
+
+    if (processRect.isEmpty()) return dst;
 
     KisSequentialConstIterator srcIt(src, processRect);
     KisSequentialIterator dstIt(dst, processRect);
 
-    do {
+    while (srcIt.nextPixel() && dstIt.nextPixel()) {
         const quint8 *srcPtr = srcIt.rawDataConst();
         quint8 *alpha8Ptr = dstIt.rawData();
 
@@ -286,7 +225,7 @@ KisPaintDeviceSP KisPainter::convertToAlphaAsAlpha(KisPaintDeviceSP src)
         const quint8 alpha = srcCS->opacityU8(srcPtr);
 
         *alpha8Ptr = KoColorSpaceMaths<quint8>::multiply(alpha, KoColorSpaceMathsTraits<quint8>::unitValue - white);
-    } while (srcIt.nextPixel() && dstIt.nextPixel());
+    }
 
     return dst;
 }
@@ -295,21 +234,46 @@ KisPaintDeviceSP KisPainter::convertToAlphaAsGray(KisPaintDeviceSP src)
 {
     const KoColorSpace *srcCS = src->colorSpace();
     const QRect processRect = src->extent();
-    KisPaintDeviceSP dst = new KisPaintDevice(KoColorSpaceRegistry::instance()->alpha8());
+    KisPaintDeviceSP dst(new KisPaintDevice(KoColorSpaceRegistry::instance()->alpha8()));
+
+    if (processRect.isEmpty()) return dst;
 
     KisSequentialConstIterator srcIt(src, processRect);
     KisSequentialIterator dstIt(dst, processRect);
 
-    do {
+    while (srcIt.nextPixel() && dstIt.nextPixel()) {
         const quint8 *srcPtr = srcIt.rawDataConst();
         quint8 *alpha8Ptr = dstIt.rawData();
 
         *alpha8Ptr = srcCS->intensity8(srcPtr);
-    } while (srcIt.nextPixel() && dstIt.nextPixel());
+    }
 
     return dst;
 }
 
+bool KisPainter::checkDeviceHasTransparency(KisPaintDeviceSP dev)
+{
+    const QRect deviceBounds = dev->exactBounds();
+    const QRect imageBounds = dev->defaultBounds()->bounds();
+
+    if (deviceBounds.isEmpty() ||
+        (deviceBounds & imageBounds) != imageBounds) {
+
+        return true;
+    }
+
+    const KoColorSpace *cs = dev->colorSpace();
+    KisSequentialConstIterator it(dev, deviceBounds);
+
+    while(it.nextPixel()) {
+        if (cs->opacityU8(it.rawDataConst()) != OPACITY_OPAQUE_U8) {
+            return true;
+        }
+    }
+
+    return false;
+}
+
 void KisPainter::begin(KisPaintDeviceSP device)
 {
     begin(device, d->selection);
@@ -417,6 +381,8 @@ KisTransaction* KisPainter::takeTransaction()
     return temp;
 }
 
+
+
 QVector<QRect> KisPainter::takeDirtyRegion()
 {
     QVector<QRect> vrect = d->dirtyRects;
@@ -449,6 +415,8 @@ inline bool KisPainter::Private::tryReduceSourceRect(const KisPaintDevice *srcDe
      * the image.
      */
     if (compositeOp->id() != COMPOSITE_COPY &&
+        compositeOp->id() != COMPOSITE_DESTINATION_IN  &&
+        compositeOp->id() != COMPOSITE_DESTINATION_ATOP &&
         !srcDev->defaultBounds()->wrapAroundMode()) {
 
         /**
@@ -654,7 +622,7 @@ void KisPainter::bitBltImpl(qint32 dstX, qint32 dstY,
     the other bit blit operations. This one is longer than the rest in an effort to
     optimize speed and memory use */
     if (d->selection) {
-        KisPaintDeviceSP selectionProjection = d->selection->projection();
+        KisPaintDeviceSP selectionProjection(d->selection->projection());
         KisRandomConstAccessorSP maskIt = selectionProjection->createRandomConstAccessorNG(dstX, dstY);
 
         while (rowsRemaining > 0) {
@@ -806,7 +774,7 @@ void KisPainter::fill(qint32 x, qint32 y, qint32 width, qint32 height, const KoC
     KisRandomAccessorSP dstIt = d->device->createRandomAccessorNG(x, y);
 
     if(d->selection) {
-        KisPaintDeviceSP selectionProjection = d->selection->projection();
+        KisPaintDeviceSP selectionProjection(d->selection->projection());
         KisRandomConstAccessorSP maskIt = selectionProjection->createRandomConstAccessorNG(x, y);
 
         while(rowsRemaining > 0) {
@@ -934,7 +902,7 @@ void KisPainter::bltFixed(qint32 dstX, qint32 dstY,
     if (d->selection) {
         /* d->selection is a KisPaintDevice, so first a readBytes is performed to
         get the area of interest... */
-        KisPaintDeviceSP selectionProjection = d->selection->projection();
+        KisPaintDeviceSP selectionProjection(d->selection->projection());
         quint8* selBytes = 0;
         try {
             selBytes = new quint8[srcWidth * srcHeight * selectionProjection->pixelSize()];
@@ -1093,19 +1061,21 @@ void KisPainter::paintLine(const KisPaintInformation &pi1,
 void KisPainter::paintPolyline(const vQPointF &points,
                                int index, int numPoints)
 {
-    if (index >= (int) points.count())
+    if (index >= points.count())
         return;
 
     if (numPoints < 0)
         numPoints = points.count();
 
-    if (index + numPoints > (int) points.count())
+    if (index + numPoints > points.count())
         numPoints = points.count() - index;
 
-
-    KisDistanceInformation saveDist;
-    for (int i = index; i < index + numPoints - 1; i++) {
-        paintLine(points [i], points [i + 1], &saveDist);
+    if (numPoints > 1) {
+        KisDistanceInformation saveDist(points[0],
+                                        KisAlgebra2D::directionBetweenPoints(points[0], points[1], 0.0));
+        for (int i = index; i < index + numPoints - 1; i++) {
+            paintLine(points [i], points [i + 1], &saveDist);
+        }
     }
 }
 
@@ -1268,7 +1238,8 @@ void KisPainter::paintPolygon(const vQPointF& points)
 
     if (d->strokeStyle != StrokeStyleNone) {
         if (points.count() > 1) {
-            KisDistanceInformation distance;
+            KisDistanceInformation distance(points[0],
+                                            KisAlgebra2D::directionBetweenPoints(points[0], points[1], 0.0));
 
             for (int i = 0; i < points.count() - 1; i++) {
                 paintLine(KisPaintInformation(points[i]), KisPaintInformation(points[i + 1]), &distance);
@@ -1547,10 +1518,10 @@ inline void KisPainter::compositeOnePixel(quint8 *dst, const KoColor &color)
 
 /**/
 void KisPainter::drawLine(const QPointF& start, const QPointF& end, qreal width, bool antialias){
-    int x1 = start.x();
-    int y1 = start.y();
-    int x2 = end.x();
-    int y2 = end.y();
+    int x1 = qFloor(start.x());
+    int y1 = qFloor(start.y());
+    int x2 = qFloor(end.x());
+    int y2 = qFloor(end.y());
 
     if ((x2 == x1 ) && (y2 == y1)) return;
 
@@ -1576,8 +1547,8 @@ void KisPainter::drawLine(const QPointF& start, const QPointF& end, qreal width,
     int X2_ = x2;
     int Y2_ = y2;
 
-    if (x2<x1) qSwap(x1,x2);
-    if (y2<y1) qSwap(y1,y2);
+    if (x2<x1) std::swap(x1,x2);
+    if (y2<y1) std::swap(y1,y2);
 
     qreal denominator = sqrt(pow((double)dstY,2) + pow((double)dstX,2));
     if (denominator == 0.0) {
@@ -1636,11 +1607,11 @@ void KisPainter::drawLine(const QPointF & start, const QPointF & end)
 
 void KisPainter::drawDDALine(const QPointF & start, const QPointF & end)
 {
-    int x = int(start.x());
-    int y = int(start.y());
+    int x = qFloor(start.x());
+    int y = qFloor(start.y());
 
-    int x2 = int(end.x());
-    int y2 = int(end.y());
+    int x2 = qFloor(end.x());
+    int y2 = qFloor(end.y());
 
     // Width and height of the line
     int xd = x2 - x;
@@ -1702,18 +1673,18 @@ void KisPainter::drawDDALine(const QPointF & start, const QPointF & end)
 
 void KisPainter::drawWobblyLine(const QPointF & start, const QPointF & end)
 {
-    KisRandomAccessorSP accessor = d->device->createRandomAccessorNG(start.x(), start.y());
-    KisRandomConstAccessorSP selectionAccessor;
-    if (d->selection) {
-        selectionAccessor = d->selection->projection()->createRandomConstAccessorNG(start.x(), start.y());
-    }
-
     KoColor mycolor(d->paintColor);
 
-    int x1 = start.x();
-    int y1 = start.y();
-    int x2 = end.x();
-    int y2 = end.y();
+    int x1 = qFloor(start.x());
+    int y1 = qFloor(start.y());
+    int x2 = qFloor(end.x());
+    int y2 = qFloor(end.y());
+
+    KisRandomAccessorSP accessor = d->device->createRandomAccessorNG(x1, y1);
+    KisRandomConstAccessorSP selectionAccessor;
+    if (d->selection) {
+        selectionAccessor = d->selection->projection()->createRandomConstAccessorNG(x1, y1);
+    }
 
     // Width and height of the line
     int xd = (x2 - x1);
@@ -1735,8 +1706,8 @@ void KisPainter::drawWobblyLine(const QPointF & start, const QPointF & end)
             y = y + inc;
             x = qRound(fx);
 
-            float br1 = int(fx + 1) - fx;
-            float br2 = fx - (int)fx;
+            float br1 = qFloor(fx + 1) - fx;
+            float br2 = fx - qFloor(fx);
 
             accessor->moveTo(x, y);
             if (selectionAccessor) selectionAccessor->moveTo(x, y);
@@ -1762,8 +1733,8 @@ void KisPainter::drawWobblyLine(const QPointF & start, const QPointF & end)
             x = x + inc;
             y = qRound(fy);
 
-            float br1 = int(fy + 1) - fy;
-            float br2 = fy - (int)fy;
+            float br1 = qFloor(fy + 1) - fy;
+            float br2 = fy - qFloor(fy);
 
             accessor->moveTo(x, y);
             if (selectionAccessor) selectionAccessor->moveTo(x, y);
@@ -1787,19 +1758,18 @@ void KisPainter::drawWobblyLine(const QPointF & start, const QPointF & end)
 
 void KisPainter::drawWuLine(const QPointF & start, const QPointF & end)
 {
-    KisRandomAccessorSP accessor = d->device->createRandomAccessorNG(start.x(), start.y());
-    KisRandomConstAccessorSP selectionAccessor;
-    if (d->selection) {
-        selectionAccessor = d->selection->projection()->createRandomConstAccessorNG(start.x(), start.y());
-    }
-
     KoColor lineColor(d->paintColor);
 
-    int x1 = start.x();
-    int y1 = start.y();
-    int x2 = end.x();
-    int y2 = end.y();
+    int x1 = qFloor(start.x());
+    int y1 = qFloor(start.y());
+    int x2 = qFloor(end.x());
+    int y2 = qFloor(end.y());
 
+    KisRandomAccessorSP accessor = d->device->createRandomAccessorNG(x1, y1);
+    KisRandomConstAccessorSP selectionAccessor;
+    if (d->selection) {
+        selectionAccessor = d->selection->projection()->createRandomConstAccessorNG(x1, y1);
+    }
 
     float grad, xd, yd;
     float xgap, ygap, xend, yend, yf, xf;
@@ -1815,9 +1785,9 @@ void KisPainter::drawWuLine(const QPointF & start, const QPointF & end)
     if (yd == 0) {
         /* Horizontal line */
         int incr = (x1 < x2) ? 1 : -1;
-        ix1 = (int)x1;
-        ix2 = (int)x2;
-        iy1 = (int)y1;
+        ix1 = x1;
+        ix2 = x2;
+        iy1 = y1;
         while (ix1 != ix2) {
             ix1 = ix1 + incr;
 
@@ -1834,9 +1804,9 @@ void KisPainter::drawWuLine(const QPointF & start, const QPointF & end)
     if (xd == 0) {
         /* Vertical line */
         int incr = (y1 < y2) ? 1 : -1;
-        iy1 = (int)y1;
-        iy2 = (int)y2;
-        ix1 = (int)x1;
+        iy1 = y1;
+        iy2 = y2;
+        ix1 = x1;
         while (iy1 != iy2) {
             iy1 = iy1 + incr;
 
@@ -1854,21 +1824,20 @@ void KisPainter::drawWuLine(const QPointF & start, const QPointF & end)
         // horizontal line
         // line have to be paint from left to right
         if (x1 > x2) {
-            float tmp;
-            tmp = x1; x1 = x2; x2 = tmp;
-            tmp = y1; y1 = y2; y2 = tmp;
+            std::swap(x1, x2);
+            std::swap(y1, y2);
             xd = (x2 - x1);
             yd = (y2 - y1);
         }
         grad = yd / xd;
-        // nearest X,Y interger coordinates
-        xend = static_cast<int>(x1 + 0.5f);
+        // nearest X,Y integer coordinates
+        xend = x1;
         yend = y1 + grad * (xend - x1);
 
         xgap = invertFrac(x1 + 0.5f);
 
-        ix1 = static_cast<int>(xend);
-        iy1 = static_cast<int>(yend);
+        ix1 = x1;
+        iy1 = qFloor(yend);
 
         // calc the intensity of the other end point pixel pair.
         brightness1 = invertFrac(yend) * xgap;
@@ -1896,13 +1865,13 @@ void KisPainter::drawWuLine(const QPointF & start, const QPointF & end)
         // calc first Y-intersection for main loop
         yf = yend + grad;
 
-        xend = trunc(x2 + 0.5f);
+        xend = x2;
         yend = y2 + grad * (xend - x2);
 
         xgap = invertFrac(x2 - 0.5f);
 
-        ix2 = static_cast<int>(xend);
-        iy2 = static_cast<int>(yend);
+        ix2 = x2;
+        iy2 = qFloor(yend);
 
         brightness1 = invertFrac(yend) * xgap;
         brightness2 =    frac(yend) * xgap;
@@ -1933,16 +1902,16 @@ void KisPainter::drawWuLine(const QPointF & start, const QPointF & end)
             c1 = (int)(brightness1 * OPACITY_OPAQUE_U8);
             c2 = (int)(brightness2 * OPACITY_OPAQUE_U8);
 
-            accessor->moveTo(x, int (yf));
-            if (selectionAccessor) selectionAccessor->moveTo(x, int (yf));
+            accessor->moveTo(x, qFloor(yf));
+            if (selectionAccessor) selectionAccessor->moveTo(x, qFloor(yf));
 
             if (!selectionAccessor || *selectionAccessor->oldRawData() > SELECTION_THRESHOLD) {
                 lineColor.setOpacity(c1);
                 compositeOnePixel(accessor->rawData(), lineColor);
             }
 
-            accessor->moveTo(x, int (yf) + 1);
-            if (selectionAccessor) selectionAccessor->moveTo(x, int (yf) + 1);
+            accessor->moveTo(x, qFloor(yf) + 1);
+            if (selectionAccessor) selectionAccessor->moveTo(x, qFloor(yf) + 1);
 
             if (!selectionAccessor || *selectionAccessor->oldRawData() > SELECTION_THRESHOLD) {
                 lineColor.setOpacity(c2);
@@ -1955,23 +1924,22 @@ void KisPainter::drawWuLine(const QPointF & start, const QPointF & end)
         //vertical
         // line have to be painted from left to right
         if (y1 > y2) {
-            float tmp;
-            tmp = x1; x1 = x2; x2 = tmp;
-            tmp = y1; y1 = y2; y2 = tmp;
+            std::swap(x1, x2);
+            std::swap(y1, y2);
             xd = (x2 - x1);
             yd = (y2 - y1);
         }
 
         grad = xd / yd;
 
-        // nearest X,Y interger coordinates
-        yend = static_cast<int>(y1 + 0.5f);
+        // nearest X,Y integer coordinates
+        yend = y1;
         xend = x1 + grad * (yend - y1);
 
-        ygap = invertFrac(y1 + 0.5f);
+        ygap = y1;
 
-        ix1 = static_cast<int>(xend);
-        iy1 = static_cast<int>(yend);
+        ix1 = qFloor(xend);
+        iy1 = y1;
 
         // calc the intensity of the other end point pixel pair.
         brightness1 = invertFrac(xend) * ygap;
@@ -1999,13 +1967,13 @@ void KisPainter::drawWuLine(const QPointF & start, const QPointF & end)
         // calc first Y-intersection for main loop
         xf = xend + grad;
 
-        yend = trunc(y2 + 0.5f);
+        yend = y2;
         xend = x2 + grad * (yend - y2);
 
         ygap = invertFrac(y2 - 0.5f);
 
-        ix2 = static_cast<int>(xend);
-        iy2 = static_cast<int>(yend);
+        ix2 = qFloor(xend);
+        iy2 = y2;
 
         brightness1 = invertFrac(xend) * ygap;
         brightness2 =    frac(xend) * ygap;
@@ -2036,16 +2004,16 @@ void KisPainter::drawWuLine(const QPointF & start, const QPointF & end)
             c1 = (int)(brightness1 * OPACITY_OPAQUE_U8);
             c2 = (int)(brightness2 * OPACITY_OPAQUE_U8);
 
-            accessor->moveTo(int (xf), y);
-            if (selectionAccessor) selectionAccessor->moveTo(int (xf), y);
+            accessor->moveTo(qFloor(xf), y);
+            if (selectionAccessor) selectionAccessor->moveTo(qFloor(xf), y);
 
             if (!selectionAccessor || *selectionAccessor->oldRawData() > SELECTION_THRESHOLD) {
                 lineColor.setOpacity(c1);
                 compositeOnePixel(accessor->rawData(), lineColor);
             }
 
-            accessor->moveTo(int (xf) + 1, y);
-            if (selectionAccessor) selectionAccessor->moveTo(int (xf) + 1, y);
+            accessor->moveTo(qFloor(xf) + 1, y);
+            if (selectionAccessor) selectionAccessor->moveTo(qFloor(xf) + 1, y);
 
             if (!selectionAccessor || *selectionAccessor->oldRawData() > SELECTION_THRESHOLD) {
                 lineColor.setOpacity(c2);
@@ -2235,11 +2203,11 @@ void KisPainter::drawThickLine(const QPointF & start, const QPointF & end, int s
         yfb = y0b + gradb;
 
         for (x = ix1 + 1; x <= ix2 - 1; x++) {
-            fraca = yfa - int (yfa);
+            fraca = yfa - qFloor(yfa);
             b1a = 1 - fraca;
             b2a = fraca;
 
-            fracb = yfb - int (yfb);
+            fracb = yfb - qFloor(yfb);
             b1b = 1 - fracb;
             b2b = fracb;
 
@@ -2247,8 +2215,8 @@ void KisPainter::drawThickLine(const QPointF & start, const QPointF & end, int s
             opacity = ((x - ix1) / dstX) * c2.opacityF() + (1 - (x - ix1) / dstX) * c1.opacityF();
             c3.setOpacity(opacity);
 
-            accessor->moveTo(x, (int)yfa);
-            if (selectionAccessor) selectionAccessor->moveTo(x, (int)yfa);
+            accessor->moveTo(x, qFloor(yfa));
+            if (selectionAccessor) selectionAccessor->moveTo(x, qFloor(yfa));
 
             if (!selectionAccessor || *selectionAccessor->oldRawData() > SELECTION_THRESHOLD) {
                 qreal alpha = cs->opacityF(accessor->rawData());
@@ -2259,8 +2227,8 @@ void KisPainter::drawThickLine(const QPointF & start, const QPointF & end, int s
 
             // color first pixel of top line
             if (!(startWidth == 1 && endWidth == 1)) {
-                accessor->moveTo(x, (int)yfb);
-                if (selectionAccessor) selectionAccessor->moveTo(x, (int)yfb);
+                accessor->moveTo(x, qFloor(yfb));
+                if (selectionAccessor) selectionAccessor->moveTo(x, qFloor(yfb));
 
                 if (!selectionAccessor || *selectionAccessor->oldRawData() > SELECTION_THRESHOLD) {
                     qreal alpha = cs->opacityF(accessor->rawData());
@@ -2273,8 +2241,8 @@ void KisPainter::drawThickLine(const QPointF & start, const QPointF & end, int s
             // color second pixel of bottom line
             if (grada != 0 && grada != 1) { // if not flat or exact diagonal
 
-                accessor->moveTo(x, int (yfa) + 1);
-                if (selectionAccessor) selectionAccessor->moveTo(x, int (yfa) + 1);
+                accessor->moveTo(x, qFloor(yfa) + 1);
+                if (selectionAccessor) selectionAccessor->moveTo(x, qFloor(yfa) + 1);
 
                 if (!selectionAccessor || *selectionAccessor->oldRawData() > SELECTION_THRESHOLD) {
                     qreal alpha = cs->opacityF(accessor->rawData());
@@ -2288,8 +2256,8 @@ void KisPainter::drawThickLine(const QPointF & start, const QPointF & end, int s
             // color second pixel of top line
             if (gradb != 0 && gradb != 1 && !(startWidth == 1 && endWidth == 1)) {
 
-                accessor->moveTo(x, int (yfb) + 1);
-                if (selectionAccessor) selectionAccessor->moveTo(x, int (yfb) + 1);
+                accessor->moveTo(x, qFloor(yfb) + 1);
+                if (selectionAccessor) selectionAccessor->moveTo(x, qFloor(yfb) + 1);
 
                 if (!selectionAccessor || *selectionAccessor->oldRawData() > SELECTION_THRESHOLD) {
                     qreal alpha = cs->opacityF(accessor->rawData());
@@ -2303,7 +2271,7 @@ void KisPainter::drawThickLine(const QPointF & start, const QPointF & end, int s
             // fill remaining pixels
             if (!(startWidth == 1 && endWidth == 1)) {
                 if (yfa < yfb)
-                    for (int i = yfa + 1; i <= yfb; i++) {
+                    for (int i = qFloor(yfa) + 1; i <= qFloor(yfb); i++) {
 
                         accessor->moveTo(x, i);
                         if (selectionAccessor) selectionAccessor->moveTo(x, i);
@@ -2313,7 +2281,7 @@ void KisPainter::drawThickLine(const QPointF & start, const QPointF & end, int s
                         }
                     }
                 else
-                    for (int i = yfa + 1; i >= yfb; i--) {
+                    for (int i = qFloor(yfa) + 1; i >= qFloor(yfb); i--) {
 
                         accessor->moveTo(x, i);
                         if (selectionAccessor) selectionAccessor->moveTo(x, i);
@@ -2353,11 +2321,11 @@ void KisPainter::drawThickLine(const QPointF & start, const QPointF & end, int s
         xfb = x0b + gradb;
 
         for (y = iy1 + 1; y <= iy2 - 1; y++) {
-            fraca = xfa - int (xfa);
+            fraca = xfa - qFloor(xfa);
             b1a = 1 - fraca;
             b2a = fraca;
 
-            fracb = xfb - int (xfb);
+            fracb = xfb - qFloor(xfb);
             b1b = 1 - fracb;
             b2b = fracb;
 
@@ -2365,8 +2333,8 @@ void KisPainter::drawThickLine(const QPointF & start, const QPointF & end, int s
             opacity = ((y - iy1) / dstY) * c2.opacityF() + (1 - (y - iy1) / dstY) * c1.opacityF();
             c3.setOpacity(opacity);
 
-            accessor->moveTo(int (xfa), y);
-            if (selectionAccessor) selectionAccessor->moveTo(int (xfa), y);
+            accessor->moveTo(qFloor(xfa), y);
+            if (selectionAccessor) selectionAccessor->moveTo(qFloor(xfa), y);
 
             if (!selectionAccessor || *selectionAccessor->oldRawData() > SELECTION_THRESHOLD) {
                 qreal alpha = cs->opacityF(accessor->rawData());
@@ -2378,8 +2346,8 @@ void KisPainter::drawThickLine(const QPointF & start, const QPointF & end, int s
             // color first pixel of right line
             if (!(startWidth == 1 && endWidth == 1)) {
 
-                accessor->moveTo(int(xfb), y);
-                if (selectionAccessor) selectionAccessor->moveTo(int(xfb), y);
+                accessor->moveTo(qFloor(xfb), y);
+                if (selectionAccessor) selectionAccessor->moveTo(qFloor(xfb), y);
 
                 if (!selectionAccessor || *selectionAccessor->oldRawData() > SELECTION_THRESHOLD) {
                     qreal alpha = cs->opacityF(accessor->rawData());
@@ -2392,8 +2360,8 @@ void KisPainter::drawThickLine(const QPointF & start, const QPointF & end, int s
             // color second pixel of left line
             if (grada != 0 && grada != 1) { // if not flat or exact diagonal
 
-                accessor->moveTo(int(xfa) + 1, y);
-                if (selectionAccessor) selectionAccessor->moveTo(int(xfa) + 1, y);
+                accessor->moveTo(qFloor(xfa) + 1, y);
+                if (selectionAccessor) selectionAccessor->moveTo(qFloor(xfa) + 1, y);
 
                 if (!selectionAccessor || *selectionAccessor->oldRawData() > SELECTION_THRESHOLD) {
                     qreal alpha = cs->opacityF(accessor->rawData());
@@ -2407,8 +2375,8 @@ void KisPainter::drawThickLine(const QPointF & start, const QPointF & end, int s
             // color second pixel of right line
             if (gradb != 0 && gradb != 1 && !(startWidth == 1 && endWidth == 1)) {
 
-                accessor->moveTo(int(xfb) + 1, y);
-                if (selectionAccessor) selectionAccessor->moveTo(int(xfb) + 1, y);
+                accessor->moveTo(qFloor(xfb) + 1, y);
+                if (selectionAccessor) selectionAccessor->moveTo(qFloor(xfb) + 1, y);
 
                 if (!selectionAccessor || *selectionAccessor->oldRawData() > SELECTION_THRESHOLD) {
                     qreal alpha = cs->opacityF(accessor->rawData());
@@ -2421,7 +2389,7 @@ void KisPainter::drawThickLine(const QPointF & start, const QPointF & end, int s
             // fill remaining pixels between current xfa,xfb
             if (!(startWidth == 1 && endWidth == 1)) {
                 if (xfa < xfb)
-                    for (int i = (int) xfa + 1; i <= (int) xfb; i++) {
+                    for (int i = qFloor(xfa) + 1; i <= qFloor(xfb); i++) {
 
                         accessor->moveTo(i, y);
                         if (selectionAccessor) selectionAccessor->moveTo(i, y);
@@ -2431,7 +2399,7 @@ void KisPainter::drawThickLine(const QPointF & start, const QPointF & end, int s
                         }
                     }
                 else
-                    for (int i = (int) xfb; i <= (int) xfa + 1; i++) {
+                    for (int i = qFloor(xfb); i <= qFloor(xfa) + 1; i++) {
 
                         accessor->moveTo(i, y);
                         if (selectionAccessor) selectionAccessor->moveTo(i, y);
@@ -2573,6 +2541,20 @@ void KisPainter::setOpacityUpdateAverage(quint8 opacity)
     d->paramInfo.updateOpacityAndAverage(float(opacity) / 255.0f);
 }
 
+void KisPainter::setAverageOpacity(qreal averageOpacity)
+{
+    d->paramInfo.setOpacityAndAverage(d->paramInfo.opacity, averageOpacity);
+}
+
+qreal KisPainter::blendAverageOpacity(qreal opacity, qreal averageOpacity)
+{
+    const float exponent = 0.1;
+
+    return averageOpacity < opacity ?
+        opacity :
+        exponent * opacity + (1.0 - exponent) * (averageOpacity);
+}
+
 void KisPainter::setOpacity(quint8 opacity)
 {
     d->isOpacityUnit = opacity == OPACITY_OPAQUE_U8;
@@ -2658,16 +2640,21 @@ void KisPainter::setMirrorInformation(const QPointF& axesCenter, bool mirrorHori
     d->mirrorVertically = mirrorVertically;
 }
 
-void KisPainter::copyMirrorInformation(KisPainter* painter)
-{
-    painter->setMirrorInformation(d->axesCenter, d->mirrorHorizontally, d->mirrorVertically);
-}
-
 bool KisPainter::hasMirroring() const
 {
     return d->mirrorHorizontally || d->mirrorVertically;
 }
 
+bool KisPainter::hasHorizontalMirroring() const
+{
+    return d->mirrorHorizontally;
+}
+
+bool KisPainter::hasVerticalMirroring() const
+{
+    return d->mirrorVertically;
+}
+
 void KisPainter::setMaskImageSize(qint32 width, qint32 height)
 {
 
@@ -2712,6 +2699,23 @@ void KisPainter::setColorConversionFlags(KoColorConversionTransformation::Conver
     d->conversionFlags = conversionFlags;
 }
 
+void KisPainter::setRunnableStrokeJobsInterface(KisRunnableStrokeJobsInterface *interface)
+{
+    d->runnableStrokeJobsInterface = interface;
+}
+
+KisRunnableStrokeJobsInterface *KisPainter::runnableStrokeJobsInterface() const
+{
+    if (!d->runnableStrokeJobsInterface) {
+        if (!d->fakeRunnableStrokeJobsInterface) {
+            d->fakeRunnableStrokeJobsInterface.reset(new KisFakeRunnableStrokeJobsExecutor());
+        }
+        return d->fakeRunnableStrokeJobsInterface.data();
+    }
+
+    return d->runnableStrokeJobsInterface;
+}
+
 void KisPainter::renderMirrorMaskSafe(QRect rc, KisFixedPaintDeviceSP dab, bool preserveDab)
 {
     if (!d->mirrorHorizontally && !d->mirrorVertically) return;
@@ -2740,7 +2744,7 @@ void KisPainter::renderMirrorMask(QRect rc, KisFixedPaintDeviceSP dab)
     int y = rc.topLeft().y();
 
     KisLodTransform t(d->device);
-    QPointF effectiveAxesCenter = t.map(d->axesCenter);
+    QPoint effectiveAxesCenter = t.map(d->axesCenter).toPoint();
 
     int mirrorX = -((x+rc.width()) - effectiveAxesCenter.x()) + effectiveAxesCenter.x();
     int mirrorY = -((y+rc.height()) - effectiveAxesCenter.y()) + effectiveAxesCenter.y();
@@ -2771,7 +2775,7 @@ void KisPainter::renderMirrorMask(QRect rc, KisFixedPaintDeviceSP dab, KisFixedP
     int y = rc.topLeft().y();
 
     KisLodTransform t(d->device);
-    QPointF effectiveAxesCenter = t.map(d->axesCenter);
+    QPoint effectiveAxesCenter = t.map(d->axesCenter).toPoint();
 
     int mirrorX = -((x+rc.width()) - effectiveAxesCenter.x()) + effectiveAxesCenter.x();
     int mirrorY = -((y+rc.height()) - effectiveAxesCenter.y()) + effectiveAxesCenter.y();
@@ -2805,10 +2809,10 @@ void KisPainter::renderMirrorMask(QRect rc, KisFixedPaintDeviceSP dab, KisFixedP
 
 void KisPainter::renderMirrorMask(QRect rc, KisPaintDeviceSP dab){
     if (d->mirrorHorizontally || d->mirrorVertically){
-        KisFixedPaintDeviceSP mirrorDab = new KisFixedPaintDevice(dab->colorSpace());
+        KisFixedPaintDeviceSP mirrorDab(new KisFixedPaintDevice(dab->colorSpace()));
         QRect dabRc( QPoint(0,0), QSize(rc.width(),rc.height()) );
         mirrorDab->setRect(dabRc);
-        mirrorDab->initialize();
+        mirrorDab->lazyGrowBufferWithoutInitialization();
 
         dab->readBytes(mirrorDab->data(),rc);
 
@@ -2819,10 +2823,10 @@ void KisPainter::renderMirrorMask(QRect rc, KisPaintDeviceSP dab){
 void KisPainter::renderMirrorMask(QRect rc, KisPaintDeviceSP dab, int sx, int sy, KisFixedPaintDeviceSP mask)
 {
     if (d->mirrorHorizontally || d->mirrorVertically){
-        KisFixedPaintDeviceSP mirrorDab = new KisFixedPaintDevice(dab->colorSpace());
+        KisFixedPaintDeviceSP mirrorDab(new KisFixedPaintDevice(dab->colorSpace()));
         QRect dabRc( QPoint(0,0), QSize(rc.width(),rc.height()) );
         mirrorDab->setRect(dabRc);
-        mirrorDab->initialize();
+        mirrorDab->lazyGrowBufferWithoutInitialization();
         dab->readBytes(mirrorDab->data(),QRect(QPoint(sx,sy),rc.size()));
         renderMirrorMask(rc, mirrorDab, mask);
     }
@@ -2836,7 +2840,7 @@ void KisPainter::renderDabWithMirroringNonIncremental(QRect rc, KisPaintDeviceSP
     int y = rc.topLeft().y();
 
     KisLodTransform t(d->device);
-    QPointF effectiveAxesCenter = t.map(d->axesCenter);
+    QPoint effectiveAxesCenter = t.map(d->axesCenter).toPoint();
 
     int mirrorX = -((x+rc.width()) - effectiveAxesCenter.x()) + effectiveAxesCenter.x();
     int mirrorY = -((y+rc.height()) - effectiveAxesCenter.y()) + effectiveAxesCenter.y();
@@ -2884,3 +2888,23 @@ void KisPainter::renderDabWithMirroringNonIncremental(QRect rc, KisPaintDeviceSP
     }
 }
 
+bool KisPainter::hasDirtyRegion() const
+{
+    return !d->dirtyRects.isEmpty();
+}
+
+void KisPainter::mirrorRect(Qt::Orientation direction, QRect *rc) const
+{
+    KisLodTransform t(d->device);
+    QPoint effectiveAxesCenter = t.map(d->axesCenter).toPoint();
+
+    KritaUtils::mirrorRect(direction, effectiveAxesCenter, rc);
+}
+
+void KisPainter::mirrorDab(Qt::Orientation direction, KisRenderedDab *dab) const
+{
+    KisLodTransform t(d->device);
+    QPoint effectiveAxesCenter = t.map(d->axesCenter).toPoint();
+
+    KritaUtils::mirrorDab(direction, effectiveAxesCenter, dab);
+}
