@@ -31,6 +31,7 @@
 
 #include "kundo2magicstring.h"
 #include "kis_types.h"
+#include <kis_filter_configuration.h>
 #include <kritaimage_export.h>
 
 class QPen;
@@ -49,10 +50,11 @@ class KisUndoAdapter;
 class KisPostExecutionUndoAdapter;
 class KisTransaction;
 class KoPattern;
-class KisFilterConfiguration;
 class KisPaintInformation;
 class KisPaintOp;
 class KisDistanceInformation;
+class KisRenderedDab;
+class KisRunnableStrokeJobsInterface;
 
 /**
  * KisPainter contains the graphics primitives necessary to draw on a
@@ -103,6 +105,10 @@ public:
                                   const QRect &originalSrcRect,
                                   KisSelectionSP selection);
 
+    static KisPaintDeviceSP convertToAlphaAsAlpha(KisPaintDeviceSP src);
+    static KisPaintDeviceSP convertToAlphaAsGray(KisPaintDeviceSP src);
+    static bool checkDeviceHasTransparency(KisPaintDeviceSP dev);
+
     /**
      * Start painting on the specified device. Not undoable.
      */
@@ -291,6 +297,14 @@ public:
                   qint32 srcX, qint32 srcY,
                   qint32 srcWidth, qint32 srcHeight);
 
+
+    /**
+     * Render the area \p rc from \p srcDevices on the destination device.
+     * If \p rc doesn't cross the device's rect, then the device is not
+     * rendered at all.
+     */
+    void bltFixed(const QRect &rc, const QList<KisRenderedDab> allSrcDevices);
+
     /**
      * Convenience method that uses QPoint and QRect.
      *
@@ -414,6 +428,12 @@ public:
      */
     void renderDabWithMirroringNonIncremental(QRect rc, KisPaintDeviceSP dab);
 
+    /**
+     * @return true if the painter has some rects marked as dirty
+     * @see takeDirtyRegion(), addDirtyRect()
+     */
+    bool hasDirtyRegion() const;
+
     /**
       * The methods in this class do not tell the paintdevice to update, but they calculate the
       * dirty area. This method returns this dirty area and resets it.
@@ -615,17 +635,34 @@ public:
 
     void setMirrorInformation(const QPointF &axesCenter, bool mirrorHorizontally, bool mirrorVertically);
 
-    /**
-     * copy the mirror information to other painter
-     */
-    void copyMirrorInformation(KisPainter * painter);
-
     /**
      * Returns whether the mirroring methods will do any
      * work when called
      */
     bool hasMirroring() const;
 
+    /**
+     * Indicates if horizontal mirroring mode is activated
+     */
+    bool hasHorizontalMirroring() const;
+
+    /**
+     * Indicates if vertical mirroring mode is activated
+     */
+    bool hasVerticalMirroring() const;
+
+    /**
+     * Mirror \p rc in the requested \p direction around the center point defined
+     * in the painter.
+     */
+    void mirrorRect(Qt::Orientation direction, QRect *rc) const;
+
+    /**
+     * Mirror \p dab in the requested direction around the center point defined
+     * in the painter. The dab's offset is adjusted automatically.
+     */
+    void mirrorDab(Qt::Orientation direction, KisRenderedDab *dab) const;
+
     /// Set the current pattern
     void setPattern(const KoPattern * pattern);
 
@@ -651,10 +688,10 @@ public:
     const KoColor &backgroundColor() const;
 
     /// Set the current generator (a generator can be used to fill an area
-    void setGenerator(const KisFilterConfiguration * generator);
+    void setGenerator(KisFilterConfigurationSP generator);
 
     /// @return the current generator configuration
-    const KisFilterConfiguration * generator() const;
+    const KisFilterConfigurationSP  generator() const;
 
     /// This enum contains the styles with which we can fill things like polygons and ellipses
     enum FillStyle {
@@ -664,7 +701,7 @@ public:
         FillStylePattern,
         FillStyleGradient,
         FillStyleStrokes,
-        FillStyleGenerator
+        FillStyleGenerator,
     };
 
     /// Set the current style with which to fill
@@ -702,6 +739,16 @@ public:
      */
     void setOpacityUpdateAverage(quint8 opacity);
 
+    /**
+     * Sets average opacity, that is used to make ALPHA_DARKEN painting look correct
+     */
+    void setAverageOpacity(qreal averageOpacity);
+
+    /**
+     * Calculate average opacity value after painting a single dab with \p opacity
+     */
+    static qreal blendAverageOpacity(qreal opacity, qreal averageOpacity);
+
     /// Set the opacity which is used in painting (like filling polygons)
     void setOpacity(quint8 opacity);
 
@@ -760,6 +807,20 @@ public:
      */
     void setColorConversionFlags(KoColorConversionTransformation::ConversionFlags conversionFlags);
 
+    /**
+     * Set interface for running asynchronous jobs by paintops.
+     *
+     * NOTE: the painter does *not* own the interface device. It is the responsibility
+     *       of the caller to ensure that the interface object is alive during the lifetime
+     *       of the painter.
+     */
+    void setRunnableStrokeJobsInterface(KisRunnableStrokeJobsInterface *interface);
+
+    /**
+     * Get the interface for running asynchronous jobs. It is used by paintops mostly.
+     */
+    KisRunnableStrokeJobsInterface* runnableStrokeJobsInterface() const;
+
 protected:
     /// Initialize, set everything to '0' or defaults
     void init();
