@@ -38,24 +38,32 @@ struct KisSuspendProjectionUpdatesStrokeStrategy::Private
     class SuspendLod0Updates : public KisProjectionUpdatesFilter
     {
 
-        typedef QHash<KisNodeSP, QVector<QRect> > RectsHash;
-    public:
         struct Request {
-            Request() {}
-            Request(KisNodeSP _node, QRect _rect) : node(_node), rect(_rect) {}
-            KisNodeSP node;
+            Request() : resetAnimationCache(false) {}
+            Request(const QRect &_rect, bool _resetAnimationCache)
+                : rect(_rect), resetAnimationCache(_resetAnimationCache)
+            {
+            }
+
             QRect rect;
+            bool resetAnimationCache;
         };
+
+        typedef QHash<KisNodeSP, QVector<Request> > RectsHash;
     public:
         SuspendLod0Updates()
         {
         }
 
-        bool filter(KisImage *image, KisNode *node, const QRect &rect) override {
+        bool filter(KisImage *image, KisNode *node, const QVector<QRect> &rects,  bool resetAnimationCache) override {
             if (image->currentLevelOfDetail() > 0) return false;
 
             QMutexLocker l(&m_mutex);
-            m_requestsHash[KisNodeSP(node)].append(rect);
+
+            Q_FOREACH(const QRect &rc, rects) {
+                m_requestsHash[KisNodeSP(node)].append(Request(rc, resetAnimationCache));
+            }
+
             return true;
         }
 
@@ -84,22 +92,17 @@ struct KisSuspendProjectionUpdatesStrokeStrategy::Private
 
             for (; it != end; ++it) {
                 KisNodeSP node = it.key();
-                const QVector<QRect> &rects = it.value();
-
-                QVector<QRect>::const_iterator it = rects.constBegin();
-                QVector<QRect>::const_iterator end = rects.constEnd();
 
                 QRegion region;
 
-                for (; it != end; ++it) {
-                    region += alignRect(*it, step);
+                bool resetAnimationCache = false;
+                Q_FOREACH (const Request &req, it.value()) {
+                    region += alignRect(req.rect, step);
+                    resetAnimationCache |= req.resetAnimationCache;
                 }
 
-                Q_FOREACH (const QRect &rc, region.rects()) {
-                    // FIXME: constness: port rPU to SP
-                    listener->requestProjectionUpdate(const_cast<KisNode*>(node.data()), rc);
-
-                }
+                // FIXME: constness: port rPU to SP
+                listener->requestProjectionUpdate(const_cast<KisNode*>(node.data()), region.rects(), resetAnimationCache);
             }
         }
 
@@ -150,7 +153,7 @@ KisSuspendProjectionUpdatesStrokeStrategy::KisSuspendProjectionUpdatesStrokeStra
     /**
      * Here we add a dumb INIT job so that KisStrokesQueue would know that the
      * stroke has already started or not. When the queue reaches the resume
-     * stroke ans starts its execution, no Lod0 can execute anymore. So all the
+     * stroke and starts its execution, no Lod0 can execute anymore. So all the
      * new Lod0 strokes should go to the end of the queue and wrapped into
      * their own Suspend/Resume pair.
      */
@@ -211,16 +214,21 @@ void KisSuspendProjectionUpdatesStrokeStrategy::doStrokeCallback(KisStrokeJobDat
     Private::UpdatesBarrierData *barrierData = dynamic_cast<Private::UpdatesBarrierData*>(data);
     Private::IssueCanvasUpdatesData *canvasUpdates = dynamic_cast<Private::IssueCanvasUpdatesData*>(data);
 
+    KisImageSP image = m_d->image.toStrongRef();
+    if (!image) {
+        return;
+    }
+
     if (suspendData) {
-        m_d->image->setProjectionUpdatesFilter(
+        image->setProjectionUpdatesFilter(
             KisProjectionUpdatesFilterSP(new Private::SuspendLod0Updates()));
     } else if (resumeData) {
-        m_d->image->disableUIUpdates();
+        image->disableUIUpdates();
         resumeAndIssueUpdates(false);
     } else if (barrierData) {
-        m_d->image->enableUIUpdates();
+        image->enableUIUpdates();
     } else if (canvasUpdates) {
-        m_d->image->notifyProjectionUpdated(canvasUpdates->updateRect);
+        image->notifyProjectionUpdated(canvasUpdates->updateRect);
     }
 }
 
@@ -252,8 +260,13 @@ QList<KisStrokeJobData*> KisSuspendProjectionUpdatesStrokeStrategy::createResume
 
 void KisSuspendProjectionUpdatesStrokeStrategy::resumeAndIssueUpdates(bool dropUpdates)
 {
+    KisImageSP image = m_d->image.toStrongRef();
+    if (!image) {
+        return;
+    }
+
     KisProjectionUpdatesFilterSP filter =
-        m_d->image->projectionUpdatesFilter();
+        image->projectionUpdatesFilter();
 
     if (!filter) return;
 
@@ -261,16 +274,21 @@ void KisSuspendProjectionUpdatesStrokeStrategy::resumeAndIssueUpdates(bool dropU
         dynamic_cast<Private::SuspendLod0Updates*>(filter.data());
 
     if (localFilter) {
-        m_d->image->setProjectionUpdatesFilter(KisProjectionUpdatesFilterSP());
+        image->setProjectionUpdatesFilter(KisProjectionUpdatesFilterSP());
 
         if (!dropUpdates) {
-            localFilter->notifyUpdates(m_d->image.data());
+            localFilter->notifyUpdates(image.data());
         }
     }
 }
 
 void KisSuspendProjectionUpdatesStrokeStrategy::cancelStrokeCallback()
 {
+    KisImageSP image = m_d->image.toStrongRef();
+    if (!image) {
+        return;
+    }
+
     /**
      * We shouldn't emit any ad-hoc updates when cancelling the
      * stroke.  It generates weird temporary holes on the canvas,
@@ -282,6 +300,6 @@ void KisSuspendProjectionUpdatesStrokeStrategy::cancelStrokeCallback()
 
     if (!m_d->suspend) {
         // FIXME: optimize
-        m_d->image->refreshGraphAsync();
+        image->refreshGraphAsync();
     }
 }
