@@ -55,21 +55,21 @@
 
 struct Q_DECL_HIDDEN KisTransformMask::Private
 {
-    Private()
+    Private(KisTransformMask *q)
         : worker(0, QTransform(), 0),
           staticCacheValid(false),
           recalculatingStaticImage(false),
-          updateSignalCompressor(UPDATE_DELAY, KisSignalCompressor::POSTPONE),
+          updateSignalCompressor(UPDATE_DELAY, KisSignalCompressor::POSTPONE, q),
           offBoundsReadArea(0.5)
     {
     }
 
-    Private(const Private &rhs)
+    Private(const Private &rhs, KisTransformMask *q)
         : worker(rhs.worker),
           params(rhs.params),
           staticCacheValid(false),
           recalculatingStaticImage(rhs.recalculatingStaticImage),
-          updateSignalCompressor(UPDATE_DELAY, KisSignalCompressor::POSTPONE)
+          updateSignalCompressor(UPDATE_DELAY, KisSignalCompressor::POSTPONE, q)
     {
     }
 
@@ -99,13 +99,14 @@ struct Q_DECL_HIDDEN KisTransformMask::Private
 
 KisTransformMask::KisTransformMask()
     : KisEffectMask(),
-      m_d(new Private)
+      m_d(new Private(this))
 {
     setTransformParams(
         KisTransformMaskParamsInterfaceSP(
             new KisDumbTransformMaskParams()));
 
     connect(this, SIGNAL(initiateDelayedStaticUpdate()), &m_d->updateSignalCompressor, SLOT(start()));
+    connect(this, SIGNAL(forceTerminateDelayedStaticUpdate()), &m_d->updateSignalCompressor, SLOT(stop()));
     connect(&m_d->updateSignalCompressor, SIGNAL(timeout()), SLOT(slotDelayedStaticUpdate()));
 
     KisImageConfig cfg;
@@ -118,7 +119,7 @@ KisTransformMask::~KisTransformMask()
 
 KisTransformMask::KisTransformMask(const KisTransformMask& rhs)
     : KisEffectMask(rhs),
-      m_d(new Private(*rhs.m_d))
+      m_d(new Private(*rhs.m_d, this))
 {
     connect(&m_d->updateSignalCompressor, SIGNAL(timeout()), SLOT(slotDelayedStaticUpdate()));
 }
@@ -143,7 +144,7 @@ void KisTransformMask::setTransformParams(KisTransformMaskParamsInterfaceSP para
     m_d->params = params;
     m_d->reloadParameters();
 
-    m_d->updateSignalCompressor.stop();
+    emit forceTerminateDelayedStaticUpdate();
 }
 
 KisTransformMaskParamsInterfaceSP KisTransformMask::transformParams() const
@@ -158,7 +159,7 @@ void KisTransformMask::slotDelayedStaticUpdate()
      * meanwhile. Just ignore the updates in the case.
      */
 
-    KisLayerSP parentLayer = dynamic_cast<KisLayer*>(parent().data());
+    KisLayerSP parentLayer = qobject_cast<KisLayer*>(parent().data());
     if (!parentLayer) return;
 
     KisImageSP image = parentLayer->image();
@@ -175,7 +176,7 @@ KisPaintDeviceSP KisTransformMask::buildPreviewDevice()
      * is not entirely safe.
      */
 
-    KisLayerSP parentLayer = dynamic_cast<KisLayer*>(parent().data());
+    KisLayerSP parentLayer = qobject_cast<KisLayer*>(parent().data());
     KIS_ASSERT_RECOVER(parentLayer) { return new KisPaintDevice(colorSpace()); }
 
     KisPaintDeviceSP device =
@@ -195,7 +196,7 @@ void KisTransformMask::recaclulateStaticImage()
      * is not entirely safe.
      */
 
-    KisLayerSP parentLayer = dynamic_cast<KisLayer*>(parent().data());
+    KisLayerSP parentLayer = qobject_cast<KisLayer*>(parent().data());
     KIS_ASSERT_RECOVER_RETURN(parentLayer);
 
     if (!m_d->staticCacheDevice) {
@@ -228,7 +229,7 @@ QRect KisTransformMask::decorateRect(KisPaintDeviceSP &src,
                                      PositionToFilthy maskPos) const
 {
     Q_ASSERT_X(src != dst, "KisTransformMask::decorateRect",
-               "src must be != dst, because we cant create transactions "
+               "src must be != dst, because we can't create transactions "
                "during merge, as it breaks reentrancy");
 
     KIS_ASSERT_RECOVER(m_d->params) { return rc; }
@@ -386,7 +387,7 @@ QRect KisTransformMask::extent() const
 
     QRect partialChangeRect;
     QRect existentProjection;
-    KisLayerSP parentLayer = dynamic_cast<KisLayer*>(parent().data());
+    KisLayerSP parentLayer = qobject_cast<KisLayer*>(parent().data());
     if (parentLayer) {
         partialChangeRect = parentLayer->partialChangeRect(const_cast<KisTransformMask*>(this), rc);
         existentProjection = parentLayer->projection()->extent();
@@ -401,7 +402,7 @@ QRect KisTransformMask::exactBounds() const
 
     QRect partialChangeRect;
     QRect existentProjection;
-    KisLayerSP parentLayer = dynamic_cast<KisLayer*>(parent().data());
+    KisLayerSP parentLayer = qobject_cast<KisLayer*>(parent().data());
     if (parentLayer) {
         partialChangeRect = parentLayer->partialChangeRect(const_cast<KisTransformMask*>(this), rc);
         existentProjection = parentLayer->projection()->exactBounds();
@@ -424,6 +425,16 @@ void KisTransformMask::setY(qint32 y)
     KisEffectMask::setY(y);
 }
 
+void KisTransformMask::forceUpdateTimedNode()
+{
+    if (m_d->updateSignalCompressor.isActive()) {
+        KIS_SAFE_ASSERT_RECOVER_NOOP(!m_d->staticCacheValid);
+
+        emit forceTerminateDelayedStaticUpdate();
+        slotDelayedStaticUpdate();
+    }
+}
+
 KisKeyframeChannel *KisTransformMask::requestKeyframeChannel(const QString &id)
 {
     if (id == KisKeyframeChannel::TransformArguments.id() ||
