@@ -289,6 +289,7 @@ void KisImageTest::testCloneImage()
         QVERIFY(newAnnotation);
         QVERIFY(newAnnotation != annotation);
 
+
         KisProofingConfigurationSP newProofing = newImage->proofingConfiguration();
         QVERIFY(newProofing);
         QVERIFY(newProofing != proofing);
@@ -529,7 +530,7 @@ KisLayerSP flattenLayerHelper(ContainerTest &p, KisLayerSP layer, bool nothingHa
     QList<QVariant> arguments = spy.takeFirst();
     KisNodeSP newNode = arguments.first().value<KisNodeSP>();
 
-    KisLayerSP newLayer = dynamic_cast<KisLayer*>(newNode.data());
+    KisLayerSP newLayer = qobject_cast<KisLayer*>(newNode.data());
     return newLayer;
 }
 
@@ -600,7 +601,7 @@ KisLayerSP mergeHelper(ContainerTest &p, KisLayerSP layer)
     //KisLayerUtils::mergeDown(p.image, layer, KisMetaData::MergeStrategyRegistry::instance()->get("Drop"));
     p.image->waitForDone();
 
-    KisLayerSP newLayer = dynamic_cast<KisLayer*>(parent->at(newIndex).data());
+    KisLayerSP newLayer = qobject_cast<KisLayer*>(parent->at(newIndex).data());
     return newLayer;
 }
 
@@ -891,7 +892,7 @@ void testMergeCrossColorSpaceImpl(bool useProjectionColorSpace, bool swapSpaces)
     const KoColorSpace *cs3 = KoColorSpaceRegistry::instance()->rgb16();
 
     if (swapSpaces) {
-        qSwap(cs2, cs3);
+        std::swap(cs2, cs3);
     }
 
     dbgKrita << "Testing testMergeCrossColorSpaceImpl:";
