@@ -34,6 +34,10 @@ private Q_SLOTS:
     void testSerialization();
     void testDefaultValues();
     void testNotSavedValues();
+    void testCopy();
+
+    void testLists();
+
 private:
     KisPropertiesConfigurationSP createConfig();
     void testConfig(KisPropertiesConfigurationSP config);
