@@ -30,6 +30,7 @@
 #include <QDir>
 #include <QProcessEnvironment>
 #include <QGlobalStatic>
+#include <QMutexLocker>
 
 KoJsonTrader::KoJsonTrader()
 {
@@ -44,7 +45,7 @@ KoJsonTrader::KoJsonTrader()
 
         QDir appDir(qApp->applicationDirPath());
         appDir.cdUp();
-#ifdef Q_OS_MAC
+#ifdef Q_OS_OSX
         // Help Krita run without deployment
         QDir d(appDir);
         d.cd("../../../");
@@ -58,7 +59,7 @@ KoJsonTrader::KoJsonTrader()
         Q_FOREACH (const QDir& dir, searchDirs) {
             Q_FOREACH (QString entry, dir.entryList()) {
                 QFileInfo info(dir, entry);
-#ifdef Q_OS_MAC
+#ifdef Q_OS_OSX
                 if (info.isDir() && info.fileName().contains("PlugIns")) {
                     m_pluginPath = info.absoluteFilePath();
                     break;
@@ -110,6 +111,8 @@ KoJsonTrader* KoJsonTrader::instance()
 
 QList<QPluginLoader *> KoJsonTrader::query(const QString &servicetype, const QString &mimetype) const
 {
+    QMutexLocker l(&m_mutex);
+
     QList<QPluginLoader *>list;
     QDirIterator dirIter(m_pluginPath, QDirIterator::Subdirectories);
     while (dirIter.hasNext()) {
