@@ -125,7 +125,7 @@ public:
     /// Return a list describing all the channels this color model has. The order
     /// of the channels in the list is the order of channels in the pixel. To find
     /// out the preferred display position, use KoChannelInfo::displayPosition.
-    virtual QList<KoChannelInfo *> channels() const;
+    QList<KoChannelInfo *> channels() const;
 
     /**
      * The total number of channels for a single pixel in this color model
@@ -199,13 +199,13 @@ public:
      * it is usually the concatenation of the id of the color model and of the color
      * depth, for instance "RGBA8" or "CMYKA16" or "XYZA32f".
      */
-    virtual QString id() const;
+    QString id() const;
 
     /**
      * User visible name which contains the name of the color model and of the color depth.
-     * For intance "RGBA (8-bits)" or "CMYKA (16-bits)".
+     * For instance "RGBA (8-bits)" or "CMYKA (16-bits)".
      */
-    virtual QString name() const;
+    QString name() const;
 
     /**
      * @return a string that identify the color model (for instance "RGB" or "CMYK" ...)
@@ -251,7 +251,7 @@ public:
      * Retrieve a single composite op from the ones this colorspace offers.
      * If the requeste composite op does not exist, COMPOSITE_OVER is returned.
      */
-    virtual const KoCompositeOp * compositeOp(const QString & id) const;
+    const KoCompositeOp * compositeOp(const QString & id) const;
 
     /**
      * add a composite op to this colorspace.
@@ -632,7 +632,11 @@ protected:
 
 inline QDebug operator<<(QDebug dbg, const KoColorSpace *cs)
 {
-    dbg.nospace() << cs->name() << " (" << cs->colorModelId().id() << "," << cs->colorDepthId().id() << " )";
+    if (cs) {
+        dbg.nospace() << cs->name() << " (" << cs->colorModelId().id() << "," << cs->colorDepthId().id() << " )";
+    } else {
+        dbg.nospace() << "0x0";
+    }
 
     return dbg.space();
 }
