@@ -84,7 +84,7 @@
 #include <kis_debug.h>
 #include "kis_action_registry.h"
 #include <kis_brush_server.h>
-#include <KisResourceServerProvider.h>
+#include <kis_resource_server_provider.h>
 #include <KoResourceServerProvider.h>
 #include "kis_image_barrier_locker.h"
 #include "opengl/kis_opengl.h"
@@ -230,7 +230,6 @@ void KisApplication::initializeGlobals(const KisApplicationArguments &args)
 
 void KisApplication::addResourceTypes()
 {
-//    qDebug() << "addResourceTypes();";
     // All Krita's resource types
     KoResourcePaths::addResourceType("kis_pics", "data", "/pics/");
     KoResourcePaths::addResourceType("kis_images", "data", "/images/");
@@ -274,7 +273,6 @@ void KisApplication::addResourceTypes()
     d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/tags/");
     d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/asl/");
     d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/bundles/");
-    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/brushes/");
     d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/gradients/");
     d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/paintoppresets/");
     d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/palettes/");
@@ -284,7 +282,6 @@ void KisApplication::addResourceTypes()
     d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/input/");
     d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/pykrita/");
     d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/symbols/");
-    d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/color-schemes/");
     d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/preset_icons/");
     d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/preset_icons/tool_icons/");
     d.mkpath(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/preset_icons/emblem_icons/");
@@ -295,46 +292,41 @@ void KisApplication::addResourceTypes()
 
 void KisApplication::loadResources()
 {
-//    qDebug() << "loadResources();";
-
-    setSplashScreenLoadingText(i18n("Loading Resources..."));
+    setSplashScreenLoadingText(i18n("Loading Gradients..."));
     processEvents();
-    KoResourceServerProvider::instance();
+    KoResourceServerProvider::instance()->gradientServer(true);
 
-    setSplashScreenLoadingText(i18n("Loading Brush Presets..."));
+
+    // Load base resources
+    setSplashScreenLoadingText(i18n("Loading Patterns..."));
     processEvents();
-    KisResourceServerProvider::instance();
+    KoResourceServerProvider::instance()->patternServer(true);
+
+    setSplashScreenLoadingText(i18n("Loading Palettes..."));
+    processEvents();
+    KoResourceServerProvider::instance()->paletteServer(false);
 
     setSplashScreenLoadingText(i18n("Loading Brushes..."));
     processEvents();
-    KisBrushServer::instance()->brushServer();
+    KisBrushServer::instance()->brushServer(true);
 
-    setSplashScreenLoadingText(i18n("Loading Bundles..."));
+    // load paintop presets
+    setSplashScreenLoadingText(i18n("Loading Paint Operations..."));
     processEvents();
-    KisResourceBundleServerProvider::instance();
-}
+    KisResourceServerProvider::instance()->paintOpPresetServer(true);
 
-void KisApplication::loadResourceTags()
-{
-//    qDebug() << "loadResourceTags()";
+    // load symbols
+    setSplashScreenLoadingText(i18n("Loading SVG Symbol Collections..."));
+    processEvents();
+    KoResourceServerProvider::instance()->svgSymbolCollectionServer(true);
 
-    KoResourceServerProvider::instance()->patternServer()->loadTags();
-    KoResourceServerProvider::instance()->gradientServer()->loadTags();
-    KoResourceServerProvider::instance()->paletteServer()->loadTags();
-    KoResourceServerProvider::instance()->svgSymbolCollectionServer()->loadTags();
-    KisBrushServer::instance()->brushServer()->loadTags();
-    KisResourceServerProvider::instance()->workspaceServer()->loadTags();
-    KisResourceServerProvider::instance()->layerStyleCollectionServer()->loadTags();
-    KisResourceBundleServerProvider::instance()->resourceBundleServer()->loadTags();
-    KisResourceServerProvider::instance()->paintOpPresetServer()->loadTags();
-
-    KisResourceServerProvider::instance()->paintOpPresetServer()->clearOldSystemTags();
+    setSplashScreenLoadingText(i18n("Loading Resource Bundles..."));
+    processEvents();
+    KisResourceServerProvider::instance()->resourceBundleServer();
 }
 
 void KisApplication::loadPlugins()
 {
-//    qDebug() << "loadPlugins();";
-
     KoShapeRegistry* r = KoShapeRegistry::instance();
     r->add(new KisShapeSelectionFactory());
 
@@ -343,15 +335,10 @@ void KisApplication::loadPlugins()
     KisGeneratorRegistry::instance();
     KisPaintOpRegistry::instance();
     KoColorSpaceRegistry::instance();
-}
 
-void KisApplication::loadGuiPlugins()
-{
-//    qDebug() << "loadGuiPlugins();";
     // Load the krita-specific tools
     setSplashScreenLoadingText(i18n("Loading Plugins for Krita/Tool..."));
     processEvents();
-//    qDebug() << "loading tools";
     KoPluginLoader::instance()->load(QString::fromLatin1("Krita/Tool"),
                                      QString::fromLatin1("[X-Krita-Version] == 28"));
 
@@ -359,17 +346,16 @@ void KisApplication::loadGuiPlugins()
     // Load dockers
     setSplashScreenLoadingText(i18n("Loading Plugins for Krita/Dock..."));
     processEvents();
-//    qDebug() << "loading dockers";
     KoPluginLoader::instance()->load(QString::fromLatin1("Krita/Dock"),
                                      QString::fromLatin1("[X-Krita-Version] == 28"));
 
     // XXX_EXIV: make the exiv io backends real plugins
     setSplashScreenLoadingText(i18n("Loading Plugins Exiv/IO..."));
     processEvents();
-//    qDebug() << "loading exiv2";
     KisExiv2::initialize();
 }
 
+
 bool KisApplication::start(const KisApplicationArguments &args)
 {
     KisConfig cfg;
@@ -439,17 +425,11 @@ bool KisApplication::start(const KisApplicationArguments &args)
     processEvents();
     addResourceTypes();
 
-    // Load the plugins
-    loadPlugins();
-
-    // Load all resources
+    // Load all resources and tags before the plugins do that
     loadResources();
 
-    // Load all the tags
-    loadResourceTags();
-
-    // Load the gui plugins
-    loadGuiPlugins();
+    // Load the plugins
+    loadPlugins();
 
     if (needsMainWindow) {
         // show a mainWindow asap, if we want that
