@@ -33,6 +33,7 @@
 #include <KoDocumentInfo.h>
 #include <KoDpi.h>
 #include <KoUnit.h>
+#include <KoFileDialog.h>
 #include <KoID.h>
 #include <KoOdfReadStore.h>
 #include <KoProgressProxy.h>
@@ -103,7 +104,7 @@
 #include "kis_statusbar.h"
 #include "widgets/kis_progress_widget.h"
 #include "kis_canvas_resource_provider.h"
-#include "KisResourceServerProvider.h"
+#include "kis_resource_server_provider.h"
 #include "kis_node_manager.h"
 #include "KisPart.h"
 #include "KisApplication.h"
@@ -482,7 +483,6 @@ KisDocument::~KisDocument()
 
         KisImageWSP sanityCheckPointer = d->image;
         Q_UNUSED(sanityCheckPointer);
-
         // The following line trigger the deletion of the image
         d->image.clear();
 
@@ -490,6 +490,7 @@ KisDocument::~KisDocument()
         KIS_SAFE_ASSERT_RECOVER_NOOP(!sanityCheckPointer.isValid());
     }
 
+
     delete d;
 }
 
@@ -722,13 +723,9 @@ bool KisDocument::initiateSavingInBackground(const QString actionName,
                                                            exportConfiguration);
 
     if (!started) {
-        // the state should have been deinitialized in slotChildCompletedSavingInBackground()
-
-        KIS_SAFE_ASSERT_RECOVER (!d->backgroundSaveDocument && !d->backgroundSaveJob.isValid()) {
-            d->backgroundSaveDocument.take()->deleteLater();
-            d->savingMutex.unlock();
-            d->backgroundSaveJob = KritaUtils::ExportFileJob();
-        }
+        d->backgroundSaveDocument.take()->deleteLater();
+        d->savingMutex.unlock();
+        d->backgroundSaveJob = KritaUtils::ExportFileJob();
     }
 
     return started;
@@ -1540,6 +1537,9 @@ bool KisDocument::newImage(const QString& name,
 
     image->assignImageProfile(cs->profile());
     documentInfo()->setAboutInfo("title", name);
+    if (name != i18n("Unnamed") && !name.isEmpty()) {
+        setUrl(QUrl::fromLocalFile(QStandardPaths::writableLocation(QStandardPaths::PicturesLocation) + '/' + name + ".kra"));
+    }
     documentInfo()->setAboutInfo("abstract", description);
 
     layer = new KisPaintLayer(image.data(), image->nextLayerName(), OPACITY_OPAQUE_U8, cs);
@@ -1593,10 +1593,8 @@ bool KisDocument::isSaving() const
 
 void KisDocument::waitForSavingToComplete()
 {
-    if (isSaving()) {
-        KisAsyncActionFeedback f(i18nc("progress dialog message when the user closes the document that is being saved", "Waiting for saving to complete..."), 0);
-        f.waitForMutex(&d->savingMutex);
-    }
+    KisAsyncActionFeedback f(i18nc("progress dialog message when the user closes the document that is being saved", "Waiting for saving to complete..."), 0);
+    f.waitForMutex(&d->savingMutex);
 }
 
 KoShapeBasedDocumentBase *KisDocument::shapeController() const
@@ -1658,14 +1656,6 @@ void KisDocument::setCurrentImage(KisImageSP image)
     d->image->initialRefreshGraph();
 }
 
-void KisDocument::hackPreliminarySetImage(KisImageSP image)
-{
-    KIS_SAFE_ASSERT_RECOVER_RETURN(!d->image);
-
-    d->setImageAndInitIdleWatcher(image);
-    d->shapeController->setImage(image);
-}
-
 void KisDocument::setImageModified()
 {
     setModified(true);
