@@ -1,6 +1,6 @@
 /* This file is part of the Krita project
  *
- * Copyright (C) 2014 Boudewijn Rempt <boud@kogmbh.com>
+ * Copyright (C) 2014 Boudewijn Rempt <boud@valdyas.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -31,10 +31,12 @@
 #include <KoDocumentBase.h>
 #include <kundo2stack.h>
 
+#include <KisImportExportFilter.h>
 #include <kis_properties_configuration.h>
 #include <kis_types.h>
 #include <kis_painting_assistant.h>
 #include <kis_debug.h>
+#include <KisImportExportUtils.h>
 
 #include "kritaui_export.h"
 
@@ -50,8 +52,6 @@ class KoShapeLayer;
 class KoStore;
 class KoOdfReadStore;
 class KoDocumentInfo;
-class KoProgressUpdater;
-class KoProgressProxy;
 class KoDocumentInfoDlg;
 class KisImportExportManager;
 class KisUndoStore;
@@ -75,19 +75,27 @@ class KisPart;
 class KRITAUI_EXPORT KisDocument : public QObject, public KoDocumentBase
 {
     Q_OBJECT
-    Q_PROPERTY(bool backupFile READ backupFile WRITE setBackupFile)
-    Q_PROPERTY(int pageCount READ pageCount)
 
 protected:
 
     explicit KisDocument();
 
-public:
+    /**
+     * @brief KisDocument makes a deep copy of the document \p rhs.
+     *        The caller *must* ensure that the image is properly
+     *        locked and is in consistent state before asking for
+     *        cloning.
+     * @param rhs the source document to copy from
+     */
+    explicit KisDocument(const KisDocument &rhs);
 
-    enum OpenUrlFlags {
-        OPEN_URL_FLAG_NONE                       = 1 << 0,
-        OPEN_URL_FLAG_DO_NOT_ADD_TO_RECENT_FILES = 1 << 1,
+public:
+    enum OpenFlag {
+        None = 0,
+        DontAddToRecent = 0x1,
+        RecoveryFile = 0x2
     };
+    Q_DECLARE_FLAGS(OpenFlags, OpenFlag)
 
     /**
      *  Destructor.
@@ -95,7 +103,7 @@ public:
      * The destructor does not delete any attached KisView objects and it does not
      * delete the attached widget as returned by widget().
      */
-    virtual ~KisDocument();
+    ~KisDocument() override;
 
     /**
      * @brief reload Reloads the document from the original url
@@ -103,13 +111,20 @@ public:
      */
     bool reload();
 
+
+    /**
+     * @brief creates a clone of the document and returns it. Please make sure that you
+     * hold all the necessary locks on the image before asking for a clone!
+     */
+    KisDocument* clone();
+
     /**
      * @brief openUrl Open an URL
      * @param url The URL to open
      * @param flags Control specific behavior
      * @return success status
      */
-    bool openUrl(const QUrl &url, OpenUrlFlags flags = OPEN_URL_FLAG_NONE);
+    bool openUrl(const QUrl &url, OpenFlags flags = None);
 
     /**
      * Opens the document given by @p url, without storing the URL
@@ -128,13 +143,15 @@ public:
      * KisDocument (URL, modified flag etc.). Call this instead of
      * KisParts::ReadWritePart::saveAs() to implement KisMainWindow's
      * File --> Export feature.
-     *
-     * @note This will call KisDocument::saveAs(). To differentiate this
-     *       from an ordinary Save operation (in any reimplementation of
-     *       saveFile()) call isExporting().
      */
-    bool exportDocument(const QUrl &url, KisPropertiesConfigurationSP exportConfiguration = 0);
+    bool exportDocument(const QUrl &url, const QByteArray &mimeType, bool showWarnings = false, KisPropertiesConfigurationSP exportConfiguration = 0);
 
+    bool exportDocumentSync(const QUrl &url, const QByteArray &mimeType, KisPropertiesConfigurationSP exportConfiguration = 0);
+
+private:
+    bool exportDocumentImpl(const KritaUtils::ExportFileJob &job, KisPropertiesConfigurationSP exportConfiguration);
+
+public:
     /**
      * @brief Sets whether the document can be edited or is read only.
      *
@@ -159,25 +176,10 @@ public:
     /// be saved by KisDocument without a filter, in *addition* to the main one
     static QStringList extraNativeMimeTypes() { return QStringList() << KIS_MIME_TYPE; }
 
-
-    /// Enum values used by specialOutputFlag - note that it's a bitfield for supportedSpecialFormats
-    enum { /*SaveAsCalligra1dot1 = 1,*/ // old and removed
-        SaveAsDirectoryStore = 2,
-        SaveAsFlatXML = 4,
-        SaveEncrypted = 8
-                        // bitfield! next value is 16
-    };
-
-    /**
-     * Return the set of SupportedSpecialFormats that the application wants to
-     * offer in the "Save" file dialog.
-     */
-    virtual int supportedSpecialFormats() const;
-
     /**
      * Returns the actual mimetype of the document
      */
-    QByteArray mimeType() const;
+    QByteArray mimeType() const override;
 
     /**
      * @brief Sets the mime type for the document.
@@ -185,37 +187,7 @@ public:
      * When choosing "save as" this is also the mime type
      * selected by default.
      */
-    void setMimeType(const QByteArray & mimeType);
-
-    /**
-     * @brief Set the format in which the document should be saved.
-     *
-     * This is called on loading, and in "save as", so you shouldn't
-     * have to call it.
-     *
-     * @param mimeType the mime type (format) to use.
-     * @param specialOutputFlag is for "save as older version" etc.
-     */
-    void setOutputMimeType(const QByteArray & mimeType, int specialOutputFlag = 0);
-    QByteArray outputMimeType() const;
-    int specialOutputFlag() const;
-
-    /**
-     * Returns true if this document was the result of opening a foreign
-     * file format and if the user hasn't yet saved the document (in any
-     * format).
-     *
-     * Used by KisMainWindow to warn the user when s/he lazily presses
-     * CTRL+S to save in the same foreign format, putting all his/her
-     * formatting at risk (normally an export confirmation only comes up
-     * with Save As).
-     *
-     * @param exporting specifies whether this is the setting for a
-     * File --> Export or File --> Save/Save As operation.
-     */
-    bool confirmNonNativeSave(const bool exporting) const;
-    void setConfirmNonNativeSave(const bool exporting, const bool on);
-
+    void setMimeType(const QByteArray & mimeType) override;
 
     /**
      * @return true if file operations should inhibit the option dialog
@@ -240,15 +212,18 @@ public:
      */
     QString errorMessage() const;
 
+    /**
+     * Sets the warning message to be shown to the user (use i18n()!)
+     * when loading or saving fails.
+     */
+    void setWarningMessage(const QString& warningMsg);
 
     /**
-     * Show the last error message in a message box.
-     * The dialog box will mention a loading problem.
-     * openUrl/openFile takes care of doing it, but not loadNativeFormat itself,
-     * so this is often called after loadNativeFormat returned false.
+     * Return the last warning message set by loading or saving. Warnings
+     * mean that the document could not be completely loaded, but the errors
+     * were not absolutely fatal.
      */
-    void showLoadingErrorDialog();
-
+    QString warningMessage() const;
 
     /**
      * @brief Generates a preview picture of the document
@@ -263,11 +238,6 @@ public:
      */
     void setTitleModified();
 
-    /**
-     *  @return true if the document is empty.
-     */
-    virtual bool isEmpty() const;
-
     /**
      *  @brief Sets the document to empty.
      *
@@ -276,41 +246,7 @@ public:
      *
      *  @see isEmpty()
      */
-    void setEmpty();
-
-    /**
-     *  @brief Loads a document from a store.
-     *
-     *  You should never have to reimplement.
-     *
-     *  @param store The store to load from
-     *  @param url An internal url, like tar:/1/2
-     */
-    bool loadFromStore(KoStore *store, const QString& url);
-
-    /// Unused
-    virtual bool loadOdf(KoOdfReadStore & odfStore);
-    /// Unused
-    virtual bool saveOdf(SavingContext &documentContext);
-
-    /**
-     *  @brief Saves a sub-document to a store.
-     *
-     *  You should not have to reimplement this.
-     */
-    virtual bool saveToStore(KoStore *store, const QString& path);
-
-    /**
-     *  Reimplement this method to load the contents of your Calligra document,
-     *  from the XML document. This is for the pre-Oasis file format (maindoc.xml).
-     */
-    virtual bool loadXML(const KoXmlDocument & doc, KoStore *store);
-
-    /**
-     *  Reimplement this to save the contents of the %Calligra document into
-     *  a QDomDocument. The framework takes care of saving it to the store.
-     */
-    QDomDocument saveXML();
+    void setEmpty(bool empty = true);
 
     /**
      *  Return a correctly created QDomDocument for this KisDocument,
@@ -330,15 +266,7 @@ public:
      */
     static QDomDocument createDomDocument(const QString& appName, const QString& tagName, const QString& version);
 
-    /**
-     *  The first thing to do in loadOasis is get hold of the office:body tag, then its child.
-     *  If the child isn't the expected one, the error message can indicate what it is instead.
-     *  This method returns a translated name for the type of document,
-     *  e.g. i18n("Word Processing") for office:text.
-     */
-    static QString tagNameToDocumentType(const QString& localName);
-
-    /**
+   /**
      *  Loads a document in the native format from a given URL.
      *  Reimplement if your native format isn't XML.
      *
@@ -346,42 +274,11 @@ public:
      */
     bool loadNativeFormat(const QString & file);
 
-    /**
-     *  Saves the document in native format, to a given file
-     *  You should never have to reimplement.
-     *  Made public for writing templates.
-     */
-    bool saveNativeFormat(const QString & file);
-
-    /**
-     * Saves the document in the native format to the given store. Does *no*
-     * cloning of the image
-     */
-    bool saveNativeFormatCalligraDirect(KoStore *store);
-
     /**
      * Activate/deactivate/configure the autosave feature.
      * @param delay in seconds, 0 to disable
      */
-    void setAutoSave(int delay);
-
-    /**
-     * Set whether the next openUrl call should show error message boxes in case
-     * of errors. This is usually the case, but e.g. not when generating thumbnail
-     * previews.
-     */
-    void setAutoErrorHandlingEnabled(bool b);
-
-    /**
-     * Checks whether error message boxes should be shown.
-     */
-    bool isAutoErrorHandlingEnabled() const;
-
-    /**
-     * Retrieve the default value for autosave in seconds.
-     * Called by the applications to use the correct default in their config
-     */
-    static int defaultAutoSave();
+    void setAutoSaveDelay(int delay);
 
     /**
      * @return the information concerning this document.
@@ -389,66 +286,15 @@ public:
      */
     KoDocumentInfo *documentInfo() const;
 
-    /**
-     * @return the object to report progress to.
-     *
-     * This is only not zero if loading or saving is in progress.
-     *
-     * One can add more KoUpdaters to it to make the progress reporting more
-     * accurate. If no active progress reporter is present, 0 is returned.
-     **/
-    KoProgressUpdater *progressUpdater() const;
-
-    /**
-     * Set a custom progress proxy to use to report loading
-     * progress to.
-     */
-    void setProgressProxy(KoProgressProxy *progressProxy);
-    KoProgressProxy* progressProxy() const;
-
-    /**
-     * Return true if url() is a real filename, false if url() is
-     * an internal url in the store, like "tar:/..."
-     */
-    virtual bool isStoredExtern() const;
-
-    /**
-     * @return the page layout associated with this document (margins, pageSize, etc).
-     * Override this if you want to provide different sized pages.
-     *
-     * @see KoPageLayout
-     */
-    KoPageLayout pageLayout(int pageNumber = 0) const;
-    void setPageLayout(const KoPageLayout &pageLayout);
-
     /**
      * Performs a cleanup of unneeded backup files
      */
     void removeAutoSaveFiles();
 
-    void setBackupFile(bool _b);
-
-    bool backupFile()const;
-
     /**
      * Returns true if this document or any of its internal child documents are modified.
      */
-    bool isModified() const;
-
-    /**
-     * Returns true during loading (openUrl can be asynchronous)
-     */
-    bool isLoading() const;
-
-    /**
-     * Sets the backup path of the document
-     */
-    void setBackupPath(const QString & _path);
-
-    /**
-     * @return path to the backup document
-     */
-    QString backupPath()const;
+    bool isModified() const override;
 
     /**
      * @return caption of the document
@@ -468,28 +314,11 @@ public:
      */
     void resetURL();
 
-    /**
-     * Set when you want an external embedded document to be stored internally
-     */
-    void setStoreInternal(bool i);
-
-    /**
-     * @return true when external embedded documents are stored internally
-     */
-    bool storeInternal() const;
-
-    bool hasExternURL() const;
-
     /**
      * @internal (public for KisMainWindow)
      */
     void setMimeTypeAfterLoading(const QString& mimeType);
 
-    /**
-     * @return returns the number of pages in the document.
-     */
-    virtual int pageCount() const;
-
     /**
      * Returns the unit used to display all measures/distances.
      */
@@ -500,13 +329,6 @@ public:
      */
     void setUnit(const KoUnit &unit);
 
-    /**
-     * Save the unit to the settings writer
-     *
-     * @param settingsWriter
-     */
-    bool loadNativeFormatFromByteArray(QByteArray &data);
-
     KisGridConfig gridConfig() const;
     void setGridConfig(const KisGridConfig &config);
 
@@ -523,12 +345,10 @@ public:
      */
     void setModified(bool _mod);
 
-    void updateEditingTime(bool forceStoreElapsed);
+    void setRecovered(bool value);
+    bool isRecovered() const;
 
-    /**
-     * Initialize an empty document using default values
-     */
-    void initEmpty();
+    void updateEditingTime(bool forceStoreElapsed);
 
     /**
      * Returns the global undo stack
@@ -549,6 +369,13 @@ public:
      */
     QByteArray serializeToNativeByteArray();
 
+
+    /**
+     * @brief isInSaving shown if the document has any (background) saving process or not
+     * @return true if there is some saving in action
+     */
+    bool isInSaving() const;
+
 public Q_SLOTS:
 
     /**
@@ -577,24 +404,11 @@ Q_SIGNALS:
      */
     void unitChanged(const KoUnit &unit);
 
-    /**
-     * Progress info while loading or saving. The value is in percents (i.e. a number between 0 and 100)
-     * Your KisDocument-derived class should emit the signal now and then during load/save.
-     * KisMainWindow will take care of displaying a progress bar automatically.
-     */
-    void sigProgress(int value);
-
-    /**
-     * Progress cancel button pressed
-     * This is emitted by KisDocument
-     */
-    void sigProgressCanceled();
-
     /**
      * Emitted e.g. at the beginning of a save operation
      * This is emitted by KisDocument and used by KisView to display a statusbar message
      */
-    void statusBarMessage(const QString& text);
+    void statusBarMessage(const QString& text, int timeout = 0);
 
     /**
      * Emitted e.g. at the end of a save operation
@@ -615,17 +429,38 @@ Q_SIGNALS:
 
     void sigGuidesConfigChanged(const KisGuidesConfig &config);
 
+    void sigBackgroundSavingFinished(KisImportExportFilter::ConversionStatus status, const QString &errorMessage);
+
+    void sigCompleteBackgroundSaving(const KritaUtils::ExportFileJob &job, KisImportExportFilter::ConversionStatus status, const QString &errorMessage);
+
+private Q_SLOTS:
+    void finishExportInBackground();
+    void slotChildCompletedSavingInBackground(KisImportExportFilter::ConversionStatus status, const QString &errorMessage);
+    void slotCompleteAutoSaving(const KritaUtils::ExportFileJob &job, KisImportExportFilter::ConversionStatus status, const QString &errorMessage);
+
+    void slotCompleteSavingDocument(const KritaUtils::ExportFileJob &job, KisImportExportFilter::ConversionStatus status, const QString &errorMessage);
 private:
 
     friend class KisPart;
     friend class SafeSavingLocker;
 
+    bool initiateSavingInBackground(const QString actionName,
+                                    const QObject *receiverObject, const char *receiverMethod,
+                                    const KritaUtils::ExportFileJob &job,
+                                    KisPropertiesConfigurationSP exportConfiguration);
+
+    bool startExportInBackground(const QString &actionName, const QString &location,
+                                 const QString &realLocation,
+                                 const QByteArray &mimeType,
+                                 bool showWarnings,
+                                 KisPropertiesConfigurationSP exportConfiguration);
+
     /**
      * Generate a name for the document.
      */
     QString newObjectName();
 
-    QString autoSaveFile(const QString & path) const;
+    QString generateAutoSaveFileName(const QString & path) const;
 
     /**
      *  Loads a document
@@ -639,105 +474,28 @@ private:
      */
     bool openFile();
 
-    /**
-     *  Saves a document
-     *
-     *  Applies a filter if necessary, and calls saveNativeFormat in any case
-     *  You should not have to reimplement, except for very special cases.
-     */
-    bool saveFile(KisPropertiesConfigurationSP exportConfiguration = 0);
-
-    /**
-     *  Overload this function if you have to load additional files
-     *  from a store. This function is called after loadXML()
-     *  and after loadChildren() have been called.
-     */
-    bool completeLoading(KoStore *store);
-
-    /**
-     *  If you want to write additional files to a store,
-     *  then you must do it here.
-     *  In the implementation, you should prepend the document
-     *  url (using url().url()) before the filename, so that everything is kept relative
-     *  to this document. For instance it will produce urls such as
-     *  tar:/1/pictures/picture0.png, if the doc url is tar:/1
-     *  But do this ONLY if the document is not stored extern (see isStoredExtern() ).
-     *  If it is, then the pictures should be saved to tar:/pictures.
-     */
-    bool completeSaving(KoStore *store);
-
-
     /** @internal */
     void setModified();
 
-    /**
-     *  Returns whether or not the current openUrl() or openFile() call is
-     *  actually an import operation (like File --> Import).
-     *  This is for informational purposes only.
-     */
-    bool isImporting() const;
+public:
 
-    /**
-     *  Returns whether or not the current saveFile() call is actually an export
-     *  operation (like File --> Export).
-     *  If this function returns true during saveFile() and you are changing
-     *  some sort of state, you _must_ restore it before the end of saveFile();
-     *  otherwise, File --> Export will not work properly.
-     */
-    bool isExporting() const;
-
-    /**
-     * Legacy method from KoDocumentBase. Don't use it anywhere
-     * outside KisDocument!
-     */
-    bool isAutosaving() const;
+    bool isAutosaving() const override;
 
 public:
 
-    QString localFilePath() const;
+    QString localFilePath() const override;
     void setLocalFilePath( const QString &localFilePath );
 
     KoDocumentInfoDlg* createDocumentInfoDialog(QWidget *parent, KoDocumentInfo *docInfo) const;
 
     bool isReadWrite() const;
 
-    QUrl url() const;
-    void setUrl(const QUrl &url);
+    QUrl url() const override;
+    void setUrl(const QUrl &url) override;
 
     bool closeUrl(bool promptToSave = true);
 
-    bool saveAs(const QUrl &url, KisPropertiesConfigurationSP exportConfigration = 0);
-
-public Q_SLOTS:
-
-    bool save(KisPropertiesConfigurationSP exportConfiguration = 0);
-    bool waitSaveComplete();
-
-Q_SIGNALS:
-
-    void completed();
-    void canceled(const QString &);
-
-private Q_SLOTS:
-
-    void setImageModified();
-
-    void slotAutoSave();
-
-    /// Called by the undo stack when undo or redo is called
-    void slotUndoStackIndexChanged(int idx);
-
-protected:
-
-    bool oldLoadAndParse(KoStore *store, const QString& filename, KoXmlDocument& doc);
-
-public:
-
-    /**
-     * Create a new image that has this document as a parent and
-     * replace the current image with this image.
-     */
-    bool newImage(const QString& name, qint32 width, qint32 height, const KoColorSpace * cs, const KoColor &bgColor, const QString &imageDescription, const double imageResolution);
+    bool saveAs(const QUrl &url, const QByteArray &mimeType, bool showWarnings, KisPropertiesConfigurationSP exportConfigration = 0);
 
     /**
      * Create a new image that has this document as a parent and
@@ -746,11 +504,8 @@ public:
     bool newImage(const QString& name, qint32 width, qint32 height, const KoColorSpace * cs, const KoColor &bgColor, bool backgroundAsLayer,
                   int numberOfLayers, const QString &imageDescription, const double imageResolution);
 
-    /**
-     * Create a new image that has this document as a parent and
-     * replace the current image with this image.
-     */
-    KisImageWSP newImage(const QString& name, qint32 width, qint32 height, const KoColorSpace * colorspace);
+    bool isSaving() const;
+    void waitForSavingToComplete();
 
 
     KisImageWSP image() const;
@@ -764,31 +519,6 @@ public:
      */
     KisImageSP savingImage() const;
 
-    /**
-     * Makes an otherwise empty document ready for import/export
-     */
-    void prepareForImport();
-
-    /**
-     * Adds progressproxy for file operations
-     */
-    void setFileProgressProxy();
-
-    /**
-     * Clears progressproxy for file operations
-     */
-    void clearFileProgressProxy();
-
-    /**
-     * Adds progressupdater for file operations
-     */
-    void setFileProgressUpdater(const QString &text);
-
-    /**
-     * Clears progressupdater for file operations
-     */
-    void clearFileProgressUpdater();
-
     /**
      * Set the current image to the specified image and turn undo on.
      */
@@ -805,47 +535,57 @@ public:
     KoShapeLayer* shapeForNode(KisNodeSP layer) const;
 
     /**
-     * @return a list of all layers that are active in all current views
-     */
-    vKisNodeSP activeNodes() const;
-
-    /**
-     * set the list of nodes that were marked as currently active
+     * Set the list of nodes that was marked as currently active. Used *only*
+     * for saving loading. Never use it for tools or processing.
      */
     void setPreActivatedNode(KisNodeSP activatedNode);
 
     /**
-     * @return the node that was set as active during loading
+     * @return the node that was set as active during loading. Used *only*
+     * for saving loading. Never use it for tools or processing.
      */
     KisNodeSP preActivatedNode() const;
 
     QList<KisPaintingAssistantSP> assistants() const;
     void setAssistants(const QList<KisPaintingAssistantSP> value);
 
+    bool save(bool showWarnings, KisPropertiesConfigurationSP exportConfiguration);
+
+Q_SIGNALS:
+
+    void completed();
+    void canceled(const QString &);
+
+private Q_SLOTS:
+
+    void setImageModified();
+
+    void slotAutoSave();
+
+    void slotUndoStackCleanChanged(bool value);
+
+    void slotConfigChanged();
+
+
 private:
+    /**
+     * @brief try to clone the image. This method handles all the locking for you. If locking
+     *        has failed, no cloning happens
+     * @return cloned document on success, null otherwise
+     */
+    KisDocument *lockAndCloneForSaving();
 
-    void init();
-
-    bool prepareLocksForSaving();
-    void unlockAfterSaving();
-
-    bool saveNativeFormatCalligraImpl(KoStore *store);
-
-    bool saveToStream(QIODevice *dev);
-
-    bool loadNativeFormatFromStoreInternal(KoStore *store);
-
-    bool savePreview(KoStore *store);
+    QString exportErrorToUserMessage(KisImportExportFilter::ConversionStatus status, const QString &errorMessage);
 
     QString prettyPathOrUrl() const;
 
-    bool saveToUrl();
     bool openUrlInternal(const QUrl &url);
 
     class Private;
     Private *const d;
 };
 
+Q_DECLARE_OPERATORS_FOR_FLAGS(KisDocument::OpenFlags)
 Q_DECLARE_METATYPE(KisDocument*)
 
 #endif
