@@ -112,7 +112,7 @@
 #include "KisPart.h"
 #include "KisPrintJob.h"
 #include <KoUpdater.h>
-#include "KisResourceServerProvider.h"
+#include "kis_resource_server_provider.h"
 #include "kis_selection.h"
 #include "kis_selection_manager.h"
 #include "kis_shape_controller.h"
@@ -127,13 +127,12 @@
 #include "kis_zoom_manager.h"
 #include "widgets/kis_floating_message.h"
 #include "kis_signal_auto_connection.h"
+#include "kis_script_manager.h"
 #include "kis_icon_utils.h"
 #include "kis_guides_manager.h"
 #include "kis_derived_resources.h"
 #include "dialogs/kis_delayed_save_dialog.h"
 #include <kis_image.h>
-#include <KisMainWindow.h>
-#include "kis_signals_blocker.h"
 
 
 class BlockingUserInputEventFilter : public QObject
@@ -191,6 +190,7 @@ public:
         , actionCollection(_actionCollection)
         , mirrorManager(_q)
         , inputManager(_q)
+        , scriptManager(_q)
         , actionAuthor(0)
         , showPixelGrid(0)
     {
@@ -245,6 +245,7 @@ public:
     KisInputManager inputManager;
 
     KisSignalAutoConnectionsStore viewConnections;
+    KisScriptManager scriptManager;
     KSelectAction *actionAuthor; // Select action for author profile.
     KisAction *showPixelGrid;
 
@@ -312,7 +313,6 @@ KisViewManager::KisViewManager(QWidget *parent, KActionCollection *_actionCollec
     connect(KisPart::instance(), SIGNAL(sigViewRemoved(KisView*)), SLOT(slotViewRemoved(KisView*)));
 
     connect(KisConfigNotifier::instance(), SIGNAL(configChanged()), SLOT(slotUpdateAuthorProfileActions()));
-    connect(KisConfigNotifier::instance(), SIGNAL(pixelGridModeChanged()), SLOT(slotUpdatePixelGridAction()));
 
     KisInputProfileManager::instance()->loadProfiles();
 
@@ -411,23 +411,10 @@ void KisViewManager::setCurrentView(KisView *view)
         if (first) {
             KisConfig cfg;
             KisPaintOpPresetResourceServer * rserver = KisResourceServerProvider::instance()->paintOpPresetServer();
-            QString defaultPresetName = "basic_tip_default";
-            bool foundTip = false;
-            for (int i=0; i<rserver->resourceCount(); i++) {
-                KisPaintOpPresetSP resource = rserver->resources().at(i);
-                if (resource->name().toLower().contains("basic_tip_default")) {
-                    defaultPresetName = resource->name();
-                    foundTip = true;
-                } else if (foundTip == false && (resource->name().toLower().contains("default") ||
-                                                 resource->filename().toLower().contains("default"))) {
-                    defaultPresetName = resource->name();
-                    foundTip = true;
-                }
-            }
-            QString lastPreset = cfg.readEntry("LastPreset", defaultPresetName);
+            QString lastPreset = cfg.readEntry("LastPreset", QString("Basic_tip_default"));
             KisPaintOpPresetSP preset = rserver->resourceByName(lastPreset);
             if (!preset) {
-                preset = rserver->resourceByName(defaultPresetName);
+                preset = rserver->resourceByName("Basic_tip_default");
             }
 
             if (!preset) {
@@ -740,7 +727,8 @@ void KisViewManager::createActions()
     slotUpdateAuthorProfileActions();
 
     d->showPixelGrid = actionManager()->createAction("view_pixel_grid");
-    slotUpdatePixelGridAction();
+    d->showPixelGrid->setChecked(cfg.pixelGridEnabled());
+
 }
 
 void KisViewManager::setupManagers()
@@ -766,6 +754,8 @@ void KisViewManager::setupManagers()
     d->canvasControlsManager.setup(actionManager());
 
     d->mirrorManager.setup(actionCollection());
+
+    d->scriptManager.setup(actionCollection(), actionManager());
 }
 
 void KisViewManager::updateGUI()
@@ -807,6 +797,11 @@ KisDocument *KisViewManager::document() const
     return 0;
 }
 
+KisScriptManager *KisViewManager::scriptManager() const
+{
+  return &d->scriptManager;
+}
+
 int KisViewManager::viewCount() const
 {
     KisMainWindow *mw  = qobject_cast<KisMainWindow*>(d->mainWindow);
@@ -897,12 +892,6 @@ void KisViewManager::slotSaveIncremental()
 {
     if (!document()) return;
 
-    if (document()->url().isEmpty()) {
-        KisMainWindow *mw = qobject_cast<KisMainWindow*>(d->mainWindow);
-        mw->saveDocument(document(), true, false);
-        return;
-    }
-
     bool foundVersion;
     bool fileAlreadyExists;
     bool isBackup;
@@ -999,12 +988,6 @@ void KisViewManager::slotSaveIncrementalBackup()
 {
     if (!document()) return;
 
-    if (document()->url().isEmpty()) {
-        KisMainWindow *mw = qobject_cast<KisMainWindow*>(d->mainWindow);
-        mw->saveDocument(document(), true, false);
-        return;
-    }
-
     bool workingOnBackup;
     bool fileAlreadyExists;
     QString version = "000";
@@ -1276,6 +1259,7 @@ void KisViewManager::updateIcons()
             while (!objects.isEmpty()) {
                 QObject* object = objects.takeFirst();
                 objects.append(object->children());
+
                 KisIconUtils::updateIconCommon(object);
             }
         }
@@ -1395,13 +1379,3 @@ void KisViewManager::slotUpdateAuthorProfileActions()
         d->actionAuthor->setCurrentAction(profileName);
     }
 }
-
-void KisViewManager::slotUpdatePixelGridAction()
-{
-    KIS_SAFE_ASSERT_RECOVER_RETURN(d->showPixelGrid);
-
-    KisSignalsBlocker b(d->showPixelGrid);
-
-    KisConfig cfg;
-    d->showPixelGrid->setChecked(cfg.pixelGridEnabled());
-}
