@@ -50,6 +50,7 @@ class KisUndoAdapter;
 class KisZoomManager;
 class KisPaintopBox;
 class KisActionManager;
+class KisScriptManager;
 class KisInputManager;
 class KoUpdater;
 class KoProgressUpdater;
@@ -163,6 +164,8 @@ public:  // Krita specific interfaces
 
     KisDocument *document() const;
 
+    KisScriptManager *scriptManager() const;
+
     int viewCount() const;
 
     /**
@@ -252,7 +255,6 @@ private Q_SLOTS:
     void guiUpdateTimeout();
     void changeAuthorProfile(const QString &profileName);
     void slotUpdateAuthorProfileActions();
-    void slotUpdatePixelGridAction();
     void slotSaveShowRulersState(bool value);
     void slotSaveRulersTrackMouseState(bool value);
 
