@@ -33,7 +33,7 @@
 #include <QSlider>
 #include <QToolButton>
 #include <QThread>
-#include <QDesktopServices>
+#include <QStandardPaths>
 #include <QGridLayout>
 #include <QRadioButton>
 #include <QGroupBox>
@@ -79,9 +79,14 @@
 
 // for the performance update
 #include <kis_cubic_curve.h>
+#include <kis_signals_blocker.h>
 
 #include "input/config/kis_input_configuration_page.h"
 
+#ifdef Q_OS_WIN
+#  include <kis_tablet_support_win8.h>
+#endif
+
 
 GeneralTab::GeneralTab(QWidget *_parent, const char *_name)
     : WdgGeneralSettings(_parent, _name)
@@ -103,6 +108,10 @@ GeneralTab::GeneralTab(QWidget *_parent, const char *_name)
     m_cmbOutlineShape->addItem(i18n("Preview Outline"));
     m_cmbOutlineShape->addItem(i18n("Tilt Outline"));
 
+    m_cmbKineticScrollingGesture->addItem(i18n("Disabled"));
+    m_cmbKineticScrollingGesture->addItem(i18n("On Touch Drag"));
+    m_cmbKineticScrollingGesture->addItem(i18n("On Click Drag"));
+
     m_cmbCursorShape->setCurrentIndex(cfg.newCursorStyle());
     m_cmbOutlineShape->setCurrentIndex(cfg.newOutlineStyle());
 
@@ -120,6 +129,8 @@ GeneralTab::GeneralTab(QWidget *_parent, const char *_name)
     KConfigGroup group = KSharedConfig::openConfig()->group("File Dialogs");
     m_chkNativeFileDialog->setChecked(!group.readEntry("DontUseNativeFileDialog", true));
 
+    intMaxBrushSize->setValue(cfg.readEntry("maximumBrushSize", 1000));
+
     m_cmbMDIType->setCurrentIndex(cfg.readEntry<int>("mdi_viewmode", (int)QMdiArea::TabbedView));
     m_chkRubberBand->setChecked(cfg.readEntry<int>("mdi_rubberband", cfg.useOpenGL()));
     m_favoritePresetsSpinBox->setValue(cfg.favoritePresets());
@@ -136,9 +147,17 @@ GeneralTab::GeneralTab(QWidget *_parent, const char *_name)
     m_chkSingleApplication->setChecked(kritarc.value("EnableSingleApplication", true).toBool());
 
     m_radioToolOptionsInDocker->setChecked(cfg.toolOptionsInDocker());
+    m_cmbKineticScrollingGesture->setCurrentIndex(cfg.kineticScrollingGesture());
+    m_kineticScrollingSensitivity->setValue(cfg.kineticScrollingSensitivity());
+    m_chkKineticScrollingScrollbar->setChecked(cfg.kineticScrollingScrollbar());
     m_chkSwitchSelectionCtrlAlt->setChecked(cfg.switchSelectionCtrlAlt());
     chkEnableTouch->setChecked(!cfg.disableTouchOnCanvas());
     m_chkConvertOnImport->setChecked(cfg.convertToImageColorspaceOnImport());
+    m_chkCacheAnimatioInBackground->setChecked(cfg.calculateAnimationCacheInBackground());
+
+    KoColor cursorColor(KoColorSpaceRegistry::instance()->rgb8());
+    cursorColor.fromQColor(cfg.getCursorMainColor());
+    cursorColorBtutton->setColor(cursorColor);
 
     connect(m_bnFileName, SIGNAL(clicked()), SLOT(getBackgroundImage()));
     connect(clearBgImageButton, SIGNAL(clicked()), SLOT(clearBackgroundImage()));
@@ -159,6 +178,7 @@ void GeneralTab::setDefault()
     m_showOutlinePainting->setChecked(cfg.showOutlineWhilePainting(true));
     m_hideSplashScreen->setChecked(cfg.hideSplashScreen(true));
     m_chkNativeFileDialog->setChecked(false);
+    intMaxBrushSize->setValue(1000);
 
     m_cmbMDIType->setCurrentIndex((int)QMdiArea::TabbedView);
     m_chkRubberBand->setChecked(cfg.useOpenGL(true));
@@ -174,10 +194,17 @@ void GeneralTab::setDefault()
 
     m_chkHiDPI->setChecked(true);
     m_radioToolOptionsInDocker->setChecked(cfg.toolOptionsInDocker(true));
+    m_cmbKineticScrollingGesture->setCurrentIndex(cfg.kineticScrollingGesture(true));
+    m_kineticScrollingSensitivity->setValue(cfg.kineticScrollingSensitivity(true));
+    m_chkKineticScrollingScrollbar->setChecked(cfg.kineticScrollingScrollbar(true));
     m_chkSwitchSelectionCtrlAlt->setChecked(cfg.switchSelectionCtrlAlt(true));
-    chkEnableTouch->setChecked(cfg.disableTouchOnCanvas(true));
+    chkEnableTouch->setChecked(!cfg.disableTouchOnCanvas(true));
     m_chkConvertOnImport->setChecked(cfg.convertToImageColorspaceOnImport(true));
+    m_chkCacheAnimatioInBackground->setChecked(cfg.calculateAnimationCacheInBackground(true));
 
+    KoColor cursorColor(KoColorSpaceRegistry::instance()->rgb8());
+    cursorColor.fromQColor(cfg.getCursorMainColor(true));
+    cursorColorBtutton->setColor(cursorColor);
 }
 
 CursorStyle GeneralTab::cursorStyle()
@@ -241,6 +268,21 @@ bool GeneralTab::toolOptionsInDocker()
     return m_radioToolOptionsInDocker->isChecked();
 }
 
+int GeneralTab::kineticScrollingGesture()
+{
+    return m_cmbKineticScrollingGesture->currentIndex();
+}
+
+int GeneralTab::kineticScrollingSensitivity()
+{
+    return m_kineticScrollingSensitivity->value();
+}
+
+bool GeneralTab::kineticScrollingScrollbar()
+{
+    return m_chkKineticScrollingScrollbar->isChecked();
+}
+
 bool GeneralTab::switchSelectionCtrlAlt()
 {
     return m_chkSwitchSelectionCtrlAlt->isChecked();
@@ -252,12 +294,17 @@ bool GeneralTab::convertToImageColorspaceOnImport()
     return m_chkConvertOnImport->isChecked();
 }
 
+bool GeneralTab::calculateAnimationCacheInBackground()
+{
+    return m_chkCacheAnimatioInBackground->isChecked();
+}
+
 
 void GeneralTab::getBackgroundImage()
 {
     KoFileDialog dialog(this, KoFileDialog::OpenFile, "BackgroundImages");
     dialog.setCaption(i18n("Select a Background Image"));
-    dialog.setDefaultDir(QDesktopServices::storageLocation(QDesktopServices::PicturesLocation));
+    dialog.setDefaultDir(QStandardPaths::writableLocation(QStandardPaths::PicturesLocation));
     dialog.setImageFilters();
 
     QString fn = dialog.filename();
@@ -421,7 +468,7 @@ void ColorSettingsTab::installProfile()
 {
     KoFileDialog dialog(this, KoFileDialog::OpenFiles, "OpenDocumentICC");
     dialog.setCaption(i18n("Install Color Profiles"));
-    dialog.setDefaultDir(QDesktopServices::storageLocation(QDesktopServices::HomeLocation));
+    dialog.setDefaultDir(QStandardPaths::writableLocation(QStandardPaths::HomeLocation));
     dialog.setMimeTypeFilters(QStringList() << "application/vnd.iccprofile", "application/vnd.iccprofile");
     QStringList profileNames = dialog.filenames();
 
@@ -514,19 +561,14 @@ void ColorSettingsTab::setDefault()
 }
 
 
-void ColorSettingsTab::refillMonitorProfiles(const KoID & s)
+void ColorSettingsTab::refillMonitorProfiles(const KoID & colorSpaceId)
 {
-    const KoColorSpaceFactory * csf = KoColorSpaceRegistry::instance()->colorSpaceFactory(s.id());
-
     for (int i = 0; i < QApplication::desktop()->screenCount(); ++i) {
         m_monitorProfileWidgets[i]->clear();
     }
 
-    if (!csf)
-        return;
-
     QMap<QString, const KoColorProfile *>  profileList;
-    Q_FOREACH(const KoColorProfile *profile, KoColorSpaceRegistry::instance()->profilesFor(csf)) {
+    Q_FOREACH(const KoColorProfile *profile, KoColorSpaceRegistry::instance()->profilesFor(colorSpaceId.id())) {
         profileList[profile->name()] = profile;
     }
 
@@ -541,7 +583,7 @@ void ColorSettingsTab::refillMonitorProfiles(const KoID & s)
 
     for (int i = 0; i < QApplication::desktop()->screenCount(); ++i) {
         m_monitorProfileLabels[i]->setText(i18nc("The number of the screen", "Screen %1:", i + 1));
-        m_monitorProfileWidgets[i]->setCurrent(csf->defaultProfile());
+        m_monitorProfileWidgets[i]->setCurrent(KoColorSpaceRegistry::instance()->defaultProfileForColorSpace(colorSpaceId.id()));
     }
 }
 
@@ -553,6 +595,17 @@ void TabletSettingsTab::setDefault()
     KisCubicCurve curve;
     curve.fromString(DEFAULT_CURVE_STRING);
     m_page->pressureCurve->setCurve(curve);
+
+#ifdef Q_OS_WIN
+    if (KisTabletSupportWin8::isAvailable()) {
+        KisConfig cfg;
+        m_page->radioWintab->setChecked(!cfg.useWin8PointerInput(true));
+        m_page->radioWin8PointerInput->setChecked(cfg.useWin8PointerInput(true));
+    } else {
+        m_page->radioWintab->setChecked(true);
+        m_page->radioWin8PointerInput->setChecked(false);
+    }
+#endif
 }
 
 TabletSettingsTab::TabletSettingsTab(QWidget* parent, const char* name): QWidget(parent)
@@ -570,6 +623,19 @@ TabletSettingsTab::TabletSettingsTab(QWidget* parent, const char* name): QWidget
 
     m_page->pressureCurve->setMaximumSize(QSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX));
     m_page->pressureCurve->setCurve(curve);
+
+#ifdef Q_OS_WIN
+    if (KisTabletSupportWin8::isAvailable()) {
+        m_page->radioWintab->setChecked(!cfg.useWin8PointerInput());
+        m_page->radioWin8PointerInput->setChecked(cfg.useWin8PointerInput());
+    } else {
+        m_page->radioWintab->setChecked(true);
+        m_page->radioWin8PointerInput->setChecked(false);
+        m_page->grpTabletApi->setVisible(false);
+    }
+#else
+    m_page->grpTabletApi->setVisible(false);
+#endif
 }
 
 
@@ -656,6 +722,14 @@ PerformanceTab::PerformanceTab(QWidget *parent, const char *name)
     lblSwapFileLocation->setText(cfg.swapDir());
     connect(bnSwapFile, SIGNAL(clicked()), SLOT(selectSwapDir()));
 
+    sliderThreadsLimit->setRange(1, QThread::idealThreadCount());
+    sliderFrameClonesLimit->setRange(1, QThread::idealThreadCount());
+    sliderFpsLimit->setRange(20, 100);
+    sliderFpsLimit->setSuffix(i18n(" fps"));
+
+    connect(sliderThreadsLimit, SIGNAL(valueChanged(int)), SLOT(slotThreadsLimitChanged(int)));
+    connect(sliderFrameClonesLimit, SIGNAL(valueChanged(int)), SLOT(slotFrameClonesLimitChanged(int)));
+
     load(false);
 }
 
@@ -678,9 +752,18 @@ void PerformanceTab::load(bool requestDefault)
     sliderSwapSize->setValue(cfg.maxSwapSize(requestDefault) / 1024);
     lblSwapFileLocation->setText(cfg.swapDir(requestDefault));
 
+    m_lastUsedThreadsLimit = cfg.maxNumberOfThreads(requestDefault);
+    m_lastUsedClonesLimit = cfg.frameRenderingClones(requestDefault);
+
+    sliderThreadsLimit->setValue(m_lastUsedThreadsLimit);
+    sliderFrameClonesLimit->setValue(m_lastUsedClonesLimit);
+
+    sliderFpsLimit->setValue(cfg.fpsLimit(requestDefault));
+
     {
         KisConfig cfg2;
-        chkOpenGLLogging->setChecked(cfg2.enableOpenGLDebugging(requestDefault));
+        chkOpenGLFramerateLogging->setChecked(cfg2.enableOpenGLFramerateLogging(requestDefault));
+        chkBrushSpeedLogging->setChecked(cfg2.enableBrushSpeedLogging(requestDefault));
         chkDisableVectorOptimizations->setChecked(cfg2.enableAmdVectorizationWorkaround(requestDefault));
     }
 }
@@ -700,9 +783,14 @@ void PerformanceTab::save()
 
     cfg.setSwapDir(lblSwapFileLocation->text());
 
+    cfg.setMaxNumberOfThreads(sliderThreadsLimit->value());
+    cfg.setFrameRenderingClones(sliderFrameClonesLimit->value());
+    cfg.setFpsLimit(sliderFpsLimit->value());
+
     {
         KisConfig cfg2;
-        cfg2.setEnableOpenGLDebugging(chkOpenGLLogging->isChecked());
+        cfg2.setEnableOpenGLFramerateLogging(chkOpenGLFramerateLogging->isChecked());
+        cfg2.setEnableBrushSpeedLogging(chkBrushSpeedLogging->isChecked());
         cfg2.setEnableAmdVectorizationWorkaround(chkDisableVectorOptimizations->isChecked());
     }
 }
@@ -718,6 +806,20 @@ void PerformanceTab::selectSwapDir()
     lblSwapFileLocation->setText(swapDir);
 }
 
+void PerformanceTab::slotThreadsLimitChanged(int value)
+{
+    KisSignalsBlocker b(sliderFrameClonesLimit);
+    sliderFrameClonesLimit->setValue(qMin(m_lastUsedClonesLimit, value));
+    m_lastUsedThreadsLimit = value;
+}
+
+void PerformanceTab::slotFrameClonesLimitChanged(int value)
+{
+    KisSignalsBlocker b(sliderThreadsLimit);
+    sliderThreadsLimit->setValue(qMax(m_lastUsedThreadsLimit, value));
+    m_lastUsedClonesLimit = value;
+}
+
 //---------------------------------------------------------------------------------------------------
 
 #include "KoColor.h"
@@ -727,7 +829,44 @@ DisplaySettingsTab::DisplaySettingsTab(QWidget *parent, const char *name)
 {
     KisConfig cfg;
 
+    const QString rendererOpenGLText = i18nc("canvas renderer", "OpenGL");
+    const QString rendererAngleText = i18nc("canvas renderer", "Direct3D 11 via ANGLE");
+#ifdef Q_OS_WIN
+    cmbRenderer->clear();
+    QString qtPreferredRendererText;
+    if (KisOpenGL::getQtPreferredOpenGLRenderer() == KisOpenGL::RendererAngle) {
+        qtPreferredRendererText = rendererAngleText;
+    } else {
+        qtPreferredRendererText = rendererOpenGLText;
+    }
+    cmbRenderer->addItem(i18nc("canvas renderer", "Auto (%1)", qtPreferredRendererText), KisOpenGL::RendererAuto);
+    cmbRenderer->setCurrentIndex(0);
+    if (KisOpenGL::getSupportedOpenGLRenderers() & KisOpenGL::RendererDesktopGL) {
+        cmbRenderer->addItem(rendererOpenGLText, KisOpenGL::RendererDesktopGL);
+        if (KisOpenGL::getNextUserOpenGLRendererConfig() == KisOpenGL::RendererDesktopGL) {
+            cmbRenderer->setCurrentIndex(cmbRenderer->count() - 1);
+        }
+    }
+    if (KisOpenGL::getSupportedOpenGLRenderers() & KisOpenGL::RendererAngle) {
+        cmbRenderer->addItem(rendererAngleText, KisOpenGL::RendererAngle);
+        if (KisOpenGL::getNextUserOpenGLRendererConfig() == KisOpenGL::RendererAngle) {
+            cmbRenderer->setCurrentIndex(cmbRenderer->count() - 1);
+        }
+    }
+#else
+    lblRenderer->setEnabled(false);
+    cmbRenderer->setEnabled(false);
+    cmbRenderer->clear();
+    cmbRenderer->addItem(rendererOpenGLText);
+    cmbRenderer->setCurrentIndex(0);
+#endif
+
+#ifdef Q_OS_WIN
+    if (!(KisOpenGL::getSupportedOpenGLRenderers() &
+            (KisOpenGL::RendererDesktopGL | KisOpenGL::RendererAngle))) {
+#else
     if (!KisOpenGL::hasOpenGL()) {
+#endif
         grpOpenGL->setEnabled(false);
         grpOpenGL->setChecked(false);
         chkUseTextureBuffer->setEnabled(false);
@@ -748,6 +887,24 @@ DisplaySettingsTab::DisplaySettingsTab(QWidget *parent, const char *name)
             cmbFilterMode->removeItem(3);
         }
     }
+
+    const QStringList openglWarnings = KisOpenGL::getOpenGLWarnings();
+    if (openglWarnings.isEmpty()) {
+        lblOpenGLWarnings->setVisible(false);
+    } else {
+        QString text("<span style=\"color: yellow;\">&#x26A0;</span> ");
+        text.append(i18n("Warning(s):"));
+        text.append("<ul>");
+        Q_FOREACH (const QString &warning, openglWarnings) {
+            text.append("<li>");
+            text.append(warning.toHtmlEscaped());
+            text.append("</li>");
+        }
+        text.append("</ul>");
+        lblOpenGLWarnings->setText(text);
+        lblOpenGLWarnings->setVisible(true);
+    }
+
     if (qApp->applicationName() == "kritasketch" || qApp->applicationName() == "kritagemini") {
        grpOpenGL->setVisible(false);
        grpOpenGL->setMaximumHeight(0);
@@ -779,12 +936,23 @@ DisplaySettingsTab::DisplaySettingsTab(QWidget *parent, const char *name)
     chkHidePopups->setChecked(cfg.hidePopups());
 
     connect(grpOpenGL, SIGNAL(toggled(bool)), SLOT(slotUseOpenGLToggled(bool)));
+
+    KoColor gridColor(KoColorSpaceRegistry::instance()->rgb8());
+    gridColor.fromQColor(cfg.getPixelGridColor());
+    pixelGridColorButton->setColor(gridColor);
+    pixelGridDrawingThresholdBox->setValue(cfg.getPixelGridDrawingThreshold() * 100);
 }
 
 void DisplaySettingsTab::setDefault()
 {
     KisConfig cfg;
+    cmbRenderer->setCurrentIndex(0);
+#ifdef Q_OS_WIN
+    if (!(KisOpenGL::getSupportedOpenGLRenderers() &
+            (KisOpenGL::RendererDesktopGL | KisOpenGL::RendererAngle))) {
+#else
     if (!KisOpenGL::hasOpenGL()) {
+#endif
         grpOpenGL->setEnabled(false);
         grpOpenGL->setChecked(false);
         chkUseTextureBuffer->setEnabled(false);
@@ -819,6 +987,10 @@ void DisplaySettingsTab::setDefault()
     chkChannelsAsColor->setChecked(cfg.showSingleChannelAsColor(true));
     chkHidePopups->setChecked(cfg.hidePopups(true));
 
+    KoColor gridColor(KoColorSpaceRegistry::instance()->rgb8());
+    gridColor.fromQColor(cfg.getPixelGridColor(true));
+    pixelGridColorButton->setColor(gridColor);
+    pixelGridDrawingThresholdBox->setValue(cfg.getPixelGridDrawingThreshold(true) * 100);
 }
 
 void DisplaySettingsTab::slotUseOpenGLToggled(bool isChecked)
@@ -864,7 +1036,7 @@ KisDlgPreferences::KisDlgPreferences(QWidget* parent, const char* name)
     setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel | QDialogButtonBox::RestoreDefaults);
     button(QDialogButtonBox::Ok)->setDefault(true);
 
-    setFaceType(KPageDialog::List);
+    setFaceType(KPageDialog::Tree);
 
     // General
     KoVBox *vbox = new KoVBox();
@@ -947,6 +1119,7 @@ KisDlgPreferences::KisDlgPreferences(QWidget* parent, const char* name)
 
 
     QPushButton *restoreDefaultsButton = button(QDialogButtonBox::RestoreDefaults);
+    restoreDefaultsButton->setText("Restore Defaults");
 
     connect(this, SIGNAL(accepted()), m_inputConfiguration, SLOT(saveChanges()));
     connect(this, SIGNAL(rejected()), m_inputConfiguration, SLOT(revertChanges()));
@@ -1017,10 +1190,13 @@ bool KisDlgPreferences::editPreferences()
         cfg.setShowRootLayer(dialog->m_general->showRootLayer());
         cfg.setShowOutlineWhilePainting(dialog->m_general->showOutlineWhilePainting());
         cfg.setHideSplashScreen(dialog->m_general->hideSplashScreen());
+        cfg.setCalculateAnimationCacheInBackground(dialog->m_general->calculateAnimationCacheInBackground());
 
         KConfigGroup group = KSharedConfig::openConfig()->group("File Dialogs");
         group.writeEntry("DontUseNativeFileDialog", !dialog->m_general->m_chkNativeFileDialog->isChecked());
 
+        cfg.writeEntry<int>("maximumBrushSize", dialog->m_general->intMaxBrushSize->value());
+
         cfg.writeEntry<int>("mdi_viewmode", dialog->m_general->mdiMode());
         cfg.setMDIBackgroundColor(dialog->m_general->m_mdiColor->color().toQColor());
         cfg.setMDIBackgroundImage(dialog->m_general->m_backgroundimage->text());
@@ -1035,20 +1211,12 @@ bool KisDlgPreferences::editPreferences()
         kritarc.setValue("EnableSingleApplication", dialog->m_general->m_chkSingleApplication->isChecked());
 
         cfg.setToolOptionsInDocker(dialog->m_general->toolOptionsInDocker());
+        cfg.setKineticScrollingGesture(dialog->m_general->kineticScrollingGesture());
+        cfg.setKineticScrollingSensitivity(dialog->m_general->kineticScrollingSensitivity());
+        cfg.setKineticScrollingScrollbar(dialog->m_general->kineticScrollingScrollbar());
         cfg.setSwitchSelectionCtrlAlt(dialog->m_general->switchSelectionCtrlAlt());
         cfg.setDisableTouchOnCanvas(!dialog->m_general->chkEnableTouch->isChecked());
         cfg.setConvertToImageColorspaceOnImport(dialog->m_general->convertToImageColorspaceOnImport());
-
-        KisPart *part = KisPart::instance();
-        if (part) {
-            Q_FOREACH (QPointer<KisDocument> doc, part->documents()) {
-                if (doc) {
-                    doc->setAutoSave(dialog->m_general->autoSaveInterval());
-                    doc->setBackupFile(dialog->m_general->m_backupFileCheckBox->isChecked());
-                    doc->undoStack()->setUndoLimit(dialog->m_general->undoStackSize());
-                }
-            }
-        }
         cfg.setUndoStackLimit(dialog->m_general->undoStackSize());
         cfg.setFavoritePresets(dialog->m_general->favoritePresets());
 
@@ -1081,9 +1249,25 @@ bool KisDlgPreferences::editPreferences()
 
         // Tablet settings
         cfg.setPressureTabletCurve( dialog->m_tabletSettings->m_page->pressureCurve->curve().toString() );
+#ifdef Q_OS_WIN
+        if (KisTabletSupportWin8::isAvailable()) {
+            cfg.setUseWin8PointerInput(dialog->m_tabletSettings->m_page->radioWin8PointerInput->isChecked());
+        }
+#endif
 
         dialog->m_performanceSettings->save();
 
+#ifdef Q_OS_WIN
+        {
+            KisOpenGL::OpenGLRenderer renderer = static_cast<KisOpenGL::OpenGLRenderer>(
+                    dialog->m_displaySettings->cmbRenderer->itemData(
+                            dialog->m_displaySettings->cmbRenderer->currentIndex()).toInt());
+            KisOpenGL::setNextUserOpenGLRendererConfig(renderer);
+            const QString configPath = QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation);
+            QSettings kritarc(configPath + QStringLiteral("/kritadisplayrc"), QSettings::IniFormat);
+            kritarc.setValue("OpenGLRenderer", KisOpenGL::convertOpenGLRendererToConfig(renderer));
+        }
+#endif
         if (!cfg.useOpenGL() && dialog->m_displaySettings->grpOpenGL->isChecked())
             cfg.setCanvasState("TRY_OPENGL");
         cfg.setUseOpenGL(dialog->m_displaySettings->grpOpenGL->isChecked());
@@ -1112,6 +1296,10 @@ bool KisDlgPreferences::editPreferences()
         cfg.setHideTitlebarFullscreen(dialog->m_fullscreenSettings->chkTitlebar->checkState());
         cfg.setHideToolbarFullscreen(dialog->m_fullscreenSettings->chkToolbar->checkState());
 
+        cfg.setCursorMainColor(dialog->m_general->cursorColorBtutton->color().toQColor());
+        cfg.setPixelGridColor(dialog->m_displaySettings->pixelGridColorButton->color().toQColor());
+        cfg.setPixelGridDrawingThreshold(dialog->m_displaySettings->pixelGridDrawingThresholdBox->value() / 100);
+
         dialog->m_authorPage->apply();
 
     }
