@@ -25,7 +25,7 @@
 #include <klocalizedstring.h>
 #include <QApplication>
 #include <QTouchEvent>
-#include <QTouchEvent>
+#include <QElapsedTimer>
 
 #include <KoToolManager.h>
 
@@ -248,6 +248,10 @@ bool KisInputManager::eventFilterImpl(QEvent * event)
 {
     bool retval = false;
 
+    if (event->type() != QEvent::Wheel) {
+        d->accumulatedScrollDelta = 0;
+    }
+
     switch (event->type()) {
     case QEvent::MouseButtonPress:
     case QEvent::MouseButtonDblClick: {
@@ -326,13 +330,29 @@ bool KisInputManager::eventFilterImpl(QEvent * event)
     case QEvent::Wheel: {
         d->debugEvent<QWheelEvent, false>(event);
         QWheelEvent *wheelEvent = static_cast<QWheelEvent*>(event);
+
+#ifdef Q_OS_OSX
+        // Some QT wheel events are actually touch pad pan events. From the QT docs:
+        // "Wheel events are generated for both mouse wheels and trackpad scroll gestures."
+
+        // We differentiate between touchpad events and real mouse wheels by inspecting the
+        // event source.
+
+        if (wheelEvent->source() == Qt::MouseEventSource::MouseEventSynthesizedBySystem) {
+            KisAbstractInputAction::setInputManager(this);
+            retval = d->matcher.wheelEvent(KisSingleActionShortcut::WheelTrackpad, wheelEvent);
+            break;
+        }
+#endif
+
+        d->accumulatedScrollDelta += wheelEvent->delta();
         KisSingleActionShortcut::WheelAction action;
 
         /**
          * Ignore delta 0 events on OSX, since they are triggered by tablet
          * proximity when using Wacom devices.
          */
-#ifdef Q_OS_MAC
+#ifdef Q_OS_OSX
         if(wheelEvent->delta() == 0) {
             retval = true;
             break;
@@ -356,9 +376,15 @@ bool KisInputManager::eventFilterImpl(QEvent * event)
             }
         }
 
-        //Make sure the input actions know we are active.
-        KisAbstractInputAction::setInputManager(this);
-        retval = d->matcher.wheelEvent(action, wheelEvent);
+        if (qAbs(d->accumulatedScrollDelta) >= QWheelEvent::DefaultDeltasPerStep) {
+            //Make sure the input actions know we are active.
+            KisAbstractInputAction::setInputManager(this);
+            retval = d->matcher.wheelEvent(action, wheelEvent);
+            d->accumulatedScrollDelta = 0;
+        }
+        else {
+            retval = true;
+        }
         break;
     }
     case QEvent::Enter:
@@ -440,6 +466,10 @@ bool KisInputManager::eventFilterImpl(QEvent * event)
         QTabletEvent *tabletEvent = static_cast<QTabletEvent*>(event);
         retval = compressMoveEventCommon(tabletEvent);
 
+        if (d->tabletLatencyTracker) {
+            d->tabletLatencyTracker->push(tabletEvent->timestamp());
+        }
+
         /**
          * The flow of tablet events means the tablet is in the
          * proximity area, so activate it even when the
@@ -455,7 +485,6 @@ bool KisInputManager::eventFilterImpl(QEvent * event)
 #ifdef Q_OS_MAC
         d->allowMouseEvents();
 #endif
-        if (d->touchHasBlockedPressEvents) break;
         d->debugEvent<QTabletEvent, false>(event);
 
         QTabletEvent *tabletEvent = static_cast<QTabletEvent*>(event);
@@ -467,16 +496,10 @@ bool KisInputManager::eventFilterImpl(QEvent * event)
 
     case QEvent::TouchBegin:
     {
-        QTouchEvent *tevent = static_cast<QTouchEvent*>(event);
-        qDebug() << "QEvent::TouchBegin" << KisConfig().disableTouchOnCanvas() << "touch points" << tevent->touchPoints().count() << "pressure" << tevent->touchPoints().first().pressure() << "tablet active" << d->tabletActive;
-        d->touchHasBlockedPressEvents = KisConfig().disableTouchOnCanvas();
-        // Touch rejection: if touch is disabled on canvas, no need to block mouse press events
-        if (KisConfig().disableTouchOnCanvas()) d->eatOneMousePress();
-        if (d->tryHidePopupPalette()) {
-            retval = true;
-        } else {
+        if (startTouch(retval)) {
+            QTouchEvent *tevent = static_cast<QTouchEvent*>(event);
             KisAbstractInputAction::setInputManager(this);
-            retval = d->matcher.touchBeginEvent(static_cast<QTouchEvent*>(event));
+            retval = d->matcher.touchBeginEvent(tevent);
             event->accept();
         }
         break;
@@ -484,7 +507,6 @@ bool KisInputManager::eventFilterImpl(QEvent * event)
     case QEvent::TouchUpdate:
     {
         QTouchEvent *tevent = static_cast<QTouchEvent*>(event);
-        qDebug() << "QEvent::TouchUpdate" << KisConfig().disableTouchOnCanvas() << "touch points" << tevent->touchPoints().count()<< "pressure" << tevent->touchPoints().first().pressure();
 #ifdef Q_OS_MAC
         int count = 0;
         Q_FOREACH (const QTouchEvent::TouchPoint &point, tevent->touchPoints()) {
@@ -501,7 +523,7 @@ bool KisInputManager::eventFilterImpl(QEvent * event)
             d->touchHasBlockedPressEvents = KisConfig().disableTouchOnCanvas();
             KisAbstractInputAction::setInputManager(this);
             retval = d->matcher.touchUpdateEvent(tevent);
-#ifdef Q_OS_MAC
+#ifdef Q_OS_OSX
         }
 #endif
         event->accept();
@@ -509,13 +531,44 @@ bool KisInputManager::eventFilterImpl(QEvent * event)
     }
     case QEvent::TouchEnd:
     {
+        endTouch();
         QTouchEvent *tevent = static_cast<QTouchEvent*>(event);
-        qDebug() << "QEvent::TouchEnd" << KisConfig().disableTouchOnCanvas() << "touch points" << tevent->touchPoints().count()<< "pressure" << tevent->touchPoints().first().pressure();
-        d->touchHasBlockedPressEvents = false;
-        retval = d->matcher.touchEndEvent(static_cast<QTouchEvent*>(event));
+        retval = d->matcher.touchEndEvent(tevent);
         event->accept();
         break;
     }
+
+    case QEvent::NativeGesture:
+    {
+        QNativeGestureEvent *gevent = static_cast<QNativeGestureEvent*>(event);
+        switch (gevent->gestureType()) {
+            case Qt::BeginNativeGesture:
+            {
+                if (startTouch(retval)) {
+                    KisAbstractInputAction::setInputManager(this);
+                    retval = d->matcher.nativeGestureBeginEvent(gevent);
+                    event->accept();
+                }
+                break;
+            }
+            case Qt::EndNativeGesture:
+            {
+                endTouch();
+                retval = d->matcher.nativeGestureEndEvent(gevent);
+                event->accept();
+                break;
+            }
+            default:
+            {
+                KisAbstractInputAction::setInputManager(this);
+                retval = d->matcher.nativeGestureEvent(gevent);
+                event->accept();
+                break;
+            }
+        }
+        break;
+    }
+
     default:
         break;
     }
@@ -523,6 +576,26 @@ bool KisInputManager::eventFilterImpl(QEvent * event)
     return !retval ? d->processUnhandledEvent(event) : true;
 }
 
+bool KisInputManager::startTouch(bool &retval)
+{
+    d->touchHasBlockedPressEvents = KisConfig().disableTouchOnCanvas();
+    // Touch rejection: if touch is disabled on canvas, no need to block mouse press events
+    if (KisConfig().disableTouchOnCanvas()) {
+        d->eatOneMousePress();
+    }
+    if (d->tryHidePopupPalette()) {
+        retval = true;
+        return false;
+    } else {
+        return true;
+    }
+}
+
+void KisInputManager::endTouch()
+{
+    d->touchHasBlockedPressEvents = false;
+}
+
 void KisInputManager::slotCompressedMoveEvent()
 {
     if (d->compressedMoveEvent) {
@@ -593,7 +666,9 @@ void KisInputManager::profileChanged()
                 d->addWheelShortcut(shortcut->action(), shortcut->mode(), shortcut->keys(), shortcut->wheel());
                 break;
             case KisShortcutConfiguration::GestureType:
-                d->addTouchShortcut(shortcut->action(), shortcut->mode(), shortcut->gesture());
+                if (!d->addNativeGestureShortcut(shortcut->action(), shortcut->mode(), shortcut->gesture())) {
+                    d->addTouchShortcut(shortcut->action(), shortcut->mode(), shortcut->gesture());
+                }
                 break;
             default:
                 break;
