@@ -544,8 +544,13 @@ public:
 
     class WheelEvent : public InputEvent {
     public:
+#if QT_VERSION >= 0x050700
+        WheelEvent(QWindow *w, ulong time, const QPointF & local, const QPointF & global, QPoint pixelD, QPoint angleD, int qt4D, Qt::Orientation qt4O,
+                   Qt::KeyboardModifiers mods, Qt::ScrollPhase phase = Qt::NoScrollPhase, Qt::MouseEventSource src = Qt::MouseEventNotSynthesized)
+#else
         WheelEvent(QWindow *w, ulong time, const QPointF & local, const QPointF & global, QPoint pixelD, QPoint angleD, int qt4D, Qt::Orientation qt4O,
                    Qt::KeyboardModifiers mods, Qt::ScrollPhase phase = Qt::ScrollUpdate, Qt::MouseEventSource src = Qt::MouseEventNotSynthesized)
+#endif
             : InputEvent(w, time, Wheel, mods), pixelDelta(pixelD), angleDelta(angleD), qt4Delta(qt4D), qt4Orientation(qt4O), localPos(local), globalPos(global), phase(phase), source(src) { }
         QPoint pixelDelta;
         QPoint angleDelta;
@@ -577,7 +582,7 @@ void QWindowSystemInterface::handleTabletEvent(QWindow *w, const QPointF &local,
                                                qreal tangentialPressure, qreal rotation, int z, qint64 uid,
                                                Qt::KeyboardModifiers modifiers)
 {
-    qint64 timestamp = g_eventTimer.elapsed();
+    qint64 timestamp = g_eventTimer.msecsSinceReference() + g_eventTimer.elapsed();
 
     QWindowSystemInterfacePrivate::TabletEvent *e =
         new QWindowSystemInterfacePrivate::TabletEvent(w, timestamp, local, global, device, pointerType, buttons, pressure,
@@ -677,7 +682,7 @@ void processTabletEvent(QWindowSystemInterfacePrivate::TabletEvent *e)
 
 void QWindowSystemInterface::handleTabletEnterProximityEvent(int device, int pointerType, qint64 uid)
 {
-    qint64 timestamp = g_eventTimer.elapsed();
+    qint64 timestamp = g_eventTimer.msecsSinceReference() + g_eventTimer.elapsed();
 
     QTabletEvent ev(QEvent::TabletEnterProximity, QPointF(), QPointF(),
                     device, pointerType, 0, 0, 0,
@@ -689,7 +694,7 @@ void QWindowSystemInterface::handleTabletEnterProximityEvent(int device, int poi
 
 void QWindowSystemInterface::handleTabletLeaveProximityEvent(int device, int pointerType, qint64 uid)
 {
-    qint64 timestamp = g_eventTimer.elapsed();
+    qint64 timestamp = g_eventTimer.msecsSinceReference() + g_eventTimer.elapsed();
 
     QTabletEvent ev(QEvent::TabletLeaveProximity, QPointF(), QPointF(),
                     device, pointerType, 0, 0, 0,
