@@ -103,6 +103,9 @@ public:
     CursorStyle newCursorStyle(bool defaultValue = false) const;
     void setNewCursorStyle(CursorStyle style);
 
+    QColor getCursorMainColor(bool defaultValue = false) const;
+    void setCursorMainColor(const QColor& v) const;
+
     OutlineStyle newOutlineStyle(bool defaultValue = false) const;
     void setNewOutlineStyle(OutlineStyle style);
 
@@ -145,6 +148,12 @@ public:
     bool showRulers(bool defaultValue = false) const;
     void setShowRulers(bool rulers) const;
 
+    bool forceShowSaveMessages(bool defaultValue = true) const;
+    void setForceShowSaveMessages(bool value) const;
+
+    bool forceShowAutosaveMessages(bool defaultValue = true) const;
+    void setForceShowAutosaveMessages(bool ShowAutosaveMessages) const;
+
     bool rulersTrackMouse(bool defaultValue = false) const;
     void setRulersTrackMouse(bool value) const;
 
@@ -174,9 +183,6 @@ public:
     int openGLTextureSize(bool defaultValue = false) const;
     int textureOverlapBorder() const;
 
-    qint32 maxNumberOfThreads(bool defaultValue = false) const;
-    void setMaxNumberOfThreads(qint32 numberOfThreads);
-
     quint32 getGridMainStyle(bool defaultValue = false) const;
     void setGridMainStyle(quint32 v) const;
 
@@ -189,6 +195,15 @@ public:
     QColor getGridSubdivisionColor(bool defaultValue = false) const;
     void setGridSubdivisionColor(const QColor & v) const;
 
+    QColor getPixelGridColor(bool defaultValue = false) const;
+    void setPixelGridColor(const QColor & v) const;
+
+    qreal getPixelGridDrawingThreshold(bool defaultValue = false) const;
+    void setPixelGridDrawingThreshold(qreal v) const;
+
+    bool pixelGridEnabled(bool defaultValue = false) const;
+    void enablePixelGrid(bool v) const;
+
     quint32 guidesLineStyle(bool defaultValue = false) const;
     void setGuidesLineStyle(quint32 v) const;
     QColor guidesColor(bool defaultValue = false) const;
@@ -267,12 +282,19 @@ public:
     QString pressureTabletCurve(bool defaultValue = false) const;
     void setPressureTabletCurve(const QString& curveString) const;
 
+    bool useWin8PointerInput(bool defaultValue = false) const;
+    void setUseWin8PointerInput(bool value) const;
+
     qreal vastScrolling(bool defaultValue = false) const;
     void setVastScrolling(const qreal factor) const;
 
     int presetChooserViewMode(bool defaultValue = false) const;
     void setPresetChooserViewMode(const int mode) const;
 
+    int presetIconSize(bool defaultValue = false) const;
+    void setPresetIconSize(const int value) const;
+
+
     bool firstRun(bool defaultValue = false) const;
     void setFirstRun(const bool firstRun) const;
 
@@ -319,7 +341,10 @@ public:
     void setFavoriteCompositeOps(const QStringList& compositeOps) const;
 
     QString exportConfiguration(const QString &filterId, bool defaultValue = false) const;
-    void setExportConfiguration(const QString &filterId, const KisPropertiesConfiguration &properties) const;
+    void setExportConfiguration(const QString &filterId, KisPropertiesConfigurationSP properties) const;
+
+    QString importConfiguration(const QString &filterId, bool defaultValue = false) const;
+    void setImportConfiguration(const QString &filterId, KisPropertiesConfigurationSP properties) const;
 
     bool useOcio(bool defaultValue = false) const;
     void setUseOcio(bool useOCIO) const;
@@ -428,6 +453,9 @@ public:
     int tabletEventsDelay(bool defaultValue = false) const;
     void setTabletEventsDelay(int value);
 
+    bool trackTabletEventLatency(bool defaultValue = false) const;
+    void setTrackTabletEventLatency(bool value);
+
     bool testingAcceptCompressedTabletEvents(bool defaultValue = false) const;
     void setTestingAcceptCompressedTabletEvents(bool value);
 
@@ -447,15 +475,13 @@ public:
 
     bool useEraserBrushOpacity(bool defaultValue = false) const;
     void setUseEraserBrushOpacity(bool value);
-    
+
     QColor getMDIBackgroundColor(bool defaultValue = false) const;
     void setMDIBackgroundColor(const QColor & v) const;
 
     QString getMDIBackgroundImage(bool defaultValue = false) const;
     void setMDIBackgroundImage(const QString & fileName) const;
 
-    bool useVerboseOpenGLDebugOutput(bool defaultValue = false) const;
-
     int workaroundX11SmoothPressureSteps(bool defaultValue = false) const;
 
     bool showCanvasMessages(bool defaultValue = false) const;
@@ -467,8 +493,20 @@ public:
     bool toolOptionsInDocker(bool defaultValue = false) const;
     void setToolOptionsInDocker(bool inDocker);
 
-    void setEnableOpenGLDebugging(bool value) const;
-    bool enableOpenGLDebugging(bool defaultValue = false) const;
+    int kineticScrollingGesture(bool defaultValue = false) const;
+    void setKineticScrollingGesture(int kineticScroll);
+
+    int kineticScrollingSensitivity(bool defaultValue = false) const;
+    void setKineticScrollingSensitivity(int sensitivity);
+
+    bool kineticScrollingScrollbar(bool defaultValue = false) const;
+    void setKineticScrollingScrollbar(bool scrollbar);
+
+    void setEnableOpenGLFramerateLogging(bool value) const;
+    bool enableOpenGLFramerateLogging(bool defaultValue = false) const;
+
+    void setEnableBrushSpeedLogging(bool value) const;
+    bool enableBrushSpeedLogging(bool defaultValue = false) const;
 
     void setEnableAmdVectorizationWorkaround(bool value);
     bool enableAmdVectorizationWorkaround(bool defaultValue = false) const;
@@ -494,8 +532,8 @@ public:
     int stabilizerSampleSize(bool defaultValue = false) const;
     void setStabilizerSampleSize(int value);
 
-    int stabilizerDelayedPaintInterval(bool defaultValue = false) const;
-    void setStabilizerDelayedPaintInterval(int value);
+    bool stabilizerDelayedPaint(bool defaultValue = false) const;
+    void setStabilizerDelayedPaint(bool value);
 
     QString customFFMpegPath(bool defaultValue = false) const;
     void setCustomFFMpegPath(const QString &value) const;
@@ -506,7 +544,8 @@ public:
     QString brushHudSetting(bool defaultValue = false) const;
     void setBrushHudSetting(const QString &value) const;
 
-    bool disableColorizeMaskFeature() const;
+    bool calculateAnimationCacheInBackground(bool defaultValue = false) const;
+    void setCalculateAnimationCacheInBackground(bool value);
 
     template<class T>
     void writeEntry(const QString& name, const T& value) {
@@ -529,7 +568,7 @@ public:
     }
 
 
-    /// get the profile the color managment system has stored for the given screen
+    /// get the profile the color management system has stored for the given screen
     static const KoColorProfile* getScreenProfile(int screen);
 
 private:
