@@ -25,7 +25,7 @@
 #include <QDialog>
 #include <QVBoxLayout>
 #include <QFileInfo>
-#include <QDesktopServices>
+#include <QStandardPaths>
 
 #include <kactioncollection.h>
 #include <klocalizedstring.h>
@@ -81,6 +81,7 @@
 #include "commands/kis_image_commands.h"
 #include "commands/kis_layer_command.h"
 #include "commands/kis_node_commands.h"
+#include "kis_change_file_layer_command.h"
 #include "kis_canvas_resource_provider.h"
 #include "kis_selection_manager.h"
 #include "kis_statusbar.h"
@@ -192,20 +193,16 @@ void KisLayerManager::setup(KisActionManager* actionManager)
 
 void KisLayerManager::updateGUI()
 {
-    KisImageWSP image = m_view->image();
+    KisImageSP image = m_view->image();
+    KisLayerSP layer = activeLayer();
 
-    KisLayerSP layer;
-    qint32 nlayers = 0;
+    const bool isGroupLayer = layer && layer->inherits("KisGroupLayer");
 
-    if (image) {
-        layer = activeLayer();
-        nlayers = image->nlayers();
-    }
-
-    // XXX these should be named layer instead of image
-    m_imageFlatten->setEnabled(nlayers > 1);
-    m_imageMergeLayer->setEnabled(nlayers > 1 && layer && layer->prevSibling());
-    m_flattenLayer->setEnabled(nlayers > 1 && layer && layer->firstChild());
+    m_imageMergeLayer->setText(
+        isGroupLayer ?
+            i18nc("@action:inmenu", "Merge Group") :
+            i18nc("@action:inmenu", "Merge with Layer Below"));
+    m_flattenLayer->setVisible(!isGroupLayer);
 
     if (m_view->statusBar())
         m_view->statusBar()->setProfile(image);
@@ -243,13 +240,14 @@ void KisLayerManager::layerProperties()
     if (!m_view->document()) return;
 
     KisLayerSP layer = activeLayer();
+    if (!layer) return;
+
     QList<KisNodeSP> selectedNodes = m_view->nodeManager()->selectedNodes();
     const bool multipleLayersSelected = selectedNodes.size() > 1;
 
-    if (!layer) return;
-
     KisAdjustmentLayerSP alayer = KisAdjustmentLayerSP(dynamic_cast<KisAdjustmentLayer*>(layer.data()));
     KisGeneratorLayerSP glayer = KisGeneratorLayerSP(dynamic_cast<KisGeneratorLayer*>(layer.data()));
+    KisFileLayerSP flayer = KisFileLayerSP(dynamic_cast<KisFileLayer*>(layer.data()));
 
     if (alayer && !multipleLayersSelected) {
 
@@ -332,6 +330,37 @@ void KisLayerManager::layerProperties()
             }
 
         }
+    }  else if (flayer && !multipleLayersSelected){
+        QString basePath = QFileInfo(m_view->document()->url().toLocalFile()).absolutePath();
+        QString fileNameOld = flayer->fileName();
+        KisFileLayer::ScalingMethod scalingMethodOld = flayer->scalingMethod();
+        KisDlgFileLayer dlg(basePath, flayer->name(), m_view->mainWindow());
+        dlg.setCaption(i18n("File Layer Properties"));
+        dlg.setFileName(fileNameOld);
+        dlg.setScalingMethod(scalingMethodOld);
+
+        if (dlg.exec() == QDialog::Accepted) {
+            const QString fileNameNew = dlg.fileName();
+            KisFileLayer::ScalingMethod scalingMethodNew = dlg.scaleToImageResolution();
+
+            if(fileNameNew.isEmpty()){
+                QMessageBox::critical(m_view->mainWindow(), i18nc("@title:window", "Krita"), i18n("No file name specified"));
+                return;
+            }
+            flayer->setName(dlg.layerName());
+
+            if (fileNameOld!= fileNameNew || scalingMethodOld != scalingMethodNew) {
+                KisChangeFileLayerCmd *cmd
+                        = new KisChangeFileLayerCmd(flayer,
+                                                basePath,
+                                                fileNameOld,
+                                                scalingMethodOld,
+                                                basePath,
+                                                fileNameNew,
+                                                scalingMethodNew);
+                m_view->undoAdapter()->addCommand(cmd);
+            }
+        }
     } else { // If layer == normal painting layer, vector layer, or group layer
         QList<KisNodeSP> selectedNodes = m_view->nodeManager()->selectedNodes();
 
@@ -351,7 +380,7 @@ void KisLayerManager::convertNodeToPaintLayer(KisNodeSP source)
     if (!image) return;
 
 
-    KisLayer *srcLayer = dynamic_cast<KisLayer*>(source.data());
+    KisLayer *srcLayer = qobject_cast<KisLayer*>(source.data());
     if (srcLayer && (srcLayer->inherits("KisGroupLayer") || srcLayer->layerStyle() || srcLayer->childCount() > 0)) {
         image->flattenLayer(srcLayer);
         return;
@@ -439,6 +468,86 @@ void KisLayerManager::convertGroupToAnimated()
     m_commandsAdapter->endMacro();
 }
 
+void KisLayerManager::convertLayerToFileLayer(KisNodeSP source)
+{
+    KisImageSP image = m_view->image();
+    if (!image) return;
+
+    QStringList listMimeFilter = KisImportExportManager::mimeFilter(KisImportExportManager::Export);
+
+    KoDialog dlg;
+    QWidget *page = new QWidget(&dlg);
+    dlg.setMainWidget(page);
+    QBoxLayout *layout = new QVBoxLayout(page);
+    dlg.setWindowTitle(i18n("Save layers to..."));
+    QLabel *lbl = new QLabel(i18n("Choose the location where the layer will be saved to. The new file layer will then reference this location."));
+    lbl->setWordWrap(true);
+    layout->addWidget(lbl);
+    KisFileNameRequester *urlRequester = new KisFileNameRequester(page);
+    urlRequester->setMode(KoFileDialog::SaveFile);
+    urlRequester->setMimeTypeFilters(listMimeFilter);
+    urlRequester->setFileName(m_view->document()->url().toLocalFile());
+    if (m_view->document()->url().isLocalFile()) {
+        QFileInfo location = QFileInfo(m_view->document()->url().toLocalFile()).baseName();
+        location.setFile(location.dir(), location.baseName()+"_"+ source->name()+".kra");
+        urlRequester->setFileName(location.absoluteFilePath());
+    } else {
+        const QFileInfo location = QFileInfo(QStandardPaths::writableLocation(QStandardPaths::HomeLocation));
+        const QString proposedFileName = QDir(location.absoluteFilePath()).absoluteFilePath(source->name() + ".kra");
+        urlRequester->setFileName(proposedFileName);
+    }
+
+    layout->addWidget(urlRequester);
+    if (!dlg.exec()) return;
+
+    QString path = urlRequester->fileName();
+
+    if (path.isEmpty()) return;
+
+    QFileInfo f(path);
+
+    QString mimeType= KisMimeDatabase::mimeTypeForFile(f.fileName());
+    if (mimeType.isEmpty()) {
+        mimeType = "image/png";
+    }
+    QScopedPointer<KisDocument> doc(KisPart::instance()->createDocument());
+
+    QRect bounds = source->exactBounds();
+
+    KisImageSP dst = new KisImage(doc->createUndoStore(),
+                                  image->width(),
+                                  image->height(),
+                                  image->projection()->compositionSourceColorSpace(),
+                                  source->name());
+    dst->setResolution(image->xRes(), image->yRes());
+    doc->setFileBatchMode(false);
+    doc->setCurrentImage(dst);
+    KisNodeSP node = source->clone();
+    dst->addNode(node);
+    dst->initialRefreshGraph();
+    dst->cropImage(bounds);
+    dst->waitForDone();
+
+    bool r = doc->exportDocumentSync(QUrl::fromLocalFile(path), mimeType.toLatin1());
+    if (!r) {
+        qDebug()<< "Path:"<<path;
+        qWarning() << doc->errorMessage();
+    } else {
+        QString basePath = QFileInfo(m_view->document()->url().toLocalFile()).absolutePath();
+        QString relativePath = QDir(basePath).relativeFilePath(path);
+        KisFileLayer *fileLayer = new KisFileLayer(image, basePath, relativePath, KisFileLayer::None, source->name(), OPACITY_OPAQUE_U8);
+        fileLayer->setX(bounds.x());
+        fileLayer->setY(bounds.y());
+        KisNodeSP dstParent = source->parent();
+        KisNodeSP dstAboveThis = source->prevSibling();
+        m_commandsAdapter->beginMacro(kundo2_i18n("Convert to a file layer"));
+        m_commandsAdapter->removeNode(source);
+        m_commandsAdapter->addNode(fileLayer, dstParent, dstAboveThis);
+        m_commandsAdapter->endMacro();
+    }
+    doc->closeUrl(false);
+}
+
 void KisLayerManager::adjustLayerPosition(KisNodeSP node, KisNodeSP activeNode, KisNodeSP &parent, KisNodeSP &above)
 {
     Q_ASSERT(activeNode);
@@ -563,35 +672,11 @@ void KisLayerManager::addGeneratorLayer(KisNodeSP activeNode)
 
 }
 
-void KisLayerManager::rotateLayer(double radians)
-{
-    if (!m_view->image()) return;
-
-    KisLayerSP layer = activeLayer();
-    if (!layer) return;
-
-    if (!m_view->blockUntillOperationsFinished(m_view->image())) return;
-
-    m_view->image()->rotateNode(layer, radians);
-}
-
-void KisLayerManager::shearLayer(double angleX, double angleY)
-{
-    if (!m_view->image()) return;
-
-    KisLayerSP layer = activeLayer();
-    if (!layer) return;
-
-    if (!m_view->blockUntillOperationsFinished(m_view->image())) return;
-
-    m_view->image()->shearNode(layer, angleX, angleY);
-}
-
 void KisLayerManager::flattenImage()
 {
     KisImageSP image = m_view->image();
 
-    if (!m_view->blockUntillOperationsFinished(image)) return;
+    if (!m_view->blockUntilOperationsFinished(image)) return;
 
     if (image) {
         bool doIt = true;
@@ -638,6 +723,21 @@ bool tryMergeSelectionMasks(KisNodeSP currentNode, KisImageSP image)
     return result;
 }
 
+bool tryFlattenGroupLayer(KisNodeSP currentNode, KisImageSP image)
+{
+    bool result = false;
+
+    if (currentNode->inherits("KisGroupLayer")) {
+        KisGroupLayer *layer = qobject_cast<KisGroupLayer*>(currentNode.data());
+        KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(layer, false);
+
+        image->flattenLayer(layer);
+        result = true;
+    }
+
+    return result;
+}
+
 void KisLayerManager::mergeLayer()
 {
     KisImageSP image = m_view->image();
@@ -646,19 +746,30 @@ void KisLayerManager::mergeLayer()
     KisLayerSP layer = activeLayer();
     if (!layer) return;
 
-    if (!m_view->blockUntillOperationsFinished(image)) return;
+    if (!m_view->blockUntilOperationsFinished(image)) return;
 
     QList<KisNodeSP> selectedNodes = m_view->nodeManager()->selectedNodes();
-    if (selectedNodes.size() > 1) {
+      if (selectedNodes.size() > 1) {
         image->mergeMultipleLayers(selectedNodes, m_view->activeNode());
+    }
 
-    } else if (!tryMergeSelectionMasks(m_view->activeNode(), image)) {
+      else if (tryMergeSelectionMasks(m_view->activeNode(), image)) {
+        // already done!
+    } else if (tryFlattenGroupLayer(m_view->activeNode(), image)) {
+        // already done!
+    } else {
 
         if (!layer->prevSibling()) return;
-        KisLayer *prevLayer = dynamic_cast<KisLayer*>(layer->prevSibling().data());
+        KisLayer *prevLayer = qobject_cast<KisLayer*>(layer->prevSibling().data());
         if (!prevLayer) return;
+        if (prevLayer->userLocked()) {
+            m_view->showFloatingMessage(
+                i18nc("floating message in layer manager",
+                      "Layer is locked "),
+                QIcon(), 2000, KisFloatingMessage::Low);
+        }
 
-        if (layer->metaData()->isEmpty() && prevLayer->metaData()->isEmpty()) {
+        else if (layer->metaData()->isEmpty() && prevLayer->metaData()->isEmpty()) {
             image->mergeDown(layer, KisMetaData::MergeStrategyRegistry::instance()->get("Drop"));
         }
         else {
@@ -679,9 +790,9 @@ void KisLayerManager::flattenLayer()
     KisLayerSP layer = activeLayer();
     if (!layer) return;
 
-    if (!m_view->blockUntillOperationsFinished(image)) return;
+    if (!m_view->blockUntilOperationsFinished(image)) return;
 
-    image->flattenLayer(layer);
+    convertNodeToPaintLayer(layer);
     m_view->updateGUI();
 }
 
@@ -693,7 +804,7 @@ void KisLayerManager::rasterizeLayer()
     KisLayerSP layer = activeLayer();
     if (!layer) return;
 
-    if (!m_view->blockUntillOperationsFinished(image)) return;
+    if (!m_view->blockUntilOperationsFinished(image)) return;
 
     KisPaintLayerSP paintLayer = new KisPaintLayer(image, layer->name(), layer->opacity());
     KisPainter gc(paintLayer->paintDevice());
@@ -754,7 +865,7 @@ void KisLayerManager::saveGroupLayers()
 
     QFileInfo f(path);
 
-    QString mimeType= KisMimeDatabase::mimeTypeForFile(f.fileName());
+    QString mimeType= KisMimeDatabase::mimeTypeForFile(f.fileName(), false);
     if (mimeType.isEmpty()) {
         mimeType = "image/png";
     }
@@ -817,7 +928,7 @@ void KisLayerManager::layerStyle()
     KisLayerSP layer = activeLayer();
     if (!layer) return;
 
-    if (!m_view->blockUntillOperationsFinished(image)) return;
+    if (!m_view->blockUntilOperationsFinished(image)) return;
 
     KisPSDLayerStyleSP oldStyle;
     if (layer->layerStyle()) {
