@@ -33,7 +33,7 @@
 #include "kis_node_insertion_adapter.h"
 #include "kis_dummies_facade_base.h"
 #include "kis_node_dummies_graph.h"
-#include "kis_image_barrier_locker.h"
+#include "KisImportExportManager.h"
 #include "KisImageBarrierLockerWithFeedback.h"
 
 #include <KoProperties.h>
@@ -198,7 +198,7 @@ void KisMimeData::initializeExternalNode(KisNodeSP *node,
                                          KisShapeController *shapeController)
 {
     // layers store a link to the image, so update it
-    KisLayer *layer = dynamic_cast<KisLayer*>(node->data());
+    KisLayer *layer = qobject_cast<KisLayer*>(node->data());
     if (layer) {
         layer->setImage(image);
     }
@@ -285,10 +285,12 @@ QList<KisNodeSP> KisMimeData::loadNodes(const QMimeData *data,
     QList<KisNodeSP> nodes;
 
     if (data->hasFormat("application/x-krita-node")) {
-        QByteArray ba = data->data("application/x-krita-node");
-
         KisDocument *tempDoc = KisPart::instance()->createDocument();
-        bool result = tempDoc->loadNativeFormatFromByteArray(ba);
+        QByteArray ba = data->data("application/x-krita-node");
+        QBuffer buf(&ba);
+        KisImportExportFilter *filter = tempDoc->importExportManager()->filterForMimeType(tempDoc->nativeFormatMimeType(), KisImportExportManager::Import);
+        filter->setBatchMode(true);
+        bool result = (filter->convert(tempDoc, &buf) == KisImportExportFilter::OK);
 
         if (result) {
             KisImageWSP tempImage = tempDoc->image();
@@ -298,15 +300,15 @@ QList<KisNodeSP> KisMimeData::loadNodes(const QMimeData *data,
                 nodes << node;
             }
         }
+        delete filter;
         delete tempDoc;
     }
 
     if (nodes.isEmpty() && data->hasFormat("application/x-krita-node-url")) {
         QByteArray ba = data->data("application/x-krita-node-url");
-        QString localFile = QUrl::fromEncoded(ba).toLocalFile();
-
         KisDocument *tempDoc = KisPart::instance()->createDocument();
-        bool result = tempDoc->loadNativeFormat(localFile);
+        Q_ASSERT(QUrl::fromEncoded(ba).isLocalFile());
+        bool result = tempDoc->openUrl(QUrl::fromEncoded(ba));
 
         if (result) {
             KisImageWSP tempImage = tempDoc->image();
@@ -317,8 +319,7 @@ QList<KisNodeSP> KisMimeData::loadNodes(const QMimeData *data,
             }
         }
         delete tempDoc;
-
-        QFile::remove(localFile);
+        QFile::remove(QUrl::fromEncoded(ba).toLocalFile());
     }
 
     if (nodes.isEmpty() && data->hasImage()) {
@@ -326,7 +327,10 @@ QList<KisNodeSP> KisMimeData::loadNodes(const QMimeData *data,
 
         KisPaintDeviceSP device = new KisPaintDevice(KoColorSpaceRegistry::instance()->rgb8());
         device->convertFromQImage(qimage, 0);
-        nodes << new KisPaintLayer(image.data(), image->nextLayerName(), OPACITY_OPAQUE_U8, device);
+
+		if (image) {
+			nodes << new KisPaintLayer(image.data(), image->nextLayerName(), OPACITY_OPAQUE_U8, device);
+		}
 
         alwaysRecenter = true;
     }
@@ -401,13 +405,11 @@ bool correctNewNodeLocation(KisNodeList nodes,
     return result;
 }
 
-bool KisMimeData::insertMimeLayers(const QMimeData *data,
-                                   KisImageSP image,
-                                   KisShapeController *shapeController,
-                                   KisNodeDummy *parentDummy,
-                                   KisNodeDummy *aboveThisDummy,
-                                   bool copyNode,
-                                   KisNodeInsertionAdapter *nodeInsertionAdapter)
+KisNodeList KisMimeData::loadNodesFast(
+    const QMimeData *data,
+    KisImageSP image,
+    KisShapeController *shapeController,
+    bool &copyNode)
 {
     QList<KisNodeSP> nodes =
         KisMimeData::tryLoadInternalNodes(data,
@@ -428,6 +430,19 @@ bool KisMimeData::insertMimeLayers(const QMimeData *data,
         copyNode = true;
     }
 
+    return nodes;
+}
+
+bool KisMimeData::insertMimeLayers(const QMimeData *data,
+                                   KisImageSP image,
+                                   KisShapeController *shapeController,
+                                   KisNodeDummy *parentDummy,
+                                   KisNodeDummy *aboveThisDummy,
+                                   bool copyNode,
+                                   KisNodeInsertionAdapter *nodeInsertionAdapter)
+{
+    QList<KisNodeSP> nodes = loadNodesFast(data, image, shapeController, copyNode /* IN-OUT */);
+
     if (nodes.isEmpty()) return false;
 
     bool result = true;
