@@ -32,6 +32,7 @@
 #include <QWidgetAction>
 #include <QApplication>
 #include <QMenu>
+#include <QTime>
 
 #include <kis_debug.h>
 
@@ -116,6 +117,9 @@ KisPaintopBox::KisPaintopBox(KisViewManager *view, QWidget *parent, const char *
 
     setWindowTitle(i18n("Painter's Toolchest"));
 
+    m_favoriteResourceManager = new KisFavoriteResourceManager(this);
+
+
     KConfigGroup grp =  KSharedConfig::openConfig()->group("krita").group("Toolbar BrushesAndStuff");
     int iconsize = grp.readEntry("IconSize", 32);
 
@@ -170,45 +174,31 @@ KisPaintopBox::KisPaintopBox(KisViewManager *view, QWidget *parent, const char *
     // mirror tool options for the X Mirror
     QMenu *toolbarMenuXMirror = new QMenu();
 
-    KisAction* hideCanvasDecorationsX = m_viewManager->actionManager()->createAction("mirrorX-hideDecorations");
-    hideCanvasDecorationsX->setCheckable(true);
-    hideCanvasDecorationsX->setText(i18n("Hide Mirror Line"));
+    hideCanvasDecorationsX = m_viewManager->actionManager()->createAction("mirrorX-hideDecorations");
     toolbarMenuXMirror->addAction(hideCanvasDecorationsX);
 
-    KisAction* lockActionX = m_viewManager->actionManager()->createAction("mirrorX-lock");
-    lockActionX->setText(i18n("Lock"));
-    lockActionX->setCheckable(true);
+    lockActionX = m_viewManager->actionManager()->createAction("mirrorX-lock");
     toolbarMenuXMirror->addAction(lockActionX);
 
-    KisAction* moveToCenterActionX = m_viewManager->actionManager()->createAction("mirrorX-moveToCenter");
-    moveToCenterActionX->setCheckable(false);
-    moveToCenterActionX->setText(i18n("Move to Canvas Center"));
+    moveToCenterActionX = m_viewManager->actionManager()->createAction("mirrorX-moveToCenter");
     toolbarMenuXMirror->addAction(moveToCenterActionX);
 
 
-
     // mirror tool options for the Y Mirror
     QMenu *toolbarMenuYMirror = new QMenu();
 
-    KisAction* hideCanvasDecorationsY = m_viewManager->actionManager()->createAction("mirrorY-hideDecorations");
-    hideCanvasDecorationsY->setCheckable(true);
-    hideCanvasDecorationsY->setText(i18n("Hide Mirror Line"));
+    hideCanvasDecorationsY = m_viewManager->actionManager()->createAction("mirrorY-hideDecorations");
     toolbarMenuYMirror->addAction(hideCanvasDecorationsY);
 
 
-    KisAction* lockActionY = m_viewManager->actionManager()->createAction("mirrorY-lock");
-    lockActionY->setText(i18n("Lock"));
-    lockActionY->setCheckable(true);
+    lockActionY = m_viewManager->actionManager()->createAction("mirrorY-lock");
     toolbarMenuYMirror->addAction(lockActionY);
 
-    KisAction* moveToCenterActionY = m_viewManager->actionManager()->createAction("mirrorY-moveToCenter");
-    moveToCenterActionY->setCheckable(false);
-    moveToCenterActionY->setText(i18n("Move to Canvas Center"));
+    moveToCenterActionY = m_viewManager->actionManager()->createAction("mirrorY-moveToCenter");
     toolbarMenuYMirror->addAction(moveToCenterActionY);
 
 
 
-
     // create horizontal and vertical mirror buttons
 
     m_hMirrorButton = new KisHighlightedToolButton(this);
@@ -283,7 +273,7 @@ KisPaintopBox::KisPaintopBox(KisViewManager *view, QWidget *parent, const char *
         slFlow->setFixedHeight(iconsize);
         slFlow->setBlockUpdateSignalOnDrag(true);
 
-        slSize->setRange(0, 1000, 2);
+        slSize->setRange(0, cfg.readEntry("maximumBrushSize", 1000), 2);
         slSize->setValue(100);
 
         slSize->setSingleStep(1);
@@ -428,14 +418,20 @@ KisPaintopBox::KisPaintopBox(KisViewManager *view, QWidget *parent, const char *
         m_toolOptionsPopup->switchDetached(false);
     }
 
-    m_presetsPopup = new KisPaintOpPresetsPopup(m_resourceProvider);
+
+    m_savePresetWidget = new KisPresetSaveWidget(this);
+
+    m_presetsPopup = new KisPaintOpPresetsPopup(m_resourceProvider, m_favoriteResourceManager, m_savePresetWidget);
     m_brushEditorPopupButton->setPopupWidget(m_presetsPopup);
-    m_presetsPopup->switchDetached(false);
+    m_presetsPopup->parentWidget()->setWindowTitle(i18n("Brush Editor"));
+
+
     connect(m_presetsPopup, SIGNAL(brushEditorShown()), SLOT(slotUpdateOptionsWidgetPopup()));
     connect(m_viewManager->mainWindow(), SIGNAL(themeChanged()), m_presetsPopup, SLOT(updateThemedIcons()));
 
     m_presetsChooserPopup = new KisPaintOpPresetsChooserPopup();
-    m_presetsChooserPopup->setFixedSize(500, 600);
+    m_presetsChooserPopup->setMinimumHeight(550);
+    m_presetsChooserPopup->setMinimumWidth(450);
     m_presetSelectorPopupButton->setPopupWidget(m_presetsChooserPopup);
 
     m_currCompositeOpID = KoCompositeOpRegistry::instance().getDefaultCompositeOp().id();
@@ -451,7 +447,6 @@ KisPaintopBox::KisPaintopBox(KisViewManager *view, QWidget *parent, const char *
     m_presetsPopup->setPaintOpList(factoryList);
 
     connect(m_presetsPopup       , SIGNAL(paintopActivated(QString))          , SLOT(slotSetPaintop(QString)));
-    connect(m_presetsPopup       , SIGNAL(savePresetClicked())                , SLOT(slotSaveActivePreset()));
     connect(m_presetsPopup       , SIGNAL(defaultPresetClicked())             , SLOT(slotSetupDefaultPreset()));
     connect(m_presetsPopup       , SIGNAL(signalResourceSelected(KoResource*)), SLOT(resourceSelected(KoResource*)));
     connect(m_presetsPopup       , SIGNAL(reloadPresetClicked())              , SLOT(slotReloadPreset()));
@@ -459,6 +454,8 @@ KisPaintopBox::KisPaintopBox(KisViewManager *view, QWidget *parent, const char *
     connect(m_presetsPopup       , SIGNAL(eraserBrushSizeToggled(bool))       , SLOT(slotEraserBrushSizeToggled(bool)));
     connect(m_presetsPopup       , SIGNAL(eraserBrushOpacityToggled(bool))       , SLOT(slotEraserBrushOpacityToggled(bool)));
 
+    connect(m_presetsPopup, SIGNAL(createPresetFromScratch(QString)), this, SLOT(slotCreatePresetFromScratch(QString)));
+
     connect(m_presetsChooserPopup, SIGNAL(resourceSelected(KoResource*))      , SLOT(resourceSelected(KoResource*)));
     connect(m_presetsChooserPopup, SIGNAL(resourceClicked(KoResource*))      , SLOT(resourceSelected(KoResource*)));
 
@@ -487,7 +484,7 @@ KisPaintopBox::KisPaintopBox(KisViewManager *view, QWidget *parent, const char *
     //Needed to connect canvas to favorite resource manager
     connect(m_viewManager->resourceProvider(), SIGNAL(sigFGColorChanged(KoColor)), SLOT(slotUnsetEraseMode()));
 
-    m_favoriteResourceManager = new KisFavoriteResourceManager(this);
+
     connect(m_resourceProvider, SIGNAL(sigFGColorUsed(KoColor)), m_favoriteResourceManager, SLOT(slotAddRecentColor(KoColor)));
 
     connect(m_resourceProvider, SIGNAL(sigFGColorChanged(KoColor)), m_favoriteResourceManager, SLOT(slotChangeFGColorSelector(KoColor)));
@@ -519,7 +516,7 @@ KisPaintopBox::~KisPaintopBox()
             cfg.writeEntry(QString("LastPreset_%1").arg(iter.key().uniqueID) , iter.value().preset->name());
         }
     }
-    // Do not delete the widget, since it it is global to the application, not owned by the view
+    // Do not delete the widget, since it is global to the application, not owned by the view
     m_presetsPopup->setPaintOpSettingsWidget(0);
     qDeleteAll(m_paintopOptionWidgets);
     delete m_favoriteResourceManager;
@@ -531,6 +528,7 @@ KisPaintopBox::~KisPaintopBox()
 
 void KisPaintopBox::restoreResource(KoResource* resource)
 {
+
     KisPaintOpPreset* preset = dynamic_cast<KisPaintOpPreset*>(resource);
     //qDebug() << "restoreResource" << resource << preset;
     if (preset) {
@@ -550,6 +548,8 @@ void KisPaintopBox::newOptionWidgets(const QList<QPointer<QWidget> > &optionWidg
 
 void KisPaintopBox::resourceSelected(KoResource* resource)
 {
+    m_presetsPopup->setCreatingBrushFromScratch(false); // show normal UI elements when we are not creating
+
     KisPaintOpPreset* preset = dynamic_cast<KisPaintOpPreset*>(resource);
     if (preset && preset != m_resourceProvider->currentPreset()) {
         if (!preset->settings()->isLoadable())
@@ -567,6 +567,8 @@ void KisPaintopBox::resourceSelected(KoResource* resource)
         m_presetsPopup->setPresetImage(preset->image());
         m_presetsPopup->resourceSelected(resource);
     }
+
+
 }
 
 void KisPaintopBox::setCurrentPaintop(const KoID& paintop)
@@ -630,7 +632,8 @@ void KisPaintopBox::setCurrentPaintop(KisPaintOpPresetSP preset)
     QString pixFilename = KoResourcePaths::findResource("kis_images", paintOp->pixmap());
 
     m_brushEditorPopupButton->setIcon(QIcon(pixFilename));
-    m_presetsPopup->setCurrentPaintOp(paintop.id());
+    m_presetsPopup->setCurrentPaintOpId(paintop.id());
+
 
     ////qDebug() << "\tsetting the new preset for" << m_currTabletToolID.uniqueID << "to" << preset->name();
     m_paintOpPresetMap[m_resourceProvider->currentPreset()->paintOp()] = preset;
@@ -638,7 +641,7 @@ void KisPaintopBox::setCurrentPaintop(KisPaintOpPresetSP preset)
     m_tabletToolMap[m_currTabletToolID].paintOpID = preset->paintOp();
 
 
-    if (m_presetsPopup->currentPaintOp() != paintop.id()) {
+    if (m_presetsPopup->currentPaintOpId() != paintop.id()) {
         // Must change the paintop as the current one is not supported
         // by the new colorspace.
         dbgKrita << "current paintop " << paintop.name() << " was not set, not supported by colorspace";
@@ -655,6 +658,10 @@ void KisPaintopBox::slotUpdateOptionsWidgetPopup()
 
     m_presetsPopup->resourceSelected(preset.data());
     m_presetsPopup->updateViewSettings();
+
+    // the m_viewManager->image() is set earlier, but the reference will be missing when the stamp button is pressed
+    // need to later do some research on how and when we should be using weak shared pointers (WSP) that creates this situation
+    m_optionWidget->setImage(m_viewManager->image());
 }
 
 KisPaintOpPresetSP KisPaintopBox::defaultPreset(const KoID& paintOp)
@@ -794,6 +801,15 @@ void KisPaintopBox::slotInputDeviceChanged(const KoInputDevice& inputDevice)
     }
 }
 
+void KisPaintopBox::slotCreatePresetFromScratch(QString paintop)
+{
+    slotSetPaintop(paintop);  // change the paintop settings area and update the UI
+    m_presetsPopup->setCreatingBrushFromScratch(true); // disable UI elements while creating from scratch
+
+    KisPaintOpPresetSP preset = m_resourceProvider->currentPreset();
+    m_presetsPopup->resourceSelected(preset.data());  // this helps update the UI on the brush editor
+}
+
 void KisPaintopBox::slotCanvasResourceChanged(int key, const QVariant &value)
 {
     if (m_viewManager) {
@@ -809,6 +825,7 @@ void KisPaintopBox::slotCanvasResourceChanged(int key, const QVariant &value)
          * Update currently selected preset in both the popup widgets
          */
         m_presetsChooserPopup->canvasResourceChanged(preset);
+
         m_presetsPopup->currentPresetChanged(preset);
 
         if (key == KisCanvasResourceProvider::CurrentCompositeOp) {
@@ -841,49 +858,6 @@ void KisPaintopBox::slotCanvasResourceChanged(int key, const QVariant &value)
     }
 }
 
-void KisPaintopBox::slotSaveActivePreset()
-{
-    KisPaintOpPresetSP curPreset = m_resourceProvider->currentPreset();
-
-    if (!curPreset)
-        return;
-
-    m_favoriteResourceManager->setBlockUpdates(true);
-
-    KisPaintOpPresetSP newPreset = curPreset->clone();
-    KisPaintOpPresetResourceServer * rServer = KisResourceServerProvider::instance()->paintOpPresetServer();
-    QString saveLocation = rServer->saveLocation();
-    QString presetName = m_presetsPopup->getPresetName();
-    QString presetFilename = saveLocation + presetName + newPreset->defaultFileExtension();
-
-    QStringList tags;
-    KisPaintOpPresetSP resource = rServer->resourceByName(presetName);
-    if (resource) {
-
-        tags = rServer->assignedTagsList(resource.data());
-        rServer->removeResourceAndBlacklist(resource);
-
-    }
-
-    newPreset->setImage(m_presetsPopup->cutOutOverlay());
-    newPreset->setFilename(presetFilename);
-    newPreset->setName(presetName);
-    newPreset->setPresetDirty(false);
-
-    rServer->addResource(newPreset);
-    Q_FOREACH (const QString & tag, tags) {
-        rServer->addTag(newPreset.data(), tag);
-    }
-
-    // HACK ALERT! the server does not notify the observers
-    // automatically, so we need to call theupdate manually!
-    rServer->tagCategoryMembersChanged();
-
-    restoreResource(newPreset.data());
-
-    m_favoriteResourceManager->setBlockUpdates(false);
-}
-
 void KisPaintopBox::slotUpdatePreset()
 {
     if (!m_resourceProvider->currentPreset()) return;
@@ -918,6 +892,10 @@ void KisPaintopBox::slotSetupDefaultPreset()
     KisPaintOpPresetSP preset = defaultPreset(m_resourceProvider->currentPreset()->paintOp());
     preset->setOptionsWidget(m_optionWidget);
     m_resourceProvider->setPaintOpPreset(preset);
+
+    // tell the brush editor that the resource has changed
+    // so it can update everything
+    m_presetsPopup->resourceSelected(preset.data());
 }
 
 void KisPaintopBox::slotNodeChanged(const KisNodeSP node)
@@ -1105,36 +1083,47 @@ void KisPaintopBox::slotPreviousFavoritePreset()
 {
     if (!m_favoriteResourceManager) return;
 
-    int i = 0;
-    Q_FOREACH (KisPaintOpPresetSP preset, m_favoriteResourceManager->favoritePresetList()) {
-        if (m_resourceProvider->currentPreset() && m_resourceProvider->currentPreset()->name() == preset->name()) {
+    QVector<KisPaintOpPresetSP> presets = m_favoriteResourceManager->favoritePresetList();
+    for (int i=0; i < presets.size(); ++i) {
+        if (m_resourceProvider->currentPreset() &&
+                m_resourceProvider->currentPreset()->name() == presets[i]->name()) {
             if (i > 0) {
                 m_favoriteResourceManager->slotChangeActivePaintop(i - 1);
             } else {
                 m_favoriteResourceManager->slotChangeActivePaintop(m_favoriteResourceManager->numFavoritePresets() - 1);
             }
+            //floating message should have least 2 lines, otherwise
+            //preset thumbnail will be too small to distinguish
+            //(because size of image on floating message depends on amount of lines in msg)
+            m_viewManager->showFloatingMessage(
+                i18n("%1\nselected",
+                        m_resourceProvider->currentPreset()->name()),
+                QIcon(QPixmap::fromImage(m_resourceProvider->currentPreset()->image())));
+
             return;
         }
-        i++;
     }
-
 }
 
 void KisPaintopBox::slotNextFavoritePreset()
 {
     if (!m_favoriteResourceManager) return;
 
-    int i = 0;
-    Q_FOREACH (KisPaintOpPresetSP preset, m_favoriteResourceManager->favoritePresetList()) {
-        if (m_resourceProvider->currentPreset()->name() == preset->name()) {
+    QVector<KisPaintOpPresetSP> presets = m_favoriteResourceManager->favoritePresetList();
+    for(int i = 0; i < presets.size(); ++i) {
+        if (m_resourceProvider->currentPreset()->name() == presets[i]->name()) {
             if (i < m_favoriteResourceManager->numFavoritePresets() - 1) {
                 m_favoriteResourceManager->slotChangeActivePaintop(i + 1);
             } else {
                 m_favoriteResourceManager->slotChangeActivePaintop(0);
             }
+            m_viewManager->showFloatingMessage(
+                i18n("%1\nselected",
+                        m_resourceProvider->currentPreset()->name()),
+                QIcon(QPixmap::fromImage(m_resourceProvider->currentPreset()->image())));
+
             return;
         }
-        i++;
     }
 }
 
@@ -1143,6 +1132,10 @@ void KisPaintopBox::slotSwitchToPreviousPreset()
     if (m_resourceProvider->previousPreset()) {
         //qDebug() << "slotSwitchToPreviousPreset();" << m_resourceProvider->previousPreset();
         setCurrentPaintop(m_resourceProvider->previousPreset());
+        m_viewManager->showFloatingMessage(
+                i18n("%1\nselected",
+                        m_resourceProvider->currentPreset()->name()),
+            QIcon(QPixmap::fromImage(m_resourceProvider->currentPreset()->image())));
     }
 }
 
