@@ -66,7 +66,7 @@ public:
     ~Private() {
         delete displayShader;
         delete checkerShader;
-        delete cursorShader;
+        delete solidColorShader;
         Sync::deleteSync(glSyncObject);
     }
 
@@ -77,7 +77,7 @@ public:
     KisOpenGLShaderLoader shaderLoader;
     KisShaderProgram *displayShader{0};
     KisShaderProgram *checkerShader{0};
-    KisShaderProgram *cursorShader{0};
+    KisShaderProgram *solidColorShader{0};
     bool displayShaderCompiledWithDisplayFilterSupport{false};
 
     GLfloat checkSizeScale;
@@ -106,6 +106,11 @@ public:
     QOpenGLFunctions_2_1 *glFn201;
 #endif
 
+    qreal pixelGridDrawingThreshold;
+    bool pixelGridEnabled;
+    QColor gridColor;
+    QColor cursorColor;
+
     int xToColWithWrapCompensation(int x, const QRect &imageRect) {
         int firstImageColumn = openGLImageTextures->xToCol(imageRect.left());
         int lastImageColumn = openGLImageTextures->xToCol(imageRect.right());
@@ -173,11 +178,6 @@ KisOpenGLCanvas2::~KisOpenGLCanvas2()
     delete d;
 }
 
-bool KisOpenGLCanvas2::needsFpsDebugging() const
-{
-    return KisOpenglCanvasDebugger::instance()->showFpsOnCanvas();
-}
-
 void KisOpenGLCanvas2::setDisplayFilter(QSharedPointer<KisDisplayFilter> displayFilter)
 {
     setDisplayFilterImpl(displayFilter, false);
@@ -231,9 +231,13 @@ void KisOpenGLCanvas2::initializeGL()
     KisOpenGL::initializeContext(context());
     initializeOpenGLFunctions();
 #ifndef Q_OS_OSX
-    d->glFn201 = context()->versionFunctions<QOpenGLFunctions_2_1>();
-    if (!d->glFn201) {
-        warnUI << "Cannot obtain QOpenGLFunctions_2_1, glLogicOp cannot be used";
+    if (!KisOpenGL::hasOpenGLES()) {
+        d->glFn201 = context()->versionFunctions<QOpenGLFunctions_2_1>();
+        if (!d->glFn201) {
+            warnUI << "Cannot obtain QOpenGLFunctions_2_1, glLogicOp cannot be used";
+        }
+    } else {
+        d->glFn201 = nullptr;
     }
 #endif
 
@@ -293,13 +297,13 @@ void KisOpenGLCanvas2::initializeShaders()
     KIS_SAFE_ASSERT_RECOVER_RETURN(!d->canvasInitialized);
 
     delete d->checkerShader;
-    delete d->cursorShader;
+    delete d->solidColorShader;
     d->checkerShader = 0;
-    d->cursorShader = 0;
+    d->solidColorShader = 0;
 
     try {
         d->checkerShader = d->shaderLoader.loadCheckerShader();
-        d->cursorShader = d->shaderLoader.loadCursorShader();
+        d->solidColorShader = d->shaderLoader.loadSolidColorShader();
     } catch (const ShaderLoaderException &e) {
         reportFailedShaderCompilation(e.what());
     }
@@ -332,10 +336,6 @@ void KisOpenGLCanvas2::reportFailedShaderCompilation(const QString &context)
 {
     KisConfig cfg;
 
-    if (cfg.useVerboseOpenGLDebugOutput()) {
-        dbgUI << "GL-log:" << context;
-    }
-
     qDebug() << "Shader Compilation Failure: " << context;
     QMessageBox::critical(this, i18nc("@title:window", "Krita"),
                           QString(i18n("Krita could not initialize the OpenGL canvas:\n\n%1\n\n Krita will disable OpenGL and close now.")).arg(context),
@@ -380,31 +380,40 @@ void KisOpenGLCanvas2::paintGL()
 
 void KisOpenGLCanvas2::paintToolOutline(const QPainterPath &path)
 {
-    d->cursorShader->bind();
+    if (!d->solidColorShader->bind()) {
+        return;
+    }
 
     // setup the mvp transformation
-    KisCoordinatesConverter *converter = coordinatesConverter();
-
     QMatrix4x4 projectionMatrix;
     projectionMatrix.setToIdentity();
     projectionMatrix.ortho(0, width(), height(), 0, NEAR_VAL, FAR_VAL);
 
     // Set view/projection matrices
-    QMatrix4x4 modelMatrix(converter->flakeToWidgetTransform());
+    QMatrix4x4 modelMatrix(coordinatesConverter()->flakeToWidgetTransform());
     modelMatrix.optimize();
     modelMatrix = projectionMatrix * modelMatrix;
-    d->cursorShader->setUniformValue(d->cursorShader->location(Uniform::ModelViewProjection), modelMatrix);
+    d->solidColorShader->setUniformValue(d->solidColorShader->location(Uniform::ModelViewProjection), modelMatrix);
 
-    glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
+    if (!KisOpenGL::hasOpenGLES()) {
+        glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
 
-    glEnable(GL_COLOR_LOGIC_OP);
+        glEnable(GL_COLOR_LOGIC_OP);
 #ifndef Q_OS_OSX
-    if (d->glFn201) {
-        d->glFn201->glLogicOp(GL_XOR);
-    }
+        if (d->glFn201) {
+            d->glFn201->glLogicOp(GL_XOR);
+        }
 #else
-    glLogicOp(GL_XOR);
+        glLogicOp(GL_XOR);
 #endif
+    } else {
+        glEnable(GL_BLEND);
+        glBlendFuncSeparate(GL_ONE_MINUS_DST_COLOR, GL_ZERO, GL_ONE, GL_ONE);
+    }
+
+    d->solidColorShader->setUniformValue(
+                d->solidColorShader->location(Uniform::FragmentColor),
+                QVector4D(d->cursorColor.redF(), d->cursorColor.greenF(), d->cursorColor.blueF(), 1.0f));
 
     // Paint the tool outline
     if (KisOpenGL::hasOpenGL3()) {
@@ -419,6 +428,7 @@ void KisOpenGLCanvas2::paintToolOutline(const QPainterPath &path)
 
         QVector<QVector3D> vertices;
         vertices.resize(polygon.count());
+
         for (int j = 0; j < polygon.count(); j++) {
             QPointF p = polygon.at(j);
             vertices[j].setX(p.x());
@@ -428,8 +438,8 @@ void KisOpenGLCanvas2::paintToolOutline(const QPainterPath &path)
             d->lineBuffer.allocate(vertices.constData(), 3 * vertices.size() * sizeof(float));
         }
         else {
-            d->cursorShader->enableAttributeArray(PROGRAM_VERTEX_ATTRIBUTE);
-            d->cursorShader->setAttributeArray(PROGRAM_VERTEX_ATTRIBUTE, vertices.constData());
+            d->solidColorShader->enableAttributeArray(PROGRAM_VERTEX_ATTRIBUTE);
+            d->solidColorShader->setAttributeArray(PROGRAM_VERTEX_ATTRIBUTE, vertices.constData());
         }
 
         glDrawArrays(GL_LINE_STRIP, 0, vertices.size());
@@ -440,9 +450,13 @@ void KisOpenGLCanvas2::paintToolOutline(const QPainterPath &path)
         d->outlineVAO.release();
     }
 
-    glDisable(GL_COLOR_LOGIC_OP);
+    if (!KisOpenGL::hasOpenGLES()) {
+        glDisable(GL_COLOR_LOGIC_OP);
+    } else {
+        glDisable(GL_BLEND);
+    }
 
-    d->cursorShader->release();
+    d->solidColorShader->release();
 }
 
 bool KisOpenGLCanvas2::isBusy() const
@@ -496,7 +510,6 @@ void KisOpenGLCanvas2::drawCheckers()
     //Setup the geometry for rendering
     if (KisOpenGL::hasOpenGL3()) {
         rectToVertices(d->vertices, modelRect);
-
         d->quadBuffers[0].bind();
         d->quadBuffers[0].write(0, d->vertices, 3 * 6 * sizeof(float));
 
@@ -525,6 +538,74 @@ void KisOpenGLCanvas2::drawCheckers()
     glBindBuffer(GL_ARRAY_BUFFER, 0);
 }
 
+void KisOpenGLCanvas2::drawGrid()
+{
+    if (!d->solidColorShader->bind()) {
+        return;
+    }
+
+    QMatrix4x4 projectionMatrix;
+    projectionMatrix.setToIdentity();
+    projectionMatrix.ortho(0, width(), height(), 0, NEAR_VAL, FAR_VAL);
+
+    // Set view/projection matrices
+    QMatrix4x4 modelMatrix(coordinatesConverter()->imageToWidgetTransform());
+    modelMatrix.optimize();
+    modelMatrix = projectionMatrix * modelMatrix;
+    d->solidColorShader->setUniformValue(d->solidColorShader->location(Uniform::ModelViewProjection), modelMatrix);
+
+    glEnable(GL_BLEND);
+    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+
+    d->solidColorShader->setUniformValue(
+                d->solidColorShader->location(Uniform::FragmentColor),
+                QVector4D(d->gridColor.redF(), d->gridColor.greenF(), d->gridColor.blueF(), 0.5f));
+
+    if (KisOpenGL::hasOpenGL3()) {
+        d->outlineVAO.bind();
+        d->lineBuffer.bind();
+    }
+
+    QRectF widgetRect(0,0, width(), height());
+    QRectF widgetRectInImagePixels = coordinatesConverter()->documentToImage(coordinatesConverter()->widgetToDocument(widgetRect));
+    QRect wr = widgetRectInImagePixels.toAlignedRect();
+
+    if (!d->wrapAroundMode) {
+        wr &= d->openGLImageTextures->storedImageBounds();
+    }
+
+    QPoint topLeftCorner = wr.topLeft();
+    QPoint bottomRightCorner = wr.bottomRight() + QPoint(1, 1);
+    QVector<QVector3D> grid;
+
+    for (int i = topLeftCorner.x(); i <= bottomRightCorner.x(); ++i) {
+        grid.append(QVector3D(i, topLeftCorner.y(), 0));
+        grid.append(QVector3D(i, bottomRightCorner.y(), 0));
+    }
+    for (int i = topLeftCorner.y(); i <= bottomRightCorner.y(); ++i) {
+        grid.append(QVector3D(topLeftCorner.x(), i, 0));
+        grid.append(QVector3D(bottomRightCorner.x(), i, 0));
+    }
+
+    if (KisOpenGL::hasOpenGL3()) {
+        d->lineBuffer.allocate(grid.constData(), 3 * grid.size() * sizeof(float));
+    }
+    else {
+        d->solidColorShader->enableAttributeArray(PROGRAM_VERTEX_ATTRIBUTE);
+        d->solidColorShader->setAttributeArray(PROGRAM_VERTEX_ATTRIBUTE, grid.constData());
+    }
+
+    glDrawArrays(GL_LINES, 0, grid.size());
+
+    if (KisOpenGL::hasOpenGL3()) {
+        d->lineBuffer.release();
+        d->outlineVAO.release();
+    }
+
+    d->solidColorShader->release();
+    glDisable(GL_BLEND);
+}
+
 void KisOpenGLCanvas2::drawImage()
 {
     if (!d->displayShader) {
@@ -543,7 +624,7 @@ void KisOpenGLCanvas2::drawImage()
     projectionMatrix.ortho(0, width(), height(), 0, NEAR_VAL, FAR_VAL);
 
     // Set view/projection matrices
-    QMatrix4x4 modelMatrix(coordinatesConverter()->imageToWidgetTransform());
+    QMatrix4x4 modelMatrix(converter->imageToWidgetTransform());
     modelMatrix.optimize();
     modelMatrix = projectionMatrix * modelMatrix;
     d->displayShader->setUniformValue(d->displayShader->location(Uniform::ModelViewProjection), modelMatrix);
@@ -622,7 +703,6 @@ void KisOpenGLCanvas2::drawImage()
             //Setup the geometry for rendering
             if (KisOpenGL::hasOpenGL3()) {
                 rectToVertices(d->vertices, modelRect);
-
                 d->quadBuffers[0].bind();
                 d->quadBuffers[0].write(0, d->vertices, 3 * 6 * sizeof(float));
 
@@ -691,6 +771,7 @@ void KisOpenGLCanvas2::drawImage()
     glBindTexture(GL_TEXTURE_2D, 0);
     d->displayShader->release();
     glBindBuffer(GL_ARRAY_BUFFER, 0);
+    glDisable(GL_BLEND);
 }
 
 void KisOpenGLCanvas2::slotConfigChanged()
@@ -703,6 +784,11 @@ void KisOpenGLCanvas2::slotConfigChanged()
     d->openGLImageTextures->updateConfig(cfg.useOpenGLTextureBuffer(), cfg.numMipmapLevels());
     d->filterMode = (KisOpenGL::FilterMode) cfg.openGLFilteringMode();
 
+    d->pixelGridDrawingThreshold = cfg.getPixelGridDrawingThreshold();
+    d->pixelGridEnabled = cfg.pixelGridEnabled();
+    d->gridColor = cfg.getPixelGridColor();
+    d->cursorColor = cfg.getCursorMainColor();
+
     notifyConfigChanged();
 }
 
@@ -736,8 +822,12 @@ void KisOpenGLCanvas2::renderCanvasGL()
     if (KisOpenGL::hasOpenGL3()) {
         d->quadVAO.bind();
     }
+
     drawCheckers();
     drawImage();
+    if ((coordinatesConverter()->effectiveZoom() > d->pixelGridDrawingThreshold - 0.00001) && d->pixelGridEnabled) {
+        drawGrid();
+    }
     if (KisOpenGL::hasOpenGL3()) {
         d->quadVAO.release();
     }
@@ -777,7 +867,7 @@ KisUpdateInfoSP KisOpenGLCanvas2::startUpdateCanvasProjection(const QRect & rc,
         d->openGLImageTextures->setProofingConfig(canvas()->proofingConfiguration());
         canvas()->setProofingConfigUpdated(false);
     }
-    return d->openGLImageTextures->updateCache(rc);
+    return d->openGLImageTextures->updateCache(rc, d->openGLImageTextures->image());
 }
 
 
