@@ -116,10 +116,9 @@ KisTool::~KisTool()
     delete d;
 }
 
-void KisTool::activate(ToolActivation toolActivation, const QSet<KoShape*> &shapes)
+void KisTool::activate(ToolActivation activation, const QSet<KoShape*> &shapes)
 {
-    Q_UNUSED(toolActivation);
-    Q_UNUSED(shapes);
+    KoToolBase::activate(activation, shapes);
 
     resetCursorStyle();
 
@@ -151,8 +150,9 @@ void KisTool::activate(ToolActivation toolActivation, const QSet<KoShape*> &shap
         d->currentGenerator = canvas()->resourceManager()->resource(KisCanvasResourceProvider::CurrentGeneratorConfiguration).value<KisFilterConfiguration*>();
     }
 
-    connect(actions().value("toggle_fg_bg"), SIGNAL(triggered()), SLOT(slotToggleFgBg()), Qt::UniqueConnection);
-    connect(actions().value("reset_fg_bg"), SIGNAL(triggered()), SLOT(slotResetFgBg()), Qt::UniqueConnection);
+    connect(action("toggle_fg_bg"), SIGNAL(triggered()), SLOT(slotToggleFgBg()), Qt::UniqueConnection);
+    connect(action("reset_fg_bg"), SIGNAL(triggered()), SLOT(slotResetFgBg()), Qt::UniqueConnection);
+
     connect(image(), SIGNAL(sigUndoDuringStrokeRequested()), SLOT(requestUndoDuringStroke()), Qt::UniqueConnection);
     connect(image(), SIGNAL(sigStrokeCancellationRequested()), SLOT(requestStrokeCancellation()), Qt::UniqueConnection);
     connect(image(), SIGNAL(sigStrokeEndRequested()), SLOT(requestStrokeEnd()), Qt::UniqueConnection);
@@ -168,8 +168,8 @@ void KisTool::deactivate()
     result &= disconnect(image().data(), SIGNAL(sigUndoDuringStrokeRequested()), this, 0);
     result &= disconnect(image().data(), SIGNAL(sigStrokeCancellationRequested()), this, 0);
     result &= disconnect(image().data(), SIGNAL(sigStrokeEndRequested()), this, 0);
-    result &= disconnect(actions().value("toggle_fg_bg"), 0, this, 0);
-    result &= disconnect(actions().value("reset_fg_bg"), 0, this, 0);
+    result &= disconnect(action("toggle_fg_bg"), 0, this, 0);
+    result &= disconnect(action("reset_fg_bg"), 0, this, 0);
 
     if (!result) {
         warnKrita << "WARNING: KisTool::deactivate() failed to disconnect"
@@ -178,26 +178,17 @@ void KisTool::deactivate()
 
     d->m_isActive = false;
     emit isActiveChanged();
-}
 
-void KisTool::requestUndoDuringStroke()
-{
-    /**
-     * Default implementation just cancells the stroke
-     */
-    requestStrokeCancellation();
-}
-
-void KisTool::requestStrokeCancellation()
-{
-}
-
-void KisTool::requestStrokeEnd()
-{
+    KoToolBase::deactivate();
 }
 
 void KisTool::canvasResourceChanged(int key, const QVariant & v)
 {
+    QString formattedBrushName;
+    if (key == KisCanvasResourceProvider::CurrentPaintOpPreset) {
+         formattedBrushName = v.value<KisPaintOpPresetSP>()->name().replace("_", " ");
+    }
+
     switch (key) {
     case(KoCanvasResourceManager::ForegroundColor):
         d->currentFgColor = v.value<KoColor>();
@@ -218,7 +209,7 @@ void KisTool::canvasResourceChanged(int key, const QVariant & v)
         d->currentGenerator = static_cast<KisFilterConfiguration*>(v.value<void *>());
         break;
     case(KisCanvasResourceProvider::CurrentPaintOpPreset):
-        emit statusTextChanged(v.value<KisPaintOpPresetSP>()->name());
+        emit statusTextChanged(formattedBrushName);
         break;
     case(KisCanvasResourceProvider::CurrentKritaNode):
         resetCursorStyle();
@@ -287,12 +278,12 @@ QPointF KisTool::convertToPixelCoordAndSnap(const QPointF& pt, const QPointF &of
     return image()->documentToPixel(pos);
 }
 
-QPoint KisTool::convertToIntPixelCoord(KoPointerEvent *e)
+QPoint KisTool::convertToImagePixelCoordFloored(KoPointerEvent *e)
 {
     if (!image())
         return e->point.toPoint();
 
-    return image()->documentToIntPixel(e->point);
+    return image()->documentToImagePixelFloored(e->point);
 }
 
 QPointF KisTool::viewToPixel(const QPointF &viewCoord) const
@@ -310,7 +301,7 @@ QRectF KisTool::convertToPt(const QRectF &rect)
     QRectF r;
     //We add 1 in the following to the extreme coords because a pixel always has size
     r.setCoords(int(rect.left()) / image()->xRes(), int(rect.top()) / image()->yRes(),
-                int(1 + rect.right()) / image()->xRes(), int(1 + rect.bottom()) / image()->yRes());
+                int(rect.right()) / image()->xRes(), int( rect.bottom()) / image()->yRes());
     return r;
 }
 
@@ -447,6 +438,11 @@ KisTool::ToolMode KisTool::mode() const {
     return d->m_mode;
 }
 
+void KisTool::setCursor(const QCursor &cursor)
+{
+    d->cursor = cursor;
+}
+
 KisTool::AlternateAction KisTool::actionToAlternateAction(ToolAction action) {
     KIS_ASSERT_RECOVER_RETURN_VALUE(action != Primary, Secondary);
     return (AlternateAction)action;
@@ -550,9 +546,9 @@ void KisTool::deleteSelection()
     KisResourcesSnapshotSP resources =
         new KisResourcesSnapshot(image(), currentNode(), this->canvas()->resourceManager());
 
-    KisCanvas2 * kiscanvas = static_cast<KisCanvas2*>(canvas());
-    KisViewManager* viewManager = kiscanvas->viewManager();
-    viewManager->blockUntillOperationsFinished(image());
+    if (!blockUntilOperationsFinished()) {
+        return;
+    }
 
     if (!KisToolUtils::clearImage(image(), resources->currentNode(), resources->activeSelection())) {
         KoToolBase::deleteSelection();
@@ -585,9 +581,11 @@ void KisTool::paintToolOutline(QPainter* painter, const QPainterPath &path)
         painter->endNativePainting();
     }
     else {
+        painter->save();
         painter->setCompositionMode(QPainter::RasterOp_SourceXorDestination);
         painter->setPen(QColor(128, 255, 128));
         painter->drawPath(path);
+        painter->restore();
     }
 }
 
@@ -610,6 +608,20 @@ bool KisTool::overrideCursorIfNotEditable()
     return false;
 }
 
+bool KisTool::blockUntilOperationsFinished()
+{
+    KisCanvas2 * kiscanvas = static_cast<KisCanvas2*>(canvas());
+    KisViewManager* viewManager = kiscanvas->viewManager();
+    return viewManager->blockUntilOperationsFinished(image());
+}
+
+void KisTool::blockUntilOperationsFinishedForced()
+{
+    KisCanvas2 * kiscanvas = static_cast<KisCanvas2*>(canvas());
+    KisViewManager* viewManager = kiscanvas->viewManager();
+    viewManager->blockUntilOperationsFinishedForced(image());
+}
+
 bool KisTool::isActive() const
 {
     return d->m_isActive;
@@ -640,14 +652,6 @@ void KisTool::slotResetFgBg()
     resourceManager->setForegroundColor(KoColor(Qt::black, KoColorSpaceRegistry::instance()->rgb8()));
 }
 
-
-void KisTool::setCurrentNodeLocked(bool locked)
-{
-    if (currentNode()) {
-        currentNode()->setSystemLocked(locked, false);
-    }
-}
-
 bool KisTool::nodeEditable()
 {
     KisNodeSP node = currentNode();
@@ -655,7 +659,20 @@ bool KisTool::nodeEditable()
         return false;
     }
 
-    bool nodeEditable = node->isEditable();
+    bool blockedNoIndirectPainting = false;
+
+    const bool presetUsesIndirectPainting =
+        !currentPaintOpPreset()->settings()->paintIncremental();
+
+    if (!presetUsesIndirectPainting) {
+        const KisIndirectPaintingSupport *indirectPaintingLayer =
+                dynamic_cast<const KisIndirectPaintingSupport*>(node.data());
+        if (indirectPaintingLayer) {
+            blockedNoIndirectPainting = !indirectPaintingLayer->supportsNonIndirectPainting();
+        }
+    }
+
+    bool nodeEditable = node->isEditable() && !blockedNoIndirectPainting;
 
     if (!nodeEditable) {
         KisCanvas2 * kiscanvas = static_cast<KisCanvas2*>(canvas());
@@ -666,6 +683,8 @@ bool KisTool::nodeEditable()
             message = i18n("Layer is locked.");
         } else if(!node->visible()) {
             message = i18n("Layer is invisible.");
+        } else if (blockedNoIndirectPainting) {
+            message = i18n("Layer can be painted in Wash Mode only.");
         } else {
             message = i18n("Group not editable.");
         }
