@@ -26,7 +26,8 @@
 #include <KoPointerEvent.h>
 #include <KoCanvasResourceManager.h>
 
-#include <kis_distance_information.h>
+#include "kis_algebra_2d.h"
+#include "kis_distance_information.h"
 #include "kis_painting_information_builder.h"
 #include "kis_recording_adapter.h"
 #include "kis_image.h"
@@ -39,11 +40,29 @@
 #include "KisStabilizerDelayedPaintHelper.h"
 #include "kis_config.h"
 
+#include "kis_random_source.h"
+#include "KisPerStrokeRandomSource.h"
+
+#include "strokes/freehand_stroke.h"
+#include "strokes/KisFreehandStrokeInfo.h"
 
 #include <math.h>
 
 //#define DEBUG_BEZIER_CURVES
 
+// Factor by which to scale the airbrush timer's interval, relative to the actual airbrushing rate.
+// Setting this less than 1 makes the timer-generated pseudo-events happen faster than the desired
+// airbrush rate, which can improve responsiveness.
+const qreal AIRBRUSH_INTERVAL_FACTOR = 0.5;
+
+// The amount of time, in milliseconds, to allow between updates of the spacing information. Only
+// used when spacing updates between dabs are enabled.
+const qreal SPACING_UPDATE_INTERVAL = 50.0;
+
+// The amount of time, in milliseconds, to allow between updates of the timing information. Only
+// used when airbrushing.
+const qreal TIMING_UPDATE_INTERVAL = 50.0;
+
 struct KisToolFreehandHelper::Private
 {
     KisPaintingInformationBuilder *infoBuilder;
@@ -60,7 +79,7 @@ struct KisToolFreehandHelper::Private
     QTime strokeTime;
     QTimer strokeTimeoutTimer;
 
-    QVector<PainterInfo*> painterInfos;
+    QVector<KisFreehandStrokeInfo*> strokeInfos;
     KisResourcesSnapshotSP resources;
     KisStrokeId strokeId;
 
@@ -69,12 +88,21 @@ struct KisToolFreehandHelper::Private
 
     KisSmoothingOptionsSP smoothingOptions;
 
+    // fake random sources for hovering outline *only*
+    KisRandomSourceSP fakeDabRandomSource;
+    KisPerStrokeRandomSourceSP fakeStrokeRandomSource;
+
+    // Timer used to generate paint updates periodically even without input events. This is only
+    // used for paintops that depend on timely updates even when the cursor is not moving, e.g. for
+    // airbrushing effects.
     QTimer airbrushingTimer;
 
     QList<KisPaintInformation> history;
     QList<qreal> distanceHistory;
 
-    KisPaintOpUtils::PositionHistory lastOutlinePos;
+    // Keeps track of past cursor positions. This is used to determine the drawing angle when
+    // drawing the brush outline or starting a stroke.
+    KisPaintOpUtils::PositionHistory lastCursorPos;
 
     // Stabilizer data
     QQueue<KisPaintInformation> stabilizerDeque;
@@ -82,13 +110,11 @@ struct KisToolFreehandHelper::Private
     KisStabilizedEventsSampler stabilizedSampler;
     KisStabilizerDelayedPaintHelper stabilizerDelayedPaintHelper;
 
+    QTimer asynchronousUpdatesThresholdTimer;
+
     int canvasRotation;
     bool canvasMirroredH;
 
-    KisPaintInformation
-    getStabilizedPaintInfo(const QQueue<KisPaintInformation> &queue,
-                           const KisPaintInformation &lastPaintInfo);
-
     qreal effectiveSmoothnessDistance() const;
 };
 
@@ -106,9 +132,13 @@ KisToolFreehandHelper::KisToolFreehandHelper(KisPaintingInformationBuilder *info
                 smoothingOptions ? smoothingOptions : new KisSmoothingOptions());
     m_d->canvasRotation = 0;
 
+    m_d->fakeDabRandomSource = new KisRandomSource();
+    m_d->fakeStrokeRandomSource = new KisPerStrokeRandomSource();
+
     m_d->strokeTimeoutTimer.setSingleShot(true);
     connect(&m_d->strokeTimeoutTimer, SIGNAL(timeout()), SLOT(finishStroke()));
     connect(&m_d->airbrushingTimer, SIGNAL(timeout()), SLOT(doAirbrushing()));
+    connect(&m_d->asynchronousUpdatesThresholdTimer, SIGNAL(timeout()), SLOT(doAsynchronousUpdate()));
     connect(&m_d->stabilizerPollTimer, SIGNAL(timeout()), SLOT(stabilizerPollAndPaint()));
 
     m_d->stabilizerDelayedPaintHelper.setPaintLineCallback(
@@ -143,11 +173,13 @@ QPainterPath KisToolFreehandHelper::paintOpOutline(const QPointF &savedCursorPos
 {
     KisPaintOpSettingsSP settings = globalSettings;
     KisPaintInformation info = m_d->infoBuilder->hover(savedCursorPos, event);
+    QPointF prevPoint = m_d->lastCursorPos.pushThroughHistory(savedCursorPos);
+    qreal startAngle = KisAlgebra2D::directionBetweenPoints(prevPoint, savedCursorPos, 0);
     info.setCanvasRotation(m_d->canvasRotation);
     info.setCanvasHorizontalMirrorState( m_d->canvasMirroredH );
-    KisDistanceInformation distanceInfo(m_d->lastOutlinePos.pushThroughHistory(savedCursorPos), 0);
+    KisDistanceInformation distanceInfo(prevPoint, startAngle);
 
-    if (!m_d->painterInfos.isEmpty()) {
+    if (!m_d->strokeInfos.isEmpty()) {
         settings = m_d->resources->currentPaintOpPreset()->settings();
         if (m_d->stabilizerDelayedPaintHelper.running() &&
                 m_d->stabilizerDelayedPaintHelper.hasLastPaintInformation()) {
@@ -166,7 +198,7 @@ QPainterPath KisToolFreehandHelper::paintOpOutline(const QPointF &savedCursorPos
          * object.
          */
         KisDistanceInformation *buddyDistance =
-            m_d->painterInfos.first()->buddyDragDistance();
+            m_d->strokeInfos.first()->buddyDragDistance();
 
         if (buddyDistance) {
             /**
@@ -176,20 +208,21 @@ QPainterPath KisToolFreehandHelper::paintOpOutline(const QPointF &savedCursorPos
              * local copy of the coordinates.
              */
             distanceInfo = *buddyDistance;
-            distanceInfo.overrideLastValues(m_d->lastOutlinePos.pushThroughHistory(savedCursorPos), 0);
+            distanceInfo.overrideLastValues(prevPoint, startAngle);
 
-        } else if (m_d->painterInfos.first()->dragDistance->isStarted()) {
-            distanceInfo = *m_d->painterInfos.first()->dragDistance;
+        } else if (m_d->strokeInfos.first()->dragDistance->isStarted()) {
+            distanceInfo = *m_d->strokeInfos.first()->dragDistance;
         }
     }
 
     KisPaintInformation::DistanceInformationRegistrar registrar =
         info.registerDistanceInformation(&distanceInfo);
 
+    info.setRandomSource(m_d->fakeDabRandomSource);
+    info.setPerStrokeRandomSource(m_d->fakeStrokeRandomSource);
+
     QPainterPath outline = settings->brushOutline(info, mode);
 
-
-
     if (m_d->resources &&
         m_d->smoothingOptions->smoothingType() == KisSmoothingOptions::STABILIZER &&
         m_d->smoothingOptions->useDelayDistance()) {
@@ -203,17 +236,27 @@ QPainterPath KisToolFreehandHelper::paintOpOutline(const QPointF &savedCursorPos
     return outline;
 }
 
+void KisToolFreehandHelper::cursorMoved(const QPointF &cursorPos)
+{
+    m_d->lastCursorPos.pushThroughHistory(cursorPos);
+}
+
 void KisToolFreehandHelper::initPaint(KoPointerEvent *event,
+                                      const QPointF &pixelCoords,
                                       KoCanvasResourceManager *resourceManager,
                                       KisImageWSP image, KisNodeSP currentNode,
                                       KisStrokesFacade *strokesFacade,
                                       KisNodeSP overrideNode,
                                       KisDefaultBoundsBaseSP bounds)
 {
+    QPointF prevPoint = m_d->lastCursorPos.pushThroughHistory(pixelCoords);
+    m_d->strokeTime.start();
     KisPaintInformation pi =
         m_d->infoBuilder->startStroke(event, elapsedStrokeTime(), resourceManager);
+    qreal startAngle = KisAlgebra2D::directionBetweenPoints(prevPoint, pixelCoords, 0.0);
 
-    initPaintImpl(pi,
+    initPaintImpl(startAngle,
+                  pi,
                   resourceManager,
                   image,
                   currentNode,
@@ -227,7 +270,8 @@ bool KisToolFreehandHelper::isRunning() const
     return m_d->strokeId;
 }
 
-void KisToolFreehandHelper::initPaintImpl(const KisPaintInformation &previousPaintInformation,
+void KisToolFreehandHelper::initPaintImpl(qreal startAngle,
+                                          const KisPaintInformation &pi,
                                           KoCanvasResourceManager *resourceManager,
                                           KisImageWSP image,
                                           KisNodeSP currentNode,
@@ -235,8 +279,6 @@ void KisToolFreehandHelper::initPaintImpl(const KisPaintInformation &previousPai
                                           KisNodeSP overrideNode,
                                           KisDefaultBoundsBaseSP bounds)
 {
-    Q_UNUSED(overrideNode);
-
     m_d->strokesFacade = strokesFacade;
 
     m_d->haveTangent = false;
@@ -244,45 +286,59 @@ void KisToolFreehandHelper::initPaintImpl(const KisPaintInformation &previousPai
 
     m_d->hasPaintAtLeastOnce = false;
 
-    m_d->strokeTime.start();
-
-    m_d->previousPaintInformation = previousPaintInformation;
-
-    createPainters(m_d->painterInfos,
-                   m_d->previousPaintInformation.pos(),
-                   m_d->previousPaintInformation.currentTime());
+    m_d->previousPaintInformation = pi;
 
     m_d->resources = new KisResourcesSnapshot(image,
                                               currentNode,
                                               resourceManager,
                                               bounds);
-
     if(overrideNode) {
         m_d->resources->setCurrentNode(overrideNode);
     }
 
+    const bool airbrushing = m_d->resources->needsAirbrushing();
+    const bool useSpacingUpdates = m_d->resources->needsSpacingUpdates();
+
+    KisDistanceInitInfo startDistInfo(m_d->previousPaintInformation.pos(),
+                                      startAngle,
+                                      useSpacingUpdates ? SPACING_UPDATE_INTERVAL : LONG_TIME,
+                                      airbrushing ? TIMING_UPDATE_INTERVAL : LONG_TIME,
+                                      0);
+    KisDistanceInformation startDist = startDistInfo.makeDistInfo();
+
+    createPainters(m_d->strokeInfos,
+                   startDist);
+
     if(m_d->recordingAdapter) {
-        m_d->recordingAdapter->startStroke(image, m_d->resources);
+        m_d->recordingAdapter->startStroke(image, m_d->resources, startDistInfo);
     }
 
     KisStrokeStrategy *stroke =
-        new FreehandStrokeStrategy(m_d->resources->needsIndirectPainting(),
-                                   m_d->resources->indirectPaintingCompositeOp(),
-                                   m_d->resources, m_d->painterInfos, m_d->transactionText);
+        new FreehandStrokeStrategy(m_d->resources, m_d->strokeInfos, m_d->transactionText);
 
     m_d->strokeId = m_d->strokesFacade->startStroke(stroke);
 
     m_d->history.clear();
     m_d->distanceHistory.clear();
 
-    if(m_d->resources->needsAirbrushing()) {
-        m_d->airbrushingTimer.setInterval(m_d->resources->airbrushingRate());
+    if (airbrushing) {
+        m_d->airbrushingTimer.setInterval(computeAirbrushTimerInterval());
         m_d->airbrushingTimer.start();
+    } else if (m_d->resources->presetNeedsAsynchronousUpdates()) {
+        m_d->asynchronousUpdatesThresholdTimer.setInterval(80 /* msec */);
+        m_d->asynchronousUpdatesThresholdTimer.start();
     }
 
     if (m_d->smoothingOptions->smoothingType() == KisSmoothingOptions::STABILIZER) {
         stabilizerStart(m_d->previousPaintInformation);
     }
+
+    // If airbrushing, paint an initial dab immediately. This is a workaround for an issue where
+    // some paintops (Dyna, Particle, Sketch) might never initialize their spacing/timing
+    // information until paintAt is called.
+    if (airbrushing) {
+        paintAt(pi);
+    }
 }
 
 void KisToolFreehandHelper::paintBezierSegment(KisPaintInformation pi1, KisPaintInformation pi2,
@@ -385,7 +441,7 @@ qreal KisToolFreehandHelper::Private::effectiveSmoothnessDistance() const
     return effectiveSmoothnessDistance;
 }
 
-void KisToolFreehandHelper::paint(KoPointerEvent *event)
+void KisToolFreehandHelper::paintEvent(KoPointerEvent *event)
 {
     KisPaintInformation info =
             m_d->infoBuilder->continueStroke(event,
@@ -395,6 +451,11 @@ void KisToolFreehandHelper::paint(KoPointerEvent *event)
 
     KisUpdateTimeMonitor::instance()->reportMouseMove(info.pos());
 
+    paint(info);
+}
+
+void KisToolFreehandHelper::paint(KisPaintInformation &info)
+{
     /**
      * Smooth the coordinates out using the history and the
      * distance. This is a heavily modified version of an algo used in
@@ -408,7 +469,7 @@ void KisToolFreehandHelper::paint(KoPointerEvent *event)
      * 2) There is no 'Quality' parameter, since the number of samples
      *    is calculated automatically
      *
-     * 3) 'Tail Aggressiveness' is used for controling the end of the
+     * 3) 'Tail Aggressiveness' is used for controlling the end of the
      *    stroke
      *
      * 4) The formila is a little bit different: 'Distance' parameter
@@ -519,13 +580,22 @@ void KisToolFreehandHelper::paint(KoPointerEvent *event)
             QPointF newTangent = (info.pos() - m_d->olderPaintInformation.pos()) /
                 qMax(qreal(1.0), info.currentTime() - m_d->olderPaintInformation.currentTime());
 
-            paintBezierSegment(m_d->olderPaintInformation, m_d->previousPaintInformation,
-                               m_d->previousTangent, newTangent);
+            if (newTangent.isNull() || m_d->previousTangent.isNull())
+            {
+                paintLine(m_d->previousPaintInformation, info);
+            } else {
+                paintBezierSegment(m_d->olderPaintInformation, m_d->previousPaintInformation,
+                                   m_d->previousTangent, newTangent);
+            }
 
             m_d->previousTangent = newTangent;
         }
         m_d->olderPaintInformation = m_d->previousPaintInformation;
-        m_d->strokeTimeoutTimer.start(100);
+
+        // Enable stroke timeout only when not airbrushing.
+        if (!m_d->airbrushingTimer.isActive()) {
+            m_d->strokeTimeoutTimer.start(100);
+        }
     }
     else if (m_d->smoothingOptions->smoothingType() == KisSmoothingOptions::NO_SMOOTHING){
         paintLine(m_d->previousPaintInformation, info);
@@ -533,6 +603,11 @@ void KisToolFreehandHelper::paint(KoPointerEvent *event)
 
     if (m_d->smoothingOptions->smoothingType() == KisSmoothingOptions::STABILIZER) {
         m_d->stabilizedSampler.addEvent(info);
+        if (m_d->stabilizerDelayedPaintHelper.running()) {
+            // Paint here so we don't have to rely on the timer
+            // This is just a tricky source for a relatively stable 7ms "timer"
+            m_d->stabilizerDelayedPaintHelper.paintSome();
+        }
     } else {
         m_d->previousPaintInformation = info;
     }
@@ -555,6 +630,10 @@ void KisToolFreehandHelper::endPaint()
         m_d->airbrushingTimer.stop();
     }
 
+    if (m_d->asynchronousUpdatesThresholdTimer.isActive()) {
+        m_d->asynchronousUpdatesThresholdTimer.stop();
+    }
+
     if (m_d->smoothingOptions->smoothingType() == KisSmoothingOptions::STABILIZER) {
         stabilizerEnd();
     }
@@ -565,7 +644,10 @@ void KisToolFreehandHelper::endPaint()
      * Please note that we are not in MT here, so no mutex
      * is needed
      */
-    m_d->painterInfos.clear();
+    m_d->strokeInfos.clear();
+
+    // last update to complete rendering if there is still something pending
+    doAsynchronousUpdate(true);
 
     m_d->strokesFacade->endStroke(m_d->strokeId);
     m_d->strokeId.clear();
@@ -585,6 +667,10 @@ void KisToolFreehandHelper::cancelPaint()
         m_d->airbrushingTimer.stop();
     }
 
+    if (m_d->asynchronousUpdatesThresholdTimer.isActive()) {
+        m_d->asynchronousUpdatesThresholdTimer.stop();
+    }
+
     if (m_d->stabilizerPollTimer.isActive()) {
         m_d->stabilizerPollTimer.stop();
     }
@@ -594,7 +680,7 @@ void KisToolFreehandHelper::cancelPaint()
     }
 
     // see a comment in endPaint()
-    m_d->painterInfos.clear();
+    m_d->strokeInfos.clear();
 
     m_d->strokesFacade->cancelStroke(m_d->strokeId);
     m_d->strokeId.clear();
@@ -628,9 +714,9 @@ void KisToolFreehandHelper::stabilizerStart(KisPaintInformation firstPaintInfo)
     m_d->stabilizerPollTimer.setInterval(stabilizerSampleSize);
     m_d->stabilizerPollTimer.start();
 
-    int delayedPaintInterval = cfg.stabilizerDelayedPaintInterval();
-    if (delayedPaintInterval < stabilizerSampleSize) {
-        m_d->stabilizerDelayedPaintHelper.start(delayedPaintInterval, firstPaintInfo);
+    bool delayedPaintEnabled = cfg.stabilizerDelayedPaint();
+    if (delayedPaintEnabled) {
+        m_d->stabilizerDelayedPaintHelper.start(firstPaintInfo);
     }
 
     m_d->stabilizedSampler.clear();
@@ -638,10 +724,18 @@ void KisToolFreehandHelper::stabilizerStart(KisPaintInformation firstPaintInfo)
 }
 
 KisPaintInformation
-KisToolFreehandHelper::Private::getStabilizedPaintInfo(const QQueue<KisPaintInformation> &queue,
-                                                       const KisPaintInformation &lastPaintInfo)
+KisToolFreehandHelper::getStabilizedPaintInfo(const QQueue<KisPaintInformation> &queue,
+                                              const KisPaintInformation &lastPaintInfo)
 {
-    KisPaintInformation result(lastPaintInfo);
+    KisPaintInformation result(lastPaintInfo.pos(),
+                               lastPaintInfo.pressure(),
+                               lastPaintInfo.xTilt(),
+                               lastPaintInfo.yTilt(),
+                               lastPaintInfo.rotation(),
+                               lastPaintInfo.tangentialPressure(),
+                               lastPaintInfo.perspective(),
+                               elapsedStrokeTime(),
+                               lastPaintInfo.drawingSpeed());
 
     if (queue.size() > 1) {
         QQueue<KisPaintInformation>::const_iterator it = queue.constBegin();
@@ -653,17 +747,17 @@ KisToolFreehandHelper::Private::getStabilizedPaintInfo(const QQueue<KisPaintInfo
         it++;
         int i = 2;
 
-        if (smoothingOptions->stabilizeSensors()) {
+        if (m_d->smoothingOptions->stabilizeSensors()) {
             while (it != end) {
                 qreal k = qreal(i - 1) / i; // coeff for uniform averaging
-                result = KisPaintInformation::mix(k, *it, result);
+                result.KisPaintInformation::mixOtherWithoutTime(k, *it);
                 it++;
                 i++;
             }
         } else{
             while (it != end) {
                 qreal k = qreal(i - 1) / i; // coeff for uniform averaging
-                result = KisPaintInformation::mixOnlyPosition(k, *it, result);
+                result.KisPaintInformation::mixOtherOnlyPosition(k, *it);
                 it++;
                 i++;
             }
@@ -692,12 +786,18 @@ void KisToolFreehandHelper::stabilizerPollAndPaint()
             QPointF diff = sampledInfo.pos() - m_d->previousPaintInformation.pos();
             qreal dx = sqrt(pow2(diff.x()) + pow2(diff.y()));
 
-            canPaint = dx > R;
+            if (!(dx > R)) {
+                if (m_d->resources->needsAirbrushing()) {
+                    sampledInfo.setPos(m_d->previousPaintInformation.pos());
+                }
+                else {
+                    canPaint = false;
+                }
+            }
         }
 
         if (canPaint) {
-            KisPaintInformation newInfo =
-                    m_d->getStabilizedPaintInfo(m_d->stabilizerDeque, sampledInfo);
+            KisPaintInformation newInfo = getStabilizedPaintInfo(m_d->stabilizerDeque, sampledInfo);
 
             if (m_d->stabilizerDelayedPaintHelper.running()) {
                 delayedPaintTodoItems.append(newInfo);
@@ -750,11 +850,6 @@ void KisToolFreehandHelper::stabilizerEnd()
     }
 }
 
-const KisPaintOp* KisToolFreehandHelper::currentPaintOp() const
-{
-    return !m_d->painterInfos.isEmpty() ? m_d->painterInfos.first()->painter->paintOp() : 0;
-}
-
 void KisToolFreehandHelper::finishStroke()
 {
     if (m_d->haveTangent) {
@@ -772,39 +867,63 @@ void KisToolFreehandHelper::finishStroke()
 
 void KisToolFreehandHelper::doAirbrushing()
 {
-    if(!m_d->painterInfos.isEmpty()) {
-        paintAt(m_d->previousPaintInformation);
+    // Check that the stroke hasn't ended.
+    if (!m_d->strokeInfos.isEmpty()) {
+
+        // Add a new painting update at a point identical to the previous one, except for the time
+        // and speed information.
+        const KisPaintInformation &prevPaint = m_d->previousPaintInformation;
+        KisPaintInformation nextPaint(prevPaint.pos(),
+                                      prevPaint.pressure(),
+                                      prevPaint.xTilt(),
+                                      prevPaint.yTilt(),
+                                      prevPaint.rotation(),
+                                      prevPaint.tangentialPressure(),
+                                      prevPaint.perspective(),
+                                      elapsedStrokeTime(),
+                                      0.0);
+        paint(nextPaint);
     }
 }
 
-void KisToolFreehandHelper::paintAt(int painterInfoId,
+void KisToolFreehandHelper::doAsynchronousUpdate(bool forceUpdate)
+{
+    m_d->strokesFacade->addJob(m_d->strokeId,
+                               new FreehandStrokeStrategy::UpdateData(forceUpdate));
+}
+
+int KisToolFreehandHelper::computeAirbrushTimerInterval() const
+{
+    qreal realInterval = m_d->resources->airbrushingInterval() * AIRBRUSH_INTERVAL_FACTOR;
+    return qMax(1, qFloor(realInterval));
+}
+
+void KisToolFreehandHelper::paintAt(int strokeInfoId,
                                     const KisPaintInformation &pi)
 {
     m_d->hasPaintAtLeastOnce = true;
     m_d->strokesFacade->addJob(m_d->strokeId,
-                               new FreehandStrokeStrategy::Data(m_d->resources->currentNode(),
-                                                                painterInfoId, pi));
+                               new FreehandStrokeStrategy::Data(strokeInfoId, pi));
 
     if(m_d->recordingAdapter) {
         m_d->recordingAdapter->addPoint(pi);
     }
 }
 
-void KisToolFreehandHelper::paintLine(int painterInfoId,
+void KisToolFreehandHelper::paintLine(int strokeInfoId,
                                       const KisPaintInformation &pi1,
                                       const KisPaintInformation &pi2)
 {
     m_d->hasPaintAtLeastOnce = true;
     m_d->strokesFacade->addJob(m_d->strokeId,
-                               new FreehandStrokeStrategy::Data(m_d->resources->currentNode(),
-                                                                painterInfoId, pi1, pi2));
+                               new FreehandStrokeStrategy::Data(strokeInfoId, pi1, pi2));
 
     if(m_d->recordingAdapter) {
         m_d->recordingAdapter->addLine(pi1, pi2);
     }
 }
 
-void KisToolFreehandHelper::paintBezierCurve(int painterInfoId,
+void KisToolFreehandHelper::paintBezierCurve(int strokeInfoId,
                                              const KisPaintInformation &pi1,
                                              const QPointF &control1,
                                              const QPointF &control2,
@@ -836,8 +955,7 @@ void KisToolFreehandHelper::paintBezierCurve(int painterInfoId,
 
     m_d->hasPaintAtLeastOnce = true;
     m_d->strokesFacade->addJob(m_d->strokeId,
-                               new FreehandStrokeStrategy::Data(m_d->resources->currentNode(),
-                                                                painterInfoId,
+                               new FreehandStrokeStrategy::Data(strokeInfoId,
                                                                 pi1, control1, control2, pi2));
 
     if(m_d->recordingAdapter) {
@@ -845,11 +963,10 @@ void KisToolFreehandHelper::paintBezierCurve(int painterInfoId,
     }
 }
 
-void KisToolFreehandHelper::createPainters(QVector<PainterInfo*> &painterInfos,
-                                           const QPointF &lastPosition,
-                                           int lastTime)
+void KisToolFreehandHelper::createPainters(QVector<KisFreehandStrokeInfo*> &strokeInfos,
+                                           const KisDistanceInformation &startDist)
 {
-    painterInfos << new PainterInfo(lastPosition, lastTime);
+    strokeInfos << new KisFreehandStrokeInfo(startDist);
 }
 
 void KisToolFreehandHelper::paintAt(const KisPaintInformation &pi)
@@ -889,4 +1006,3 @@ void KisToolFreehandHelper::setCanvasHorizontalMirrorState(bool mirrored)
 {
    m_d->canvasMirroredH = mirrored;
 }
-
