@@ -20,6 +20,7 @@
 #define __KIS_TOOL_FREEHAND_HELPER_H
 
 #include <QObject>
+#include <QVector>
 
 #include "kis_types.h"
 #include "kritaui_export.h"
@@ -27,7 +28,8 @@
 #include "kis_default_bounds.h"
 #include <brushengine/kis_paintop_settings.h>
 #include "kis_smoothing_options.h"
-#include "strokes/freehand_stroke.h"
+#include "kundo2magicstring.h"
+
 
 class KoPointerEvent;
 class KoCanvasResourceManager;
@@ -36,16 +38,13 @@ class KisRecordingAdapter;
 class KisStrokesFacade;
 class KisPostExecutionUndoAdapter;
 class KisPaintOp;
+class KisFreehandStrokeInfo;
 
 
 class KRITAUI_EXPORT KisToolFreehandHelper : public QObject
 {
     Q_OBJECT
 
-protected:
-
-    typedef FreehandStrokeStrategy::PainterInfo PainterInfo;
-
 public:
 
     KisToolFreehandHelper(KisPaintingInformationBuilder *infoBuilder,
@@ -59,17 +58,22 @@ public:
 
     bool isRunning() const;
 
+    void cursorMoved(const QPointF &cursorPos);
+
+    /**
+     * @param pixelCoords - The position of the KoPointerEvent, in pixel coordinates.
+     */
     void initPaint(KoPointerEvent *event,
+                   const QPointF &pixelCoords,
                    KoCanvasResourceManager *resourceManager,
                    KisImageWSP image,
                    KisNodeSP currentNode,
                    KisStrokesFacade *strokesFacade,
                    KisNodeSP overrideNode = 0,
                    KisDefaultBoundsBaseSP bounds = 0);
-    void paint(KoPointerEvent *event);
+    void paintEvent(KoPointerEvent *event);
     void endPaint();
 
-    const KisPaintOp* currentPaintOp() const;
     QPainterPath paintOpOutline(const QPointF &savedCursorPos,
                                 const KoPointerEvent *event,
                                 const KisPaintOpSettingsSP globalSettings,
@@ -91,7 +95,8 @@ protected:
     void cancelPaint();
     int elapsedStrokeTime() const;
 
-    void initPaintImpl(const KisPaintInformation &previousPaintInformation,
+    void initPaintImpl(qreal startAngle,
+                       const KisPaintInformation &pi,
                        KoCanvasResourceManager *resourceManager,
                        KisImageWSP image,
                        KisNodeSP node,
@@ -101,19 +106,18 @@ protected:
 
 protected:
 
-    virtual void createPainters(QVector<PainterInfo*> &painterInfos,
-                                const QPointF &lastPosition,
-                                int lastTime);
+    virtual void createPainters(QVector<KisFreehandStrokeInfo*> &strokeInfos,
+                                const KisDistanceInformation &startDist);
 
     // lo-level methods for painting primitives
 
-    void paintAt(int painterInfoId, const KisPaintInformation &pi);
+    void paintAt(int strokeInfoId, const KisPaintInformation &pi);
 
-    void paintLine(int painterInfoId,
+    void paintLine(int strokeInfoId,
                    const KisPaintInformation &pi1,
                    const KisPaintInformation &pi2);
 
-    void paintBezierCurve(int painterInfoId,
+    void paintBezierCurve(int strokeInfoId,
                           const KisPaintInformation &pi1,
                           const QPointF &control1,
                           const QPointF &control2,
@@ -132,16 +136,21 @@ protected:
                                   const KisPaintInformation &pi2);
 
 private:
+    void paint(KisPaintInformation &info);
     void paintBezierSegment(KisPaintInformation pi1, KisPaintInformation pi2,
                                                    QPointF tangent1, QPointF tangent2);
 
     void stabilizerStart(KisPaintInformation firstPaintInfo);
     void stabilizerEnd();
+    KisPaintInformation getStabilizedPaintInfo(const QQueue<KisPaintInformation> &queue,
+                                               const KisPaintInformation &lastPaintInfo);
+    int computeAirbrushTimerInterval() const;
 
 private Q_SLOTS:
 
     void finishStroke();
     void doAirbrushing();
+    void doAsynchronousUpdate(bool forceUpdate = false);
     void stabilizerPollAndPaint();
 
 private:
