@@ -628,9 +628,6 @@ KisToolPaint::NodePaintAbility KisToolPaint::nodePaintAbility()
     if (node->inherits("KisShapeLayer")) {
         return VECTOR;
     }
-    if (node->inherits("KisCloneLayer")) {
-        return CLONE;
-    }
     if (node->paintDevice()) {
         return PAINT;
     }
