@@ -108,6 +108,11 @@ KisCustomImageWidget::KisCustomImageWidget(QWidget* parent, qint32 defWidth, qin
     connect(createButton, SIGNAL(clicked()), this, SLOT(createImage()));
     createButton->setDefault(true);
 
+    // parent widget contains the window
+    connect(cancelNewDocumentButton, SIGNAL(clicked(bool)), this->parentWidget(), SLOT(close()));
+    connect(cancelNewDocumentButton, SIGNAL(clicked(bool)), this->parentWidget(), SLOT(deleteLater()));
+
+
     bnPortrait->setIcon(KisIconUtils::loadIcon("portrait"));
     connect(bnPortrait, SIGNAL(clicked()), SLOT(setPortrait()));
     connect(bnLandscape, SIGNAL(clicked()), SLOT(setLandscape()));
@@ -120,6 +125,7 @@ KisCustomImageWidget::KisCustomImageWidget(QWidget* parent, qint32 defWidth, qin
     colorSpaceSelector->setCurrentColorModel(KoID(defColorModel));
     colorSpaceSelector->setCurrentColorDepth(KoID(defColorDepth));
     colorSpaceSelector->setCurrentProfile(defColorProfile);
+    connect(colorSpaceSelector, SIGNAL(colorSpaceChanged(const KoColorSpace*)), this, SLOT(changeDocumentInfoLabel()));
 
     //connect(chkFromClipboard,SIGNAL(stateChanged(int)),this,SLOT(clipboardDataChanged()));
     connect(QApplication::clipboard(), SIGNAL(dataChanged()), this, SLOT(clipboardDataChanged()));
@@ -146,13 +152,14 @@ KisCustomImageWidget::KisCustomImageWidget(QWidget* parent, qint32 defWidth, qin
     switchPortraitLandscape();
 
     // this makes the portrait and landscape buttons more
-    // obvious what is selected by changing the higlight color
+    // obvious what is selected by changing the highlight color
     QPalette p = QApplication::palette();
     QPalette palette_highlight(p );
     QColor c = p.color(QPalette::Highlight);
     palette_highlight.setColor(QPalette::Button, c);
     bnLandscape->setPalette(palette_highlight);
     bnPortrait->setPalette(palette_highlight);
+    changeDocumentInfoLabel();
 }
 
 void KisCustomImageWidget::showEvent(QShowEvent *)
@@ -178,6 +185,7 @@ void KisCustomImageWidget::resolutionChanged(double res)
         m_heightUnit.setFactor(res / 72.0);
         m_height = m_heightUnit.fromUserValue(doubleHeight->value());
     }
+    changeDocumentInfoLabel();
 }
 
 
@@ -196,11 +204,13 @@ void KisCustomImageWidget::widthUnitChanged(int index)
     doubleWidth->setValue(KoUnit::ptToUnit(m_width, m_widthUnit));
 
     doubleWidth->blockSignals(false);
+    changeDocumentInfoLabel();
 }
 
 void KisCustomImageWidget::widthChanged(double value)
 {
     m_width = m_widthUnit.fromUserValue(value);
+    changeDocumentInfoLabel();
 }
 
 void KisCustomImageWidget::heightUnitChanged(int index)
@@ -218,11 +228,13 @@ void KisCustomImageWidget::heightUnitChanged(int index)
     doubleHeight->setValue(KoUnit::ptToUnit(m_height, m_heightUnit));
 
     doubleHeight->blockSignals(false);
+    changeDocumentInfoLabel();
 }
 
 void KisCustomImageWidget::heightChanged(double value)
 {
     m_height = m_heightUnit.fromUserValue(value);
+    changeDocumentInfoLabel();
 }
 
 void KisCustomImageWidget::createImage()
@@ -370,6 +382,7 @@ void KisCustomImageWidget::predefinedClicked(int index)
 
     doubleWidth->setValue(predefined->getDouble("width"));
     doubleHeight->setValue(predefined->getDouble("height"));
+    changeDocumentInfoLabel();
 
 }
 
@@ -380,7 +393,7 @@ void KisCustomImageWidget::saveAsPredefined()
         return;
     }
     QString saveLocation = KoResourcePaths::saveLocation("data", "predefined_image_sizes/", true);
-    QFile f(saveLocation + '/' + fileName.replace(' ', '_').replace('(', '_').replace(')', '_') + ".predefinedimage");
+    QFile f(saveLocation + '/' + fileName.replace(' ', '_').replace('(', '_').replace(')', '_').replace(':', '_') + ".predefinedimage");
 
     f.open(QIODevice::WriteOnly | QIODevice::Truncate);
     KisPropertiesConfigurationSP predefined = new KisPropertiesConfiguration();
@@ -453,6 +466,7 @@ void KisCustomImageWidget::switchWidthHeight()
 
     widthChanged(doubleWidth->value());
     heightChanged(doubleHeight->value());
+    changeDocumentInfoLabel();
 }
 
 void KisCustomImageWidget::switchPortraitLandscape()
@@ -463,3 +477,41 @@ void KisCustomImageWidget::switchPortraitLandscape()
         bnPortrait->setChecked(true);
 }
 
+void KisCustomImageWidget::changeDocumentInfoLabel()
+{
+
+    qint64 width, height;
+    double resolution;
+    resolution = doubleResolution->value() / 72.0;  // internal resolution is in pixels per pt
+
+    width = static_cast<qint64>(0.5  + KoUnit::ptToUnit(m_width, KoUnit(KoUnit::Pixel, resolution)));
+    height = static_cast<qint64>(0.5 + KoUnit::ptToUnit(m_height, KoUnit(KoUnit::Pixel, resolution)));
+
+    qint64 layerSize = width * height;
+    const KoColorSpace *cs = colorSpaceSelector->currentColorSpace();
+    int bitSize = 8 * cs->pixelSize(); //pixelsize is in bytes.
+    layerSize = layerSize * cs->pixelSize();
+    QString byte = i18n("bytes");
+    if (layerSize>1024) {
+        layerSize/=1024;
+        byte = i18nc("Abbreviation for kilobyte", "KB");
+    }
+    if (layerSize>1024) {
+        layerSize/=1024;
+        byte = i18nc("Abbreviation for megabyte", "MB");
+    }
+    if (layerSize>1024) {
+        layerSize/=1024;
+        byte = i18nc("Abbreviation for gigabyte", "GB");
+    }
+    QString text = i18nc("arg1: width. arg2: height. arg3: colorspace name. arg4: size of a channel in bits. arg5: size in unites of arg6. Arg6: KB, MB or GB",
+                         "This document will be %1 pixels by %2 pixels in %3, which means the pixel size is %4 bit. A single paint layer will thus take up %5 %6 of RAM.",
+                         width,
+                         height,
+                         cs->name(),
+                         bitSize,
+                         layerSize,
+                         byte);
+    lblDocumentInfo->setText(text);
+}
+
