@@ -30,7 +30,7 @@
 
 #include <ui_wdgpaintoppresets.h>
 #include <kis_config.h>
-#include <KisResourceServerProvider.h>
+#include <kis_resource_server_provider.h>
 #include <brushengine/kis_paintop_preset.h>
 #include <kis_icon.h>
 #include <brushengine/kis_paintop_settings.h>
@@ -40,7 +40,6 @@ struct KisPaintOpPresetsChooserPopup::Private
 public:
     Ui_WdgPaintOpPresets uiWdgPaintOpPresets;
     bool firstShown;
-    QToolButton *viewModeButton;
 };
 
 KisPaintOpPresetsChooserPopup::KisPaintOpPresetsChooserPopup(QWidget * parent)
@@ -90,10 +89,8 @@ KisPaintOpPresetsChooserPopup::KisPaintOpPresetsChooserPopup(QWidget * parent)
     m_d->uiWdgPaintOpPresets.wdgPresetChooser->showTaggingBar(true);
 
     m_d->uiWdgPaintOpPresets.wdgPresetChooser->itemChooser()->setViewModeButtonVisible(true);
-    m_d->viewModeButton = m_d->uiWdgPaintOpPresets.wdgPresetChooser->itemChooser()->viewModeButton();
-    m_d->viewModeButton->setMenu(menu);
-    m_d->viewModeButton->setIcon(KisIconUtils::loadIcon("configure"));
-
+    QToolButton *viewModeButton = m_d->uiWdgPaintOpPresets.wdgPresetChooser->itemChooser()->viewModeButton();
+    viewModeButton->setMenu(menu);
 
     connect(m_d->uiWdgPaintOpPresets.wdgPresetChooser, SIGNAL(resourceSelected(KoResource*)),
             this, SIGNAL(resourceSelected(KoResource*)));
@@ -153,11 +150,6 @@ void KisPaintOpPresetsChooserPopup::canvasResourceChanged(KisPaintOpPresetSP  pr
     m_d->uiWdgPaintOpPresets.wdgPresetChooser->updateViewSettings();
 }
 
-void KisPaintOpPresetsChooserPopup::slotThemeChanged()
-{
-    m_d->viewModeButton->setIcon(KisIconUtils::loadIcon("configure"));
-}
-
 void KisPaintOpPresetsChooserPopup::updateViewSettings()
 {
    m_d->uiWdgPaintOpPresets.wdgPresetChooser->updateViewSettings();
