@@ -29,6 +29,7 @@
 #include <QSortFilterProxyModel>
 #include <QApplication>
 
+#include <kis_config.h>
 #include <klocalizedstring.h>
 
 
@@ -53,11 +54,11 @@ class KisPresetDelegate : public QAbstractItemDelegate
 {
 public:
     KisPresetDelegate(QObject * parent = 0) : QAbstractItemDelegate(parent), m_showText(false), m_useDirtyPresets(false) {}
-    virtual ~KisPresetDelegate() {}
+    ~KisPresetDelegate() override {}
     /// reimplemented
-    virtual void paint(QPainter *, const QStyleOptionViewItem &, const QModelIndex &) const;
+    void paint(QPainter *, const QStyleOptionViewItem &, const QModelIndex &) const override;
     /// reimplemented
-    QSize sizeHint(const QStyleOptionViewItem & option, const QModelIndex &) const {
+    QSize sizeHint(const QStyleOptionViewItem & option, const QModelIndex &) const override {
         return option.decorationSize;
     }
 
@@ -99,7 +100,27 @@ void KisPresetDelegate::paint(QPainter * painter, const QStyleOptionViewItem & o
         painter->drawImage(paintRect.x(), paintRect.y(),
                            preview.scaled(pixSize, Qt::KeepAspectRatio, Qt::SmoothTransformation));
 
-        painter->drawText(pixSize.width() + 10, option.rect.y() + option.rect.height() - 10, preset->name());
+        // Put an asterisk after the preset if it is dirty. This will help in case the pixmap icon is too small
+         QString dirtyPresetIndicator = QString("");
+        if (m_useDirtyPresets && preset->isPresetDirty()) {
+            dirtyPresetIndicator = QString("*");
+        }
+
+        qreal brushSize = preset->settings()->paintOpSize();
+        QString brushSizeText;
+
+        // Disable displayed decimal precision beyond a certain brush size
+        if (brushSize < 100) {
+            brushSizeText = QString::number(brushSize, 'g', 3);
+        } else {
+            brushSizeText = QString::number(brushSize, 'f', 0);
+        }
+
+        painter->drawText(pixSize.width() + 10, option.rect.y() + option.rect.height() - 10, brushSizeText); // brush size
+
+        QString presetDisplayName = preset->name().replace("_", " "); // don't need underscores that might be part of the file name
+        painter->drawText(pixSize.width() + 40, option.rect.y() + option.rect.height() - 10, presetDisplayName.append(dirtyPresetIndicator));
+
     }
     if (m_useDirtyPresets && preset->isPresetDirty()) {
         const QIcon icon = KisIconUtils::loadIcon(koIconName("dirty-preset"));
@@ -134,9 +155,9 @@ public:
     {
         setSortingEnabled(true);
     }
-    virtual ~KisPresetProxyAdapter() {}
+    ~KisPresetProxyAdapter() override {}
 
-    virtual QList< KoResource* > resources() {
+    QList< KoResource* > resources() override {
 
         QList<KoResource*> serverResources =
             KisPaintOpPresetResourceServerAdapter::resources();
@@ -188,6 +209,7 @@ KisPresetChooser::KisPresetChooser(QWidget *parent, const char *name)
     m_adapter = QSharedPointer<KoAbstractResourceServerAdapter>(new KisPresetProxyAdapter(rserver));
 
     m_chooser = new KoResourceItemChooser(m_adapter, this);
+    m_chooser->setObjectName("ResourceChooser");
     m_chooser->setColumnCount(10);
     m_chooser->setRowHeight(50);
     m_delegate = new KisPresetDelegate(this);
@@ -195,6 +217,11 @@ KisPresetChooser::KisPresetChooser(QWidget *parent, const char *name)
     m_chooser->setSynced(true);
     layout->addWidget(m_chooser);
 
+    KisConfig cfg;
+    m_chooser->configureKineticScrolling(cfg.kineticScrollingGesture(),
+                                         cfg.kineticScrollingSensitivity(),
+                                         cfg.kineticScrollingScrollbar());
+
     connect(m_chooser, SIGNAL(resourceSelected(KoResource*)),
             this, SIGNAL(resourceSelected(KoResource*)));
     connect(m_chooser, SIGNAL(resourceClicked(KoResource*)),
@@ -204,6 +231,8 @@ KisPresetChooser::KisPresetChooser(QWidget *parent, const char *name)
 
     connect(KisConfigNotifier::instance(), SIGNAL(configChanged()),
             SLOT(notifyConfigChanged()));
+
+
     notifyConfigChanged();
 }
 
@@ -232,6 +261,8 @@ void KisPresetChooser::notifyConfigChanged()
 {
     KisConfig cfg;
     m_delegate->setUseDirtyPresets(cfg.useDirtyPresets());
+    setIconSize(cfg.presetIconSize());
+
     updateViewSettings();
 }
 
@@ -243,6 +274,8 @@ void KisPresetChooser::updateViewSettings()
     } else if (m_mode == DETAIL) {
         m_chooser->setSynced(false);
         m_chooser->setColumnCount(1);
+        m_chooser->setColumnWidth(m_chooser->width());
+
         KoResourceItemChooserSync* chooserSync = KoResourceItemChooserSync::instance();
         m_chooser->setRowHeight(chooserSync->baseLength());
         m_delegate->setShowText(true);
@@ -280,7 +313,7 @@ void KisPresetChooser::setCurrentResource(KoResource *resource)
     m_chooser->setCurrentResource(resource);
 }
 
-KoResource* KisPresetChooser::currentResource()
+KoResource* KisPresetChooser::currentResource() const
 {
     return m_chooser->currentResource();
 }
@@ -295,6 +328,7 @@ KoResourceItemChooser *KisPresetChooser::itemChooser()
     return m_chooser;
 }
 
+
 void KisPresetChooser::setPresetFilter(const QString& paintOpId)
 {
     KisPresetProxyAdapter *adapter = static_cast<KisPresetProxyAdapter*>(m_adapter.data());
@@ -305,5 +339,25 @@ void KisPresetChooser::setPresetFilter(const QString& paintOpId)
     }
 }
 
+void KisPresetChooser::setIconSize(int newSize)
+{
+    KoResourceItemChooserSync* chooserSync = KoResourceItemChooserSync::instance();
+    chooserSync->setBaseLength(newSize);
+    updateViewSettings();
+}
+
+int KisPresetChooser::iconSize()
+{
+    KoResourceItemChooserSync* chooserSync = KoResourceItemChooserSync::instance();
+
+    return chooserSync->baseLength();
+}
 
 
+
+void KisPresetChooser::saveIconSize()
+{
+    // save icon size
+    KisConfig cfg;
+    cfg.setPresetIconSize(iconSize());
+}
