@@ -59,11 +59,14 @@ public:
     {
     }
 
-    void requestProjectionUpdate(KisNode *node, const QRect& rect) {
-        KisNodeGraphListener::requestProjectionUpdate(node, rect);
+    void requestProjectionUpdate(KisNode *node, const QVector<QRect> &rects, bool resetAnimationCache) override {
+        KisNodeGraphListener::requestProjectionUpdate(node, rects, resetAnimationCache);
 
         QMutexLocker locker(&m_lock);
-        m_scratchPad->imageUpdated(rect);
+
+        Q_FOREACH (const QRect &rc, rects) {
+            m_scratchPad->imageUpdated(rc);
+        }
     }
 
 private:
@@ -81,9 +84,9 @@ public:
     {
     }
 
-    virtual ~KisScratchPadDefaultBounds() {}
+    ~KisScratchPadDefaultBounds() override {}
 
-    virtual QRect bounds() const {
+    QRect bounds() const override {
         return m_scratchPad->imageBounds();
     }
 
@@ -102,6 +105,7 @@ KisScratchPad::KisScratchPad(QWidget *parent)
     , m_resourceProvider(0)
 {
     setAutoFillBackground(false);
+    setMouseTracking(true);
 
     m_cursor = KisCursor::load("tool_freehand_cursor.png", 5, 5);
     setCursor(m_cursor);
@@ -190,6 +194,7 @@ void KisScratchPad::pointerRelease(KoPointerEvent *event)
 
 void KisScratchPad::pointerMove(KoPointerEvent *event)
 {
+    m_helper->cursorMoved(documentToWidget().map(event->point));
     if (m_toolMode == PAINTING) {
         doStroke(event);
         event->accept();
@@ -207,20 +212,19 @@ void KisScratchPad::pointerMove(KoPointerEvent *event)
 void KisScratchPad::beginStroke(KoPointerEvent *event)
 {
     KoCanvasResourceManager *resourceManager = m_resourceProvider->resourceManager();
-
     m_helper->initPaint(event,
+                        documentToWidget().map(event->point),
                         resourceManager,
                         0,
                         0,
                         m_updateScheduler,
-                        m_undoAdapter,
                         m_paintLayer,
                         m_paintLayer->paintDevice()->defaultBounds());
 }
 
 void KisScratchPad::doStroke(KoPointerEvent *event)
 {
-    m_helper->paint(event);
+    m_helper->paintEvent(event);
 }
 
 void KisScratchPad::endStroke(KoPointerEvent *event)
@@ -377,6 +381,7 @@ QImage KisScratchPad::cutoutOverlay() const
     if(!m_paintLayer) return QImage();
     KisPaintDeviceSP paintDevice = m_paintLayer->paintDevice();
 
+
     QRect rc = widgetToDocument().mapRect(m_cutoutOverlay);
     QImage rawImage = paintDevice->convertToQImage(0, rc.x(), rc.y(), rc.width(), rc.height(), KoColorConversionTransformation::internalRenderingIntent(), KoColorConversionTransformation::internalConversionFlags());
 
@@ -392,18 +397,42 @@ void KisScratchPad::setPresetImage(const QImage& image)
     m_presetImage = image;
 }
 
+
+void KisScratchPad::paintCustomImage(const QImage& loadedImage)
+{
+    // this is 99% copied from the normal paintPresetImage()
+    // we don't want to save over the preset image, so we don't
+    // want to store it in the m_presetImage
+    if(!m_paintLayer) return;
+    KisPaintDeviceSP paintDevice = m_paintLayer->paintDevice();
+
+
+    QRect overlayRect = widgetToDocument().mapRect(m_cutoutOverlay);
+    QRect imageRect(QPoint(), overlayRect.size());
+
+    QImage scaledImage = loadedImage.scaled(overlayRect.size(),
+                                              Qt::IgnoreAspectRatio,
+                                              Qt::SmoothTransformation);
+    KisPaintDeviceSP device = new KisPaintDevice(paintDevice->colorSpace());
+    device->convertFromQImage(scaledImage, 0);
+
+    KisPainter painter(paintDevice);
+    painter.bitBlt(overlayRect.topLeft(), device, imageRect);
+    update();
+}
+
 void KisScratchPad::paintPresetImage()
 {
     if(!m_paintLayer) return;
     KisPaintDeviceSP paintDevice = m_paintLayer->paintDevice();
 
+
     QRect overlayRect = widgetToDocument().mapRect(m_cutoutOverlay);
     QRect imageRect(QPoint(), overlayRect.size());
 
     QImage scaledImage = m_presetImage.scaled(overlayRect.size(),
                                               Qt::IgnoreAspectRatio,
                                               Qt::SmoothTransformation);
-
     KisPaintDeviceSP device = new KisPaintDevice(paintDevice->colorSpace());
     device->convertFromQImage(scaledImage, 0);
 
@@ -430,6 +459,19 @@ void KisScratchPad::fillDefault()
     update();
 }
 
+void KisScratchPad::fillTransparent() {
+    if(!m_paintLayer) return;
+    KisPaintDeviceSP paintDevice = m_paintLayer->paintDevice();
+
+    QColor transQColor(0,0,0,0);
+    KoColor transparentColor(transQColor, KoColorSpaceRegistry::instance()->rgb8());
+    transparentColor.setOpacity(0.0);
+
+    paintDevice->setDefaultPixel(transparentColor);
+    paintDevice->clear();
+    update();
+}
+
 void KisScratchPad::fillGradient()
 {
     if(!m_paintLayer) return;
