@@ -106,6 +106,7 @@ KoResourceItemChooser::KoResourceItemChooser(QSharedPointer<KoAbstractResourceSe
     connect(d->model, SIGNAL(afterResourcesLayoutReset()), SLOT(slotAfterResourcesLayoutReset()));
 
     d->view = new KoResourceItemView(this);
+    d->view->setObjectName("ResourceItemview");
     d->view->setModel(d->model);
     d->view->setItemDelegate(new KoResourceItemDelegate(this));
     d->view->setSelectionMode(QAbstractItemView::SingleSelection);
@@ -501,9 +502,9 @@ void KoResourceItemChooser::baseLengthChanged(int length)
     if (d->synced) {
         int resourceCount = d->model->resourcesCount();
         int width = d->view->width();
-        int maxColums = width / length;
+        int maxColumns = width / length;
         int cols = width / (2 * length) + 1;
-        while (cols <= maxColums) {
+        while (cols <= maxColumns) {
             int size = width / cols;
             int rows = ceil(resourceCount / (double)cols);
             if (rows * size < (d->view->height() - 5)) {
@@ -532,6 +533,61 @@ bool KoResourceItemChooser::eventFilter(QObject *object, QEvent *event)
     return QObject::eventFilter(object, event);
 }
 
+void KoResourceItemChooser::configureKineticScrolling(int gesture, int sensitivity, bool scrollbar)
+{
+    QScroller::ScrollerGestureType gestureType;
+
+    switch (gesture) {
+    case 1: {
+        gestureType = QScroller::TouchGesture;
+        break;
+    }
+    case 2: {
+        gestureType = QScroller::LeftMouseButtonGesture;
+        break;
+    }
+    default:
+        return;
+    }
+
+    KoResourceItemView *view = itemView();
+
+    view->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
+    if (!scrollbar) {
+        view->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    }
+
+    QScroller *scroller = QScroller::scroller(view);
+    scroller->grabGesture(view, gestureType);
+
+    QScrollerProperties sp;
+
+    // DragStartDistance seems to be based on meter per second; though it's
+    // not explicitly documented, other QScroller values are in that metric.
+
+    // To start kinetic scrolling, with minimal sensitity, we expect a drag
+    // of 10 mm, with minimum sensitity any > 0 mm.
+
+    const float mm = 0.001f; // 1 millimeter
+    const float resistance = 1.0f - (sensitivity / 100.0f);
+
+    sp.setScrollMetric(QScrollerProperties::DragStartDistance, resistance * 10.0f * mm);
+    sp.setScrollMetric(QScrollerProperties::DragVelocitySmoothingFactor, 1.0f);
+    sp.setScrollMetric(QScrollerProperties::MinimumVelocity, 0.0f);
+    sp.setScrollMetric(QScrollerProperties::AxisLockThreshold, 1.0f);
+    sp.setScrollMetric(QScrollerProperties::MaximumClickThroughVelocity, 0.0f);
+    sp.setScrollMetric(QScrollerProperties::MousePressEventDelay, 1.0f - 0.75f * resistance);
+    sp.setScrollMetric(QScrollerProperties::AcceleratingFlickSpeedupFactor, 1.5f);
+
+    sp.setScrollMetric(QScrollerProperties::VerticalOvershootPolicy, QScrollerProperties::OvershootAlwaysOn);
+    sp.setScrollMetric(QScrollerProperties::OvershootDragResistanceFactor, 0.1);
+    sp.setScrollMetric(QScrollerProperties::OvershootDragDistanceFactor, 0.3);
+    sp.setScrollMetric(QScrollerProperties::OvershootScrollDistanceFactor, 0.1);
+    sp.setScrollMetric(QScrollerProperties::OvershootScrollTime, 0.4);
+
+    scroller->setScrollerProperties(sp);
+}
+
 void KoResourceItemChooser::resizeEvent(QResizeEvent *event)
 {
     QWidget::resizeEvent(event);
