@@ -340,7 +340,7 @@ QMenu *KToolBar::Private::contextMenu(const QPoint &globalPos)
         QList<int> avSizes;
         avSizes << 16 << 22 << 24 << 32 << 48 << 64 << 128 << 256;
 
-        qSort(avSizes);
+        std::sort(avSizes.begin(), avSizes.end());
 
         if (avSizes.count() < 10) {
             // Fixed or threshold type icons
@@ -784,18 +784,6 @@ void KToolBar::Private::slotLockToolBars(bool lock)
     q->setToolBarsLocked(lock);
 }
 
-KToolBar::KToolBar(QWidget *parent, bool isMainToolBar, bool readConfig)
-    : QToolBar(parent),
-      d(new Private(this))
-{
-    d->init(readConfig, isMainToolBar);
-
-    // KToolBar is auto-added to the top area of the main window if parent is a QMainWindow
-    if (QMainWindow *mw = qobject_cast<QMainWindow *>(parent)) {
-        mw->addToolBar(this);
-    }
-}
-
 KToolBar::KToolBar(const QString &objectName, QWidget *parent, bool readConfig)
     : QToolBar(parent),
       d(new Private(this))
@@ -811,25 +799,6 @@ KToolBar::KToolBar(const QString &objectName, QWidget *parent, bool readConfig)
     }
 }
 
-KToolBar::KToolBar(const QString &objectName, QMainWindow *parent, Qt::ToolBarArea area,
-                   bool newLine, bool isMainToolBar, bool readConfig)
-    : QToolBar(parent),
-      d(new Private(this))
-{
-    setObjectName(objectName);
-    d->init(readConfig, isMainToolBar);
-
-    if (newLine) {
-        mainWindow()->addToolBarBreak(area);
-    }
-
-    mainWindow()->addToolBar(area, this);
-
-    if (newLine) {
-        mainWindow()->addToolBarBreak(area);
-    }
-}
-
 KToolBar::~KToolBar()
 {
     delete d->contextLockAction;
@@ -1155,7 +1124,7 @@ void KToolBar::dragMoveEvent(QDragMoveEvent *event)
 
 void KToolBar::dragLeaveEvent(QDragLeaveEvent *event)
 {
-    // Want to clear this even if toolBarsEditable was changed mid-drag (unlikey)
+    // Want to clear this even if toolBarsEditable was changed mid-drag (unlikely)
     delete d->dropIndicatorAction;
     d->dropIndicatorAction = 0L;
     d->actionsBeingDragged.clear();
@@ -1179,7 +1148,7 @@ void KToolBar::dropEvent(QDropEvent *event)
         }
     }
 
-    // Want to clear this even if toolBarsEditable was changed mid-drag (unlikey)
+    // Want to clear this even if toolBarsEditable was changed mid-drag (unlikely)
     delete d->dropIndicatorAction;
     d->dropIndicatorAction = 0L;
     d->actionsBeingDragged.clear();
@@ -1249,7 +1218,7 @@ void KToolBar::mouseMoveEvent(QMouseEvent *event)
 
 void KToolBar::mouseReleaseEvent(QMouseEvent *event)
 {
-    // Want to clear this even if toolBarsEditable was changed mid-drag (unlikey)
+    // Want to clear this even if toolBarsEditable was changed mid-drag (unlikely)
     if (d->dragAction) {
         d->dragAction = 0L;
         event->accept();
