@@ -41,7 +41,7 @@ git_pull_rebase_helper()
 
 
 # Workaround for: On CentOS 6, .pc files in /usr/lib/pkgconfig are not recognized
-# However, this is where .pc files get installed when bulding libraries... (FIXME)
+# However, this is where .pc files get installed when building libraries... (FIXME)
 # I found this by comparing the output of librevenge's "make install" command
 # between Ubuntu and CentOS 6
 ln -sf /usr/share/pkgconfig /usr/lib/pkgconfig
@@ -55,31 +55,6 @@ ln -sf /usr/share/pkgconfig /usr/lib/pkgconfig
 # krita_build -- build directory for krita itself
 # krita.appdir -- install directory for krita and the dependencies
 
-# fetch and build gmic
-if [ ! -d /gmic ] ; then
-	git clone  --depth 1 https://github.com/dtschump/gmic.git
-fi
-
-cd /gmic/
-git_pull_rebase_helper
-cd /
-make -C gmic/src CImg.h gmic_stdlib.h
-
-# fetch and build gmic-qt
-if [ ! -d /gmic-qt ] ; then
-	git clone  --depth 1 https://github.com/c-koi/gmic-qt.git
-fi
-
-cd /gmic-qt/
-git_pull_rebase_helper
-
-cd /
-mkdir gmic-qt-build
-cd  gmic-qt-build
-cmake3 ../gmic-qt -DGMIC_QT_HOST=krita
-cp gmic_krita_qt /krita.appdir/usr/bin
-
-
 # Get Krita
 if [ ! -d /krita ] ; then
 	git clone  --depth 1 https://github.com/KDE/krita.git /krita
