@@ -1,6 +1,6 @@
 #!/bin/bash
 
-RELEASE=3.2.0-rc.3
+RELEASE=3.3.2.1
 
 # Enter a CentOS 6 chroot (you could use other methods)
 # git clone https://github.com/probonopd/AppImageKit.git
@@ -96,10 +96,7 @@ cp gmic_krita_qt /krita.appdir/usr/bin
 
 # fetch and build krita
 cd /
-#wget http://files.kde.org/krita/krita-$RELEASE.tar.gz
-#wget http://www.valdyas.org/~boud/krita-$RELEASE.tar.gz
-#wget http://download.kde.org/unstable/krita/$RELEASE/krita-$RELEASE.tar.gz
-tar -xf krita-$RELEASE.tar.gz
+tar -xf krita-$RELEASE.tar.xz
 cd /krita_build
 cmake3 ../krita-$RELEASE \
     -DCMAKE_INSTALL_PREFIX:PATH=/krita.appdir/usr \
