@@ -273,13 +273,7 @@ void KisAssistantTool::beginPrimaryAction(KoPointerEvent *event)
     Q_FOREACH (KisPaintingAssistantSP assistant, m_canvas->paintingAssistantsDecoration()->assistants()) {
 
         // This code contains the click event behavior.
-        QTransform initialTransform = m_canvas->coordinatesConverter()->documentToWidgetTransform();
-        QPointF actionsPosition = initialTransform.map(assistant->buttonPosition());
-
-
-        // for UI editor widget controls with move, show, and delete -- disregard document transforms like rotating and mirroring.
-        // otherwise the UI controls get awkward to use when they are at 45 degree angles or the order of controls gets flipped backwards
-        QPointF uiMousePosition = initialTransform.map( canvasDecoration->snapToGuide(event, QPointF(), false));
+        QPointF actionsPosition = m_canvas->viewConverter()->documentToView(assistant->buttonPosition());
 
         AssistantEditorData editorShared; // shared position data between assistant tool and decoration
 
@@ -287,12 +281,11 @@ void KisAssistantTool::beginPrimaryAction(KoPointerEvent *event)
         QPointF iconSnapPosition(actionsPosition + editorShared.snapIconPosition);
         QPointF iconDeletePosition(actionsPosition + editorShared.deleteIconPosition);
 
-
         QRectF deleteRect(iconDeletePosition, QSizeF(editorShared.deleteIconSize, editorShared.deleteIconSize));
         QRectF visibleRect(iconSnapPosition, QSizeF(editorShared.snapIconSize, editorShared.snapIconSize));
         QRectF moveRect(iconMovePosition, QSizeF(editorShared.moveIconSize, editorShared.moveIconSize));
 
-        if (moveRect.contains(uiMousePosition)) {
+        if (moveRect.contains(mousePos)) {
             m_assistantDrag = assistant;
             m_cursorStart = event->point;
             m_currentAdjustment = QPointF();
@@ -300,7 +293,7 @@ void KisAssistantTool::beginPrimaryAction(KoPointerEvent *event)
             return;
         }
 
-        if (deleteRect.contains(uiMousePosition)) {
+        if (deleteRect.contains(mousePos)) {
             removeAssistant(assistant);
             if(m_canvas->paintingAssistantsDecoration()->assistants().isEmpty()) {
                 m_internalMode = MODE_CREATION;
@@ -310,7 +303,7 @@ void KisAssistantTool::beginPrimaryAction(KoPointerEvent *event)
             m_canvas->updateCanvas();
             return;
         }
-        if (visibleRect.contains(uiMousePosition)) {
+        if (visibleRect.contains(mousePos)) {
             newAssistantAllowed = false;
             assistant->setSnappingActive(!assistant->isSnappingActive()); // toggle
             assistant->uncache();//this updates the chache of the assistant, very important.
