@@ -22,6 +22,7 @@
 #include <QColor>
 #include <QMimeData>
 #include <QPointer>
+#include <KoResourceModel.h>
 
 #include "kis_layer.h"
 #include "kis_config.h"
@@ -46,6 +47,10 @@
 #include "kis_projection_leaf.h"
 #include "kis_time_range.h"
 
+#include "kis_node_view_color_scheme.h"
+#include "krita_utils.h"
+#include <QApplication>
+
 struct TimelineFramesModel::Private
 {
     Private()
@@ -59,7 +64,7 @@ struct TimelineFramesModel::Private
 
     int activeLayerIndex;
 
-    KisDummiesFacadeBase *dummiesFacade;
+    QPointer<KisDummiesFacadeBase> dummiesFacade;
     KisImageWSP image;
     bool needFinishInsertRows;
     bool needFinishRemoveRows;
@@ -97,10 +102,7 @@ struct TimelineFramesModel::Private
     bool specialKeyframeExists(int row, int column) {
         KisNodeDummy *dummy = converter->dummyFromRow(row);
         if (!dummy) return false;
-
-        QList<KisKeyframeChannel *> channels = dummy->node()->keyframeChannels();
-
-        Q_FOREACH(KisKeyframeChannel *channel, channels) {
+        Q_FOREACH(KisKeyframeChannel *channel, dummy->node()->keyframeChannels()) {
             if (channel->id() != KisKeyframeChannel::Content.id() && channel->keyframeAt(column)) {
                 return true;
             }
@@ -134,6 +136,12 @@ struct TimelineFramesModel::Private
         frame->setColorLabel(color);
     }
 
+    int layerColorLabel(int row) const {
+        KisNodeDummy *dummy = converter->dummyFromRow(row);
+        if (!dummy) return -1;
+        return dummy->node()->colorLabelIndex();
+    }
+
     QVariant layerProperties(int row) const {
         KisNodeDummy *dummy = converter->dummyFromRow(row);
         if (!dummy) return QVariant();
@@ -157,7 +165,8 @@ struct TimelineFramesModel::Private
         KisNodeSP node = dummy->node();
         if (!KisAnimationUtils::supportsContentFrames(node)) return false;
 
-        return KisAnimationUtils::createKeyframeLazy(image, node, KisKeyframeChannel::Content.id(), column, copy);
+        KisAnimationUtils::createKeyframeLazy(image, node, KisKeyframeChannel::Content.id(), column, copy);
+        return true;
     }
 
     bool addNewLayer(int row) {
@@ -206,10 +215,15 @@ void TimelineFramesModel::setNodeManipulationInterface(NodeManipulationInterface
 
 KisNodeSP TimelineFramesModel::nodeAt(QModelIndex index) const
 {
-    return m_d->converter->dummyFromRow(index.row())->node();
+    /**
+     * The dummy might not exist because the user could (quickly) change
+     * active layer and the list of the nodes in m_d->converter will change.
+     */
+    KisNodeDummy *dummy = m_d->converter->dummyFromRow(index.row());
+    return dummy ? dummy->node() : 0;
 }
 
-QList<KisKeyframeChannel *> TimelineFramesModel::channelsAt(QModelIndex index) const
+QMap<QString, KisKeyframeChannel*> TimelineFramesModel::channelsAt(QModelIndex index) const
 {
     KisNodeDummy *srcDummy = m_d->converter->dummyFromRow(index.row());
     return srcDummy->node()->keyframeChannels();
@@ -219,7 +233,7 @@ void TimelineFramesModel::setDummiesFacade(KisDummiesFacadeBase *dummiesFacade,
 {
     KisDummiesFacadeBase *oldDummiesFacade = m_d->dummiesFacade;
 
-    if (m_d->dummiesFacade) {
+    if (m_d->dummiesFacade && m_d->image) {
         m_d->image->animationInterface()->disconnect(this);
         m_d->image->disconnect(this);
         m_d->dummiesFacade->disconnect(this);
@@ -244,7 +258,8 @@ void TimelineFramesModel::setDummiesFacade(KisDummiesFacadeBase *dummiesFacade,
     }
 
     if (m_d->dummiesFacade != oldDummiesFacade) {
-        reset();
+        beginResetModel();
+        endResetModel();
     }
 
     if (m_d->dummiesFacade) {
@@ -321,16 +336,32 @@ QVariant TimelineFramesModel::data(const QModelIndex &index, int role) const
     case SpecialKeyframeExists: {
         return m_d->specialKeyframeExists(index.row(), index.column());
     }
-    case ColorLabel: {
+    case FrameColorLabelIndexRole: {
         int label = m_d->frameColorLabel(index.row(), index.column());
         return label > 0 ? label : QVariant();
     }
     case Qt::DisplayRole: {
-        return QVariant();
+        return m_d->layerName(index.row());
     }
     case Qt::TextAlignmentRole: {
         return QVariant(Qt::AlignHCenter | Qt::AlignVCenter);
     }
+    case KoResourceModel::LargeThumbnailRole: {
+        KisNodeDummy *dummy = m_d->converter->dummyFromRow(index.row());
+        if (!dummy) {
+            return  QVariant();
+        }
+        const int maxSize = 200;
+
+        QSize size = dummy->node()->extent().size();
+        size.scale(maxSize, maxSize, Qt::KeepAspectRatio);
+        if (size.width() == 0 || size.height() == 0) {
+            // No thumbnail can be shown if there isn't width or height...
+            return QVariant();
+        }
+        QImage image(dummy->node()->createThumbnailForFrame(size.width(), size.height(), index.column()));
+        return image;
+    }
     }
 
     return ModelWithExternalNotifications::data(index, role);
@@ -362,7 +393,7 @@ bool TimelineFramesModel::setData(const QModelIndex &index, const QVariant &valu
         }
         break;
     }
-    case ColorLabel: {
+    case FrameColorLabelIndexRole: {
         m_d->setFrameColorLabel(index.row(), index.column(), value.toInt());
     }
         break;
@@ -428,6 +459,18 @@ QVariant TimelineFramesModel::headerData(int section, Qt::Orientation orientatio
             if (!dummy) return QVariant();
             return dummy->node()->useInTimeline();
         }
+        case Qt::BackgroundRole: {
+            int label = m_d->layerColorLabel(section);
+            if (label > 0) {
+                KisNodeViewColorScheme scm;
+                QColor color = scm.colorLabel(label);
+                QPalette pal = qApp->palette();
+                color = KritaUtils::blendColors(color, pal.color(QPalette::Button), 0.3);
+                return QBrush(color);
+            } else {
+                return QVariant();
+            }
+        }
         }
     }
 
@@ -631,24 +674,14 @@ bool TimelineFramesModel::createFrame(const QModelIndex &dstIndex)
 {
     if (!dstIndex.isValid()) return false;
 
-    bool result = m_d->addKeyframe(dstIndex.row(), dstIndex.column(), false);
-    if (result) {
-        emit dataChanged(dstIndex, dstIndex);
-    }
-
-    return result;
+    return m_d->addKeyframe(dstIndex.row(), dstIndex.column(), false);
 }
 
 bool TimelineFramesModel::copyFrame(const QModelIndex &dstIndex)
 {
     if (!dstIndex.isValid()) return false;
 
-    bool result = m_d->addKeyframe(dstIndex.row(), dstIndex.column(), true);
-    if (result) {
-        emit dataChanged(dstIndex, dstIndex);
-    }
-
-    return result;
+    return m_d->addKeyframe(dstIndex.row(), dstIndex.column(), true);
 }
 
 QString TimelineFramesModel::audioChannelFileName() const
