@@ -91,11 +91,47 @@
 
 #include "ui_wdglayerbox.h"
 
-inline void KisLayerBox::connectActionToButton(KisViewManager* view, QAbstractButton *button, const QString &id)
-{
-    if (!view || !button) return;
+#include <QProxyStyle>
+#include <QPainter>
 
-    KisAction *action = view->actionManager()->actionByName(id);
+class KisLayerBoxStyle : public QProxyStyle
+{
+public:
+    KisLayerBoxStyle(QStyle *baseStyle = 0) : QProxyStyle(baseStyle) {}
+
+    void drawPrimitive(PrimitiveElement element, const QStyleOption *option,
+                       QPainter *painter, const QWidget *widget) const
+    {
+        if (element == QStyle::PE_IndicatorItemViewItemDrop)
+        {
+            QColor color(widget->palette().color(QPalette::Highlight).lighter());
+
+            if (option->rect.height() == 0) {
+                QBrush brush(color);
+
+                QRect r(option->rect);
+                r.setTop(r.top() - 2);
+                r.setBottom(r.bottom() + 2);
+
+                painter->fillRect(r, brush);
+            } else {
+                color.setAlpha(200);
+                QBrush brush(color);
+                painter->fillRect(option->rect, brush);
+            }
+        }
+        else
+        {
+            QProxyStyle::drawPrimitive(element, option, painter, widget);
+        }
+    }
+};
+
+inline void KisLayerBox::connectActionToButton(KisViewManager* viewManager, QAbstractButton *button, const QString &id)
+{
+    if (!viewManager || !button) return;
+
+    KisAction *action = viewManager->actionManager()->actionByName(id);
 
     if (!action) return;
 
@@ -125,6 +161,8 @@ KisLayerBox::KisLayerBox()
 
     m_wdgLayerBox->setupUi(mainWidget);
 
+    m_wdgLayerBox->listLayers->setStyle(new KisLayerBoxStyle(m_wdgLayerBox->listLayers->style()));
+
     connect(m_wdgLayerBox->listLayers,
             SIGNAL(contextMenuRequested(const QPoint&, const QModelIndex&)),
             this, SLOT(slotContextMenuRequested(const QPoint&, const QModelIndex&)));
@@ -295,7 +333,7 @@ void KisLayerBox::setMainWindow(KisViewManager* kisview)
 
 void KisLayerBox::setCanvas(KoCanvasBase *canvas)
 {
-    if(m_canvas == canvas)
+    if (m_canvas == canvas)
         return;
 
     setEnabled(canvas != 0);
@@ -367,11 +405,7 @@ void KisLayerBox::setCanvas(KoCanvasBase *canvas)
         m_newLayerMenu->addSeparator();
         addActionToMenu(m_newLayerMenu, "add_new_transparency_mask");
         addActionToMenu(m_newLayerMenu, "add_new_filter_mask");
-
-        if (!KisConfig().disableColorizeMaskFeature()) {
-            addActionToMenu(m_newLayerMenu, "add_new_colorize_mask");
-        }
-
+        addActionToMenu(m_newLayerMenu, "add_new_colorize_mask");
         addActionToMenu(m_newLayerMenu, "add_new_transform_mask");
         addActionToMenu(m_newLayerMenu, "add_new_selection_mask");
     }
@@ -431,14 +465,9 @@ void KisLayerBox::updateUI()
     m_wdgLayerBox->doubleOpacity->setEnabled(activeNode && activeNode->isEditable(false));
 
     m_wdgLayerBox->cmbComposite->setEnabled(activeNode && activeNode->isEditable(false));
+    m_wdgLayerBox->cmbComposite->validate(m_image->colorSpace());
 
     if (activeNode) {
-        if (m_nodeManager->activePaintDevice()) {
-            slotFillCompositeOps(m_nodeManager->activeColorSpace());
-        } else {
-            slotFillCompositeOps(m_image->colorSpace());
-        }
-
         if (activeNode->inherits("KisColorizeMask") ||
                 activeNode->inherits("KisLayer")) {
 
@@ -466,7 +495,7 @@ void KisLayerBox::updateUI()
 
 
 /**
- * This method is callen *only* when non-GUI code requested the
+ * This method is called *only* when non-GUI code requested the
  * change of the current node
  */
 void KisLayerBox::setCurrentNode(KisNodeSP node)
@@ -505,11 +534,6 @@ void KisLayerBox::slotSetCompositeOp(const KoCompositeOp* compositeOp)
     m_wdgLayerBox->cmbComposite->blockSignals(false);
 }
 
-void KisLayerBox::slotFillCompositeOps(const KoColorSpace* colorSpace)
-{
-    m_wdgLayerBox->cmbComposite->validate(colorSpace);
-}
-
 // range: 0-100
 void KisLayerBox::slotSetOpacity(double opacity)
 {
@@ -568,21 +592,17 @@ void KisLayerBox::slotContextMenuRequested(const QPoint &pos, const QModelIndex
             addActionToMenu(groupMenu, "create_quick_group");
             addActionToMenu(groupMenu, "create_quick_clipping_group");
             addActionToMenu(groupMenu, "quick_ungroup");
-            QMenu *locksMenu = menu.addMenu(i18n("&Locks && visibility"));
-            addActionToMenu(locksMenu, "toggle_layer_lock");
+            QMenu *locksMenu = menu.addMenu(i18n("&Toggle Locks && Visibility"));
             addActionToMenu(locksMenu, "toggle_layer_visibility");
-            addActionToMenu(locksMenu, "toggle_layer_alpha_lock");
+            addActionToMenu(locksMenu, "toggle_layer_lock");
             addActionToMenu(locksMenu, "toggle_layer_inherit_alpha");
+            addActionToMenu(locksMenu, "toggle_layer_alpha_lock");
 
             if (singleLayer) {
                 QMenu *addLayerMenu = menu.addMenu(i18n("&Add"));
                 addActionToMenu(addLayerMenu, "add_new_transparency_mask");
                 addActionToMenu(addLayerMenu, "add_new_filter_mask");
-
-                if (!KisConfig().disableColorizeMaskFeature()) {
-                    addActionToMenu(addLayerMenu, "add_new_colorize_mask");
-                }
-
+                addActionToMenu(addLayerMenu, "add_new_colorize_mask");
                 addActionToMenu(addLayerMenu, "add_new_transform_mask");
                 addActionToMenu(addLayerMenu, "add_new_selection_mask");
 
@@ -591,6 +611,7 @@ void KisLayerBox::slotContextMenuRequested(const QPoint &pos, const QModelIndex
                 addActionToMenu(convertToMenu, "convert_to_transparency_mask");
                 addActionToMenu(convertToMenu, "convert_to_filter_mask");
                 addActionToMenu(convertToMenu, "convert_to_selection_mask");
+                addActionToMenu(convertToMenu, "convert_layer_to_file_layer");
 
                 QMenu *splitAlphaMenu = menu.addMenu(i18n("S&plit Alpha"));
                 addActionToMenu(splitAlphaMenu, "split_alpha_into_mask");
@@ -600,9 +621,9 @@ void KisLayerBox::slotContextMenuRequested(const QPoint &pos, const QModelIndex
 
             menu.addSeparator();
 
-            if (singleLayer) {
-                addActionToMenu(&menu, "show_in_timeline");
+            addActionToMenu(&menu, "show_in_timeline");
 
+            if (singleLayer) {
                 KisNodeSP node = m_filteringModel->nodeFromIndex(index);
                 if (node && !node->inherits("KisTransformMask")) {
                     addActionToMenu(&menu, "isolate_layer");
@@ -615,12 +636,6 @@ void KisLayerBox::slotContextMenuRequested(const QPoint &pos, const QModelIndex
     }
 }
 
-void KisLayerBox::slotMergeLayer()
-{
-    if (!m_canvas) return;
-    m_nodeManager->mergeLayer();
-}
-
 void KisLayerBox::slotMinimalView()
 {
     m_wdgLayerBox->listLayers->setDisplayMode(KisNodeView::MinimalMode);
