@@ -18,16 +18,18 @@
 #ifndef H_IMAGEDOCKER_DOCK_H_
 #define H_IMAGEDOCKER_DOCK_H_
 
+#include <QPointer>
 #include <QDockWidget>
 #include <KoCanvasObserverBase.h>
 #include <QStringList>
 #include <QPixmap>
 #include <QMap>
 
+#include <KoCanvasBase.h>
+
 class QModelIndex;
 class QFileSystemModel;
 class QButtonGroup;
-class KoCanvasBase;
 class ImageFilter;
 class ImageStripScene;
 class ImageListModel;
@@ -83,7 +85,8 @@ protected:
     void dragEnterEvent(QDragEnterEvent *event) override;
     void dropEvent(QDropEvent *event) override;
     void showEvent(QShowEvent *) override;
-private:
+    void hideEvent(QHideEvent *event) override;
+ private:
     void addCurrentPathToHistory();
     void updatePath(const QString& path);
     qint64 generateImageID() const;
@@ -91,10 +94,14 @@ private:
     bool isImageLoaded() const { return m_currImageID != -1; }
     void setZoom(const ImageInfo& info);
     
+    void saveConfigState();
+    void loadConfigState();
+
+
 private:
     QFileSystemModel*      m_model;
     QButtonGroup*          m_zoomButtons;
-    KoCanvasBase*          m_canvas;
+    QPointer<KoCanvasBase> m_canvas;
     ImageFilter*           m_proxyModel;
     ImageListModel*        m_imgListModel;
     QStringList            m_history;
