@@ -23,7 +23,7 @@
 
 #include <QStringList>
 #include <QDir>
-#include <QDesktopServices>
+#include <QStandardPaths>
 #include <QMessageBox>
 
 #include <klocalizedstring.h>
@@ -62,12 +62,11 @@ Imagesplit::~Imagesplit()
 
 bool Imagesplit::saveAsImage(const QRect &imgSize, const QString &mimeType, const QString &url)
 {
-    KisImageWSP image = m_view->image();
+    KisImageSP image = m_view->image();
 
     KisDocument *document = KisPart::instance()->createDocument();
-    document->prepareForImport();
 
-    KisImageWSP dst = new KisImage(document->createUndoStore(), imgSize.width(), imgSize.height(), image->colorSpace(), image->objectName());
+    KisImageSP dst = new KisImage(document->createUndoStore(), imgSize.width(), imgSize.height(), image->colorSpace(), image->objectName());
     dst->setResolution(image->xRes(), image->yRes());
     document->setCurrentImage(dst);
 
@@ -77,9 +76,8 @@ bool Imagesplit::saveAsImage(const QRect &imgSize, const QString &mimeType, cons
 
     dst->addNode(paintLayer, KisNodeSP(0));
     dst->refreshGraph();
-    document->setOutputMimeType(mimeType.toLatin1());
     document->setFileBatchMode(true);
-    if (!document->exportDocument(QUrl::fromLocalFile(url))) {
+    if (!document->exportDocumentSync(QUrl::fromLocalFile(url), mimeType.toLatin1())) {
         if (document->errorMessage().isEmpty()) {
             QMessageBox::critical(0, i18nc("@title:window", "Krita"), i18n("Could not save\n%1", document->localFilePath()));
         } else {
@@ -140,7 +138,7 @@ void Imagesplit::slotImagesplit()
         if (dlgImagesplit->autoSave()) {
             KoFileDialog dialog(m_view->mainWindow(), KoFileDialog::OpenDirectory, "OpenDocument");
             dialog.setCaption(i18n("Save Image on Split"));
-            dialog.setDefaultDir(QDesktopServices::storageLocation(QDesktopServices::PicturesLocation));
+            dialog.setDefaultDir(QStandardPaths::writableLocation(QStandardPaths::PicturesLocation));
             QStringList mimeFilter = m_view->document()->importExportManager()->mimeFilter(KisImportExportManager::Export);
             QString defaultMime = QString::fromLatin1(m_view->document()->mimeType());
             dialog.setMimeTypeFilters(mimeFilter, defaultMime);
@@ -181,12 +179,12 @@ void Imagesplit::slotImagesplit()
                 for (int j = 0; j < (numHorizontalLines + 1); j++) {
                     KoFileDialog dialog(m_view->mainWindow(), KoFileDialog::SaveFile, "OpenDocument");
                     dialog.setCaption(i18n("Save Image on Split"));
-                    dialog.setDefaultDir(QDesktopServices::storageLocation(QDesktopServices::PicturesLocation));
+                    dialog.setDefaultDir(QStandardPaths::writableLocation(QStandardPaths::PicturesLocation));
                     dialog.setMimeTypeFilters(listMimeFilter, defaultMime);
 
                     QUrl url = QUrl::fromUserInput(dialog.filename());
 
-                    QString mimefilter = KisMimeDatabase::mimeTypeForFile(url.toLocalFile());
+                    QString mimefilter = KisMimeDatabase::mimeTypeForFile(url.toLocalFile(), false);
 
                     if (url.isEmpty())
                         return;
