@@ -91,7 +91,7 @@ void KisFilterThreshold::processImpl(KisPaintDeviceSP device,
     KisSequentialIterator it(device, applyRect);
     int p = 0;
     const int pixelSize = device->colorSpace()->pixelSize();
-    do {
+    while (it.nextPixel()) {
         if (device->colorSpace()->intensity8(it.oldRawData()) > threshold) {
             memcpy(it.rawData(), white.data(), pixelSize);
         }
@@ -101,12 +101,12 @@ void KisFilterThreshold::processImpl(KisPaintDeviceSP device,
 
         if (progressUpdater) progressUpdater->setValue(p++);
 
-    } while (it.nextPixel());
+    }
 
 }
 
 
-KisFilterConfigurationSP KisFilterThreshold::factoryConfiguration(const KisPaintDeviceSP) const
+KisFilterConfigurationSP KisFilterThreshold::factoryConfiguration() const
 {
     KisFilterConfigurationSP config = new KisFilterConfiguration("threshold", 1);
     config->setProperty("threshold", 128);
