@@ -62,11 +62,11 @@ KisConfigWidget * KisUnsharpFilter::createConfigurationWidget(QWidget* parent, c
     return new KisWdgUnsharp(parent);
 }
 
-KisFilterConfiguration* KisUnsharpFilter::factoryConfiguration(const KisPaintDeviceSP) const
+KisFilterConfigurationSP KisUnsharpFilter::factoryConfiguration() const
 {
-    KisFilterConfiguration* config = new KisFilterConfiguration(id().id(), 1);
+    KisFilterConfigurationSP config = new KisFilterConfiguration(id().id(), 1);
     config->setProperty("halfSize", 1);
-    config->setProperty("amount", 50);
+    config->setProperty("amount", 0.5);
     config->setProperty("threshold", 0);
     config->setProperty("lightnessOnly", true);
     return config;
@@ -74,7 +74,7 @@ KisFilterConfiguration* KisUnsharpFilter::factoryConfiguration(const KisPaintDev
 
 void KisUnsharpFilter::processImpl(KisPaintDeviceSP device,
                                    const QRect& applyRect,
-                                   const KisFilterConfiguration* config,
+                                   const KisFilterConfigurationSP _config,
                                    KoUpdater* progressUpdater
                                    ) const
 {
@@ -91,14 +91,14 @@ void KisUnsharpFilter::processImpl(KisPaintDeviceSP device,
         filterUpdater = updater->startSubtask();
     }
 
-    if (!config) config = new KisFilterConfiguration(id().id(), 1);
+    KisFilterConfigurationSP config = _config ? _config : new KisFilterConfiguration(id().id(), 1);
 
     QVariant value;
 
     KisLodTransformScalar t(device);
 
     const qreal halfSize = t.scale(config->getProperty("halfSize", value) ? value.toDouble() : 1.0);
-    const qreal amount = (config->getProperty("amount", value)) ? value.toDouble() : 25;
+    const qreal amount = (config->getProperty("amount", value)) ? value.toDouble() : 0.5;
     const uint threshold = (config->getProperty("threshold", value)) ? value.toUInt() : 0;
     const uint lightnessOnly = (config->getProperty("lightnessOnly", value)) ? value.toBool() : true;
 
@@ -148,7 +148,7 @@ void KisUnsharpFilter::processRaw(KisPaintDeviceSP device,
     for (int j = 0; j < rect.height(); j++) {
         do {
             quint8 diff = cs->difference(dstIt->oldRawData(), dstIt->rawDataConst());
-            if (diff > threshold) {
+            if (diff >= threshold) {
                 memcpy(colors[0], dstIt->oldRawData(), pixelSize);
                 memcpy(colors[1], dstIt->rawDataConst(), pixelSize);
                 convolutionOp->convolveColors(colors, weights, dstIt->rawData(), factor, 0, 2, channelFlags);
@@ -185,7 +185,7 @@ void KisUnsharpFilter::processLightnessOnly(KisPaintDeviceSP device,
     for (int j = 0; j < rect.height(); j++) {
         do {
             quint8 diff = cs->differenceA(dstIt->oldRawData(), dstIt->rawDataConst());
-            if (diff > threshold) {
+            if (diff >= threshold) {
                 cs->toLabA16(dstIt->oldRawData(), (quint8*)labColorSrc, 1);
                 cs->toLabA16(dstIt->rawDataConst(), (quint8*)labColorDst, 1);
 
@@ -208,7 +208,7 @@ void KisUnsharpFilter::processLightnessOnly(KisPaintDeviceSP device,
     }
 }
 
-QRect KisUnsharpFilter::neededRect(const QRect & rect, const KisFilterConfiguration* config, int lod) const
+QRect KisUnsharpFilter::neededRect(const QRect & rect, const KisFilterConfigurationSP config, int lod) const
 {
     KisLodTransformScalar t(lod);
 
@@ -218,7 +218,7 @@ QRect KisUnsharpFilter::neededRect(const QRect & rect, const KisFilterConfigurat
     return rect.adjusted(-halfSize * 2, -halfSize * 2, halfSize * 2, halfSize * 2);
 }
 
-QRect KisUnsharpFilter::changedRect(const QRect & rect, const KisFilterConfiguration* config, int lod) const
+QRect KisUnsharpFilter::changedRect(const QRect & rect, const KisFilterConfigurationSP config, int lod) const
 {
     KisLodTransformScalar t(lod);
 
