@@ -29,9 +29,9 @@ class KisBMPImport : public KisImportExportFilter
     Q_OBJECT
 public:
     KisBMPImport(QObject *parent, const QVariantList &);
-    virtual ~KisBMPImport();
+    ~KisBMPImport() override;
 public:
-    virtual KisImportExportFilter::ConversionStatus convert(const QByteArray& from, const QByteArray& to, KisPropertiesConfigurationSP configuration = 0);
+    KisImportExportFilter::ConversionStatus convert(KisDocument *document, QIODevice *io,  KisPropertiesConfigurationSP configuration = 0) override;
 };
 
 #endif
