@@ -23,15 +23,37 @@
 #include <QVariant>
 
 #include <KisImportExportFilter.h>
+#include <ui_wdg_export_gih.h>
+#include <kis_config_widget.h>
+#include <kis_properties_configuration.h>
+
+class KisWdgOptionsBrush : public KisConfigWidget, public Ui::WdgExportGih
+{
+    Q_OBJECT
+
+public:
+    KisWdgOptionsBrush(QWidget *parent)
+        : KisConfigWidget(parent)
+    {
+        setupUi(this);
+    }
+
+    void setConfiguration(const KisPropertiesConfigurationSP  cfg) override;
+    KisPropertiesConfigurationSP configuration() const override;
+};
+
 
 class KisBrushExport : public KisImportExportFilter
 {
     Q_OBJECT
 public:
     KisBrushExport(QObject *parent, const QVariantList &);
-    virtual ~KisBrushExport();
-public:
-    virtual KisImportExportFilter::ConversionStatus convert(const QByteArray& from, const QByteArray& to, KisPropertiesConfigurationSP configuration = 0);
+    ~KisBrushExport() override;
+    KisImportExportFilter::ConversionStatus convert(KisDocument *document, QIODevice *io,  KisPropertiesConfigurationSP configuration = 0) override;
+    KisPropertiesConfigurationSP defaultConfiguration(const QByteArray& from = "", const QByteArray& to = "") const override;
+    KisConfigWidget *createConfigurationWidget(QWidget *parent, const QByteArray& from = "", const QByteArray& to = "") const override;
+
+    void initializeCapabilities() override;
 };
 
 #endif
