@@ -188,7 +188,7 @@ bool PSDLayerMaskSection::readLayerInfoImpl(QIODevice* io)
                 channelInfo->channelDataLength = channelInfo->channelDataLength - (channelInfo->channelDataStart - channelStartPos);
 
                 dbgFile << "\t\tchannel record" << j << "for layer" << i << "with id" << channelInfo->channelId
-                        << "starting postion" << channelInfo->channelDataStart
+                        << "starting position" << channelInfo->channelDataStart
                         << "with length" << channelInfo->channelDataLength
                         << "and has compression type" << channelInfo->compressionType;
 
@@ -230,8 +230,6 @@ bool PSDLayerMaskSection::readImpl(QIODevice* io)
         return true;
     }
 
-    KIS_ASSERT_RECOVER(m_header.version == 1) { return false; }
-
     if (!readLayerInfoImpl(io)) {
         return false;
     }
@@ -369,7 +367,7 @@ KisNodeSP findOnlyTransparencyMask(KisNodeSP node, FlattenedNode::Type type)
         return 0;
     }
 
-    KisLayer *layer = dynamic_cast<KisLayer*>(node.data());
+    KisLayer *layer = qobject_cast<KisLayer*>(node.data());
     QList<KisEffectMaskSP> masks = layer->effectMasks();
 
     if (masks.size() != 1) return 0;
@@ -564,7 +562,8 @@ void PSDLayerMaskSection::writeImpl(QIODevice* io, KisNodeSP rootLayer)
                                    onlyTransparencyMask,
                                    maskRect,
                                    sectionType,
-                                   stylesXmlDoc);
+                                   stylesXmlDoc,
+                                   node->inherits("KisGroupLayer"));
             }
 
             dbgFile << "start writing layer pixel data" << io->pos();
