@@ -21,7 +21,7 @@
  */
 
 #include "kis_brushop_settings_widget.h"
-#include <kis_brush_based_paintop_settings.h>
+#include <KisBrushOpSettings.h>
 #include <kis_pressure_darken_option.h>
 #include <kis_pressure_opacity_option.h>
 #include <kis_pressure_flow_option.h>
@@ -40,11 +40,15 @@
 #include <kis_compositeop_option.h>
 #include <kis_pressure_flow_opacity_option_widget.h>
 #include <kis_pressure_spacing_option_widget.h>
+#include <kis_pressure_rate_option.h>
 #include "kis_texture_option.h"
 #include "kis_curve_option_widget.h"
 #include <kis_pressure_mirror_option_widget.h>
 #include "kis_pressure_texture_strength_option.h"
+#include <KisMaskingBrushOption.h>
 
+#include <KisPrefixedPaintOpOptionWrapper.h>
+#include <KisPaintopSettingsIds.h>
 
 KisBrushOpSettingsWidget::KisBrushOpSettingsWidget(QWidget* parent)
     : KisBrushBasedPaintopOptionWidget(parent)
@@ -75,11 +79,51 @@ KisBrushOpSettingsWidget::KisBrushOpSettingsWidget(QWidget* parent)
     addPaintOpOption(new KisCurveOptionWidget(KisPressureHSVOption::createSaturationOption(), KisPressureHSVOption::saturationMinLabel(), KisPressureHSVOption::saturationmaxLabel()), i18n("Saturation"));
     addPaintOpOption(new KisCurveOptionWidget(KisPressureHSVOption::createValueOption(), KisPressureHSVOption::valueMinLabel(), KisPressureHSVOption::valuemaxLabel()), i18n("Value"));
     addPaintOpOption(new KisAirbrushOption(false), i18n("Airbrush"));
-    addPaintOpOption(new KisPaintActionTypeOption(), i18n("Painting Mode"));
+    addPaintOpOption(new KisCurveOptionWidget(new KisPressureRateOption(), i18n("0%"), i18n("100%")), i18n("Rate"));
+
+
+    KisPaintActionTypeOption *actionTypeOption = new KisPaintActionTypeOption();
+    addPaintOpOption(actionTypeOption, i18n("Painting Mode"));
 
     addPaintOpOption(new KisTextureOption(), i18n("Pattern"));
     addPaintOpOption(new KisCurveOptionWidget(new KisPressureTextureStrengthOption(), i18n("Weak"), i18n("Strong")), i18n("Strength"));
 
+    KisMaskingBrushOption::MasterBrushSizeAdapter sizeAdapter =
+        [this] () { return this->brush()->userEffectiveSize(); };
+
+    KisMaskingBrushOption *maskingOption = new KisMaskingBrushOption(sizeAdapter);
+    addPaintOpOption(maskingOption, i18n("Brush Tip"));
+
+    connect(maskingOption, SIGNAL(sigCheckedChanged(bool)),
+            actionTypeOption, SLOT(slotForceWashMode(bool)));
+
+    {
+        KisCurveOption *maskingSizeOption = new KisPressureSizeOption();
+        maskingSizeOption->setChecked(false);
+
+        addPaintOpOption(
+                    new KisPrefixedPaintOpOptionWrapper<KisCurveOptionWidget>(
+                        KisPaintOpUtils::MaskingBrushPresetPrefix,
+                        maskingSizeOption,
+                        i18n("0%"), i18n("100%")),
+                    i18n("Size"), KisPaintOpOption::MASKING_BRUSH);
+    }
+
+    addPaintOpOption(
+        new KisPrefixedPaintOpOptionWrapper<KisPressureMirrorOptionWidget>(
+            KisPaintOpUtils::MaskingBrushPresetPrefix),
+        i18n("Mirror"), KisPaintOpOption::MASKING_BRUSH);
+
+    addPaintOpOption(
+        new KisPrefixedPaintOpOptionWrapper<KisCurveOptionWidget>(
+            KisPaintOpUtils::MaskingBrushPresetPrefix,
+            new KisPressureRotationOption(), i18n("-180°"), i18n("180°")),
+        i18n("Rotation"), KisPaintOpOption::MASKING_BRUSH);
+
+    addPaintOpOption(
+        new KisPrefixedPaintOpOptionWrapper<KisPressureScatterOptionWidget>(
+            KisPaintOpUtils::MaskingBrushPresetPrefix),
+        i18n("Scatter"), KisPaintOpOption::MASKING_BRUSH);
 }
 
 KisBrushOpSettingsWidget::~KisBrushOpSettingsWidget()
@@ -88,7 +132,7 @@ KisBrushOpSettingsWidget::~KisBrushOpSettingsWidget()
 
 KisPropertiesConfigurationSP KisBrushOpSettingsWidget::configuration() const
 {
-    KisBrushBasedPaintOpSettingsSP config = new KisBrushBasedPaintOpSettings();
+    KisBrushBasedPaintOpSettingsSP config = new KisBrushOpSettings();
     config->setOptionsWidget(const_cast<KisBrushOpSettingsWidget*>(this));
     config->setProperty("paintop", "paintbrush"); // XXX: make this a const id string
     writeConfiguration(config);
