@@ -30,12 +30,12 @@
 #include <kis_vec.h>
 
 #include <kis_hairy_ink_option.h>
-#include <kis_hairy_shape_option.h>
 #include <kis_hairy_bristle_option.h>
 #include <kis_brush_option.h>
 #include <kis_brush_based_paintop_settings.h>
 #include <kis_fixed_paint_device.h>
 #include <kis_lod_transform.h>
+#include <kis_spacing_information.h>
 
 
 #include "kis_brush.h"
@@ -49,7 +49,7 @@ KisHairyPaintOp::KisHairyPaintOp(const KisPaintOpSettingsSP settings, KisPainter
     m_dev = node ? node->paintDevice() : 0;
 
     KisBrushOption brushOption;
-    brushOption.readOptionSettingForceCopy(settings);
+    brushOption.readOptionSetting(settings);
     KisBrushSP brush = brushOption.brush();
     KisFixedPaintDeviceSP dab = cachedDab(painter->device()->compositionSourceColorSpace());
     if (brush->brushType() == IMAGE || brush->brushType() == PIPE_IMAGE) {
@@ -103,6 +103,11 @@ void KisHairyPaintOp::loadSettings(const KisBrushBasedPaintOpSettings *settings)
 
 
 KisSpacingInformation KisHairyPaintOp::paintAt(const KisPaintInformation& info)
+{
+    return updateSpacingImpl(info);
+}
+
+KisSpacingInformation KisHairyPaintOp::updateSpacingImpl(const KisPaintInformation &info) const
 {
     Q_UNUSED(info);
     return KisSpacingInformation(0.5);
