@@ -236,10 +236,6 @@ void KisToolMove::activate(ToolActivation toolActivation, const QSet<KoShape*> &
         }
     }
 
-    if (image()->globalSelection()) {
-        totalBounds &= image()->globalSelection()->selectedRect();
-    }
-
     m_startPosition = totalBounds.topLeft();
 
     if (m_optionsWidget)
@@ -268,15 +264,8 @@ void KisToolMove::initHandles(const KisNodeList &nodes)
 {
     m_handlesRect = QRect();
     for (KisNodeSP node : nodes) {
-        node->exactBounds();
         m_handlesRect |= node->exactBounds();
     }
-    if (image()->globalSelection()) {
-        m_handlesRect &= image()->globalSelection()->selectedRect();
-    }
-    if (m_handlesRect.topLeft() != m_startPosition) {
-        m_handlesRect.moveTopLeft(m_startPosition);
-    }
 }
 
 void KisToolMove::deactivate()
@@ -561,10 +550,6 @@ void KisToolMove::slotNodeChanged(KisNodeList nodes)
         }
     }
 
-    if (image()->globalSelection()) {
-        totalBounds &= image()->globalSelection()->selectedRect();
-    }
-
     m_startPosition = totalBounds.topLeft();
 
     if (m_optionsWidget)
