@@ -55,7 +55,7 @@ public:
      * the tool. See bug 362659
      * @return false
      */
-    bool wantsAutoScroll() const override {
+    bool wantsAutoScroll() const {
         return false;
     }
 
