@@ -110,6 +110,30 @@ enum BooleanOp {
 
 }
 
+QPolygonF selectionPolygon(KoSelection *selection)
+{
+    QPolygonF result;
+
+    QList<KoShape*> selectedShapes = selection->selectedShapes();
+
+    if (!selectedShapes.size()) {
+        return result;
+    }
+
+    if (selectedShapes.size() > 1) {
+        QTransform matrix = selection->absoluteTransformation(0);
+        result = matrix.map(QPolygonF(QRectF(QPointF(0, 0), selection->size())));
+    } else {
+        KoShape *selectedShape = selectedShapes.first();
+        QTransform matrix = selectedShape->absoluteTransformation(0);
+        result = matrix.map(QPolygonF(QRectF(QPointF(0, 0), selectedShape->size())));
+    }
+
+    return result;
+}
+
+
+
 class NopInteractionStrategy : public KoInteractionStrategy
 {
 public:
@@ -677,12 +701,12 @@ void DefaultTool::paint(QPainter &painter, const KoViewConverter &converter)
     // this tool only works on a vector layer right now, so give a warning if another layer type is trying to use it
     KisNodeSP currentNode = canvas()->resourceManager()->resource(KisCanvasResourceProvider::CurrentKritaNode).value<KisNodeWSP>();
 
-    if (currentNode.isNull() || (!currentNode->inherits("KisShapeLayer") && !currentNode->inherits("KisSelectionMask"))) {
+    if (currentNode.isNull() || !currentNode->inherits("KisShapeLayer")) {
 
         KisCanvas2 * kiscanvas = static_cast<KisCanvas2*>(canvas());
         kiscanvas->viewManager()->showFloatingMessage(
             i18n("This tool only works on vector layers. You probably want the move tool."),
-            QIcon(), 500, KisFloatingMessage::Medium, Qt::AlignCenter);
+            QIcon(), 2000, KisFloatingMessage::Medium, Qt::AlignCenter);
 
         return;
     }
@@ -1009,17 +1033,13 @@ void DefaultTool::selectionGroup()
 
     QList<KoShape *> selectedShapes = selection->selectedEditableShapes();
     std::sort(selectedShapes.begin(), selectedShapes.end(), KoShape::compareShapeZIndex);
-    if (selectedShapes.isEmpty()) return;
-
-    const int groupZIndex = selectedShapes.last()->zIndex();
 
     KoShapeGroup *group = new KoShapeGroup();
-    group->setZIndex(groupZIndex);
     // TODO what if only one shape is left?
     KUndo2Command *cmd = new KUndo2Command(kundo2_i18n("Group shapes"));
     new KoKeepShapesSelectedCommand(selectedShapes, {}, selection, false, cmd);
     canvas()->shapeController()->addShapeDirect(group, 0, cmd);
-    new KoShapeGroupCommand(group, selectedShapes, true, cmd);
+    new KoShapeGroupCommand(group, selectedShapes, false, true, true, cmd);
     new KoKeepShapesSelectedCommand({}, {group}, selection, true, cmd);
     canvas()->addCommand(cmd);
 
@@ -1443,7 +1463,7 @@ KoInteractionStrategy *DefaultTool::createStrategy(KoPointerEvent *event)
         if (handle != KoFlake::NoHandle) {
             // resizing or shearing only with left mouse button
             if (insideSelection) {
-                return new ShapeResizeStrategy(this, event->point, handle, m_tabbedOptionWidget->useUniformScaling());
+                return new ShapeResizeStrategy(this, event->point, handle);
             }
 
             if (handle == KoFlake::TopMiddleHandle || handle == KoFlake::RightMiddleHandle ||
