@@ -37,9 +37,8 @@
 
 #include <kis_debug.h>
 
-ShapeResizeStrategy::ShapeResizeStrategy(KoToolBase *tool, const QPointF &clicked, KoFlake::SelectionHandle direction, bool forceUniformScalingMode)
-    : KoInteractionStrategy(tool),
-      m_forceUniformScalingMode(forceUniformScalingMode)
+ShapeResizeStrategy::ShapeResizeStrategy(KoToolBase *tool, const QPointF &clicked, KoFlake::SelectionHandle direction)
+    : KoInteractionStrategy(tool)
 {
     Q_ASSERT(tool->canvas()->shapeManager()->selection());
     Q_ASSERT(tool->canvas()->shapeManager()->selection()->count() > 0);
@@ -211,7 +210,7 @@ void ShapeResizeStrategy::resizeBy(const QPointF &stillPoint, qreal zoomX, qreal
         m_executedCommand.reset();
     }
 
-    const bool usePostScaling = m_selectedShapes.size() > 1 || m_forceUniformScalingMode;
+    const bool usePostScaling = m_selectedShapes.size() > 1;
 
     m_executedCommand.reset(
          new KoShapeResizeCommand(
