@@ -46,6 +46,7 @@
 #define FILES_DEFAULT_DATA_DIR "."
 #endif
 
+#include "qimage_test_util.h"
 
 /**
  * Routines that are useful for writing efficient tests
@@ -63,61 +64,12 @@ inline KisNodeSP findNode(KisNodeSP root, const QString &name) {
         child = child->nextSibling();
     }
 
-    return 0;
-}
-
-
-inline QString fetchExternalDataFileName(const QString relativeFileName)
-{
-    static QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
-    static QString unittestsDataDirPath = "KRITA_UNITTESTS_DATA_DIR";
-
-    QString path;
-    if (!env.contains(unittestsDataDirPath)) {
-        warnKrita << "Environment variable" << unittestsDataDirPath << "is not set";
-        return QString();
-    } else {
-        path = env.value(unittestsDataDirPath, "");
-    }
-
-    QString filename  =
-        path +
-        QDir::separator() +
-        relativeFileName;
-
-    return filename;
-}
-
-inline QString fetchDataFileLazy(const QString relativeFileName, bool externalTest = false)
-{
-    if (externalTest) {
-        return fetchExternalDataFileName(relativeFileName);
-    } else {
-        QString filename  =
-            QString(FILES_DATA_DIR) +
-            QDir::separator() +
-            relativeFileName;
-
-        if (QFileInfo(filename).exists()) {
-            return filename;
-        }
-
-        filename  =
-            QString(FILES_DEFAULT_DATA_DIR) +
-            QDir::separator() +
-            relativeFileName;
-
-        if (QFileInfo(filename).exists()) {
-            return filename;
-        }
-    }
-
-    return QString();
+    return KisNodeSP();
 }
 
 inline void dumpNodeStack(KisNodeSP node, QString prefix = QString("\t"))
 {
-    dbgKrita << node->name();
+    qDebug() << node->name();
     KisNodeSP child = node->firstChild();
 
     while (child) {
@@ -125,7 +77,7 @@ inline void dumpNodeStack(KisNodeSP node, QString prefix = QString("\t"))
         if (child->childCount() > 0) {
             dumpNodeStack(child, prefix + "\t");
         } else {
-            dbgKrita << prefix << child->name();
+            qDebug() << prefix << child->name();
         }
         child = child->nextSibling();
     }
@@ -137,92 +89,40 @@ public:
         : m_min(0), m_max(0), m_value(0)
     {}
 
-    int maximum() const {
+    int maximum() const override {
         return m_max;
     }
-    void setValue(int value) {
+    void setValue(int value) override {
         m_value = value;
     }
-    void setRange(int min, int max) {
+    void setRange(int min, int max) override {
         m_min = min;
         m_max = max;
     }
-    void setFormat(const QString &format) {
+    void setFormat(const QString &format) override {
         m_format = format;
     }
 
+    void setAutoNestedName(const QString &name) {
+        m_autoNestedName = name;
+        KoProgressProxy::setAutoNestedName(name);
+    }
+
     int min() { return m_min; }
     int max() { return m_max; }
     int value() { return m_value; }
     QString format() { return m_format; }
+    QString autoNestedName() { return m_autoNestedName; }
+
 
 private:
     int m_min;
     int m_max;
     int m_value;
     QString m_format;
+    QString m_autoNestedName;
 };
 
-
-inline bool compareQImages(QPoint & pt, const QImage & image1, const QImage & image2, int fuzzy = 0, int fuzzyAlpha = 0, int maxNumFailingPixels = 0)
-{
-    //     QTime t;
-    //     t.start();
-
-    const int w1 = image1.width();
-    const int h1 = image1.height();
-    const int w2 = image2.width();
-    const int h2 = image2.height();
-    const int bytesPerLine = image1.bytesPerLine();
-
-    if (w1 != w2 || h1 != h2) {
-        pt.setX(-1);
-        pt.setY(-1);
-        dbgKrita << "Images have different sizes" << image1.size() << image2.size();
-        return false;
-    }
-
-    int numFailingPixels = 0;
-
-    for (int y = 0; y < h1; ++y) {
-        const QRgb * const firstLine = reinterpret_cast<const QRgb *>(image2.scanLine(y));
-        const QRgb * const secondLine = reinterpret_cast<const QRgb *>(image1.scanLine(y));
-
-        if (memcmp(firstLine, secondLine, bytesPerLine) != 0) {
-            for (int x = 0; x < w1; ++x) {
-                const QRgb a = firstLine[x];
-                const QRgb b = secondLine[x];
-                const bool same = qAbs(qRed(a) - qRed(b)) <= fuzzy
-                                  && qAbs(qGreen(a) - qGreen(b)) <= fuzzy
-                                  && qAbs(qBlue(a) - qBlue(b)) <= fuzzy;
-                const bool sameAlpha = qAlpha(a) - qAlpha(b) <= fuzzyAlpha;
-                const bool bothTransparent = sameAlpha && qAlpha(a)==0;
-
-                if (!bothTransparent && (!same || !sameAlpha)) {
-                    pt.setX(x);
-                    pt.setY(y);
-                    numFailingPixels++;
-
-                    dbgKrita << " Different at" << pt
-                             << "source" << qRed(a) << qGreen(a) << qBlue(a) << qAlpha(a)
-                             << "dest" << qRed(b) << qGreen(b) << qBlue(b) << qAlpha(b)
-                             << "fuzzy" << fuzzy
-                             << "fuzzyAlpha" << fuzzyAlpha
-                             << "(" << numFailingPixels << "of" << maxNumFailingPixels << "allowed )";
-
-
-                    if (numFailingPixels > maxNumFailingPixels) {
-                        return false;
-                    }
-                }
-            }
-        }
-    }
-    //     dbgKrita << "compareQImages time elapsed:" << t.elapsed();
-    //    dbgKrita << "Images are identical";
-    return true;
-}
-
 inline bool comparePaintDevices(QPoint & pt, const KisPaintDeviceSP dev1, const KisPaintDeviceSP dev2)
 {
     //     QTime t;
@@ -251,7 +151,7 @@ inline bool comparePaintDevices(QPoint & pt, const KisPaintDeviceSP dev1, const
         iter1->nextRow();
         iter2->nextRow();
     }
-    //     dbgKrita << "comparePaintDevices time elapsed:" << t.elapsed();
+    //     qDebug() << "comparePaintDevices time elapsed:" << t.elapsed();
     return true;
 }
 
@@ -262,7 +162,7 @@ inline bool comparePaintDevicesClever(const KisPaintDeviceSP dev1, const KisPain
     QRect rc2 = dev2->exactBounds();
 
     if (rc1 != rc2) {
-        dbgKrita << "Devices have different size" << ppVar(rc1) << ppVar(rc2);
+        qDebug() << "Devices have different size" << ppVar(rc1) << ppVar(rc2);
         return false;
     }
 
@@ -280,9 +180,9 @@ inline bool comparePaintDevicesClever(const KisPaintDeviceSP dev1, const KisPain
 
                 if (p1[3] < alphaThreshold && p2[3] < alphaThreshold) continue;
 
-                dbgKrita << "Failed compare paint devices:" << iter1->x() << iter1->y();
-                dbgKrita << "src:" << p1[0] << p1[1] << p1[2] << p1[3];
-                dbgKrita << "dst:" << p2[0] << p2[1] << p2[2] << p2[3];
+                qDebug() << "Failed compare paint devices:" << iter1->x() << iter1->y();
+                qDebug() << "src:" << p1[0] << p1[1] << p1[2] << p1[3];
+                qDebug() << "dst:" << p2[0] << p2[1] << p2[2] << p2[3];
                 return false;
             }
         } while (iter1->nextPixel() && iter2->nextPixel());
@@ -296,103 +196,6 @@ inline bool comparePaintDevicesClever(const KisPaintDeviceSP dev1, const KisPain
 
 #ifdef FILES_OUTPUT_DIR
 
-inline bool checkQImageImpl(bool externalTest,
-                            const QImage &srcImage, const QString &testName,
-                            const QString &prefix, const QString &name,
-                            int fuzzy, int fuzzyAlpha, int maxNumFailingPixels)
-{
-    QImage image = srcImage.convertToFormat(QImage::Format_ARGB32);
-
-    if (fuzzyAlpha == -1) {
-        fuzzyAlpha = fuzzy;
-    }
-
-    QString filename(prefix + "_" + name + ".png");
-    QString dumpName(prefix + "_" + name + "_expected.png");
-
-    const QString standardPath =
-        testName + QDir::separator() +
-        prefix + QDir::separator() + filename;
-
-    QString fullPath = fetchDataFileLazy(standardPath, externalTest);
-
-    if (fullPath.isEmpty() || !QFileInfo(fullPath).exists()) {
-        // Try without the testname subdirectory
-        fullPath = fetchDataFileLazy(prefix + QDir::separator() +
-                                     filename,
-                                     externalTest);
-    }
-
-    if (fullPath.isEmpty() || !QFileInfo(fullPath).exists()) {
-        // Try without the prefix subdirectory
-        fullPath = fetchDataFileLazy(testName + QDir::separator() +
-                                     filename,
-                                     externalTest);
-    }
-
-    if (!QFileInfo(fullPath).exists()) {
-        fullPath = "";
-    }
-
-    bool canSkipExternalTest = fullPath.isEmpty() && externalTest;
-    QImage ref(fullPath);
-
-    bool valid = true;
-    QPoint t;
-    if(!compareQImages(t, image, ref, fuzzy, fuzzyAlpha, maxNumFailingPixels)) {
-        bool saveStandardResults = true;
-
-        if (canSkipExternalTest) {
-            static QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
-            static QString writeUnittestsVar = "KRITA_WRITE_UNITTESTS";
-
-            int writeUnittests = env.value(writeUnittestsVar, "0").toInt();
-            if (writeUnittests) {
-                QString path = fetchExternalDataFileName(standardPath);
-
-                QFileInfo pathInfo(path);
-                QDir directory;
-                directory.mkpath(pathInfo.path());
-
-                dbgKrita << "--- Saving reference image:" << name << path;
-                image.save(path);
-                saveStandardResults = false;
-
-            } else {
-                dbgKrita << "--- External image not found. Skipping..." << name;
-            }
-        } else {
-            dbgKrita << "--- Wrong image:" << name;
-            valid = false;
-        }
-
-        if (saveStandardResults) {
-            image.save(QString(FILES_OUTPUT_DIR) + QDir::separator() + filename);
-            ref.save(QString(FILES_OUTPUT_DIR) + QDir::separator() + dumpName);
-        }
-    }
-
-    return valid;
-}
-
-inline bool checkQImage(const QImage &image, const QString &testName,
-                        const QString &prefix, const QString &name,
-                        int fuzzy = 0, int fuzzyAlpha = -1, int maxNumFailingPixels = 0)
-{
-    return checkQImageImpl(false, image, testName,
-                           prefix, name,
-                           fuzzy, fuzzyAlpha, maxNumFailingPixels);
-}
-
-inline bool checkQImageExternal(const QImage &image, const QString &testName,
-                                const QString &prefix, const QString &name,
-                                int fuzzy = 0, int fuzzyAlpha = -1, int maxNumFailingPixels = 0)
-{
-    return checkQImageImpl(true, image, testName,
-                           prefix, name,
-                           fuzzy, fuzzyAlpha, maxNumFailingPixels);
-}
-
 struct ExternalImageChecker
 {
     ExternalImageChecker(const QString &prefix, const QString &testName)
@@ -485,8 +288,8 @@ class TestNode : public DefaultNode
 {
     Q_OBJECT
 public:
-    KisNodeSP clone() const {
-        return new TestNode(*this);
+    KisNodeSP clone() const override {
+        return KisNodeSP(new TestNode(*this));
     }
 };
 
@@ -494,32 +297,32 @@ class TestGraphListener : public KisNodeGraphListener
 {
 public:
 
-    virtual void aboutToAddANode(KisNode *parent, int index) {
+    void aboutToAddANode(KisNode *parent, int index) override {
         KisNodeGraphListener::aboutToAddANode(parent, index);
         beforeInsertRow = true;
     }
 
-    virtual void nodeHasBeenAdded(KisNode *parent, int index) {
+    void nodeHasBeenAdded(KisNode *parent, int index) override {
         KisNodeGraphListener::nodeHasBeenAdded(parent, index);
         afterInsertRow = true;
     }
 
-    virtual void aboutToRemoveANode(KisNode *parent, int index) {
+    void aboutToRemoveANode(KisNode *parent, int index) override {
         KisNodeGraphListener::aboutToRemoveANode(parent, index);
         beforeRemoveRow  = true;
     }
 
-    virtual void nodeHasBeenRemoved(KisNode *parent, int index) {
+    void nodeHasBeenRemoved(KisNode *parent, int index) override {
         KisNodeGraphListener::nodeHasBeenRemoved(parent, index);
         afterRemoveRow = true;
     }
 
-    virtual void aboutToMoveNode(KisNode *parent, int oldIndex, int newIndex) {
+    void aboutToMoveNode(KisNode *parent, int oldIndex, int newIndex) override {
         KisNodeGraphListener::aboutToMoveNode(parent, oldIndex, newIndex);
         beforeMove = true;
     }
 
-    virtual void nodeHasBeenMoved(KisNode *parent, int oldIndex, int newIndex) {
+    void nodeHasBeenMoved(KisNode *parent, int oldIndex, int newIndex) override {
         KisNodeGraphListener::nodeHasBeenMoved(parent, oldIndex, newIndex);
         afterMove = true;
     }
@@ -556,8 +359,8 @@ struct MaskParent
         const KoColorSpace * cs = KoColorSpaceRegistry::instance()->rgb8();
         undoStore = new KisSurrogateUndoStore();
         image = new KisImage(undoStore, imageRect.width(), imageRect.height(), cs, "test image");
-        layer = new KisPaintLayer(image, "paint1", OPACITY_OPAQUE_U8);
-        image->addNode(layer);
+        layer = KisPaintLayerSP(new KisPaintLayer(image, "paint1", OPACITY_OPAQUE_U8));
+        image->addNode(KisNodeSP(layer.data()));
     }
 
     KisSurrogateUndoStore *undoStore;
@@ -598,10 +401,10 @@ public:
 private:
     void printValues(bool force = false) {
         if (m_cycles > m_period || force) {
-            dbgKrita << "Val / Total:" << qreal(m_val) / qreal(m_total);
-            dbgKrita << "Avg. Val:   " << qreal(m_val) / m_cycles;
-            dbgKrita << "Avg. Total: " << qreal(m_total) / m_cycles;
-            dbgKrita << ppVar(m_val) << ppVar(m_total) << ppVar(m_cycles);
+            qDebug() << "Val / Total:" << qreal(m_val) / qreal(m_total);
+            qDebug() << "Avg. Val:   " << qreal(m_val) / m_cycles;
+            qDebug() << "Avg. Total: " << qreal(m_total) / m_cycles;
+            qDebug() << ppVar(m_val) << ppVar(m_total) << ppVar(m_cycles);
 
             m_val = 0;
             m_total = 0;
@@ -616,6 +419,71 @@ private:
     qint64 m_cycles;
 };
 
+struct MeasureDistributionStats {
+    MeasureDistributionStats(int numBins, const QString &name = QString())
+        : m_numBins(numBins),
+          m_name(name)
+    {
+        reset();
+    }
+
+    void reset() {
+        m_values.clear();
+        m_values.resize(m_numBins);
+    }
+
+    void addValue(int value) {
+        addValue(value, 1);
+    }
+
+    void addValue(int value, int increment) {
+        KIS_SAFE_ASSERT_RECOVER_RETURN(value >= 0);
+
+        if (value >= m_numBins) {
+            m_values[m_numBins - 1] += increment;
+        } else {
+            m_values[value] += increment;
+        }
+    }
+
+    void print() {
+        qCritical() << "============= Stats ==============";
+
+        if (!m_name.isEmpty()) {
+            qCritical() << "Name:" << m_name;
+        }
+
+        int total = 0;
+
+        for (int i = 0; i < m_numBins; i++) {
+            total += m_values[i];
+        }
+
+        for (int i = 0; i < m_numBins; i++) {
+            if (!m_values[i]) continue;
+
+            const QString lastMarker = i == m_numBins - 1 ? "> " : "  ";
+
+            const QString line =
+                QString("  %1%2: %3 (%4%)")
+                    .arg(lastMarker)
+                    .arg(i, 3)
+                    .arg(m_values[i], 5)
+                    .arg(qreal(m_values[i]) / total * 100.0, 7, 'g', 2);
+
+            qCritical() << qPrintable(line);
+        }
+        qCritical() << "----                          ----";
+        qCritical() << qPrintable(QString("Total: %1").arg(total));
+        qCritical() << "==================================";
+    }
+
+private:
+    QVector<int> m_values;
+    int m_numBins = 0;
+    QString m_name;
+};
+
 QStringList getHierarchy(KisNodeSP root, const QString &prefix = "");
 bool checkHierarchy(KisNodeSP root, const QStringList &expected);
 
